;+
;NAME:
;mmselectgood - select good points from mm array
;SYNTAX: n	=mmselectgood(mm,mmGood,ltI,tdbad=tdbad,lrTdI=lrTdI,ldate=ldate,hard=hard,$
;                          gain=gain,bmwid=bmwid,vg=vg,vb=vb,retall=retall,verb=verb)
;ARGS:
;   mm[m]:{}    calibration data to use as input
;KEYWORDS:
;tdbad[] :  int   specify tiedowns that have bad tensions
;                 (count 1..6 -> td12-1,td12-2,td4-1,td4-2,td8-1,td8-1
;                 the routine will copy the tension of the "other" cable
;                 for that td. If both cables on a td are bad, you are
;                 out of luck.
;ldate   : string for plots
;hard    : int    if true then make hardcopy of result
;psname  : int    if hard then psname to use. def: idl.ps
;gain    : int    if set then include gain plot
;bmwid   : int    if set then include beamwid
;vg [2]  : float   vertical range for gain plot
;vb [2]  : float   vertical range for beamwidth
;ret_all :         if set then return all, no user input
;verb    :         if set then print progress
;RETURNS:
;   n       : long    number of good points in mmGood
; mmGood[n] :{}       good data selected from mm
; ltI[n]    :{}       tdkips and platform height for each good pnt
; lrTdI[]   :{}       laser range and td info input that overlaps the
;                     the mm time range.
;DESCRIPTION: 
;   Let the user select calibration scans that are good,
;The routine will input the platform height and td tension data that
;corresponds to the mm[m] time range.
;It will then plot the azErr, zaErr, td tensions, and platform height.
;The user can then interactively exclude points that have a bad platform height
;or low td tension.
;	The array mmGood[n] are the resulting good calibration scans.
;ltI[n] holds the interplated platform height, tdKips for each mmGood entry.
; The lrTdI[] array holds the original td, platform height data input.
; 
;Before calling this routine, you must
; @lrinit,@tdinit,@corinit

function mmselectgood,mm,mmG,ltI,tdbad=tdbad,lrtdI=lrTdI,ldate=ldate,hard=hard,psname=psname,$
			gain=gain,bmwid=bmwid,vb=vb,vg=vg,ret_all=ret_all,verb=verb
	    common colph,decomposedph,colph
;
	use3=1			; if missing distomat, use 3 dist model
	verbl=keyword_set(verb)
	dohard=keyword_set(hard)
	if n_elements(ret_all) eq 0 then ret_all=0
	if (dohard) then begin
		if n_elements(psname) eq 0 then psname='idl.ps'
	endif
	if n_elements(vb) ne 2 then begin
		vbl=[0,220]
	endif else begin
		vbl=vb
	endelse
	if n_elements(vg) ne 2 then begin
		vgl=[0,12]
	endif else begin
		vgl=vg
	endelse
		
	minKipsOk=1.
	ldate=(n_elements(ldate) gt 0) ? ldate:''
;
; get the lr,td data for this time range
;
	if verbl then print,"start mmgetlrtd"
	n=mmgetlrtd(mm,lrTdI,tdbad=tdbad,verb=verb,use3=use3)
	if n eq 0 then begin
	  print,"No lr,td info found for this time range"
	  return,0
	endif
	if verbl then print,"done mmgetlrtd"
; for every mm entry record min,max,avg value for kips,platHght
	a={ min:0.,$
    	max:0.,$
    	avg:0.}
	aa={ mjd:0d,$ ; start
     	nsmp:0l,$
     	kips:replicate(a,3),$ ; use total kips td
     	plh :a}
; pattern length, fraction of a day
	patDura=321d/86400d
	eps=10/86400.
	nmm=n_elements(mm)
	ltIinp=replicate(aa,n)
;    loop through mm finding min,max,avg lr,td info for each entry.
	for i=0,nmm-1 do begin &$
    	ii=where((lrtdi.mjd  ge (mm[i].julday-eps)) and (lrtdi.mjd le (mm[i].julday+patDura+eps)),cnt) &$
;   default to mm start. update ot lr time if we find lr samples
    	ltIinp[i].mjd=mm[i].julday &$
    	if cnt gt 0 then begin &$
        	for j=0,2 do begin &$
           		ltIinp[i].kips[j].min=min(lrtdI[ii].tdkips[*,j]) &$
           		ltIinp[i].kips[j].max=max(lrtdI[ii].tdkips[*,j]) &$
           		ltIinp[i].kips[j].avg=mean(lrtdI[ii].tdkips[*,j]) &$
        	endfor &$
        ; for avg plathght, exclude samples with no readings
        	avghPl=lrtdI[ii].avghpl &$
        	jj=where(avghPl gt 1.,cnt) &$
        	if (cnt ne 0) then begin &$
            	ltIinp[i].plh.min=min(avghPl[jj]) &$
            	ltIinp[i].plh.max=max(avghPl[jj]) &$
            	ltIinp[i].plh.avg=mean(avghPl[jj]) &$
        	endif else begin &$
            	ltIinp[i].plh.min=0. &$
            	ltIinp[i].plh.max=0. &$
            	ltIinp[i].plh.avg=0. &$
        	endelse &$
        	ltIinp[i].mjd=lrtdI[ii[0]].mjd &$
    	endif &$
    	ltIinp[i].nsmp=cnt &$
	endfor
;
; 	now loop twice
; 	once to let them specify bad points
;   a second time to plot the results..
;
;  plot vs start time of mm array
redo:
	selectBadDone=0  ; see if in the select loop
	if ret_all then begin
		ok=intarr(n)+1
    	iiOk=where(ok eq 1,cntiiok)
		selectBadDone=1
	endif 
	nbad=0l
	while 1 do begin
		hor
		ver
		x0=mm[0].julday  		; start of x axis	
		xmm=(mm.julday - x0)*24. 	; put in hours from start
		xltInp=(ltIinp.mjd - x0)*24.
;
; find points with any td tension less than minkipsOk
;
		ii=where((reform(ltIinp.kips[0].min) lt minkipsok) or $
                 (reform(ltIinp.kips[1].min) lt minkipsok) or $
                 (reform(ltIinp.kips[2].min) lt minkipsok),cnt)
		az=mm.az
		za=mm.za
		azErr=mm.fit.azerr*60
		zaErr=mm.fit.zaerr*60
        gainM=mm.fit.gain
	    bmWidM=mm.fit.bmwidavg*60.
		smp=lindgen(nmm)
		symAz=-1
		symZa=-2
		col=[1,3,4,5,6,7,8,9,10]

		cs=2
	    if dohard and  selectBadDone then pscol,psname,/full
		ll=4
		if keyword_set(gain) then ll++
		if keyword_set(bmwid) then ll++
		!p.multi=[0,1,ll]
		ver,-30,30
		hor,-10,nmm+5
		xind=lindgen(nmm)
		x=xind
		sclln=.7
;
;
; plot vs index
;
		ln=2
		ver,-40,40
		mmplotsrc,x,zaerr,mm,/rise,col=col,chars=cs,sclln=sclln,ln=ln,$
    		xtit='pattern #',yit='Asec',tit=ldate + ' Za error vs pattern index'
		if cnt gt 0 then oplot,x[ii],zaerr[ii],psym=1,col=colph[2]
; flag  bad points we actually used, only after selectBadDone
		if selectBadDone then begin
    		for i=0,nbad-1 do begin &$
        		jj=where(mm.scan eq uscanbad[i]) &$
        		flag,jj,linestyle=1 &$
    		endfor &$
		endif
;---------------------------------------------------------------
		mmplotsrc,x,azerr,mm,/rise,col=col,chars=cs,/nolab,$
    		xtit='pattern #',yit='Asec',tit='Az error vs pattern index'
		if cnt gt 0 then oplot,x[ii],azerr[ii],psym=1,col=colph[2]
		if selectBadDone then begin
    		for i=0,nbad-1 do begin &$
        		jj=where(mm.scan eq uscanbad[i]) &$
        		flag,jj,linestyle=1 &$
    		endfor &$
		endif
;---------------------------------------------------------------
; plot td tensions
		ver,0,40
		stripsxy,x,transpose(ltIinp.kips.avg),0,0,/step,chars=cs ,$
    		xtit='Pattern #',ytit='Kips',Tit='Td kips vs pat index.(sum cables)'
		if cnt gt 1 then $
			oplot,x[ii],ltIinp[ii].kips[0].avg,psym=2,col=colph[5]
		ltd=['Td12','Td4','Td8']
		ln=18
		xp=.5
		xpinc=.15
;		for i=0,2 do note,ln,ltd[i],chars=csn,font=font,col=colph[i+1],xp=xp+xpinc*i
		x0= -9
		y0=5
		xinc=3
		for i=0,2 do xyouts,x0+i*xinc,y0,ltd[i],chars=csn,font=font,col=colph[i+1]
		if (selectBadDone) then begin 
    		for i=0,nbad-1 do begin &$
        		jj=where(mm.scan eq uscanbad[i]) &$
        		flag,jj,linestyle=1 &$
    		endfor &$
		endif
		avgDef=1256.22
		ver,-5,5
		plot,x,(ltIinp.plh.avg - avgDef)*12.*2.54,psym=-2,chars=cs,$
    		xtit='pattern #',ytit='Hght-defHght[cm]',tit='cm platform above default hght vs patIndex'
		if (selectBadDone) then begin 
    		for i=0,nbad-1 do begin &$
        		jj=where(mm.scan eq uscanbad[i]) &$
        		flag,jj,linestyle=1 &$
    		endfor &$
		endif
		if keyword_set(gain) then begin
		ver,vgl[0],vgl[1]
		plot,x,gainM,psym=-2,chars=cs,$
    		xtit='pattern #',ytit='K/Jy',tit='gain'
		if selectBadDone then begin
    		for i=0,nbad-1 do begin &$
        		jj=where(mm.scan eq uscanbad[i]) &$
        		flag,jj,linestyle=1 &$
    		endfor &$
		endif
		endif
		if keyword_set(bmwid) then begin
		ver,vbl[0],vbl[1]
		plot,x,bmwidM,psym=-2,chars=cs,$
    		xtit='pattern #',ytit='Asecs',tit='bmWidth'
		if selectBadDone then begin
    		for i=0,nbad-1 do begin &$
        		jj=where(mm.scan eq uscanbad[i]) &$
        		flag,jj,linestyle=1 &$
    		endfor &$
		endif
		endif
;
; to find bad points interactively
		if (selectBadDone eq 0) then begin &$
			print,"Left button flags bad scan. right button when done"

    		nbad=0
    		i=rdcur(icur) &$
   			if i gt 0 then begin
        		ibad=long(reform(icur[0,*])+.5) &$
        		scanBad=mm[ibad].scan &$
        		uscanbad=scanBad[uniq(scanbad,sort(scanbad))] &$
        		nbad=n_elements(uscanbad) &$
    		endif
    		ok=intarr(n) + 1
    		for i=0,nbad-1 do begin &$
        		jj=where(mm.scan eq uscanbad[i]) &$
        		ok[jj]=0 &$
    		endfor &$
    		iiOk=where(ok eq 1,cntiiok)
	        selectBadDone=1  ; see if in the select loop
    		continue	; at top of while loop
		endif
		break
	endwhile
;
; now overplot az,za pointing error vs za. with points that we manually excluded
;
	usrc=mm[uniq(mm.srcname,sort(mm.srcname))].srcname
	nsrc=n_elements(usrc)
	if not dohard  then  begin
		key=checkkey()
		print,'r to redo, enter to continue'
		key=checkkey(/wait)
		if key eq 'r' then begin
			goto,redo
		endif
	endif
	!p.multi=[0,1,4]
	ver,-30,30
	hor,0,20
	xp=.04
	ln=2
	scl=.7
	for isrc=0,nsrc-1 do begin &$
    	jj=where(mm.srcname eq usrc[isrc],cnt) &$
    	mmplotsrc,za[jj],zaErr[jj],mm[jj],psym=-1,chars=cs,/nolab,$
        tit=usrc[isrc] + ' az,za error vs za',/rise&$
;   plot,za[jj],zaErr[jj],psym=-1,chars=cs,$
;       tit=usrc[i] + ' az,za error vs za'&$
    	mmplotsrc,za[jj],azErr[jj],mm[jj],psym=-2,chars=cs,/nolab ,$
        tit=usrc[isrc] + ' az,za error vs za',/rise,/over,col=[4]&$
;   oplot,za[jj],azErr[jj],psym=-2 &$
;;  kk=where((mm[ii].srcname eq usrc[isrc]) and (ok eq 0),cntb) &$
    	kk=where((mm.srcname eq usrc[isrc]) and (ok eq 0),cntb) &$
    	if cntb gt 0 then begin &$
        	plots,za[kk],zaErr[kk],psym=1,col=colph[2]
        	plots,za[kk],azErr[kk],psym=1,col=colph[2]
;       for k=0,cntb-1 do begin &$
;           l=ii[kk[k]] &$
;           plots,za[l],zaErr[l],psym=1,col=colph[2] &$
;           plots,za[l],azErr[l],psym=2,col=colph[2] &$
;       endfor &$
    	endif &$
    	if (isrc mod 4) eq 0 then begin &$
        	note,ln,"+ za error",chars=csn,font=font,xp=xp &$
        	note,ln+1*scl,"* az Error",chars=csn,font=font,xp=xp,col=colph[4] &$
    	endif &$
    	if ((isrc mod 4) eq 3) and (dohard eq 0)  then key=checkkey(/wait) &$
	endfor
	if dohard then begin
		hardcopy
		x
		ldcolph
	endif
	if cntiiok eq 0 then return,0
	mmG=mm[iiok]
	ltI=ltIinp[iiok]
	return,n_elements(iiok)
end
