;+
;NAME:
;agcazencdifinpavg - average azimuth enc difference to 1 deg bins
;SYNTAX: ndays=agcazencdifinpavg(yymmdd,dIAr,ndays=ndays,azLoc=azLoc,range=range.verb=verb)
; ARGS:
; yymmdd: long     start date to input
;KEYWORDS:
;  ndays : long    number of days to process. def=1
;                  if ndays < 0 then start ndays-1 before yymmdd. (-1,1 is always current day
;  verb  :         if set the list days as they are input
;RETURNS:
; ndays:   long    number of days we actually found with data
; dIAr[ndays] : {}      holds the enc average difference info for each day
;azLoc[360];   fltarr  return the azimuth values used for the 360 bins
;range[]   :  long     return the first,last yymmdd used for the plots.
;                      if only one day, the range[1]. if multiple days then range[2]
;
;DESCRIPTION:
;	Input ndays of agc data starting at yymmdd. If ndays is supplied then input abs(ndays).
; If ndays >0 start at yymmdd. If ndays is < 0 then start at yymmdd - (abs(nday)-1). After inputting the data,
;compute the avgerage encoder differencefor each deg 0 to 360. The bins are at .5,1.5,.. to 359.5 deg azimuth
;The averaged data for each day is returned in dIAr[ndays]
;help,dIAr,/st
;** Structure <ee7608>, 5 tags, length=4328, data length=4328, refs=1:
;   YYMMDD          LONG            191106    start date in b 
;   NTOT            LONG                 0    number of total points we used for this day
;   CNTAR           LONG      Array[360]      number of pnts for each 1 deg bin
;   AZDAVG          FLOAT     Array[360]      average each 1 deg bin
;   AZDMED          FLOAT     Array[360]      median of each 1 deg bin.
;
;The routine only uses azmimuth samples where the azimuth is moving (abs(encoder velocity) > .0005  deg/sec).
;azloc contains the azimuth value for each of the 360 bins.
;-
;	
function agcazencdifinpavg,yymmdd,dIAr,ndays=ndays,azloc=azloc,range=range,verb=verb
;
;		
	if n_elements(ndays) eq 0 then ndays=1
	lndays=(ndays lt 0)?-ndays:ndays
;  limits for data
	minVel=.0005  ; deg/sec --> we're movig
	maxVel=.5     ; deg/sec 
	maxAzDif=1.   ; deg  to exclude bad points
	azloc=findgen(360) + .5
	useverb=keyword_set(verb)
;	print,"n_elements verb:",n_elements(verb)
;	if n_elements(verb) gt 0 then print,"verb",verb
;	print,"useverb:",useverb
;
	difI={ yymmdd: 0L,$
		 ntot  : 0l,$ ; points we found within vel range
         cntAr : lonarr(360),$
		 azDAvg:fltarr(360),$
		 azDMed:fltarr(360)}
;
	jd0=yymmddtojulday(yymmdd)
	dIAr=replicate(difI,lndays)
    icur=0 
	range1=yymmdd
	range2=0l
	if ndays lt 0 then begin
		istart=ndays +1
		iend=0
    endif else begin	
		istart=0
		iend=ndays-1
	endelse
	starting=1
    for iday=istart,iend do begin
;		
;		input day
;
        jd=jd0 + iday
;       back to yymmdd
        caldat,jd,m,d,y
        yymmdd1=y*10000l + m*100 + d
	    if starting then begin
			range1=yymmdd1
			starting=0
		endif
		range2=yymmdd1
	    if useverb then print,"inputing: ",yymmdd1
        n=agcinpday(yymmdd1,b)
        if n eq 0 then begin
            print,"no data found for agcinpday",yymmdd1
            continue
        endif
;
;	    compute the average
;
;	throw out points we don't wont.
;   only use when we are moving
;   use encoder vel since motor velocity has offset
;   also exclude points that have bad azvalue
		az=b.cb.pos[0]
		tm=b.cb.time
		tm[0]=tm[1]-1
		dtm=tm - shift(tm,1)
		dtm[0]=dtm[1]
		daz=az -shift(az,1)
		daz[0]=daz[1]
		velE=daz/dtm
;
		y=abs(velE)
		ii=where((abs(velE) ge minvel) and (abs(velE lt maxVel)) and (abs(b.fb.azencdif) lt maxAzDif),cnt)
		if cnt eq 0 then continue
		azDif=b[ii].fb.azencdif
		az=b[ii].cb.pos[0] mod 360.
		ntot=0L
		for i=0,359-1 do begin
			y=az - azloc[i]
		 	ii=where((y  ge -.5) and (y lt .5) ,cnt) &$
   		 	if cnt gt 0 then   begin
				ntot+=cnt
				yy=azdif[ii]
				case cnt of 
					1: begin
						diAr[icur].azDAvg[i]=yy
						dIar[icur].azDMed[i]=yy
		  	   		end
					2: begin
					 	dIAr[icur].azDAvg[i]=mean(yy)
						dIAr[icur].azDMed[i]=dIar[icur].azDAvg[i]
			   	    	end
					else: begin
						dIar[icur].azDAvg[i]=mean(yy)
						dIAr[icur].azDMed[i]=median(yy)
			   			end
				endcase
 			endif
			dIAr[icur].cntAr[i]=cnt
		endfor
		dIAr[icur].ntot=ntot
		dIAr[icur].yymmdd=yymmdd1
	    icur++
	endfor
	if icur ne lndays then dIAr=dIar[0:icur-1]
	range=((range1 eq range2) or (range2 eq 0))?range1:[range1,range2]
	return,icur
end
