;+
;NAME:
;shsclpquicklook - compute a clp spectra and display image
;SYNTAX: shsclpquicklook,fnum,fbase,dir0=dir0,$
;    fband=fband,doband=doband,
;	firstHghtKm=firsthghtKm,lastHghtKm=lastHghtKm,hghtStepUsec=hghtStepUsec,$
;    zx=zx,zy=zy,$
;    dinfo=dinfo,hdr=hdr,img1=img1,img2=img2 ,
;ARGS:
;   fnum: long		file number to process. -1 --> last file
;  fbase: string    name of subdir in dir0 to search. eg let fbase='expabc'
;                   filenum =-1 searches dir0+'expabc/*.shs
;                   filenum =val searches dir0+'expabc/expabc_nnn.shs'
;KEYWORDS:
;   dir0: string    base dir. default: /net/rserv/export/RSERVLV2/daeron/'
;   fband: float[2] frequenciy offsets (in Mhz) for bands. def:[-5.5,5.5]
;   doband: int     bands to process. 1->band 1, 2-> band 2, 12-> band 1 and 2
;                   (12 does not currently merge images).
;firstHghtKm:float  first height to display. def=90km.
;                   If less than first height then first height will be used.
; lastHghtKm:float  last height to display. def=450Km
;                   If greater than last height recorded, then clip to last height.
;hghtStepUsec:float Spacing between heights (in usec, not km). def: 2usec
;                   the baud is  normally 2usec.
;zx          :float  zoom factor in x. def: -5 if zx=0 then use default
;zy          :float  zoom factor in y. def: search for zy giving le 790 pixels
;RETURNS:
;  dinfo: {}	     struct holding info used in compuatation
; hdr   : {}         header read from file
; img1[n,m]: float   img 1 (if selected). this has not been zoomed.
; img2[n,m]: float   img 2 (if selected). this has not been zoomed.
;DESCRIPTION:
; 	Read and process an echotek clp image. It tries to read
;10 secs of data. It will always start at the begining of the 
;selected file.
;
;To select the file:
; - dir0: is the directory holding the experiment directories
;   the default is /net/rserv/export/RSERVLV2/daeron/. This the destination
;   for the copy from daeron to aonet.
;   Another alternative would be to read it directly from daeron:
;   /net/daeron/data/
; - fbase: is the base filename
;   this is the subdirectory in dir0 and well as the start of the filename.
;   eg: 'hf_20180315'
; - fnum: this is the file number in the subdirectory to input
;   fnum=-1 will use the most recent file.
;
;Bands to process:
;   There are normally 2 bands [-5.5,5.5] if up,down or [5.5,9.5] upper band 
;  The parameter doband lets you select which bands to process. 
;   doband=1 --> process only band 1, doband=2 --> process only band 2,
;   doband=12 --> process both bands. By default both  bands are processed.
;   Processing fewer bands speeds up the processing.
;
;Heights to process
;	By default all heights 90 through 450 km are processed with 2 usec spacing (same as the def baud)
;   firstHghtKm  specifies a different first height, Asking for a height before the first available
;                height was def to the first height.
;   lastHghtKm   specifies a different ending height. asking for a height greater than available will
;                default to the last available height
;   heightStepUsec: spacing between heights.
; 
;image zoom
; 	zx,zy - x,y image zoom. Positive make the image bigger, neg shrinks the dimension (zoom needs to be integral).
;	In x there are 4096 channels. a zoom of -4 would make an image with 1024 horizontal pixels.
;   In y, a standard can support 795 pixels (the axis are extra). 
; 	if zx=0 or is not supplied then it defaults to -4.
;   if zy=0 or is not supplied then the zy is computed such that there are as many pixels as possible and does
;   not exceed 795 pixels.
;	eg with 90km to 450 km there are (450-90)/.15(km/usec) = 2400 pixels. It would compute zy=-4 giving 600 pixels.
;Returned info.
;   dinfo
;IDL> help,dinfo,/st
;** Structure <1c29b08>, 13 tags, length=44056, data length=40050, refs=1:
;   GWUSEC          DOUBLE          0.20000000
;   BAUDLENUSEC     FLOAT           2.00000
;   CODELENGW       LONG              2200
;   NHGHTS          LONG              1201
;   HGHTSTEPGW      LONG                10
;   IPPAVGED        LONG              1000
;   SPCLEN          INT           4096
;   TXSMPSKIP       LONG                25
;   NUMCHN          INT              2
;   BWMHZ           DOUBLE           5.0000000
;   NSIGCLIP        FLOAT           0.00000
;   USENOISE        INT              0
;   IPPI            STRUCT    -> <Anonymous> Array[1000]
;  hdr: {}
;IDL> help,hdr,/st
;** Structure <21b62f8>, 18 tags, length=104, data length=90, refs=1:
;   TABLEREC        LONG                 0
;   TABLESIZE       LONG           9844000
;   DATAWIDTH       INT              2
;   DATATYPE        STRING    'short'
;   NUMDIMS         INT              2
;   DIM0            LONG             49220
;   DIM1            LONG               200
;   NUMCHANNELS     INT              2
;   TXSTART         LONG                 0
;   TXLEN           LONG              5200
;   DATASTART       LONG              6000
;   DATALEN         LONG             44000
;   NOISESTART      LONG             50000
;   NOISELEN        LONG                20
;   SYSTIME         STRING    '09:30:07'
;   AZ              FLOAT           347.997
;   ZAGREG          FLOAT           1.10000
;   ZACH            FLOAT           12.0000
;
; img1[n,m] : float   image band 1 (if processed)
; img2[n,m] : float   image band 2 (if processed)
;
; Time to compute:
; cpu   time  nbands hgtStep  hghtrange
;       min           usec      km
;megs3  3:30   2       4       90 -690
;megs3  2:10   2       4       90-450
;megs3  1:07   1       4       90-450
;megs3  2:12   1       2       90-450
;- 
pro shsclpquicklook,fnum,fbase,dir0=dir0,$
	firstHghtKm=firsthghtKm,lastHghtKm=lastHghtKm,hghtStepUsec=hghtStepUsec,$
	img1=img1,img2=img2 ,fband=fband,doband=doband,dinfo=dinfo,hdr=hdr,zx=zx,zy=zy

	maxy=792
	forward_function timeit
	kmToUsec=1./.15
	if n_elements(fbase) eq 0 then fbase='t2881_20160602'
	if n_elements(dir0) eq 0 then dir0='/net/rserv/export/RSERVLV2/daeron/'
	if n_elements(firstHghtKm) eq 0 then firstHghtKm=90.
	if n_elements(lastHghtKm) eq 0 then lastHghtKm=450.
	if n_elements(hghtStepUsec) eq 0 then hghtStepUsec=2. ; usecs
	if n_elements(fband) eq 0 then fband=[-5.5,5.5]
	hghtStepKm=hghtStepUsec/kmToUsec
;
; fnum lt 0.. find last
;
	if fnum lt 0 then begin
		d=dir0 + fbase + "/*.shs" 
	    cmd='ls -t ' + d 
	    spawn,cmd,inp
		file=inp[0]
	endif else begin
		if fnum le 999 then begin
			file=dir0 + fbase + "/" + fbase + string(format='("_",i03)',fnum) + ".shs"
		endif else begin
			file=dir0 + fbase + "/" + fbase + string(format='("_",i04)',fnum) + ".shs"
		endelse
		
	endelse
	istat=shsopen(file,desc)
	if istat ne 1 then begin	
		print,"Could not open file:",file
		return
	end
	nchan=desc.dhdr1.numchannels
	if n_elements(doband) eq 0 then begin
		doband=(nchan eq 2)?12:1
	endif
	if ((doband eq 12) or (doband eq 2))  and (nchan eq 1) then begin
		print,"chan 2 requested, but only 1 channel in data"
		return
	end
	; first hght in file
	hght0Km=desc.dhdr1.datastart/2./5.*.15
; if not 1 usec
; 90km first hght
	firstHkmL=(firstHghtKm lt hght0Km)?hght0Km:firstHghtKm
	nheights=long((lastHghtkm  - firstHKmL)/hghtStepKm) + 1
	ifirsth=long((firstHKmL - hght0Km)*kmTousec /hghtStepUsec)
	spclen=4096
	ippToAvg=1000
	dotm=0
	lab=string(format='("start:",a," file:",a)',systime(0),$
		basename(file))
	print,lab," start:",systime(0)," " 
	istat=shsclp(desc,spcbuf1,spcbuf2,ippToAvg=ippToAvg,spclen=spclen,$
		dinfo=dinfo,hdr=hdr,dotm=dotm,tmI=tmI,nheights=nheights,firstheight=ifirstH,$
		baudlen=hghtStepUsec,doband=doband)
	print,"end:",systime(0)
	shsclose,desc
do1=(doband eq 1 ) or (doband eq 12)
do2=(doband eq 2 ) or (doband eq 12)
;
nh=(do1)?n_elements(spcbuf1[0,*]):n_elements(spcbuf2[0,*])
;h0=(hdr.datastart/2. * .2)*.15
h0=firstHKmL
h1=h0 + nh*hghtStepKm
if do1 then begin
img1=imgflat(spcbuf1,/median)
img1=reverse(img1,1,/overwrite)
nh=n_elements(img1[0,*])
endif 
if do2 then begin
	img2=imgflat(spcbuf2,/median)
	img2=reverse(img2,1,/overwrite)
    nh=n_elements(img2[0,*])
endif
medlen=71
for ih=0,nh-1 do begin &$
;    img1[*,ih]/=bpc[*] &$
    if do1 then img1[*,ih]-= median(img1[*,ih],medlen) &$
    if do2 then img2[*,ih]-= median(img2[*,ih],medlen) &$
endfor


zxL=-4
zyL=0l
if n_elements(zx) ne 0 then zxL=(zx eq 0)?zxL:zx
if n_elements(zy) eq 0 then zyL=(zy eq 0)?zyL:zy
; search for zy
if zyL eq 0 then begin
	zyL=1
	if nh lt maxy then begin&$
       while ((zyL+1)*nh le maxy) do zyL++&$
	endif
	if nh gt maxy then begin&$
    	while ((1.*nh)/zyL gt maxy) do zyL++&$
    	zyL=-zyL&$
	endif
endif

frq1=fband[0] + [-2.5,2.5] + 430. 
frq2=fband[1] + [-2.5,2.5] + 430. 
border=90
cs=1.5
ln=1.5
csn=1.8
nsig=[-2,5]

;st=fstat(desc.lun)
tit=hdr.systime +  ".  file:" + basename(file)
;
if do1 then begin
win1=1
imgdisp,img1,zx=zxL,zy=zyL,nsigclip=nsig,xr=frq1,yr=[h0,h1],chars=cs,$
	xtitle='Freq Mhz',ytitle='Range km',border=border,xstyle=8,win=win1
note,ln,tit,chars=csn
endif
;
if do2 then begin
win2=2
imgdisp,img2,zx=zxL,zy=zyL,nsigclip=nsig,xr=frq2,yr=[h0,h1],chars=cs,$
	xtitle='Freq Mhz',ytitle='Range km',border=border,xstyle=8,win=win2
note,ln,tit,chars=csn
endif
return
end
function timeit,dif,a
        a.nsmp =a.nsmp+1
        a.sum  =a.sum+dif
        a.sumsq=a.sumsq + dif*dif
    return,a
end

