;+
;NAME:
;cgendiag - plot generator diagnostic data
;SYNTAX: cgendiag,d,title=title,drange=drange,cs=cs,font=font,sym=sym,$
;					labadate=labadate,wait=wait,hard=hard,psname=psname
;ARGS:   
;   d[n]:{cgeninfo} array of generator data
;KEYWORDS:
; title     : string title to add to the top of each page
; drange[2] : long  [yyyymmdd,numdays] to plot
;  cs       : float character size scaling if single plot. default=2
; font      : int   1-truetype, -1 hershey, 0 device
;                   def:hershey
;   sym     : int   symbol to using when plotting x vs y.
;                   1,2,3 .. plot a symbol at each measured position.
;                   1 +, 2 *, 3 . , Negative number -1,-2,-3 will plot 
;                   the symbol and the connecting lines.
;                   default:4
; title     : string add this to top of each page
;  labadate : string if supplied, then this is the output format
;                   for label_date call:
;                   %M-monName,%N-monNun,%D-dayNo,%Z-yy,%H-hr,%I-min
;                   default is "%D%M" (ddMon)
;   wait    :       if set then wait for enter between plots
;   hard    :       if set then write a .ps file.
; psname    : string name for psfile. Default="cgensum_yyyymm.ps"
; lastpwr   : ifset then print time, value for last power reading
;
;DESCRIPTION:
;   Plot the cummings generator data passed in.
;The first page plots combined data (from the 4 generators)
; 1. total power vs time.
; 2. oil pressure for each mot/gen set
; 3. oil temp for each mot/gen set
; 4. coolant temp for each mot/gen set
;Page 2:
; 1. power by generator
; 2. power factor by generator
; 3. generator current 3 phases for each gen
; 4. generator voltage (phase to gnd) 3 phases for each gen
;
;	The /hard keyword will generate a hardcopy file (ps). The default
;name is cgensum_yyyymm.ps . You can change the psname using the psname
;keyword.
;
;	You can limit the data plotted using the drange= keyword
;  	drange[0]= yyyymmdd   start of data
;   drange[1]= ndays       number of days (from drange[0] to plot).
;                         if ndays can be positive or negative.
;-
pro cgendiag,d , title=title,cs=cs,font=font,sym=sym,drange=drange ,$
				labadate=labadate,wait=wait,hard=hard,psname=psname,lastpwr=lastpwr
;
	common colph,decomposedph,colph

	oilPWarn=48.5 ; psi when error comes on
	coolTWarn=213; pre high temp warn
	lsW=2
	icolW=10
;	see if they are plotting to file

	lastpwrl=keyword_set(lastpwr)
	hdc=(!d.flags and 1) or (keyword_set(hard))
	if hdc then begin
		csL=2.0
		fontL=1
	endif else begin
		csL=2.0
		fontL=-1
	endelse

	csn=1.5
	if n_elements(cs) ne 0 then csL=cs
	if n_elements(font) ne 0 then fontL=font

	if n_elements(sym) eq 0 then sym=4
	if n_elements(title) eq 0 then title=''
	if n_elements(hhmmss) eq 0 then hhmmss=0l
	useDrange=(n_elements(drange) eq 2)

    !x.style=1
    !y.style=1
	hor
	ver
;
;	if 3 or fewer days, include hour of day
;

	n=n_elements(d)
	if n eq 0 then begin
		print,"no input data"
		return
	endif
	ii=lindgen(n)
	; make jd basedon utc
	astToUtc= 4D/24D 
	jd=d.jd - AstToUtc
	jdLast=jd[n-1]
;	jdLast=d[n-1].jd
    if useDrange then begin
;
       jd1=yymmddtojulday(drange[0])
       delta=drange[1]
       if delta lt 0. then begin 
        ii=where((jd ge (jd1+delta)) and (jd le jd1),count)
       endif else begin
        ii=where((jd ge jd1) and (jd le (jd1+ delta)),count)
       endelse
       if count le 0 then begin
            printf,-2,"no data for date,range:",drange
            return 
        endif
		jd=jd[ii]
    endif

	npnts=n_elements(ii)
	d_jd=max(jd)-min(jd)
	defLabdate=(d_jd le 3)?"%H:%I_%D%M":"%D%M"
	aa=(n_elements(labadate) gt 0)?labadate:defLabDate
	; when label_date is used to plot jd dates,
    ; - if highest resolution is day, then the tick
    ;   marks occur at utc Noon (start of jd).
    ; - if you include hours in the axis, you can
    ;   see where the utc date starts
    ; - labDateoff tries to make it so the day starts
    ;   at 0 utc (or ast since we use 4/24 to go utc to ast.
	labDateOff=(strpos(aa,"H") ne -1)?0.:.5
    a=label_date(date_format=aa)
    xtformat='label_date'
    xtitleL='date'

    xp=.02
    xpinc=.1
    scl=.8
    n=n_elements(d)
;
; 	total power vs time
;
	totPwr=total(d[ii].geni.totkw,1)
	pwrByGen=transpose(d[ii].genI.totKw)
;   amps[tm,phase,gen]
	amps=fltarr(npnts,3,4)
	amps[*,0,*]=(transpose(d[ii].genI.ampsA))
	amps[*,1,*]=(transpose(d[ii].genI.ampsB))
	amps[*,2,*]=(transpose(d[ii].genI.ampsC))
	vtog=fltarr(npnts,3,4)
	vtog[*,0,*]=(transpose(d[ii].genI.voltsA))
	vtog[*,1,*]=(transpose(d[ii].genI.voltsB))
	vtog[*,2,*]=(transpose(d[ii].genI.voltsC))
	oilP=transpose(d[ii].genI.oilPres)
	oilT=transpose(d[ii].genI.oilTemp)*9/5. + 32. ; to cent to fah
	coolT=transpose(d[ii].genI.coolantTemp)*9/5. + 32. ; to cent to fah
;   power factor
	eps=.1
	jj=where(pwrByGen le eps,cnt)
	if cnt gt 0 then pwrByGen[jj]=1.
	pfByGen=pwrByGen/transpose(d[ii].genI.totkva)
; looks like temps leave non zero values in temps if offline
;  put in zeros
	for i=0,3 do begin
		jj=where(d[ii].genI[i].state eq 0,cnt)
		if cnt gt 0 then begin
			oilT[jj,i]=0
			coolT[jj,i]=0
		endif
	endfor

	if (lastpwrL) then begin
		nn=n_elements(jd)
		pwrL=totPwr[nn-1]
		ldate=systime(0,tosecs1970(0,0,jd=jdLast))
		print,format='(f6.1," KW at:",a)',pwrL,ldate
	endif
;
	
	if  keyword_set(hard) then begin
		if n_elements(psname) eq 0 then begin
			psname="cgendiag_" + string(format='(i4,i02)',$
				d[ii[0]].yyyymmdd/10000L,$
				(d[ii[0]].yyyymmdd/100L) mod 100L) + ".ps"
		endif
		pscol,psname,/full
	endif
	vscl=1.05
	hscl=1.05
	x=jd + labDateOff
	!p.multi=[0,1,4]
	ver,0,max(totpwr)*vscl
	plot,x ,totPwr,chars=csl,font=fontL,$
		xtickformat=xtformat,$
    	xtit=xtitleL,ytit='power [KW]',$
    title=title + ' Total power vs Time (sum all generators)'
	ver,40,75
	stripsxy,x,oilP,0,0,/step,chars=csL,font=fontL,$
		xtickformat=xtformat,$
    	xtit=xtitleL,ytit="PSI",$
    	title='oil pressure by gen'
	oplot,[x[0],x[npnts-1]],[1,1]*oilPWarn,linestyle=lsW,col=colph[icolW]
	dx=(x[npnts-1]-x[0])*.02
	xyouts,x[0]+dx,oilPWarn-4,"warnThreshold",chars=csn,font=fontL,col=colph[icolW]
	;
	ver,70,250
	stripsxy,x,oilT,0,0,/step,chars=csL,font=fontL,$
		xtickformat=xtformat,$
    	xtit=xtitleL,ytit="degF",$
    	title='oil temp by generator'
	ver,70,250
	stripsxy,x,coolT,0,0,/step,chars=csL,font=fontL,$
		xtickformat=xtformat,$
    	xtit=xtitleL,ytit="degF",$
    	title='Coolant temp by generator'
	oplot,[x[0],x[npnts-1]],[1,1]*coolTWarn,linestyle=lsW,col=colph[icolW]
	xyouts,x[0]+dx,coolTWarn+4,"warnThreshold",chars=csn,font=fontL,col=colph[icolW]
	
	xpinc=.1
	ln=9
	scl=.8
	xp=.1
	for i=0,1 do begin
		note,ln+i*scl,"gen"+string(format='(i1)',i+1),chars=csn,font=fontL,xp=xp,col=colph[i+1]
		note,ln+i*scl,"gen"+string(format='(i1)',i+3),chars=csn,font=fontL,xp=xp+xpinc,col=colph[i+3]
	endfor
;------------------------------------------------------------------------------------------------------
; page 2 
;
	if not keyword_set(hard) then begin
		key=checkkey(/all)
		if keyword_set(wait) then begin
			print,"Hit enter to continue"
			key=checkkey(/wait)
		endif
	endif
!p.multi=[0,1,4]
	vscl=1.05
	ver,0,max(pwrByGen)*vscl
	stripsxy,x,pwrByGen,0,0,/step,chars=csL,font=fontL,$
		xtickformat=xtformat,$
    	xtit=xtitleL,ytit="KW",$
    	title='Output power vs time by generator'
;
; power factor
;
	ver,.5,1.
	stripsxy,x,pfByGen,0,0,/step,chars=csL,font=fontL,$
		xtickformat=xtformat,$
    	xtit=xtitleL,ytit="powerFactor",$
    	title='Power factorr vs time by generator'
;
;  amps
;
	ver,0,150.
	for i=0,2 do begin  &$
		over=(i ne 0) &$
		stripsxy,x,reform(amps[*,i,*],npnts,4),0,0,/step,chars=csL,font=fontL,$
		xtickformat=xtformat,over=over,$
    	xtit=xtitleL,ytit="Amps",$
    	title='Amps vs time for 3 phases (by generator)' &$
	endfor
;
;  volts To gnd
;
	ver,2300,2450
	for i=0,2 do begin  &$
		over=(i ne 0) &$
		stripsxy,x,reform(vtog[*,i,*],npnts,4),0,0,/step,chars=csL,font=fontL,$
		xtickformat=xtformat,over=over,$
    	xtit=xtitleL,ytit="Volts",$
    	title='Volts To Gnd vs time for 3 phases (by generator)' &$
	endfor
	
	xpinc=.1
	ln=2
	scl=.8
	xp=.04
	for i=0,1 do begin
		note,ln+i*scl,"gen"+string(format='(i1)',i+1),chars=csn,font=fontL,xp=xp,col=colph[i+1]
		note,ln+i*scl,"gen"+string(format='(i1)',i+3),chars=csn,font=fontL,xp=xp+xpinc,col=colph[i+3]
	endfor
	if keyword_set(hard) then begin
		hardcopy
		x
		ldcolph
	endif
    return
end
