;+
;NAME:
;ecalimgday - create dynamic spectra images for a days worth of data.
;SYNTAX: istat=ecalimgday(ecalI,odir)
;ARGS:
;ecalI: {}  struct with for a day. You input it  with ecalinpday()
;odir : string    output directory to write jpeg files.
;RETURNS:
;istat:  int  >= 0 number of jpeg files written.
;             <0  error with message output.
;DESCRIPTION:
;	Callistio data has spectra sampled at a .25 second interval. You can input a days worth
;of data with istat=ecalinpday(yymmdd,ecalI). Data is normally blocked in 15 minute sections (3600 .26 second 
;spectra). This routine will make dynamic spectra for each 15 minute set  and write the jpeg files to odir. 
;The jpeg file naming convention is: yymmdd_hhmm.jpg  where hhmm is the hour,min for the start of the 15 minute block.
; 	The jpeg files contain:
; - an upper frame (polA)
; - and a lower frame (polB)
; - the horizontal axis is time with tickmarks labeled as: hh:mm:ss
; - the vertical scale is freq (MHz). The routine excludes the bottom 8 channels that have no data.
;
;   When computing the dynamic spectra:
; - flatten the image using the median bandpass for the 15 minutes.
; - scale the data to +/- 5 sigma. Sigma is computed across the entire 15 minutes of data 
;    using a robust estimator (excludes outliers). 
;    Callisto uses a logarithmic sampler. I have not converted it back to linear prior to 
;    computing the sigmas and displaying the data (maybe someday..:).
;
;Notes:
;	- The polA and polB data are written to separate fits file by the callisto system. 
;     The time axis of the polA,B data will correspond to the values in the fits files 
;     (they can be different depending when the acquistions started).
;-
function ecalimgday,ecalI,odir

	
	hard=0
	if ecalI.nsets le 0 then return,0
	nsets=ecalI.nsets
	
	if n_elements(odir) gt 0 then begin
	  if odir ne '' then begin
;  see if the directory exists
	    if (file_test(odir,/dir) eq 0) then begin
	      file_mkdir,odir
		  if (file_test(odir,/dir) eq 0) then begin
			 print,"Err: unable to generate directory:",odir
			 return,-1
		  endif
		endif 
		hard=1 
	  endif
	endif
	tit0='e-Callisto spectra Arecibo Obs'

	a=label_date(); clear
	a=label_date(date_format='%H:%I:%S')
	xtickformat='label_date'

	xtit0='Time [HH:MM:SS UTC]'
	ytit='Freq [MHz]'

	lyymmdd=string(format='(i06)',ecalI.yymmdd)
	tmInc=ecalI.tmInc
;   window params
	xsize=1446
	ysize=924
	window,1,xsize=xsize,ysize=ysize
	lpolAr=[' PolA','PolB']
	frq=reverse(ecalI.freqAr)  ; increasing freq
	nfrq=ecalI.freqdim
	frqMin=min(frq)
	frqMax=max(frq)
	eps=.001
	ii=where(frq le (frqmin + eps),cnt)
	i0=cnt-1
	i1=nfrq-1
	f0=frq[i0]
	f1=frq(i1)
	yrange=[f0,f1]
;
	nsig=5
	cs=1.5
	zx=-3
	zy=4
	cth=1.5
	jd0=yymmddtojulday(ecalI.yymmdd)
	nticks=15
	for i=0,nsets-1 do begin
		if (ecalI.spcI[i].ntm[0] le 4) and  (ecalI.spcI[i].ntm[1] le 4) then continue
		!p.multi=[0,1,2]
		tit=lyymmdd + " " + tit0
		for ipol=0,1 do begin
			if ipol eq 0 then begin
				xtit=''
			endif  else begin
				xtit=xtit0
			endelse
		   ntm=ecalI.spcI[i].ntm[ipol]
		   if (ntm le 4) then begin
				plot,[0,1],[0,1],/nodata,$
				   xtit=xtit,ytit=ytit,$
				  chars=cs,charth=cth,$
				  tit=tit +  " No data "  + lpolAr[ipol]
		   endif else begin
		  	  img=imgflat(ecalI.spcI[i].spc[*,0:ntm-1,ipol],0,/median)
;            increasing freq
			  img=transpose(reverse(img,1))  ; swithch to tm,freq
			  hr0=ecalI.spcI[i].startSec[ipol]/3600d
			  hr1=(ecalI.spcI[i].ntm[ipol] * tmInc)/3600d + hr0
			  xrange=[hr0,hr1]/24. + jd0
;
; 	  compute nsig using valid data
;
;			  print,"iset:",i," ipol:",ipol," hr0,hr1:",hr0,hr1," xr:",xrange
	 		  imgdisp,img[*,i0:i1],zx=zx,zy=zy,xr=xrange,yr=yrange,nsig=nsig,xtit=xtit,$
        	  ytit=ytit,chars=cs,charth=cth,xtickformat=xtickformat,tit=tit + lpolAr[ipol],$
			  xticks=nticks
		      print,i,ipol,hr0,hr1,format='("iset:",i2," ipol:",i1," hr0,1:",f7.4,1x,f7.4)'
			endelse
			tit=''
		endfor
		empty
		if hard then begin  &$
			eps=.001
			ind=(ecalI.spcI[i].ntm[0] gt 4)?0:1
			isecmidhms3,long(ecalI.spcI[i].startsec[ind]+eps),hr,min,sec 
			fname=odir + string(format='(a,"_",i02,i02)',lyymmdd,hr,min) + ".jpg"
			write_jpeg,fname,tvrd()
		endif
	endfor
	return,nsets
end
