;+
;NAME:
;ecalinpday - input the ecallisto fits files for a day.
;SYNTAX: nfiles=ecalinpday(yymmdd,ecalI,verb=verb)
;ARGS:
;yymmdd: long   day to input
;verb  :        is set output some debug info
;RETURNS:
;nfiles :  long  > 0 number of fits files found for polA or B
;                0   no files available
;                -1  i/o error with message output.
;3
;ecalI:   {}         struct holding the data
;
;DESCRIPTION:
;	Input the callisto data for one day. Normal operation has the callisto data uploaded to
;the callistion site: http://soleil80.cs.technik.fhnw.ch/solarradio/data/2002-20yy_Callisto
;daily. A crontab then downloads the fits files from this site. This routine will
;search for the downloaded fits files for the requested date. This routine does not do the
;download. If the download did not occur (or failed) then this routine will routine 0 files.
;
; A days set of data will normally contain 44 sets.
; Each set is has 15 minutes of pola,b data. (on disc there will be 88 files
; since pola,b are in separte fits files.
; Each set has 3600 spectra of 200 channels.  the spectra are separated by .25 seconds
;
; the ecalI struct holding a days set of data contains
; help,ecalI,/st
;** Structure <21771c8>, 8 tags, length=254127544, data length=254127362, refs=1:
;   YYMMDD    LONG            211201   yymmdd utc for data
;   NSETS     LONG                44   number of 15 minute datasets input
;   TMINC     FLOAT          0.250000  time step between spectra
;   FREQDIM   INT            200       number of freq dimensions
;   TMDIM     LONG           3600      number of time steps 1 data set (15min tot) 
;   FREQAR    FLOAT     Array[200]     freq for each of the 200 channels
;   TMOFFSETS FLOAT     Array[3600]    time offset (from start of set) for time stamps
;    SPCI     STRUCT    -> <Anonymous> Array[44] the 44 sets of 15 minute data
;
; the ecali.spcI structs hold:
;
; help,ecalI.spci,/st
;** Structure <216fa58>, 10 tags, length=5775280, data length=5775276, refs=2:
;   DATEOBS   STRING    '2021'
;   TIMEOBS   STRING    '10:30:00.702'
;   DATEEND   STRING    '2021'
;   TIMEEND   STRING    '10:45:00'
;   PWMVAL    LONG       80
;   STARTSEC  fltarr(2)  instead of crval1, use timeObs since it has a fraction. 27jan22
;   NTM       LONG       3600    ; number of time samples valid
;   SPC       FLOAT     Array[200, 3600, 2] ; frq,time,pols
;
; Notes:
;  - i'm storing the spectra as (freq,tim). the fits file has (tim,freq)
;  - the freqar in decreasing.. freqAr reflects that.
;
;  - where are the fits files:
;          hhjan22:  base dir for fits files: /share/phildat/callisto/
;          bdir=ecalbase()  will give the current value
;
;  the fits file contains
;  hdu 0 - primary header and byte array of ata
;  hdu 1 - extension with 3600 timestamps, and 200 freq points
;          i checked and the freq are constant for a day
;          some files may have viewer time points.
;          I load -1 into the spectra that are missing.
; 220127 - Trying to sync polA,polB
;          - crval1 for both a,B fits files
;            but time-obs is different. Looking at the data, you need to use
;            time_obs since that determines when data taken, not crval1.
;-
function ecalinpday,yymmdd,ecalI,verb=verb
;
	integTmSecMin=300;   if start time > 300 secs from previous , then this is a new scan
;                        hopefully polA,B will not be separated by more that this.
	missingdata=-1
	fitsPre=ecalbase()
	yymmddL=yymmdd
	if yymmddL ge 220000l then yymmddL=yymmddL mod 1000000L
	lyymmdd=string(format='(i06)',yymmddL)
	fitsPath=fitsPre + string(format='(i06)',yymmddL)
	flist=file_search(fitsPath,"*.fit",count=nfiles)
	if nfiles eq 0 then begin
		print,"no files found for " + lyymmdd
		return,0
	endif
	maxsets=nfiles ; # files 1 pol in case we have  1 pol per set
	nfreq=200  ; what we expect
	ntmStepFile=3600l; what we expect
;   missing data will have -1
	npol=2

;  hold data for 1 image/ or set of 2 files (pola,b)
	spcSet={ dateObs : '',$; from header polA
	         timeObs : strarr(2),$; from headers polA,B
             dateEnd : '',$;  polA
             timeEnd : strarr(2),$;  from hdrs polA,B
		     pwmval  : lonarr(2), $; used for tuner gain polA,B
             startSec: fltarr(2), $; for first row of file computed form timeObs, not crval1
			 ntm     : lonarr(2), $; last image may have fewer entries. polA,B
; --> spc is freq,time  unlike the fits file
		     spc    : fltarr(nfreq,ntmStepFile,npol)}
			spcIAr=replicate(spcSet,maxSets)
;  define at end when we know n sets
	
	lastStartTmSec=0
	for ifile=0,nfiles-1 do begin 
;
; 	use digit at end of filename for pol. 
;   They didn't put the polValue  in the fits header.
;
	 	a=strmid(flist[ifile],5,2,/reverse_offset)
		ipol=-1
;   ind 0: my polA, focus code 1,63(lcp)
;   ind 1: my polB, focus code 2,62(Rcp)
		if (a eq '01') or (a eq '63') then begin
			ipol=0
		endif
		if (a eq '02') or (a eq '62') then begin
			ipol=1
		endif
	 	if (ipol lt 0) then begin
			print,"err: filename does not contain polCode at end (01,02,63,62):",flist[ifile]
			return,-1
	 	endif
;		if (ipol eq 0) and (ifile ne 0) then icur++
;       get data
		img=mrdfits(flist[ifile],0,hdr,status=status,/silent)
		 if status lt 0 then begin
			print,"Error stat:",status," reading hdu 0 (primary hdr,image) ",flist[ifile]
			return,-1
		 endif
;       parse the header
		a=stregex(hdr,"^([^ =]+)[ =']*([^/']+)",/sub,/extr)
		if ifile eq 0 then begin
;           for first file of day get hdu 1 that has the freq,timesteps
;           assume doesn't change for day
		    ext=mrdfits(flist[ifile],1,hdr1,status=status)
		 	if status lt 0 then begin
				print,"Error stat:",status," reading hdu 1 (time,freq) ",flist[ifile]
				return,-1
		 	endif
;          parse some fields of the primary header			
;                       key     skip    val
;           find the indices for some keywords
			icur=0l
			inaxis1 =where(a[1,*] eq "NAXIS1",cnt)
			inaxis2 =where(a[1,*] eq "NAXIS2",cnt)
			idateObs=where(a[1,*] eq "DATE-OBS",cnt)
			itimeObs=where(a[1,*] eq "TIME-OBS",cnt)
			idateEnd=where(a[1,*] eq "DATE-END",cnt)
			itimeEnd=where(a[1,*] eq "TIME-END",cnt)
			icrval1 =where(a[1,*]  eq "CRVAL1",cnt)
			icrval2 =where(a[1,*]  eq "CRVAL2",cnt)
			icdelt1 =where(a[1,*]  eq "CDELT1",cnt)
			icdelt2 =where(a[1,*]  eq "CDELT2",cnt)
			ipwmval =where(a[1,*]  eq "PWM_VAL",cnt)
			tmInc=float(a[2,icdelt1]) ; time inc in file .25 secs, should match tmoff dif
			freqAr= ext.frequency
			tmOffsets=ext.time ; these don't change for files
		endif 
		crval1=float(a[2,icrval1])  ; start time secs
  	    ltimeObs=a[2,itimeObs]
		ltimeEnd=a[2,itimeEnd]
		atm=stregex(ltimeObs,"([^:]+):([^:]+):([0-9.]+)",/sub,/extr)
		timeObs=long(atm[1])*3600. + long(atm[2])*60. + float(atm[3])
		atm=stregex(ltimeEnd,"([^:]+):([^:]+):([0-9.]+)",/sub,/extr)
		timeEnd=long(atm[1])*3600. + long(atm[2])*60. + float(atm[3])
	    ntmR=long(a[2,inaxis1]) ; number times this file 
	    nfreqR=long(a[2,inaxis2]); number freq this file
		inc=0
		if (((crval1 - lastStartTmSec) ge integTmSecMin) and (ifile ne 0)) then begin
				icur++ ; start time after previous end
				inc=1
		endif
 		if keyword_set(verb) then begin
			print,ifile,icur,inc,ipol,ntmR,crval1,lastStartTmSec,ltimeobs,ltimeEnd,$
 			format='("file:",i3," icur:",i2," inc:",i1," ipol:",i1," ntm:",i4," crval1,lastSt:",f7.1,1x,f7.1," start,endtms:",a,",",a)'
		endif
		if  (ifile eq 0 ) then lastTimeEnd=timeEnd
		spcIAr[icur].dateObs=a[2,idateObs]
		spcIAr[icur].timeObs[ipol]=ltimeObs
		spcIAr[icur].dateEnd=a[2,idateEnd]
		spcIAr[icur].timeEnd[ipol]=timeEnd
		spcIAr[icur].pwmVal[ipol] =long(a[2,ipwmVal])
		spcIAR[icur].startSec[ipol]=timeObs
		if (ntmR gt  ntmStepfile) then begin
				print,string(format=$
				'("ifile:",i4,"ipol:",i1," tmsteps:",i4," > max=3600,clipping")',$
						ifile,ipol,ntmr)
				ntmR=ntmStepFile
			    img=img[0:ntmR-1,*]
	 	endif
		ntmUse=ntmR < ntmStepFile
		spcIAR[icur].ntm[ipol]= ntmUse
		itm1=ntmUse-1 
		ifrq1=nfreqR-1 
		spcIAr[icur].spc[0:ifrq1,0:itm1,ipol]=transpose(float(img)) ; verse dimesions
		
;       load bad data (-1) in missing values
	    case 1 of 
		   (nfreqR eq nfreq) and (ntmR eq ntmStepFile):aa=1 ; do nothing
		   (nfreqR eq nFreq) and (ntmR ne ntmStepFile): $
		   		 spcIAR[icur].spc[*,itm1+1:*,ipol]=missingData
		   (nfreqR ne nfreq) and (ntmR eq ntmStepFile):$
		   		 spcIAR[icur].spc[ifrq1+1:*,*,ipol]=missingData
		   (nfreqR ne nfreq) and (ntmR ne ntmStepFile):$
		   		 spcIAr[icur].spc[ifrq1+1:*,itm1+1:*,ipol]=missingData
		endcase
		lastStartTmSec=crval1
	endfor
;
;	now fill in full struct
	nsets=((icur eq 0) and (spcIAr[icur].startSec[0] eq 0.))?0:icur+1 
	ecalI  ={yymmdd: yymmddL,$  ; date for file
			  nsets: nsets,$    ; number of image sets we have for a pol
			  tmInc: tmInc  ,$    ; seconds each freq channel in spec
		      freqDim : nfreq,$ ; in spectra assume same for all spectra.. normal 
			  tmDim   : ntmstepfile,$; default time steps in image.but check spcI[].ntm
		      freqAr: freqAr,$ ;  freq Mhz
		  tmOffsets: tmOffsets,$; add this to startSec to each image to get time stamp for
			 spcI  : spcIAr[0:icur]}
	ecalI.nsets=nsets
	return,ecalI.nsets
end
