;+
;NAME:
;ecaltpday - compute total for each sample of date.
;SYNTAX: istat=ecaltpday(yymmdd,tpI,useEcalI=useEcalI,ecalI=ecalI,freqrange=freqrange,verb=verb)
;ARGS:
;yymmdd: long  date to input
;KEYWORDS:
;useecalI:     if set then use value passed in via ecalI instead of reloading the files
;freqrange: fltarr(2) : if supplied then limit toal power to this freq range.
;verb          if set then ecalinpday will give verbose output when reading file. 
;RETURNS:
; istat:   int  1 ok, -1 error
; tpI  : {}     struct holding total power info
;ecalI : {}     if provided and useCalI not set, then return the spectra data that we input.
;
;DESCRIPTION:
; 	Input a days worth of spectra (yymmdd) and compute the total power for each time sample. If the useEcali
;keyword is set and keyword ecalI is supplied, then the total power will be computed using the data in ecalI.
;If useEcalI is not set and ecalI keyword is included, then the input spectra will also  be returned in ecalI.
;
;	By default the total power is computed over the entire freq range (although the lowest 8 freq chan are skipped since 
;they have no data). The freqRange= keyword can be use to limit the frequency range for the total power. freqrange[0]
;is the loweset freq, and freqrange[1] is the upper frequency (in Mhz) to use.
;  The data is returned in the tpI structure:
;help,tpI,/st
;** Structure <15c1798>, 8 tags, length=3800808, data length=3800800, refs=1:
; YYMMDD   LONG            220131   ; date for data 
; FRQRANGE DOUBLE    Array[2]       ; min,max freq used for tp computation (MHz)
; NSMPA    LONG            158377   ; # of samples of tpA
; HRA      DOUBLE    Array[158377]  ; time stamps for tpA (hr of day UTC)
; TPA      FLOAT     Array[158377]  ; polA total power
; NSMPB    LONG            158354   ; # of samples for tpB
; HRB      DOUBLE    Array[158354]  ; polB time stamps 
; TPB      FLOAT     Array[158354]  ; polB total power
;
; Note that polA and polB can have a different number of samples.
;
;-
function ecaltpday,yymmdd,tpI,ecalI=ecalI,useecalI=useEcalI,freqrange=freqrange,verb=verb
;
	if  keyword_set(useEcalI) then begin
		if n_elements(ecalI) eq 0 then begin
			print,"You must supply keyword ecalI=ecalI if you want useEcalI"
		    return,-1
		endif
	endif else begin
		nfiles=ecalinpday(yymmdd,ecalI,verb=verb)
		if nfiles le 0 then begin
			print,"ecalnipday( with date:" + string(format='(i06)',yymmdd) + "returned:",nfiles," 0=nodata, -1=err"
		    return,-1
		endif
	endelse
	frqAr=ecalI.freqAr
	ifrq0=0
	ifrq1=191 ; skip lowest 8
	nfrq=ifrq1 - ifrq0 + 1l
	if n_elements(freqRange) eq 2 then begin
		ii=where((frqAr ge freqRange[0]) and (frqAr le freqRange[1]),nfrq)
		if nfrq eq 0 then begin
			print,"data does not cover freqRange:",freqRange
			return,-1
		endif
		ifrq0=ii[0]
	    ifrq1=ii[nfrq-1]
	endif
;
	nsets=ecalI.nsets
	nnmax=3600l*nsets
	tpar=fltarr(nnmax,2)
	tmar=dblarr(nnmax,2)
	icurPol=lonarr(2)
	tmInc=ecalI.tminc[0]*1d
	for iset=0,nsets-1 do begin
		for ipol=0,1 do begin
			ntm=ecalI.spcI[iset].ntm[ipol]
			i0=icurPol[ipol]
			i1=i0 + ntm -1
			tpAr[i0:i1,ipol]=total(ecalI.spcI[iset].spc[ifrq0:ifrq1,0l:ntm-1,ipol],1)/nfrq
			tmAr[i0:i1,ipol]=ecalI.spcI[iset].startSec[ipol]*1d + dindgen(ntm)*tmInc
		    icurPol[ipol]+=ntm
		endfor
	endfor
	nA=icurPol[0] 
	nB=icurPol[1] 
;
;	generate struct
;
	f0=frqAr[ifrq0]
	f1=frqAr[ifrq1]
	if f0 gt f1 then begin
		f0=frqAr[ifrq1]
		f1=frqAr[ifrq0]
	endif
		
	tpI={ yymmdd: ecalI.yymmdd,$
		  frqRange: [f0,f1],$
		  nsmpA   : Na,$
		  hrA     : tmAr[0:nA-1,0]/3600D,$
		  tpA     : tpAr[0l:nA-1,0],$
		  nsmpB   : NB,$
		  hrB     : tmAr[0:nB-1,1]/3600D,$ 
		  tpB     : tpAr[0l:nB-1,1]}
	return,1
end
