;+
;NAME:
;basketweave - basket weave two maps
;
;SYNTAX: basketweave,map1,map2,map1w,map2w,deg=deg,fract=fract
;ARGS:
; map1[m,n]: float/double input map 1. it was driven in the x coordinate
; map2[m,n]: float/double input map 2. it was driven in the y coordinate
;KEYWORDS:
; deg:  int   if not zero then fit a polynomial of deg deg to the subtracted data
;             if zero, use a robust mean.
;             def: robust mean
;fract: float  fraction of fit to remove on each iteration, (0> to 1.). 
;verb :       if set then plot results as we go along
;RETURNS:
;map1w[m,n]: float/double backet weave map1 using map1 - map2
;map2w[m,n]  float/double basket weave map2 using map2 - map1
;
;DESCRIPTION:
;	Basketweave two maps. This will help remove stripes in a map.
;When making the map you can:
;  - drive in ra and step in dec
;or- drive in da and step in ra.
;Stripes typically between the strips. 
;To basketweave to maps:
;1. the maps should already be gridded to the same coord grid
;2. copy map1 to map1w, copy map2 to map2w
;3. compute the map difference mdif12= map1 - map2
;4. for each  xstrip
;   - fit a robust polynomial to the mdif12[*,i] (or just use the robust mean)
;   - remove a fraction of this difference from the map1w[*,i]  (def=.7)
;5. do the same to map 2
;   mdif21=map2 - map1
;   for each y strip
;   - fit a robust polynomial of deg to mdif21[i,*] (or just use the robust mean)
;   - remove a fraction of the fit from map2w[i,*]-=fract*yfit
;
; Return map1w,map2w  ...
;Normal processing would iterate:
; mapc1=map1
; mapc2=map2
; loop n
;  rms1=rms(mapc1-mapc2,/quiet)  ; rms of difference before
;  basksketweave(mapc1,mapc2,map1w,map2w,deg=1)
;  rms2=rms(map1w-map2w,/quiet)  ; rms of difference before
;  print.. rms before,afte
;  mapc1=map1w
;  mapc2=map2wk
;endloop
; you could then averaged the 2 basket weaved maps
;-
pro basketweave,m1,m2,mw1,mw2 ,deg=deg,fract=fract,verb=verb
;
;
	if n_elements(deg) eq 0 then deg=0
	if n_elements(fract) eq 0 then fract=.7
	doverb=keyword_set(verb)
	dofit= deg gt 0
	maxiter=5   ; for robust fit
	mw=m1
	a=size(m1)
	dimx=a[1]
	dimy=a[2]
	x=(dindgen(dimx)/(dimx-1.))*2. - 1.		; -1 to 1
	y=(dindgen(dimy)/(dimy-1.))*2. - 1.		; -1 to 1

	mdif=m1-m2
	mw1=m1
	mw2=m2
	if doverb then begin
		cs=1.4
		ytit="difference"
		xtit="map x points"
		tit="fit xstrip differences of maps"
		!p.multi=[0,1,2]
		plot,[0,0],[0,dimx],/nodat,$
		  chars=cs,font=font,$
		  xtit=xtit,ytit=ytit,tit=tit
		  ln=2
		  xp=.94
		  scl=.7
		  csn=1.5
		  note,ln,"difference",chars=csn,font=font,xp=xp
		  note,ln+1*scl,"fit to difference",chars=csn,font=font,xp=xp,col=colph[2]
	endif
	nsig=3.
	if deg eq 0 then yzeros=fltarr(dimx)
	for i=0,dimy-1 do begin
		z=mdif[*,i]
		if dofit then begin
			coef=robfit_poly(x,z,deg,nsig=nsig,/double,yfit=yfit,maxiter=maxiter)
		endif else begin
			yfit=meanrob(z,nsig=nsig)+yzeros
		endelse
		if doverb then begin
			oplot,x,z
 			oplot,x,yfit,color=2
		endif
		mw1[*,i]=m1[*,i] - yfit*fract
	endfor
;   no go the other direction
	mdif=m2-m1
	if doverb then begin
		xtit="map y points"
		tit="fit ystrip differences of maps"
		plot,[0,0],[0,dimy],/nodat,$
		  chars=cs,font=font,$
		  xtit=xtit,ytit=ytit,tit=tit
	endif
;
	for i=0,dimx-1 do begin
		z=reform(mdif[i,*])
		if dofit then begin
			coef=robfit_poly(y,z,deg,nsig=nsig,/double,yfit=yfit,maxiter=maxiter)
		endif else begin
			yfit=meanrob(z,nsig=nsig)+yzeros
		endelse
  		oplot,y,z
 		oplot,y,yfit,color=2
		mw2[i,*]=m2[i,*] - yfit*fract
	endfor
	if doverb then !p.multi=0
	return
end
