;+
;NAME:
;ephmaotordr - convert ao short to ao radar ephemeris
;SYNTAX: istat=ephmaotordr(aofileIn, rdrfileOut)
;ARGS:
; aofileIn: string  input file to convert
; rdrfileOut: string output file for radar format
;DESCRIPTION:
;	The ao pointing program supports tracking using an ephemeris file 
;(more info:http://www.naic.edu/~phil/software/datataking/vxworks/pntCmd.html#ephem).
;There are two format supported:
; - standard ao radar ephemeris (rcv and tx positions)
; - ao short format ( just the date time ra dec az el range rangeRate
; You can download ao short ephemerides from jpl horizons:
; 1. see /share/megs/x101/ephm/obs_tbl.sc (or one of the scripts that calls it
; 2. hortoao.pl < jplInp > aoOutput  .. will format the jpl output to ao short format
;	
;	This routine will then take the ao short format and generate a radar ephemeris file.
;To do this:
; 1. the ao format file should have used -q 2,4,20  options to obs)tbl.sc ..
;    This returns apparent ra,dec,az,el and well had range and range rate
; 2. I reads the ao format file. copying the comments  (#) to the output file
; 3. For each data line it generates the rcv and tx data (the tx data is a copy of the rcv data).
;    - the delay is computed as rangeAu*AutoKm/vellight 
; 20may20.. update to have a tx set that is close to correct
;          (except that the horizon's ephm using venus is not an exact match of the actual
;           radar version).
;SEE ALSO
; an example of using this routine to get radar ephm from tle's can be found at
; /share/obs4/usr/radar/r3399/mkephm.sc 
;-
function ephmaotordr,fileIn,fileOut

	txFreq=2380d6
	auKm=149597870.7D
	c   =299792.458d

 	n=readasciifile(fileIn,inp)
 	n=ephmaord(fileIn,ephmI,/rangedop)
;
;output comments until # Date .. we switch this to the new format
;
	openw,lunout,fileout,/get_lun
	icur=0l
	while (strmid(inp[icur],0,7) ne "#  Date") do printf,lunout,inp[icur++]
;
;  new header lines
;
	line0=$
'#                    ---------------------------- Receiver ------------------------- -------------------------- Transmitter -------------------------'
	line1=$
'#      UTC           ELEV AZMTH RA___apparent___DEC      Delay           Doppler      ELEV AZMTH RA___apparent___DEC    TX Delay        Freq Offset   Local'
	line2=$
'#YYY MM DD HH MM SS +DD.d DDD.d HHMMSS.ss +DDMMSS.s SSSSS.mmmuuunnn +MKKKHHH.mmmuuun +DD.d DDD.d HHMMSS.ss +DDMMSS.S SSSSS.mmmuuunnn +MKKKHHH.mmmuuun HH:MM'
	printf,lunout,line0
	printf,lunout,line1
	printf,lunout,line2
;
; parse the lines with data
; 0 - date
; 1 - time
; 2 - ra
; 3 - dec
; 4 - az
; 5 - el
; 6 - range au
; 7 - range rate
;                           date
; 	compute the round trip time.. just use 2*range
;
;
	  rttRx=2d*ephmI.dat.range*auKm/c  ; auInKm/velLightkm/sec *2 since round trip
	  dopplerRx=-2d*ephmI.dat.dop/c * txFreq
;
; 	now evaluate the tx values rtt before the rcv values
;
	rxTm=ephmI.dat.jd*86400d 
	txTm=rxTm + rttRx
	rttTx=interpol(rttRx,rxTm,txtm)
	dopplerTx=-interpol(dopplerRx,rxTm,txtm)
	raTx=interpol(ephmI.dat.raHr,rxTm,txTm)
	decTx=interpol(ephmI.dat.decD,rxTm,txTm)

	for i=0,n-1 do begin
	   secMidAst=ephmI.dat[i].secmid - 4*3600l
	   if secMidAst < 0 then secMidAst+=86400l
	   hrAst=secMidAst / 3600
	   minAst=(secMidAst - hrAst*3600)/60
	   lhhmmAst=string(format='(i02,":",i02)',hrAst,minAst)
	   ltm=fisecmidhms3(ephmI.dat[i].secmid,h,m,s)
	   ldatetm=string(format='(i04,"-",i02,"-",i02,1x,a)',$
		  ephmI.dat[i].yyyymmdd/10000l,(ephmI.dat[i].yyyymmdd/100l) mod 100l,$
		  (ephmI.dat[i].yyyymmdd mod 100l),ltm) 

;
; 
;      do the rcv then duplicate for xmit
;                            el      az     ra   dec    delay   doppler
	  lraRx=fisecmidhms3(ephmI.dat[i].raHr*3600d,h,m,s,/float,/nocolon)
	  ldecRx=fisecmidhms3(ephmI.dat[i].decD*3600d,h,m,s,/float,/nocolon); i  think this still works
	  lraTx=fisecmidhms3(raTx[i]*3600d,h,m,s,/float,/nocolon)
	  ldecTx=fisecmidhms3(decTx[i]*3600d,h,m,s,/float,/nocolon); i  think this still works
	  lrcv=string(format='(f5.1,1x,f5.1,1x,a9,1x,a9,1x,f15.9,1x,f16.7)',$
				90. - ephmI.dat[i].zaD,ephmI.dat[i].azD,lraRx,ldecRx,rttRx[i],dopplerRx[i])
	  ltx=string(format='(f5.1,1x,f5.1,1x,a9,1x,a9,1x,f15.9,1x,f16.7)',$
				90. - ephmI.dat[i].zaD,ephmI.dat[i].azD,lraTx,ldecTx,rttTx[i],dopplerTx[i])
	  line=string(format='(a,1x,a,1x,a,1x,a)',$
			ldatetm,lrcv,ltx,lhhmmAst)
	  printf,lunout,line
	endfor
	free_lun,lunout
	return,1
end
