;+
;NAME:
;ephmradarrd - read radar ephemeris file
;SYNTAX: n=ephmradarrd(fname,ephmI)
;ARGS: 
;fname: string name of ephemeris file to input
;
;RETURNS:
;      n :  > 0 number of entries read
;           < 0 error code
;ephmI[n]:  struct holding ephm info
;DESCRIPTION:
; 	Read in an ao radar ephmfile. This is the format that is passed to the
;datataking software. It is jon's format after being massaged be ??? script.
;
; The return info is:
; help,ephmI,/st
;   JD       DOUBLE   2456505.2        juldate of entry
;   SECS     LONG     1375289760       secs1970 of entry                                                                                                             
;   STEPSECS DOUBLE   60.000000        step (secs) of entries in ephmfile
;   YMD      LONG     20130731         yyyymmdd of entry (utc)                                                                                                           
;   HMS      LONG     165600           hhmmss   of entry (utc)                                                                                                          
;   LYMD     STRING   '2013-07-31'     yyyy-mm-dd as string (utc)
;   LHMS     STRING   '16:56:00'       hh:mm:ss   as string (utc)
;   RXI      STRUCT   -> <Anonymous> Array[1]     receive info                                                                                                    
;   TXI      STRUCT   -> <Anonymous> Array[1]     tx info                          
;
; help
;IDL> help,ephmI.rxI,/st                                                                                                                                      
;   EL       FLOAT    40.4000         elevation (deg) to .1 deg
;   AZS      FLOAT    59.0000         source az (deg) to .1 deg
;   AZF      FLOAT    -121.000        feed az (deg) to .1 deg
;   RAHMS    DOUBLE   123858.80       apparent ra hhmmss.sss 
;   DECDMS   DOUBLE   351243.50       apparent dec ddmmss.ddd  
;   RAH      DOUBLE   12.649667       apparent ra hh.hhh
;   DECD     DOUBLE   35.212083       apparent dec dd.ddd
;   DELAY    DOUBLE   27.067925       delay secs (rtt)
;
; ephmI.txI is similar for the tx portion 
;-
function ephmradarrd,fname,ephmI 
	rxtx={el :0.,$ ; elevation
	      azS :0.,$ ; az  (source)
	      azF :0.,$ ; az  (feed)
		  raHms:0d,$ ; rahms   .. of date
		  decDms:0d,$ ; dechms .. of date
	      raH  :0d,$  ; ra hour
	      decD  :0d,$ ; dec deg
		  delay:0d,$ ; delay in secs
        frqOffDop:0d}  ; hz

	a={$
		JD: 0d   , $ ;  julian day
		secs: 0l , $ ;  secs from 1970
	    stepSecs:0d,$ ;  between entries
		; times are utc
		ymd :0L,$; yyyymmdd
		hms :0l,$; hhmmss
        lymd:'', $ ; yyyy-mm-dd label
        lhms:'', $ ; hh:mm:ss   label (utc)	
		rxI : rxtx,$
		txI : rxtx} 
	n=readasciifile(fname,inp,comment="#")
	ephmI=replicate(a,n)
;             UTC           ELEV AZMTH RA___apparent___DEC      Delay           Doppler      ELEV AZMTH RA___apparent___DEC    TX Delay        Freq Offset   Local
;      YYY MM DD HH MM SS +DD.d DDD.d HHMMSS.ss +DDMMSS.s SSSSS.mmmuuunnn +MKKKHHH.mmmuuun +DD.d DDD.d HHMMSS.ss +DDMMSS.S SSSSS.mmmuuunnn +MKKKHHH.mmmuuun HH:MM
;                     1        2      3       4        5        6      7       8         9       10      11      12       13       14     15
;                  yymmdd    hhmmss   el      az      ra        dec   delay    dop      el       az      ra      dec     txdelay  freqOff local
	aa=stregex(inp,"([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*) *([^ ]*)",$
					/extract,/subexpr)
	ephmI.lymd=reform(aa[1,*])
	ephmI.lhms=reform(aa[2,*])
	i=3
	ephmI.rxI.el=float(reform(aa[0+i,*]))
	ephmI.rxI.azS=float(reform(aa[1+i,*]))
	ephmI.rxI.raHms=double(reform(aa[2+i,*])) 
	ephmI.rxI.decdms=double(reform(aa[3+i,*])) 
	ephmI.rxI.delay=double(reform(aa[4+i,*])) 
	ephmI.rxI.frqOffDop=double(reform(aa[5+i,*])) 
	i=9
	ephmI.txI.el=float(reform(aa[0+i,*]))
	ephmI.txI.azS=float(reform(aa[1+i,*]))
	ephmI.txI.raHms=double(reform(aa[2+i,*])) 
	ephmI.txI.decdms=double(reform(aa[3+i,*])) 
	ephmI.txI.delay=double(reform(aa[4+i,*])) 
	ephmI.txI.frqOffDop=double(reform(aa[5+i,*])) 
	bb=stregex(reform(aa[1,*]),"([0-9]*)-([0-9]*)-([0-9]*)",/sub,/extract)
	ephmI.ymd=long(reform(bb[1,*]))*10000L + long(reform(bb[2,*]))*100L + long(reform(bb[3,*]))
	bb=stregex(reform(aa[2,*]),"([0-9]*):([0-9]*):([0-9]*)",/sub,/extract)
	ephmI.hms=long(reform(bb[1,*]))*10000L + long(reform(bb[2,*]))*100L + long(reform(bb[3,*]))
; computed
	ephmI.rxI.azF=ephmI.rxI.azS - 180.
	ephmI.txI.azF=ephmI.txI.azS - 180.

	ephmI.rxI.raH=hms1_hr(ephmI.rxI.raHms)
	ephmI.txI.raH=hms1_hr(ephmI.txI.raHms)

	ephmI.rxI.decD=dms1_deg(ephmI.rxI.decdms)
	ephmI.txI.decD=dms1_deg(ephmI.txI.decdms)

	ephmI.secs=tosecs1970(ephmI.ymd,ephmI.hms)
	ephmI.stepsecs=ephmI[1].secs*1d - ephmI[0].secs
	ephmI.jd  = yymmddtojulday(ephmI.ymd) + hms1_hr(ephmI.hms)/24d
	return,n 
end
