;-
;NAME:
;flagpks1 - flag peaks in an array
;SYNTAX: npks=flagpks(xar,yar1,yar2,pkI,usey2=usey2,thrlev=thrlev,xstp=xstp,$
;ARGS:
;xar[nchn,nsets]: dblarr   x values
;yar1[nchn,nsets]: dblarr  y values
;yar2[nchn,sets]: dblarr   y values to overplot in red
;pkI[npks]       : {}       return peak info
;
;KEYWORDS:
;usey2           : int      if set then overplot y2.. def: ignore y2
;thrlev          : dbl      overplot threshold line on each plot
;xstp            : dbl      span of each plot
;                           cycle through each band with (max(xar[*,i])-min(xar[*,i])/xstp
;                           def=2.
;DESCRIPTION:
;	Use the cursor to specify the max, min values around a set of peaks.
;This is handy to use when the arrays have more points than are available on the screen.
;	Plots of xstp width will be made, then the user can flag the start,stop region for
;where pks exist (you flag max,min, not the center..in case things move).
;	The routine will cycle through  (max(xar[*,i]-min(xar[*,i])/xstp plots per set
;It will do this for all nsets.
;	Whenever a peak is specified, it will add this info to pkI[].
;
;	If the keyword usey2 is set, the the values for yar2 will be plotted in red and 
;yar1 will be overplotted in red.
;	In rfi tests, when you power a device on/off.. let yar1 be off and yar2 be on
;If you then see red spikes with no black on top of it, then you know the rfi comes
;from this device.
;                 
;Example:
; let freqAr[8192,7] be the freq from the 7 mock bands
; let spcarOn[8192,7] be the average spectra when an rfi device is on
; let spcarOff[8192,7] be the average spectra when an rfi device is off
; assume the spxAr's are normalized to 1 (== Tsys)
;
;  thrLev=.02  ... overplot line of .02 Tsys for reference
;  xstp=2d     ..  each plot 2 Mhz.
; npks=flagpks(freqAr,spcArOn,spcAroff,pkI,/usey2,thrlev=thrlev,xstp=xstp)
; 
; The left button is used to mark the left then the right edge of the rfi
; the right button says to move on to the next xstp plot
;
;IDL> help,pkI,/st
;   XMIN            DOUBLE           300.01337 ;; user flagged
;   XMAX            DOUBLE           300.02453 ;; user flagged
;   ipk             long                       ;; index into xar[*,iset] for pk
;   XPK             DOUBLE           300.01855 .. pos of ypk between xmin,xmax
;   YPK             DOUBLE          0.14707208 ;; valu of ypk betwee xmin,xmax
;   ISET            LONG                 0
;
; Note : if /usey2 then the xpk,ypk is searched in yar2.. .. else it is searched
;        in yar1
;-
function flagpks1,xar,yar1,yar2,pkI,xstp=xstp,usey2=usey2,thrlev=thrlev
;
	    common colph,decomposedph,colph
	if n_elements(xstp) eq 0 then xstp=2d
	useThr= n_elements(thrlev) ne 0 
	usey2 = keyword_set(usey2)
	ax=size(xar)
	ay=size(yar1)
	ndimx=ax[0]
	ndimy=ay[0]
	if ndimx ne ndimy then begin
		print,"err:X and Y arrays must have same number of dimensions"
		return,-1
	endif
	for i=1,ndimx do begin
		if ax[i] ne ay[i] then begin
			print,"err:dimension ",i," ne in x,y arrays"
			return,-1
		endif
	endfor
	if usey2 then begin
		ay2=size(yar2)
		if (total(ay[0:ndimy] - ay2[0:ndimy]) ne 0.)  then begin
			print,"err:yar1,yar2 must have the same dimensions"
			return,-1
		endif
	endif
	if (ndimx gt 2) then begin
		print,"err:for now flagspk1 works up to a 2d array"
		return,-1
	endif
	nsets=(ndimx eq 1)?1:(ax[2])
	nchn=ax[1]
	a={$
		xmin:0d,$
		xmax:0d,$
		ipk :0L,$
		xpk :0d,$
		ypk :0d,$
		iset:0L$  	; set we found this in
	}
	maxPks=1000L
	pkI=replicate(a,maxpks)
	ipk=0L
	eps=.001
	for iis=0,nsets-1 do begin

		f0=min(xar[*,iis])
		fmax=max(xar[*,iis])
	    nbands=long((fmax - f0)/xstp)
		xstpL=xstp 
	    if nbands eq 0 then begin
			nbands=1
			xstpL=(max(xar[*,iis] - f0))
		endif
		if (f0 + nbands*xstpL) lt (fmax-eps) then nbands++

		for iib=0,nbands-1 do begin
			hmin=f0
			hmax=(f0 + xstpL) < fmax
			print,"iis,iib: ",iis,iib," nbands:",nbands," f0:",f0," xstp=",xstpL," hmin,max:",hmin,hmax
			hor,hmin,hmax
			if usey2 then begin
				plot,[0,1],[0,1],/nodata
				oplot,xar[*,iis],yar2[*,iis],col=colph[2]
				oplot,xar[*,iis],yar1[*,iis],col=colph[1]
			endif else begin
				plot,xar[*,iis],yar1[*,iis]
			endelse
			done=0
			while not done do  begin &$
            	col=(i mod 10) + 1 &$
			    x1=0d
			    y=0d
            	cp,x=x1,y=y,/down &$
            	flag,x1,color=colph[col] &$
           		 if !mouse.button eq 4 then begin &$
                	done=1 &$
            	endif else begin &$
			    	x2=0d
                	cp,x=x2,y=y,/down &$
                	if !mouse.button eq 4 then begin &$
                   		done=1  &$
                	endif else begin &$
                    	flag,x2,color=colph[col] &$
						ii=where((xar[*,iis] ge x1) and $
							     (xar[*,iis] le x2),cnt)
						if cnt eq 0 then begin
							print,"x1 > x2.. skipping.. try again on this one"
					    endif else begin
                    		pkI[ipk].xmin=x1 &$ if !mouse.button eq 4 then stop &$ 
                    		pkI[ipk].xmax=x2 &$ if !mouse.button eq 4 then stop &$
							if (usey2) then begin
							   a=max(yar2[ii,iis],ind)
							endif else begin
							   a=max(yar1[ii,iis],ind)
							endelse
							iipk=ind + ii[0]
							pkI[ipk].ipk=iipk
							pkI[ipk].xpk=xar[iipk,iis]
							pkI[ipk].ypk=(usey2)?yar2[iipk,iis]:yar1[iipk:iis]
							pkI[ipk].iset=iis
                    		ipk+=1 &$
					   endelse
                	endelse &$
            	endelse &$
       		endwhile &
	   		f0+=xstpL
		endfor
	endfor
	npks=ipk
	if npks eq 0 then begin
		 pkI=''
	endif else begin
		pkI=pkI[0:npks-1]
	endelse
	return,npks
end
