;+
;NAME:
;flux12m - flux for some 12meter sources
;SYNTAX: istat=flux12m(srcNm,freqar,fluxI,yr=yr,list=list,cat=cat)
;ARGS:
;srcNm:  string    source to use. will match any case.
;freqAr: fltarr[n] freq to evaluate flux at (MHz)
;KEYWORDS:
;list  :  int     if not zero the just list the source we
;                 know.
;yr    :  long     year for flux. default current year.
;                  this is used if baars et al and there is
;                  a correction for variabilty eg.. casa
; cat  : string    catalog to use: P -perley and butler
;                                  B -baars et al (if available)
;                                  w -wmap combined:7yrwmap cal src. weiland 2010.
;        
;RETURNS:
;istat  :  1 ok, -1 error
;fluxI[n]: {}  array of fluxes (Jy)
; 
;fluxI:{ srcNm: string,$
;		 n     :  n ,$; number of fluxes we return
;		 cat  :  '' ,$; 'p'-perly, 'b' baars et al.
;	     freq:fltarr(n),$ ; freq in Mhz
;        flux :fltarr(n)} ; flux in Jy
;DESCRIPTION:
;	Compute the flux (JY) at the requested freq for some 
;sources. Data is taken from perley and butler 2017 or baars et al.
;	If you request a catatlog and we only have the other, then 
;you get the one we have (and flux.cat will be set accordingly).
;
;	currently just casa and cygnus A
;-
function flux12m,srcnm,freqMhz,fluxI,yr=yr,list=list,cat=cat
;
	forward_function fluxfitvla
;
	if n_elements(cat) eq 0 then cat='p' ; default to perley 2017
	catl=strlowcase(cat)
	if n_elements(yr) eq 0 then begin		
 		a=bin_date()
		yr=a[0]
	endif
	yrL=(yr lt 99)?yr+2000:yr
	;
	srcList=['3C144','3C274','3C405','3C461']
	altList=['CRAB','VIRGOA','CYGNUSA','CASA']
	nsrc=n_elements(srcList)
	if keyword_set(list) then begin
		for i=0,nsrc-1 do begin
			print,srclist[i],altlist[i],format='(1x,a,1x,a)'
		endfor
		return,1
	endif
	ind=-1
	for i=0,nsrc-1 do begin
		if (strcmp(srcnm,srcList[i],/fold_case) or $
		    strcmp(srcnm,altList[i],/fold_case)) then begin
	      ind=i
		  break 
	    endif
	endfor
	if ind eq -1 then return,-1
	nfreq=n_elements(freqMhz) 
;
;   test for source
;
	case srcList[ind] of
    '3C461': begin
    ;
	;     baars et al
	;
		case catl  of 
			'b': begin

;        	 baars flux with correction ref:1980

				yrRef=1980D
				a=5.745d
    			b=-.770d
    			coef=[a,b]
    			flux=fluxfitvla(coef,freqMhz)
    			nyears= yrL - yrRef
    			lossyr= 1d - (.97 -.3*alog10(freqMhz*.001D))*.01
       	 		losstot=lossYr^nyears
        		flux =flux*lossTot
			 end
;            wmap calibrators   ref:2000
	        'w': begin
				 a=2.204d
				 b=-.682
				 c=.028
				 yrRef=2000l
				 logfrq=alog10(freqMhz/40000d); coef for freq/40Ghz
				 flux=10d^(a + b*logfrq + c*(logfrq^2))
				 ; use baars decrease with 2000 ref
    			 lossyr= 1d - (.97 -.3*alog10(freqMhz*.001))*.01
				 nyears=yrL - yrRef
				 lossTot=lossYr^nyears
        		 flux =flux*lossTot
			 end 
			 else: begin
;        perley flux   2017 no correction
	 			coef=[3.3584d,-.7518,-.0347,-.0705d]
         		flux=fluxfitvla(coef,freqMhz,/newfit)
    			lossyr= 1d - .0046d ; from perley paperf
			    yrRef=2017d
				 nyears=yrL - yrRef
				 lossTot=lossYr^nyears
				flux=flux*lossTot
			    catl='p'   ; perley flux
		     end
	    endcase
		end
;
;   virgo  A
;
	'3C274': begin 
			  case catl  of 
				'b': begin
					 a=5.023d
    			     b=-.856d
    				coef=[a,b]
    				flux=fluxfitvla(coef,freqMhz)
			        catl='b'   ; baars flux
			       end
			   else: begin
;			   			perley flux def
				   coef=[2.4466, -0.8116, -0.0483]
       	           flux=fluxfitvla(coef,freqMhz,/newfit)
			       catl='p'   ; only perley flux
			      end
			  endcase
		    end
;
;   cygnus A
;
	'3C405': begin
			case catl of
			;  baars flux   20 to 2ghz, 2ghz to 31ghz
			'b': begin
				logFrq=alog10(freqMhz*1d)
				iil=where(freqMhz le 2000.,cntl)
				if cntl gt 0 then  begin
					a=4.695d
					b=.085d
					c=-.178d
					fluxl=10d^(a+b*logFrq[iil] + c*(logFrq[iil]^2d))
				endif
				iih=where(freqMhz gt 2000.,cnth)
				if cnth gt 0 then  begin
					a=7.161d
					b=-1.244d
					fluxh=10d^(a+b*logFrq[iih])
				endif
				flux=dblarr(n_elements(freqMhz))
				if cntl gt 0 then flux[iil]=fluxl
				if cnth gt 0 then flux[iih]=fluxh
			    end
			;             wmap flux
			'w': begin
				 a=1.482d
				 b=-1.200d
				 logfrq=alog10(freqMhz/40000d); coef for freq/40Ghz
				 flux=10d^(a + b*logfrq )
			    end
		     else: begin
;			   perley flux
			   coef=[3.3498d, -1.0022d,-0.2246d,0.0227d,0.0425d]
               flux=fluxfitvla(coef,freqMhz)
			   catl='p'   ; default
			 end
		    endcase
		end
;
;  crab 
;
	'3C144': begin
			case catl of
			 'b': begin
				 a=3.915d
				 b=-.299d
				 logfrq=alog10(freqMhz*1d); 
				 flux=10d^(a + b*logfrq )
			 	  end
			 'w': begin
				 a=2.506d
				 b=-.302d
				 logfrq=alog10(freqMhz/40000d); coef for freq/40Ghz
				 flux=10d^(a + b*logfrq )
			 	  end
		    else: begin
;			   perley flux
			   coef=[ 2.9516d,-0.2173d,-0.0473d,-0.0674]
               flux=fluxfitvla(coef,freqMhz)
			   catl='p'   ;
			 end
		 endcase
	   end
	endcase
	fluxI={src:srcList[ind],$
             n: nfreq,$
            cat: catl,$  ; p or b
		   freq:freqMhz,$
		   flux:flux}
	return,1
end
