;+
;NAME:
;foldtmseries - fold a time series
;SYNTAX: df=foldtmseries(d,smpTm,period,nbins,stphase=stphase,cnts=cnts)
;ARGS:
;   d[m,n]: float  data to fold. If 1d then just fold the time series
;                  if 2-d then each bin of the folded data will have elements
;  smpTm: float  seconds per sample in d
; period: float  period in secs to fold to.
;  nbins: long   number bins after folding
;KEYWORDS:
;sTPhase: float  the starting second for  d[0]. The default is 0.
;
;RETURNS:
;df[m,nbins]  :double folded data, normalized to the number of counts 
;cnts[nbins]:long  number of counts in each phase bin.
;
;DESCRIPTION:
;    Fold the time series of d into an array df[nbins]. If d is 2d, then the 
;the data is folded on the 2nd dimension (each bin of the folded output will have
;m entries). The folded data is normalized to the number of counts in each bin.
;Optionally return the number of counts for each bin in cnts[nbins].
;
;   The folding computations are done in double precision for more accuracy.
;EXAMPLE:
;;  Take a wapp pulsar datafile on B1937+21 and:
;;1. input the file
;;2. dedisperse it.
;;3. fold it
;
;file2='/share/wapp21/p2175.B1937+21.wapp3.53889.0001'
;period=1.55772007d-3           ; pulsar period
;dm=71.040                      ; dm 
;smpTm=80d-6                    ; sample time of the data
;npts=sp_dedisp(file2,dm,dedisp2) ; input and dedisperse
;nbins=32                       ; bins to fold into
;dfold=foldtmseries(dedisp2,smpTm,period,nbins)     ; fold it
;-
;
function foldtmseries,d,smpTm,period,nbins,stphase=stphase,cnts=cnts
;
    if n_elements(stphase) eq 0 then stphase=0D
	a=size(d)
	use2d=a[0] gt 1
	if a[0] eq 2 then begin
	   n=a[2]
	   nchan=a[1]
	endif else begin
	   n=a[1]
	endelse
    bind=long($
         (((dindgen(n)*smpTm + (smpTm*.5D + stPhase))/period) mod 1D)*nbins)
    df  =(use2d)? dblarr(nchan,nbins):dblarr(nbins)
    cnts=lonarr(nbins)
    for i=0L,nbins-1 do begin
        ii=where(bind eq i,cnt)         ; find points for this bin
        if cnt gt 0 then begin
			if (use2d) then begin
				df[*,i]=total(d[*,ii],2)/cnt
		    endif else begin
				df[i]=mean(d[ii])
			endelse
        	cnts[i]=cnt
		endif
    endfor
    return,df
end
