;+
;NAME:
;gainget12m - return 12m telescope gain(el,freq) for rcvr.
;
;SYNTAX: stat=gainget12m(el,freq,rcvrNum,gainVal,date=date)
;
;ARGS:
;      el[n]: float elevation  angle in degrees for gains
;    freq: float freq in Mhz.
; rcvrNum: int   receiver number 7 or 11
;KEYWORDS:
; date[2]: int   [year,daynumber] to use for gain computation. The default
;                is to use the current day. If they gain curves change
;                with time, this allows you to access the gain curve that
;                was in use when the data was taken.
;RETURNS:
; gainval[n]: float gain in Kelvins/Jy
;    stat: int   -1 --> error, not data returned
;                 0 --> requested outside freq range of fits. Return gain
;                       of the closed frequency value.
;                       the closest value.
;                 1 --> frequency interpolated gain value returned.
;
;DESCRIPTION:
;   Return the gain (K/Jy) for the requested receiver at the specified 
;frequency and elevations. The default is to use the current gain values. The
;date keyword allows you to access a gain curve that was valid at some 
;other epoch.
;
;   Fits have been done for g(el) at different frequencies for various
;receivers (currently only xb). This routine will input the fit information and compute the
;gain for the two closest frequencies and then interpolate to the
;requested freqeuncy. The input fit data is stored in a common block so
;the data does not have to be input from disc a second time unless you
;pick a different receiver.
;
;NOTE:
;   Some receivers have no gain fits. They will return -1 in the status.
;If a requested frequency is outside the fitted values, then the value
;at the closest frequency is returned (no extrapolation is done).
;   For a description of the gain calibration see:
;http://www.naic.edu/~phil. Look under calibration for the receiver
;of interest. The lines with the remark (gain curves) were used to compute
;the gain curves.
;
;EXAMPLES:
;
;SEE ALSO:gaininpdata12m,
;-
function gainget12m,el,freq,rcvrNum,gainval,date=date
;
; return the gain value for this receiver and  freq
; retstat: -1 error, ge 0 ok
;
    forward_function gaininpdata12m,dmtodayno
    common cmgaindata12m,gdata12m
     
    rdfile=(n_elements(gdata12m) eq 0 )
    if n_elements(date) eq 2 then begin
        year  =date[0]
        dayNum=date[1]
    endif else begin
        a=bin_date()
        year  =a[0]
        dayNum=dmtodayno(a[2],a[1],a[0])
    endelse
;	print,"rdfile:",rdfile
    if (not rdfile) then begin
       if  (gdata12m.rcvNum ne rcvrNum)  or $
           ((gdata12m.startYr  gt year) or $
            (gdata12m.endYr    lt year) or $
            ((gdata12m.startYr eq year) and $
             (gdata12m.startDaynum gt dayNum)) or $
            ((gdata12m.endYr eq year) and $
             (gdata12m.endDaynum le dayNum)) ) then rdfile=1
    endif
    if (rdfile) then begin
        datel=[year,dayNum]
        if  gaininpdata12m(rcvrNum,gdata12m,date=datel) lt 1 then return,-1
    endif
;
;   
;
    gotit=0
    eps=1e-4
    ilow= where(((gdata12m.fiti.freq-eps) le freq),countl)
    if countl gt 0 then ilow=ilow[countl-1]
    ihi = where(((gdata12m.fiti.freq+eps) ge freq),counth)
    if counth gt 0 then ihi=ihi[0]
    retstat=1           ; ok
    case 1 of 
;
;       ilow < 0 --> reqfreq< minFreq  use ihi[0]
;
      ilow lt 0:begin
                gainval=fitazeleval(0,el,gdata12m.fiti[ihi])
                if abs(gdata12m.fiti[ihi].freq - freq) gt eps then retstat=0
            end
;
;       ihigh < 0 --> reqfreq > maxFreq use ilow[0]
;
      ihi lt 0:begin
                gainval=fitazeleval(0.,el,gdata12m.fiti[ilow])
                if abs(gdata12m.fiti[ilow].freq - freq) gt eps then retstat=0
               end
;
;       ihigh == ilow  use either 
;
      ihi eq ilow:begin
                gainval=fitazeleval(0.,el,gdata12m.fiti[ihi])
                if abs(gdata12m.fiti[ihi].freq - freq) gt eps then retstat=0
                end
;
;      two freq.. interpolate
;
      else: begin
                gain1=fitazeleval(0,el,gdata12m.fiti[ilow])
                gain2=fitazeleval(0.,el,gdata12m.fiti[ihi])
                f1=gdata12m.fiti[ilow].freq
                f2=gdata12m.fiti[ihi].freq
                gainval=gain1+ (freq-f1)/(f2-f1)* (gain2-gain1)
            end
    endcase
    return,retstat
end
