;+
;NAME:
;gainInpData12m - input gain data for 12meter rcvrs.
;
;SYNTAX:
;     istat=gainInpData12m(rcvNum,gainData,fname=fname,date=date)  
;
;ARGS:
;     rcvNum:  7 or 11    sband or xband (prior to wbrcvr)
;KEYWORDS:
;      fname: to specify an alternate data file with gain values.
;            The default file is aodefdir() + 'data/gain12m.datR{rcvNum}
;      date : [year,daynum]  .. if specified the data when you want the
;                              gain for..default is most recent.
;
;RETURNS:
;      istat: 1 ok, -1 bad file/rcvnum,data probably no fit data available.
;    gainData[n]:{gainData12m} return fit info. 1 structure for each frequency
;DESCRIPTION: 
;  Input the gain fit data for all the frequncies of a particular receiver
; (rcvNum). The rcvNum can be extracted from the headers with iflohrfnum().
;
;  The default datafile is aodefdir() + 'data/gain12m.datR{rcvNum} (aodefdir() is
;  a function that returns the root of the aoroutines). The keyword
;  fname allows you to specify an alternate file. The file format is:
;  -  col 1 ; is a column, 
;  -  col 1 !yyyy dayno   starts a date section. yyyy dayno is the
;                        year daynumber for the start of this data set.
;  -  data is free format , column oriented
;  freq fitType A0 A1 .. sigFit
;  
;   c0..cN are the fit coefficients, pol is I or A or B, 
;
;The structure format for {gaindata} is:
; gainData.rcvNum       receiver number
; gainData.numFreq      number of frequencies found
; gainData.startYr      for the fit
; gainData.startDaynum  for the fit
; gainData.endYr        for the fit
; gainData.endDaynum    for the fit
; gainData.fitI[numFreq] {azelfit12m} structure holding the coef and other
;                          info for each fit
; {azelfit12m}:
; {numcoef  : 10,$; for fittype
;  fittype  :  1,$;
;  freq     :  0.,$;Mhz
;  coef     : dblarr(10),$; 10 is max coef
;  sigma    : 0.d       ,$; of fit
;  rfnum    : 0         ,$;  7 or 11
;  pol      : ' '       ,$; I, A, or B
;  type     : ''        ,$; ascii string for type: gain,tsys,sefd..
;  date     :' '    }    ;date fit run
;
;This routine is called automatically by gainget12m().
;
;How the different cal routines vary:
;
;SEE ALSO: gainget12m ,masgainget12m
;- 
;history:
;08mar23 .. started
function gainInpData12m,rcvNum,gainData12m,fname=fname,date=date
;
;    on_error,1
    on_ioerror,endio
	verb=0
    lun=-1
    if (n_elements(fname) eq 0) then begin
        fname=string(format='(a,"data/gain12m.datR",i0)',aodefdir(),rcvNum)
    endif
    case n_elements(date) of
        0   : begin
                reqYr =0
                reqDay=0
              end
        1   : begin
                reqYr =date[0]
                reqDay=0            
              end
        2   : begin
                reqYr =date[0]
                reqDay=date[1]
              end
        else: message,'gaininpdata12m illegal date request.. [year,daynum]'
    endcase
    openr,lun,fname,/get_lun,error=err
    if  err ne 0 then begin
        print,"couldn't open file",fname
        return,-1
    endif
;
;    file format is:
; ; col 1 is comment
; !yyyy ddd is first valid date for data that follows..
;
    fitI=replicate({azelfit12m},10)    ;allow for up to 10 freq.. may need to change for wb
    freqLine=0.
    inpl=" "
    i=0
    gotDate=reqYr eq 0
    startYr=0L & startDay=0L
    endYr=0L   & endDay=0L
    while (1) do begin
        readf,lun,inpl
	    if keyword_set(verb) then print,inpl
        if (strmid(inpl,0,1) ne ';') then begin
           if (strmid(inpl,0,1) eq '!') then begin
              if not gotDate then begin
                 startYr=0L & startDay=0l
                 reads,strmid(inpl,1,strlen(inpl)-1),startYr,startDay
                 if (startYr lt reqYr) or $
                    ((startYr eq reqYr) and (startDay le reqDay)) then begin
                        gotDate=1
                 endif else begin
                    endYr = startYr
                    endDay= startDay
                 endelse
              endif else begin
                 if i gt 0 then goto,endio
              endelse
              if (startYr eq 0 ) and (startDay eq 0) then begin
                   reads,strmid(inpl,1,strlen(inpl)-1),startYr,startDay
              endif
           endif else begin
                if gotDate then begin
                    line=''
                    line2=''
                    reads,inpl,freqLine,fittype,line
                    fitI.numCoef=2
                    case fittype of
                        1: begin
                            fitI.numCoef=2
                            var=fltarr(3)
                            reads,line,var,line2
                           end
                    else: begin
                           ln='file:'+fname+'  freq:'+string(freqLine) $
                                + ' fittype:'+ string(fittype) + ' illegal'
                           message,ln
                          end
                    endcase
; 
                    fitI[i].freq=freqLine
                    fitI[i].rfNum =rcvNum
                    fitI[i].type ='gain'
                    fitI[i].coef  =var[0:fitI[i].numCoef-1]
                    fitI[i].sigma =var[fitI[i].numCoef]
                    fitI[i].fittype =fittype
                    line2=strtrim(line2,1)
                    fitI[i].pol   =strmid(line2,0,1)
                    i=i+1
                endif
           endelse
        endif
    endwhile
endio:
    retstat=1
    if ((not eof(lun)) and  (strmid(inpl,0,1) ne '!') ) or (i eq 0) then begin
        retstat=-1
    endif else begin
        if (endYr eq 0) then begin
            endYr=3000L
            endDay=366L
        endif
        gainData12m={ rcvNum    :rcvNum,$
                  numFreq    :i, $
                  startYr    : startYr,$
                  startDaynum: startDay,$
                  endYr      : endYr,$
                  endDaynum  : endDay,$
                  fitI       : fitI[0:i-1]}
    endelse
    if (lun gt -1) then free_lun,lun
    return,retstat
end
