;+
;NAME:
;gsfit2d_12m - cross pattern 2d fit to total power az,el stripsI
;SYNTAX: fitCoef=gsfit2d_12m,azOff,elOff,z,el,initCoef,zfit=zfit,$
;                covar=covar,sigCoef=sigCoef,chisq=chisq,sigma=sigma,$
;                rot=rot,trouble=trouble,weights=weights
;ARGS:
;       azOff[npts]: az pos (offsets from center of strip) in gcdeg all pnts in pat
;       elOff[npts]: el pos (offsets from center of strip) in gcdeg all pnts in pat
;        z[npts]: measured data points
;       el[npnts]:   encoder offset of elevation from central elevation value.
;                    (needed since the el offset along the az strip relative to the source
;                    is zero, but the change in actual elevation is non zero (since the source
;                    is moving in az,el during the cross.
;    initCoef[m]: float. coef to fit for. first guess
;    fitCoef[m]:  float coef from fit
;    rot       :  int   if set then do the beam rotation. the new default is no rotation.
;                       the initCoef should still contain 8 elements. initCoef[6] rotation angle
;                       is ignored.
;                       For the 305m the beam was elliptical and rotation was needed
;                       For the 12meter the beam in more spherical, and there is not  enough
;                       ellipticity for this to work well. 
;KEYWORDS:
;       zfit[npts]: float return fit evaluated at input points
;       sigma=sigma : sigma for fit  
;       covar[10,10]: covariance matrix
;       sigCoef[10] : sigmas for coef.
;       chisq       : float ..
;       weights[npts]: float weights 
;       trouble     : 0 converged,
;                    -1 chisq infinite,
;                    -2 flamdacount>30*10/flstep
;                    -3 iteration > iterationmax def. 20
;                    -4 alpha/c not finite.. probably 0 in partial deriv
;                    -5 one of the coef not finite
;                    -6 a width^2 is negative
;       nostop      : if set then if trouble=-4 then don't stop, just return
;                     with trouble=-4
;       cfplot      : for curve fit: 0 noplot,1 plot,nowait, 2 plot wait at last fit, 3 wait each iter
;   
;DESCRIPTION:
; Fit to the function:
;
;z(x,y)= a0 + a1*exp[-xp^2/sigxp^2 - yp^2/sigyp^2] + a7*elOff
;
;You need to give the coef initial values when the routine is called.
;The units are:
;a0 - constant
;a1 : amplitude
;a2 : xoffset ,az coord direction offset is az units
;a3 : yoffset ,el coord direction offset in el units
;a4 : FWHM_AZ  ,in prime coordinate system: fwhm az units
;a5 : FWHM_EL  ,in prime coordinate system: fwhm el units
;a6 : theta   ,rotate unprimed to primed aligned along ellipsoid of beam,Deg
;              (only do rotation if /rot  is set 
;a7 : zaslope ,The el slope in amplitude units per el unit
;
;xm=(x-x0)
;ym=(y-y0)
;xp = xm*cos(th)  + ym*sin(th)
;yp =-xm*sin(th)  + ym*cos(th)
;
; angle theta rotates from az,za to axes aligned with the major axis
; of the beam elipse. Only rotate if  /rot is set. If not, the rot angle is forced to 0.
; 
;The x,y values are passed via a common block. The call passes in 
;an index to this common block.
;
; The fit is done in the units passed in (az,el). They should be
;greatcircle units with the same scale (don't mix hrs and deg)
;-
; 
;NAME:
;gsfit2dFunc -  cross pattern 2d fit to total power I
;
;DESCRIPTION:
;a0 - constant
;a1 : amplitude
;a2 : xoffset ,az coord direction
;a3 : yoffset ,el coord direction
;a4 : sigx^2  ,in prime coordinate system
;a5 : sigy^2  ,in prime coordinate system 
;a6 : theta   ,rotate unprimed to primed aligned along ellipsoid of beam,Rd
;              only rotate if /rot set.
;a7 : slope   ,fit for slope of Y (usually el)
;
;z(x,y)= a0 + a1*exp[-xp^2/sigxp^2* - yp^2/sigyp^2*]
;xm=(x-x0)
;ym=(y-y0)
;xp = xm*cos(th)  + ym*sin(th)
;yp =-xm*sin(th)  + ym*cos(th)
;
; angle theta rotates from az,el to axes aligned with the major axis
; of the beam elipse
;
;The x,y values are passed via a common block. The call passes in 
;an index to this common block. This routine is used by gsfit2d
;
; 
pro gsfit2dFunc_12m,i,a,z,pder
;
common  comgsfit2d_12m,comgsfit2dx,comgsfit2dy,comgsfit2del
;
;   get the x,y values in the prime system (aligned with the ellipitcal beam).
;
;   angles for rotation 
;
; test no rotation..
    costh =cos(a[6])
    sinth =sin(a[6])
    xm =comgsfit2dx[i]-a[2]
    ym =comgsfit2dy[i]-a[3]
    xp = xm*costh  + ym*sinth
    yp =-xm*sinth  + ym*costh


    sigX2=(1d-16 > a[4])
    sigY2=(1d-16 > a[5])
    z=a[0] + a[7]*comgsfit2del[i] + a[1]*exp(-(xp*xp)/sigX2 - (yp*yp)/sigY2)
    return
end
;
; no rotation function.. 
; coefs[6]=elslope, rot excluded
pro gsfit2dFuncnorot_12m,i,a,z,pder
;
common  comgsfit2d_12m,comgsfit2dx,comgsfit2dy,comgsfit2del
;
;   get the x,y values in the prime system (aligned with the ellipitcal beam).
;
;   angles for rotation 
;
    xm =comgsfit2dx[i]-a[2]
    ym =comgsfit2dy[i]-a[3]


    sigX2=(1d-16 > a[4])
    sigY2=(1d-16 > a[5])
    z=a[0] + a[6]*comgsfit2del[i] + a[1]*exp(-(xm*xm)/sigX2 - (ym*ym)/sigY2)
    return
end

function  gsfit2d_12m,azOff,elOff,z,el,a,zfit=zfit,covar=covar,sigCoef=sigCoef,$
				chisq=chisq,$
                 sigma=sigma,trouble=trouble,cfplot=cfplot,$
				 rot=rot,$
                 weights=weights,cfparms=cfparms,$
                 nostop=nostop,itmax=itmax
;
common  comgsfit2d_12m,comgsfit2dx,comgsfit2dy,comgsfit2del
    
    flambdastep=10.             ; default
    fwhm2tosig2=1.D/(4.D*alog(2.)); fwhm^2 to sig^2
	if n_elements(itmax) eq 0 then itmax    =20                ; default
    s=size(azOff)
	if s[0] eq 1 then begin
    	pntstp=s[1]
    	pnttot=s[1]
	endif else begin
    	pntstp=s[1]
    	pnttot=s[1]*s[2]
	endelse
    comgsfit2dx=reform(azOff,pnttot)  ; in fwhm
    comgsfit2dy=reform(elOff,pnttot)  ;
    lindar=lindgen(pnttot)
    comgsfit2del=reform(el,pnttot)  ;

    ad=double(a)
    if not keyword_set(weights) then begin
        weightsl=dblarr(pnttot)+1.D
    endif else begin
        weightsl=reform(weights*1.D,pnttot)
    endelse
;
;   change fwhm to sig^2 for fit
;
;   make sure the two fwhm are a little different so the
;   theta fit works..
;
    if ad[4] eq ad[5] then begin
        ad[4]=ad[4]*.99
        ad[5]=ad[5]*1.01
    endif
    fwhm=ad[4]
    ad[4]=fwhm*fwhm*fwhm2tosig2
    fwhm=ad[5]
    ad[5]=fwhm*fwhm*fwhm2tosig2
    ad[6]=ad[6]*!dtor
    zloc=reform(z,pnttot)
	if (keyword_set(rot) eq 0 ) then  begin
		ctemp=ad[7] ; slope
		ad=ad[0:6] ; only 7 coef, we will remove rotation
		ad[6]=ctemp; put el slope back
    	zfit=curvefitpp(lindar,zloc,weightsl,ad,sigCoef,chisq=chisq,$
            function_name='gsfit2dfuncnorot_12m',itmax=itmax,tol=tol,nostop=nostop,$
            covar=covar,flambdastep=flambdastep,/noderivative,trouble=trouble,$
            cfplot=cfplot,cfparms=cfparms)
	endif else begin
    	zfit=curvefitpp(lindar,zloc,weightsl,ad,sigCoef,chisq=chisq,$
            function_name='gsfit2dfunc_12m',itmax=itmax,tol=tol,nostop=nostop,$
            covar=covar,flambdastep=flambdastep,/noderivative,trouble=trouble,$
            cfplot=cfplot,cfparms=cfparms)
	endelse
;	print,"trouble:",trouble
	if trouble eq 0 then begin
		aa=finite(ad)
		ii=where(aa eq 0,cnt)
		if cnt gt 0 then trouble=-5
	endif
;
;   fix up ..
;
	if ad[4] lt 0 then begin
    	ad[4]= sqrt(-ad[4]/fwhm2tosig2)  ; back to fwhm
		trouble=(trouble ne 0)?-6:trouble 
	endif else begin
    	ad[4]= sqrt(ad[4]/fwhm2tosig2)  ; back to fwhm
	endelse
	if ad[5] lt 0 then begin
    	ad[5]= sqrt(-ad[5]/fwhm2tosig2)  ; back to fwhm
		trouble=(trouble ne 0)?-6:trouble 
	endif else begin
    	ad[5]= sqrt(ad[5]/fwhm2tosig2)  ; back to fwhm
	endelse
	if (keyword_set(rot) eq 0)  then begin
		; we fit 7 coef. with ad[6]=elslope.need to  switch to 8coef. where ad[6]=0.rotation
		aa=dblarr(8)
		aa[0:5]=ad[0:5]
		aa[7]=ad[6]    ; move elslope up, rotation will be 0.
		ad=aa
;       and sig coef
		aa=dblarr(8)
		aa[0:5]=sigcoef[0:5]
		aa[7] =sigcoef[6]
;       fix covariance (7,7) to (8,8)
		m8=dblarr(8,8)
		if trouble eq 0 then begin
		m8[0:5,0:5]=covar[0:5,0:5]
		m8[0:5,7]  =covar[0:5,6]
		m8[7,0:5]  =covar[6,0:5]
		m8[6,6]=1.
		m8[7,7]=covar[6,6]
		endif
		covar=m8
	endif else begin
    	ad[6]=ad[6]*!radeg mod 360.
	endelse
    if trouble eq 0 then  sigma=(rms(zfit-zloc,/quiet))[1]
    return,ad
end
