;+
;NAME:
;gsfit2deval_12m - evaluate fitI returned from mascrossfit2d()
;SYNTAX: z=gsfit2deval_12m(fitI)
;ARGS:
;       fitI[0]:  singleo struct (fitI) returned from mascrossfit2d().
;RETURNS:
;     z[npts,2,nbms]: float  fit evalutated at fit points (npnts,az/el)
;DESCRIPTION:
; 	mascrossfit2d() fits a 2d gaussian to crosses made on a source with the 12meter.
;It  uses gsfit2d_12m() do do the actual  fitting. mascrossfit2d then returns an array of
; structures;  fitI[n] (one for each cross done).
;	This  routine gsfit2deval_12m() will evaluate  one of these structures at the fit values 
;note:mascrossfit2d()  can return and array of structs if more than one cross is passed in.
;     this routine will only evaluate one fit at a  time.
;     eg mascrossfit2d(..fitiar)
;        z=gsfit2deval_12m{fitiar[4])  to evaluate the 5th crosss.
;the fit.  holding info from the fits. This routine will take a s
;
;   The 2d  fit is:
;z(x,y)= a0 + a1*exp[-xp^2/sigxp^2 - yp^2/sigyp^2] + a7*elOffset
;   see mascrossfit2d() or gsfit2d_12m() for a description of the coef and xp,yp
;
;  The x,y values for the evaluation are taken from fitI that is passed in.
;
;	The fiti can have more than one bm (freq band). The returned evaluation
;will be dimensioned z[npnts,2,nbms)  where 2 is az,el strip.
;
;Example:
;  1. assume you called mascrossfit2d() with xband data that had 
;     1200 samples/strip 7 bms and it returned fitI and 1 cross.
;  2. to evaluate the fit:
;    zret=gsfit2deval_12m(fitI)
;    zret would be dimensioned: zret[1200,2,7]
;
;  3. to overplot the data with the fits:
;
;     npnts=fiti.pntstrip  ; normally 1200.
;     nbms=n_elements(fit.fitibm)
;	  xaz=fiti.offard[*,0] ; az offsets
;	  xel=fiti.offard[*,1] ; el offsets
;     zaz=reform(fiti.fitibm.z[*,0],npnts,nbms) ; [1200,7] az data values 
;     zel=reform(fiti.fitibm.z[*,1],npnts,nbms ; [1200,7] el data values 
;
;	!p.multi[0,1,2]
;	stripsxy,xaz,fiti.fitibm.z[*,0],0,0,/step ; az strips data
;	stripsxy,xaz,zaz,0,0,/step,/over          ; overplot the az fits
;	stripsxy,xaz,fiti.fitibm.z[*,1],0,0,/step ; el strips data
;   stripsxy,xel,zel,0,0,/step,/over          ; overplot el fits
;
; Notes:
; 1. 14mar23 - an option was added to remove linear baselines prior to the 2d fitting.
;    This changed the number of elements in fitI[] (to include the baseline info)
;    if this was selected:
;    - the z data values in fitI.fitibm[ibm].z have had these linear baselines removed 
;      before the 2d fitting is done.
;    - if  you want the actual offsets (tsys), you need to look at the results from the 
;      linear fits
;       : fitI.fitibm.blfit.az.coef[2]: coef[0] +coef[1]*azoff  . gives the  entrire fit
;       : fitI.fitibm.blfit.el.coef[2]: coef[0] +coef[1]*eloff  . ditto
;       or 
;       : fitI.fitibm.blfit.az.avgVal       avg val from fit (tsys)
;       : fitI.fitibm.blfit.el.avgVal       ditto
;
;-
; Notes: this should actually be called masgsfit2d_eval() since it only works on the
;        output from mascrossfit2d().. being in gen/ is just a holdover from gsfit2deval()
function gsfit2deval_12m,fitI

	if n_elements(fitI) ne 1 then begin
		print,"gsfit2deval() only processes 1 cross fit at at time. (try fitI[i] in call)"
		return,-1
	endif

	nbms=n_elements(fitI.fitibm)
	npnts=fiti.pntstrip
	zret=dblarr(npnts,2,nbms)
	x=dblarr(npnts,2)  ; azoffset  for both strips
	y=dblarr(npnts,2)  ; azoffset  for both strips
	x[*,0]=fitI.OffArd[*,0]; first strips az offset, 2nd strip offset=0
	y[*,1]=fitI.OffArd[*,1]; el offset 2nd strips, 1st 0
	elAr=fitI.elOff ; for linear fit to el
	for ibm=0,nbms-1 do begin
		if fiti.fitibm[ibm].trouble ne 0 then continue  ; return not zer 0' if bad fit
		a=fitI.fitIbm[ibm].coef

    	fwhm2tosig2=1.D/(4.D*alog(2.)); fwhm^2 to sig^2
		fwhm=a[4]
		a[4]=fwhm*fwhm*fwhm2tosig2
		fwhm=a[5]
		a[5]=fwhm*fwhm*fwhm2tosig2
;
; convert rotation theta to radians
; if norotation was included in the fit the angle will have been set to 0.
		a[6]=a[6]*!dtor
		costh =cos(a[6])
    	sinth =sin(a[6])
    	xm =  x - a[2]
    	ym =  y - a[3]
    	xp = xm*costh  + ym*sinth
    	yp =-xm*sinth  + ym*costh
    	sigX2=(1d-16 > a[4])
    	sigY2=(1d-16 > a[5])
;       see if it contains linear elevation fit
    	if n_elements(a) gt 7 then begin
        	zret[*,*,ibm]=a[0] + a[7]*elAr + a[1]*exp(-(xp*xp)/sigX2 - (yp*yp)/sigY2)
    	endif else begin
        	zret[*,*,ibm]=a[0] + a[1]*exp(-(xp*xp)/sigX2 - (yp*yp)/sigY2)
    	endelse
	endfor
	return,zret
end
