;+
;NAME: 
;matrot - generate 1 or more rotation matrices
;SYNTAX: mat=matrot(order,th1,th2,th3)
;ARGS:
;order: string   "xyz" order for the rotations
;th1  :  float    first angle in radians
;th2  :  float    2nd angle in radians
;th3  :  float    3rd angle in radians
;RETURNS:
;mat[3,3]: float   rotation matrix 
;DESCRIPTION:
;	concatenate 3 rotation matrices.
; The matrices will be applied using the order string order (eg.
; if order='zxy' then a z rotation,followed by an x rotation followed by
; a rotation about the y axis.
;
;	The direction of the rotation:
; +theta about an axis generates a CCW rotation of a vector
; (Note this direction was updated on 17jun20).
; Order of data
;   
;EXAMPLE:
;   rotate 90deg about z then x then y
;   use vectors [1,0,0] and [0,1,0]
;   axis='zxy'
;	V=fltarr(3,2)
;   V[*,0]=[1,0,0]
;   V[*,1]=[0,1,0]
;	thrRd=90.*!dtor
;   M=matrot(axis,'y',thrd,thrd,thrd)
;   to apply the matrix to a vector use:
;   v1=transpose(m)#v
;   of matrix_multiply(m,v,/atranspose)
;   print,v1
;  1.00000 -4.37114e-08      0.00000
;  0.00000  1.91069e-15      1.00000
;-
function matrot,order,th1,th2,th3,double=double
;
	matI=(keyword_set(double))?dblarr(3,3):fltarr(3,3)
	matI[0,0]=1.
	matI[1,1]=1.
	matI[2,2]=1.
	mat=matI
	slen=strlen(order)
	lorder=strlowcase(order)
	for i=0,slen-1 do begin
		th=(i eq 0)?th1:(i eq 1)?th2:th3
		s=sin(th)
		c=cos(th)
		mm=matI
		;
		case strmid(lorder,i,1) of
		'x': begin
		     mm[4]=c	
		     mm[5]=-s	
		     mm[7]=s	
		     mm[8]=c	
			 end
		'y': begin
		     mm[0]=c	
		     mm[2]=s	
		     mm[6]=-s	
		     mm[8]=c	
			 end
		'z': begin
		     mm[0]=c	
		     mm[1]=-s	
		     mm[3]=s	
		     mm[4]=c	
			 end
	    endcase
		; 17jun20 previously had mm#mat but used tranposed matrices
;		mat=mat#mm
		mat=mm##mat
	endfor
	return,mat
end
