;+
;NAME:
;platrotmodel - model platform rotation 
;SYNTAX: skyRot=platrotmodel(yr,lr,rotI)
;ARGS:
;year:   long      year for lr data (lr has a dayno but no year).
;lr[n]: {}         laser ranging for dates of interest
;KEYWORDS:
;RETURNS:
;n        : long    number of returned entries
;                  < 0 --> error inputing model
;rotI[n]: {}      array of struct holding extra tilt info.
;
;DESCRIPTION:
;    A model of the platform rotations  was made for 
;different epochs. The models were created using data  when the platform was at the
;correct height, and there was tension in all 3 tiedowns.
;  This routine will take laser ranging data and compute the additional
;platform tilt (value - model). This could be caused by tiedowns out of position,
;or because we lost tension in a tiedown.
;	It will also compute the pointing error caused by the excess tilt
;and the pointing error caused by the current platform height.
;	To use this routine, you need laser ranging data with all 6 distomats
;working and valid az,za and date.
;
;	The information is returned in the roti[] array of structs. It contains:
;IDL> help,roti,/st
; OK          LONG    1               1 rotations computed. other values are:
;                                     0 Did not fall in model data range
;                                    -1 not Enough dist measurements
;                                    -2 bad az or za value
;                                    -3 bad date vale
; SEC         DOUBLE  1.5369846e+09  secs from 1970 for this data point
; YYMMDD      LONG    20180915       data point data as yyyymmdd (ast)
; HR          FLOAT   0.169444       data point time as hr from midnight (ast)
; AZ          FLOAT   95.7299        azimuth for this point (deg)
; ZA          FLOAT   6.39518        za for this point (deg)
; AVGH        FLOAT   382.896        avg platform height (meters). ref value is 382.896
; TRINP       FLOAT   Array[3]       plat translations dx,dy,dz in cm
; ROTINP      FLOAT   Array[2]       rotation angles [x,y] in deg for inp
; ROTMOD      FLOAT   Array[2]       rotation angles (x,y) for model at this az,za
; ROTEXTR     FLOAT   Array[2]       extra rotation (input - model) in deg about [x,y] axis.
; PNTERRROT   FLOAT   0.00227990     pointing error from extra rotation (deg)
; PNTERRHGHT  FLOAT   1.77322e-05    pointing error from platform height (deg)
; SAVIND      LONG    2              index for model save file used for this point
;
;NOTES:
; - the x axis is toward the west
; - the y axis is toward the north
; - the z axis points down
; - positive angles are CCW looking down the axis from the positive direction.
;-
function platrotmodel,year,lr,rotI
;
; not lr angles in radians
;     fit coef,amp in deg, phase in radians
; 
	forward_function platrottosky,pnterrplathght
;
	ftom=12d*2.54/100d
	zaMin=1.05
	zaMax=19.7
	refHght=1256.22 *ftom
	modelDir=aodefdir() + "data/"
	flist=file_search(modelDir + "lrAngFit*.sav")
	if n_elements(flist) eq 1 then begin
			if flist eq '' then begin
				print,"No model sav files found."
				return,-1
			endif
	endif
;
;	now get the date range for the save files
;
	nsav=n_elements(flist)
	nlr=n_elements(lr)
	ok=intarr(nlr)
	aa={  savname: '',$
            sec1 : 0d,$, ; sec 1970 start of save data
            sec2 : 0d} ; sec 1970 end of save data
	savI=replicate(aa,nsav)
;
;	rot,tr struct
;
	a={ ok : 0l, $   ; 1--> we computed values, else trouble
;                       0 didn't fall within sav file range?
;                      -1 need 6 dist
;                      -2 bad za, or az
;                      -3 bad date
        sec: 0d,$   ; secs from 1970
	    yymmdd:0l,$ ; ast date
	        hr:0.,$ ; ast  hour of day
	    az : 0.,$
	    za : 0.,$
        avgh: 0.,$;          ; average height in meters.1256.22=382.896
		trInp :fltarr(3),$   ; dx,dy,dz cm
        rotInp:fltarr(2),$   ; xrot,yrot, deg
        rotMod:fltarr(2),$   ; xrot,yrot, deg
        rotExtr:fltarr(2),$   ; Inp - model (x,y)
;
;       rotation errors in deg. add in quadrature (wrong but angles small).
;
	    pntErrRot:  0.     , $; pnt err from rot (deg)
	     pntErrHght:  0.    , $; pnt error from height (deg)
		savInd : 0l}          ; index into save list we used fro this entry 
	rotI=replicate(a,nlr)
		
	a=stregex(flist,"^.*lrAngFit_([0-9]+)_([0-9]+).sav",/extr,/sub)
	yymmdd1=reform(long(a[1,*]))
	yymmdd2=reform(long(a[2,*]))
;  input dates are ast.
	for i=0,nsav-1 do begin
	    savI[i].savname=flist[i]
		savI[i].sec1=tosecs1970(yymmdd1[i],040000l)
		savI[i].sec2=tosecs1970(yymmdd2[i],040000l) + 86400 -1l ; include entire last day
	endfor
;
;	flag lr data that won't work
;
	ii=where(lr.dok ne 1,cnt)
	if cnt gt 0 then  rotI[ii].ok = -1	; need 6 dist..
	ii=where((lr.az lt 0.) or (lr.az gt 720) or $
	         (lr.zagr lt zamin) or (lr.zagr gt zamax),cnt)
	if cnt gt 0 then  rotI[ii].ok = -2	; bad az or za
	ii=where((lr.date le 0.) or (lr.date ge 367),cnt) ; exclude 0.0 only one pnt a yr, lots of bad measures have 0.
	if cnt gt 0 then  rotI[ii].ok = -3	; bad date
;
; map  dates to save files.
;  each save file has a start,end date for the data
;  mapping to save files
;   use save file i if
;    sec >= savfile[i]sec0  and
;    sec <  saveFile[+1].sec0 
;    for i = 0 include all points before savefile[0].sec0
;    for i = last include all points after savefile[last].sec1
;
	secYrSt=tosecs1970(year*10000l + 0101,040000l) ; start of ast year
	lrSecAr  = secYrSt + (lr.date -1l )*86400d 
	isavIAr=lonarr(nlr) - 1 
	case 1 of 
	    nsav eq 1: begin
			ii=lindgen(nlr)
			isavIar[ii]=0
	        end
	    nsav eq 2: begin
			ii=where(lrSecAr lt savI[1].sec1,cnt)	
		    if cnt gt 0 then isavIAr[ii]=0
			ii=where(lrSecAr ge savI[1].sec1,cnt)	
		    if cnt gt 0 then isavIAr[ii]=1
			end
		else: begin
			ii=where(lrSecAr lt savI[1].sec1,cnt)	
	    	if cnt gt 0 then isavIAr[ii]=0
			ii=where(lrSecAr ge savI[nsav-1].sec1,cnt)	
	    	if cnt gt 0 then isavIAr[ii]=nsav-1
			for i=1l,nsav-2 do begin &$
				ii=where((lrSecAr ge savI[i].sec1) and (lrSecAr lt savI[i+1].sec1),cnt) &$
				if cnt gt 0 then isavIar[ii]=i &$
			endfor
			end
	endcase
;
;	now loop through the savs files using the ones we need
;
	for i=0,nsav-1 do begin
		ii=where((isavIar eq i) and (rotI.ok ge 0),cnt) ; get lr measurements that use this save file.
;		print,"ntot,indisav,cntindsav:",n_elements(lr),i,cnt
		if cnt gt 0 then begin
			restore,savI[i].savname
			; we have fits for each za. use in lr to determine which to use
			;get za coef index
			indCoefza=long(lr[ii].zagr) -1  ; start at za=1
			uind=indCoefZa[uniq(indcoefZa,sort(indcoefZa))]
		    nn=n_elements(uind)
			for j=0l,nn-1 do begin
				jj=where((long(lr[ii].zagr) eq uind[j]+1) and (lr[ii].zagr ge zamin) and $
							  (lr[ii].zagr le zamax),cnt) ; at this za
				if cnt gt 0 then begin
				kk=ii[jj]			; to index into complete arrays
				xrotM=fitsinneval((lr[kk].az mod 360.)*!dtor,coefrX[uind[j]])
				yrotM=fitsinneval((lr[kk].az mod 360.)*!dtor,coefrY[uind[j]])
				rotI[kk].ok=1
				rotI[kk].sec=lrSecAr[kk]
				rotI[kk].az =lr[kk].az mod 360.
				rotI[kk].za =lr[kk].zagr
				rotI[kk].avgh =lr[kk].avgh * ftom
				rotI[kk].trInp[0]=lr[kk].dx
				rotI[kk].trInp[1]=lr[kk].dy
				rotI[kk].trInp[2]=lr[kk].dz
				rotI[kk].rotInp[0]=lr[kk].xrot*!radeg
				rotI[kk].rotInp[1]=lr[kk].yrot*!radeg
				rotI[kk].rotMod[0]=xrotM
				rotI[kk].rotMod[1]=yrotM
				rotI[kk].rotExtr[0]= rotI[kk].rotInp[0] -xrotM
				rotI[kk].rotExtr[1]= rotI[kk].rotInp[1] -yrotM
				rotI[kk].savInd =i 
				endif
			endfor
		endif
	endfor
;
; 	convert secs1970 to yyyymmdd and hr of day
;   fudge so date comes back ast, not utc
	jd=secs1970tojd(rotI.sec) - 4d/24d
	caldat,jd,mon,day,yr,hr,min,sec
	rotI.yymmdd=yr*10000l + mon*100l + day
	rotI.hr    = hr + min/60. + sec/3600.
;
	ii=where(rotI.ok eq 1,cnt)
	if cnt gt 0 then begin
;
;		rotate unit vector in z direction  by two angles:w
;
		dtor=!dpi/180d
		thx=rotI[ii].rotExtr[0]*dtor
		thy=rotI[ii].rotExtr[1]*dtor
		cosx=cos(thx)
	    sinx=sin(thx)
		cosy=cos(thy)
	    siny=sin(thy)
		vec =dblarr(3,cnt)
		vec[2,*]=1d			; unit vector z direction
		rvec=vec
;       rotate about x
		rvec[1,*]=vec[1,*]*cosx  + vec[2,*]*sinx
		rvec[2,*]=-vec[1,*]*sinx + vec[2,*]*cosx
;       rotate about y
		vec=rvec
		rvec[0,*]=vec[0,*]*cosy - vec[2,*]*siny
		rvec[2,*]=vec[0,*]*siny + vec[2,*]*cosy
;
;		use dot prodoct between initial unit vec at rvec to get angle
;
		vec =dblarr(3,cnt)
		vec[2,*]=1d			; unit vector z direction
	    dp=dblarr(cnt)
		for i=0,2 do dp+=reform((vec[i,*])*(rvec[i,*]))
		thD=acos(dp)/dtor
		rotI[ii].pntErrRot=platrottosky(rotI[ii].za,thD)
		istat=pnterrplathght(rotI[ii].avgh/ftom,rotI[ii].za,pntErrAsecs)
		rotI[ii].pntErrHght=pntErrAsecs/3600.
	endif
	
	return,n_elements(rotI)
end
