;+
;NAME:
;readasciifile - read an ascii file into strarr
;SYNTAX: nlines=readasciifile(filename,inpLines,comment=comment,$
;                           fast=fast,maxlines=maxlines)
;ARGS:
;   filename: string the filename to read
;KEYWORDS:
;    comment: string single character that is a comment
;                    lines that start with this will be skipped
;fast: int    if set then try fast read, don't use shell,wc
;maxLine:long if fast used, then this is the max number of lines
;             to expect in file (including comments). def:1000 lines..
;             if exceeded, will return an error. It can be less
;RETURNS:
;     nlines: long number of lines read
;             -1 if file does not exist or read error
;             -2 if fast mode used and maxlines too small
;inpLines[nlines]: strarr lines read
;
;DESCRIPTION
;   Read an entire file into a string array. 1 line per string index.
;Skip any lines that start with the comment character.
;Return the string array and the number of lines read.
;
;EXAMPLES:
;   filename='savfiles.dat'
;   nlines=readasciifile(filename,inplines,comment=';')
;
;Timing:
;	For large numbers of files, /fast will run about 20 times
;faster (since it doesn't make a shell call)
;-
;history:
; 11aug16 - fast mode: if embedded null lines, it returns too few lines
;           switch to keep the last non null line
;
function readasciifile,filename,inpLines,comment=comment,$
		  fast=fast,maxLines=maxLines

	retStat=-1
	lun=-1
    if file_exists(filename) eq 0 then goto,errOut
	fMaxLines=1000L
	useFast=keyword_set(fast)
	if n_elements(maxLines) eq 1 then fMaxLines=maxLines
	nlines=fMaxLines 
	if (not useFast) then begin
    	cmd='wc ' + filename
    	spawn,cmd,inp
		nlines=long(inp[0])
		if nlines eq 0 then begin
			retStat=0
			goto,errOut
		endif
	endif
    openr,lun,filename,/get_lun
    inpLines =strarr(nlines)
	on_ioerror,cont
	err=1
    readf,lun,inpLines
	err=0
cont:
	on_ioerror,NULL
	hitEof=0
	if (err) then begin
		if (useFast) then hitEof=eof(lun)
		if not hitEof then begin
			print,"Err reading:filename:",filename," err:",!error_stat.msg
			retStat=-1
			goto,errOut
		endif
		; /fast, see number of non null entries
		ii=where(inpLines ne '',cnt)
		if (cnt eq 0 ) then begin
			retStat=0
			goto,errOut
		endif
		if cnt ne nlines then begin
			imax=ii[cnt-1]
			inpLines=inpLines[0:imax]
			nlines=n_elements(inpLines)
		endif
	endif
	; if fast, see if next read is eof. if not then error.
	if (useFast and (not hitEof)) then begin
	    on_ioerror,cont2
		temp=''
        readf,lun,temp
cont2:  hitEof=eof(lun)
		on_ioerror,NULL
		if (not hitEof) or (temp ne '')  then begin	
			print,"readasciifile, fast mode,more then:",nlines," lines in file (increase maxlines)"
			retStat=-2
			goto,errOut
		endif
	endif
		
    if keyword_set(comment) then begin
        ind=where(strmid(inpLines,0,1) ne comment,count)
        if count ne nlines then begin
		 	if count eq 0 then begin
				inplines=''
				retStat=0
			 	goto,errOut
			endif else begin
				inpLines=inpLines[ind]
				nlines=count
			endelse
		endif
    endif
	retStat=nlines
errOut:
	if (lun ne -1) then free_lun,lun
	return,retStat
end
