;+
;NAME:
;stkscalibv - stokes calibration from complex voltages
;SYNTAX: istat=stkscalibv(rcvNum,cfr,smpTimeUsec,fftlen,VcalonA,VcalOnB,VcalOffA,VcalOffB,$
;                  calibI,lcorcal=lcorcal,date=date,degPerMhz=degPerMhz,edge=edge
;ARGS:
;rcvNum    : int     receiver number to use when querying cals: 1=327,2-430,5=lbw,$
;                    7-sbw,8-sbh,9-cb,11-xband,13-cbw,17-alfa 
;cfr       : float   center freq in Mhz. A single cal value for the whole band
;                    will be computed at this freq.
;smpTmUsec :float    sample time for data in microsecs
;fftlen    :long     fftlen to use when computing the spectra
;vcalOnA[n]:complex  complex voltage data for calon polA
;vcalOnB[n]:complex  complex voltage data for calOn polB 
;vcalOffA[n]:complex complex voltage data for caloff polA
;vcalOffB[n]:complex complex voltage data for calOff polB 
;
;KEYWORDS:
;   lcorcal:        if set then use low cor cal. def: high cor cal
;  date[2] : lonarr [year,daynumber] date to use when searching for cal value.
;                   def: most recent cal values
; degPerMhz: float  initial guess for phase change polA,B per Mhz. def:8 deg/MHz
;                   comes from the ao fiber.
; edge[2]  : float  fraction of  bandpass edge to ignore on each side. def: .06
;                   Calibration and fitting will exclude data on the edges.
;
;DESCRIPTION:
;	Using complex sampled data, compute the intensity calibration and phase correction 
;for polA,polB from a calon,off measurement. The user inputs the complex voltages for
;calonA,calonB, caloffA,caloffB (the routine currently assumes these are all the same length).
; It will compute the complex spectra, intensity calibrate, and then fix the phase difference of polA
; polB using  stokesU from the cal deflection.
;
;The program steps are:
; 1. look up the cal value (it uses a single value for the entire band).
; 2. compute the complex spectra for the 4 data sets
; 3. compute the average spectra for the 4 buffers
; 4. compute the indices i0,i1 of the spectrum to use for calibrating and fitting 
;    (this comes from the edge keyword).
; 5. compute the caldeflection averaging over the band i0:i1
; 6. compute the calScale factor (a/d counts to Kelvins)
; 7. compute the bandpass correction: 1/caloff normalized to mean(caloff[i0:i1])
; 8. compute the scaling arrays (convert to kelvins and bandpass correct)
; 9. compute stokesI,Q after scaling the data from the average spectra
; 10 compute stokesU,V : compute stokes u,v for each fft, scale and average
; 11. compute the cal deflection: calon - caloff for each stokes parameter
; 12. fit sine waves (non linear fit) to .stokes u and V : a[0]+a[1]*sin((freqMhz*a[2] + a[3])*2*pi)
; 13. compute the phase difference delta for polA,B using atan(fitStksV/fitStksU)*!radeg
; 14. unwrap any 180 deg phase jumps in delta
; 15. linear fit to unwrapped delta vs freqMHz.
; 16. using the linear fit, compute the phase correction for u,v
;     phCor=exp(complex(0.,-(!dtor*poly(frqAr,calibI.phFit))))
;     cuv=complex(calDU,calDV)
;     crotated=cuv*phCor
;     U=float(crotated)
;     V=imaginary(crotated)
; help,calibI,/st
; CFR      FLOAT 2380.00       ; freq used to compute cal value
; FFTLEN   LONG  1024          ; length of fft when computing spectra
; BWMHZ    FLOAT  0.500000     ; bandwith of data in MHz  
; CALK     FLOAT  Array[2]     ; cal in degK [polA,polB]
; INDEDGE  LONG   Array[2]     ; indices into fftlen to use for calibration
; UI       STRUCT -> <Anonymous> Array[1] ; sine fit to stokes u (see below)
; VI       STRUCT -> <Anonymous> Array[1] ; sine fit to stokes V (see below)
; DELTAF   FLOAT  Array[1024]  ; phase difference PolA-polB vs freq (in degrees)
; DELTAFU  FLOAT  Array[1024]  ; phase difference polA,B and phase unwrapping
; PHFIT    FLOAT  Array[2]     ; linear fit to deltafU (deg/Mhz). y=phfit[0] +freqMHz*a[1]
; STKCALD  FLOAT  Array[1024, 4];stokes i,q,u,v for caldeflection after phase correction,
;                              and intensity calibration
; uI,vI sine fit info
;IDL> help,calibI.uI,/st                                                                                                  
; DATA     FLOAT  Array[1024]  ; data used for the sine fit
; YFIT     FLOAT  Array[1024]  ; sine fit to data
; COEFAR   FLOAT  Array[4]     ; coef for fit:y=a[0]+a[1](sin((freMHz*a[2]+a[3])*2*pi)
; SIGMA    FLOAT  Array[4]     ; fit sigmas for each parameter
; CHISQ    FLOAT   6.04890     ; chisq from fit (but weights were all set to 1.)
;
;Notes:
; 1. All of the cal deflection should come out in stokesU (since the cal is injected equally into
;    polA and polB). For some setups i've gotten all of the cal deflection in -U (less  then zero).
;    In this case there is a 180 deg. phase shift in delta.
; 2. If you need to flip the bandpass (because the original data had an odd number of high side 
;    lo's) then remember to change the sign of stokesV.
; 
;-
function stkscalibv,rcvNum,cfr,smpTimeUsec,fftlen,VcalonA,VcalOnB,VcalOffA,VcalOffB,calibI,$
				   lcorcal=lcorcal,date=date,degPerMhz=degPerMhz,edge=edge
;   non linear least squares sine fit for u,v
	forward_function fitsinnl
    common colph,decomposedph,colph
;
; initial guess for phase difference per Mhz. comes from the AO, IF fiber..
	edgeL=[.06,.06]
	if (n_elements(degPerMhz) eq 0) then degPerMhz=8.

	if (n_elements(edge) eq 2 ) then edgeL=edge
	if (n_elements(edge) eq 1 ) then edgeL=[edge,edge]
;
;	get the cal values
;
	ldate=(n_elements(date) eq 2)?date:0
	caltype=(keyword_set(lcorcal))?0:1 ; lcorcal,hcorcal
	istat=calget1(rcvnum,caltype,cfr,calK,date=ldate)
	bwMhz=1./smpTimeUsec
	npnts=n_elements(VcalOnA)
	nfft=(npnts/long(fftlen))
	if nfft eq 0 then begin
		print,"fftlen less than number of points"
		return,-1
	endif
	npntsU=nfft*fftlen
	;
	; figure out the range of fftlen to use for the calibration.. from edge
	;
	i0=long(fftlen*edgel[0]+.5)
	i1=fftlen -1 - i0
	lenC=i1-i0 + 1l
;
;	compute the complex spectra.. 
;
	cspcOnA=shift(fft(reform(vcalOnA[0l:npntsU-1]-mean(vcalOnA),fftlen,nfft),dim=1),fftlen/2,0)
	cspcOnB=shift(fft(reform(vcalOnB[0l:npntsU-1]-mean(vcalOnB),fftlen,nfft),dim=1),fftlen/2,0)
	cspcOffA=shift(fft(reform(vcalOffA[0l:npntsU-1]-mean(vcalOffA),fftlen,nfft),dim=1),fftlen/2,0)
	cspcOffB=shift(fft(reform(vcalOffB[0l:npntsU-1]-mean(vcalOffB),fftlen,nfft),dim=1),fftlen/2,0)
;
;	 use the avg cal intensity deflection for intensity calibration
;
	npol=2
	spcOnAvg=fltarr(fftlen,npol)
	spcOffAvg=fltarr(fftlen,npol)
	spcOnAvg[*,0]=total(cspcOnA*conj(cspcOnA),2)/nfft
	spcOnAvg[*,1]=total(cspcOnB*conj(cspcOnB),2)/nfft
	spcOffAvg[*,0]=total(cspcOffA*conj(cspcOffA),2)/nfft
	spcOffAvg[*,1]=total(cspcOffB*conj(cspcOffB),2)/nfft
;   average over bandpass to get scale factors
;    ignore edge
	calDifAvg=reform(total(spcOnAvg[i0:i1,*] - spcOffAvg[i0:i1,*],1)/lenC)
	calScl=calK/calDifAvg  ; convertes spectrometer counts to Kelvinsl
;
; 	compute bandpass correction using normalized cal off, and scaling to K
;
	sclA=calScl[0]/(spcOffAvg[*,0]/mean(spcOffAvg[i0:i1,0]))
	sclB=calScl[1]/(spcOffAvg[*,1]/mean(spcOffAvg[i0:i1,1]))

	sclAB=sqrt(sclA*sclB)  ; for stokes u,v
;
;	now compute  average stokesI,Q,U,V on,off 
;   the stokes values for the caldeflection;
;
	yA=spcOnAvg[*,0]*sclA
	yB=spcOnAvg[*,1]*sclB
	stkIOn=yA + yB
	stkQOn=yA - yB
;   off
	yA=spcOffAvg[*,0]*sclA
	yB=spcOffAvg[*,1]*sclB
	stkIOff=yA + yB
	stkQOff=yA - yB
;
;   .. need a -1 here for cal deflection to come out in stokes U..
;
	stkUOn =2*total(float(cspcOnA *conj(cspcOnB )),2)*sclAB/nfft
	stkUOff=2*total(float(cspcOffA*conj(cspcOffB)),2)*sclAB/nfft

	stkVOn =2*total(imaginary(cspcOnA *conj(cspcOnB )),2)*sclAB/nfft
	stkVOff=2*total(imaginary(cspcOffA*conj(cspcOffB)),2)*sclAB/nfft
;   stokes for cal deflection
;   
	calDI=(stkIon - stkIoff)
	calDQ=(stkQon - stkQoff)
	calDU=(stkUon - stkUoff)
	calDV=(stkVon - stkVoff)
;
;	get phase variation delta vs freq.
;   U= Ea*Eb*cos(delta)
;   V= Ea*Eb*sin(delta)
;   delta=atan(V/U)
;   Fit u,v with a sine wave then take the ratio of the fits..
;   to get not get biased by outliers
; 
;   fit params are: 
;	a[0] - constant offset
;	a[1] - amplitude
;   a[2] - freq: cycles/xunit
;   a[3] - phase: fractions of a cycle (
	s={  data  :fltarr(fftlen),$; data to fit 
		 yfit  :fltarr(fftlen),$; fit values 
         coefAr:fltarr(4),$  ;offset,ampl,freq,phase from fit)
         sigma : fltarr(4),$ ;each param
		 chisq : 0. } ;   probably not valid since weights set to 1.
	calibI={ cfr: cfr,$; center freq Mhz
          fftlen: fftlen,$
		   bwMhz : bwMhz ,$
		   calK  : calk,$ ; polA,B
		   indEdge:[i0,i1],$; edge indices (cnt from 0) in fftlen to use for fitting,calibrating
		   uI    : s ,$
	       vI    : s ,$
	       deltaF: fltarr(fftlen),$  atan(fitV/fitU)* !redeq
	       deltaFU:fltarr(fftlen),$  unwrapped  deltaF
		   phFit : fltarr(2),$      ; linear fit to delta vs freq [constant,slope]
		  stkCalD   :fltarr(fftlen,4)} ; stokesi,q,u,v cal defl after phase correction u,v
	a=fltarr(4)
	; fit u,v 
	w=fltarr(fftlen)+1.   ; weights
	frqAr=findgen(fftlen)/fftlen *bwMhz
	yfitAr=fltarr(fftlen,2) ; u,v
	coefAr=fltarr(4,2)
	
	for i=0,1 do begin
		y=(i eq 0)?calDU:calDV
		aa=rms(y,/quiet)  ; returns [mean,rms]
		a[0]=aa[0]; no offset
;        for amplitude estimate, assume dominated by a sine wave.. compute rms the divide by .707
		a[1]=aa[1]/.707
;        for freq, assume you're at ao, fiber gives about 
		a[2]=degPerMhz/360.   ; fraction of a cycle.
		a[3]=0.; 			  ;initial phase 
		;  y=a[0] + a[1]*sin(a[2]*x + a[3])
;		print,i,"bef:",a
		fitsinnl,frqAr[i0:i1],y[i0:i1],w[i0:i1],a,yfit,chisq,iter,sigma 
;		print,i,"aft:",a
		s.data=y
	    s.yfit=a[0] + a[1]*sin((frqAr*a[2] + a[3])*!pi*2)
		s.coefAr=a
        s.sigma=sigma
		s.chisq=chisq
		if i eq 0 then begin
			calibI.uI = s
		endif else begin
			calibI.vI = s
		endelse
	endfor
	calibI.deltaF=atan(calibI.vI.yfit/calibI.uI.yfit)*!radeg  ; put it in deg
	; now unwrap the phase
	off=0.
	delta=calibI.deltaF
	for i=0,fftlen-2 do begin
		d=off + delta[i+1]  - delta[i] 
		if abs(d) gt 180. then off=(d gt 0)?off - 360: off + 360.
		delta[i+1]=delta[i+1] + off
	endfor
	calibI.deltaFU=delta  ; unwrapped phase change
;
;	now linear fit to get correction 
;	
	calibI.phFit=poly_fit(frqAr[i0:i1],delta[i0:i1],1)
	y=[calDI,calDQ,calDU,calDV]
;
;	phase rotate u,v
;	
	phCor=exp(complex(0.,-(!dtor*poly(frqAr,calibI.phFit))))
	cuv=complex(calDU,calDV)
	crotated=cuv*phCor
;	now load the corrected cal deflection into stkCalD[fftlen,4]
	calibI.stkCalD[*,0]=calDI 
	calibI.stkCalD[*,1]=calDQ 
	calibI.stkCalD[*,2]=float(crotated)
	calibI.stkCalD[*,3]=imaginary(crotated)
	return,1
end	
