;+
;NAME:
;hf_gettxpwr - get tx power from anritsu files
;SYNTAX:n=hf_gettxpwr(dirList,pwrI,minTx=minTx,band=band,verb=verb,pwrtot=pwrtot,$
;                     pwrByTxKw=pwrByTxKw)
;ARGS:
;dirList[m]: strarr   list of directories holding anritsu hf files
;KEYWORDS:

;minTx   : int        minimum # of tx needed for pwrTot array.
;                     for entries with fewer, -1 is returned in that 
;                     entry. def=4
;band    :  int       5 or 8. decides where to look for peak power in spectra
;                     -1 --> search 5 and  8 bands for max value
;verb    :            if set, then print name of each dir when starting it.
;RETURNS:
;n      : long   number of entries returned
;pwrI[n]: {}     struct holding pwrInfo read in.
;pwrTot[n]: fltarr    total power (KW) 
;pwrByTxKw[n,6,3]:fltarr power (kw) by transmitter
;                        1..6 tx 1..6
;                        1..3 : avg, max ,min hold
;DESCRIPTION:
;	Get the power info from the  anritsu hf files. return info in pwrI[n] struct 
;In the arrays [6] correspond to tranmitters 1,2,3,4,5,6
;help,pwrI,/st
;** Structure <1121878>, 4 tags, length=160, data length=156, refs=1:
;   GOTDAT          INT       Array[6]  ; 1--> data from tx, 0--> no data this tx
;   FREQ            FLOAT     Array[6]  ; freq of peak data (between 5.0 and 5.2 Mhz)
;   TM              LONG64    Array[6]  ; tm for this measurement (secs 1970)
;   DAT             FLOAT     Array[3, 6]; rf coupler value: dbm.
;
; example:
;
;bdir='/share/phildat/hf/'
;dirAr=['151109','151110','151110N','151111','151111N','151112','151112N',$
;'151113','151113N','151114D','151114M','151114N']
;flist=bdir + dirAr
;n=hf_gettxpwr(flist,pwrI,pwrTot=pwrTot)
;- 
function hf_gettxpwr,dirL,pwrI,minTx=minTx,verb=verb,pwrTot=pwrTot,pwrbyTxKw=pwrByTxKw,band=band
	forward_function anritinpbin
	ntx=6
	ntrace=3
	iiavg=0
	iimax=1
	iimin=2
	if n_elements(minTx) eq 0 then minTx=4
	if n_elements(band) eq 0 then band=5
	ndir=n_elements(dirL)
	a={gotDat: intarr(ntx),   $ ; 0 no data, 1 got data
   		freq  : fltarr(ntx),   $ ; for each tx
   		tm    : lon64arr(ntx), $ ; sec 1970
 	    dat   :fltarr(3,ntx)   $ ; avg,max,min
	}
	feps=.1 
	nbands=1
	case band of 
		5: f0 = 5.1
		8: f0 =8.175
       -1: begin
			f0=[5.1,8.175]
			nbands=2
		   end
		else: begin
			print,"band must be 5, 8, or -1 (to search both bands) "
			return,-1
		    end
	endcase
	fminS=f0 - feps
	fmaxS=f0 + feps
	maxNum=ndir*5000L
	pwrI=replicate(a,maxNum)
	fpat="hf*.dat"
	icur=0L
	for idir=0,ndir-1 do begin
		needFreq=1
		pat= dirL[idir] + "/" + fpat	
		flist=file_search(pat)
		nfiles=n_elements(flist)
	 	if keyword_set(verb) then print,'starting:',pat
		for i=0L,nfiles-1 do begin
    		istat=anritinpbin(flist[i],ntrace,fI)
			a=stregex(flist[i],".*([0-6]).dat",/subex,/extr)
			txNum=fix(a[1])
			itx=txNum-1
			if (needFreq) then begin
			    if nbands eq 1 then begin
					iifS=where((fi.Freq ge fminS) and (fi.freq le fmaxS),cnt)
					iiFs=[iiFs[0],iiFs[cnt-1]]
			    endif else begin
					iifS=where((fi.Freq ge fminS[0]) and (fi.freq le fmaxS[0]),cnt)
					iiFs1=[iiFs[0],iiFs[cnt-1]]
					iifS=where((fi.Freq ge fminS[1]) and (fi.freq le fmaxS[1]),cnt)
					iiFs2=[iiFs[0],iiFs[cnt-1]]
					iiFS=reform([iifs1,iifs2],2,2)
			    endelse
				needFreq=0
			endif
;		print,i,itx,icur,pwrI[icur].gotDat[itx]
			if (pwrI[icur].gotDat[itx] ne 0) then icur++
			pwrI[icur].tm[itx] = fi.mtime
			pwrI[icur].gotDat[itx] = 1
			if nbands eq 1 then begin
				pwrI[icur].dat[0,itx]=max(fI.spc[iifs[0]:iifs[1],0])
				pwrI[icur].dat[1,itx]=max(fI.spc[iifs[0]:iifs[1],1],ind)
				pwrI[icur].dat[2,itx]=max(fI.spc[iifs[0]:iifs[1],2])
				indToUse=0
			endif else begin
			    ; use pkhold to determine which is larger..
				max1_pk=max(fI.spc[iifs[0,0]:iifs[1,0],1],ind1)
				max2_pk=max(fI.spc[iifs[0,1]:iifs[1,1],1],ind2)
			    indToUse=(max1_pk gt max2_pk)?0:1
				pwrI[icur].dat[0,itx]=max(fI.spc[iifs[0,indToUse]:iifs[1,indToUse],0])
                pwrI[icur].dat[1,itx]=max(fI.spc[iifs[0,indToUse]:iifs[1,indToUse],1],ind)
                pwrI[icur].dat[2,itx]=max(fI.spc[iifs[0,indToUse]:iifs[1,indToUse],2])

			endelse
			pwrI[icur].freq[itx]=fI.freq[iifs[0,indToUse] + ind]
		endfor
	endfor
	ntot=icur
	if ntot lt maxNum then pwrI=pwrI[0L:ntot-1]
;
	if (arg_present(pwrTot)) then begin
		aa=total(pwrI.gotdat,1)
		iiBad=where(aa lt minTx,cntBad) 
	endif
; 
	pwrByTxKw=fltarr(ntot,ntx,3)
	if nbands gt 1 then begin
		ii5=where(pwri.freq[0] lt 6,cnt5)
		ii8=where(pwrI.freq[0] gt 6,cnt8)
	endif
	for itx=0,ntx-1 do begin &$
		if nbands eq 1 then begin
			for itr=0,2 do begin &$
	    	  pwrByTxKw[*,itx,itr]= reform(hf_cplrtoxmitpwr(pwrI.dat[itr,itx],band=band),ntot) &$
			endfor &$
		endif else begin
			for itr=0,2 do begin &$
				if cnt5 gt 0 then begin
    	  			pwrByTxKw[ii5,itx,itr]= reform(hf_cplrtoxmitpwr(pwrI[ii5].dat[itr,itx],band=5),cnt5) &$
				endif
				if cnt8 gt 0 then begin
    	  			pwrByTxKw[ii8,itx,itr]= reform(hf_cplrtoxmitpwr(pwrI[ii8].dat[itr,itx],band=8),cnt8) &$
				endif
			endfor &$
		endelse
	endfor
	pwrTot=total(pwrByTxKw,2)
	if cntBad gt 0 then pwrTot[iiBad,*]=-1.
	return,n_elements(pwrI)
end
