;+
;NAME:
;hf_inpday - input and process hf data for a day
;SYNTAX:istat=hf_inpday(yymmdd,corfileNm,hrLim=hrlim,mintx=mintx,band=band,$
;                       maxCorRecs=maxCorRecs,corFrqRng=corFrqRng,corpol=corpol,$  
;                        hfIDay)
;ARGS:
;yymmdd   : long    date to process
;corfileNm: string  name of cor file that has the monitor data
;                   if '' then don't input any cor data
;KEYWORDS:
; hrlim[2]:float    hours to limit data (min max) ast hours.
; mintx : int       minimum number of tx needed when inputing txPwr. def:4
; band  : int       5 or 8. Where to get tx peak power.
;maxCorRecs:long    max number of cor recs to input. default: 20000
;corFrqRng[2]:float freq range in cor data to compute total power.
;                   default: 425 -435
;corPol      :int   if provided then only compute cor power from this 
;                   pol. 1-polA,2-polB, 12: both (default)
;
;RETURNS;
;	istat: 0	
;  hfIDay: {}      hf data for the day containing:
;   ntxpwr:               long  number of entries txPwrI
;   txPwrI[ntxpwr]: {}    struct holding tx power
;   txPwrHrAll[ntxpwr,6]: float hour tmstamp for each tx of each set.
;                         The spec ana reads the the tx sequentially.
;   nLogI     : long      number of synth log entries returned
;   logI[nlogI]: {}       array of structs with logI
;   logIHr[nlogI]:float   ast hour for each logI entry
;   nCorPwr   : long      number of cor pwr entries returned
;   corPwr[ncorPwr]:float corpwr.. normalized to median.
;   corpwrhr[ncorPwr]:float ast hour for each sample
;   txPwrTot[ntxpwr,3]:   float total transmitted power. sum of all tx.KW.
;                         2nd ind: [avg,pkHold,minHold]
; txPwrByTx[ntxpwr,6,3]:  float transmitted power by tx (KW).
;                         3rd ind is [avg,pkHold,minHold]
; 
;
;DESCRIPTION:
; process hf info for a day.
; run from the directory where you want to store the data.
; The program will:
;
;1. input anritsu power logs
;   plot the data
;2. input synthesizer values used
;   plot the data
;3. input correlator data (if filename not  '') 
;
; To call:
; idl71
; @phil
; @hfinit
; @corinit
; @spcanainit
; yymmdd=xxxxx
; corfileNm='/share/olcor/corfile. xxx'
; n=hf_inpday(yymmdd,corfileNm,hfIDay)
;-
function hf_inpday,yymmdd,corfileNm, hrlim=hrlim,$
		mintxNum=mintxNum,band=band,$
		maxCorRecs=maxCorRecs,corFrqRng=corFrqRng,corpol=corpol,$
		hfIDay 
;
;
	lyymmdd=string(format='(i06)',yymmdd)
	getCorData=corfileNm ne ''
	if n_elements(mintxNum) eq 0 then mintxNum=4
	if n_elements(band) eq 0 then  band=5
	hrLimL=[0,24.]
	if n_elements(hrLim) eq 2 then hrLimL=hrLim
	if n_elements(maxCorRecs) eq 0 then maxCorRecs=20000L
	corFrqRngL=[425,435.]
	if n_elements(corFrqRng) eq 2 then corFrqRngL=corFrqRng
	if n_elements(corPol) eq 0 then corPol=12
;
; input tx power
;
	dirl='/share/phildat/hf/' + lyymmdd
	ntxpwr=hf_gettxpwr(dirl,txPwrI,pwrtot=txPwrTot,pwrByTxKw=txPwrByTx,band=band)
	gotAr=total(txPwrI.gotdat,1)
	ii=where(gotar gt 4,n)
	txPwrI=txPwrI[ii]
	txPwrTot=txPwrTot[ii]
	txPwrByTx=txPwrByTx[ii,*,*]
;
; get ast hours from secs1970
;
	T0=tosecs1970(yymmdd,40000l)
	txPwrhrAll=transpose((txPwrI.tm - T0)/3600.)
	ntxPwr=n_elements(txPwrhrAll[*,0])
	ii=where((txPwrhrAll[*,0] ge hrLimL[0]) and (txPwrhrAll[*,5] le hrLimL[1]),cnt)
	if cnt ne ntxPwr then begin
		ntxPwr=cnt
		txPwrI=txPwrI[ii]
	    txPwrHrAll=txPwrhrAll[ii,*]
	    txPwrTot=txPwrTot[ii]
		txPwrByTx=txPwrByTx[ii,*,*]
	endif
;
; input synlog
;
	nLogI=hf_inpsynlog(yymmdd,logI)
	logIHr=logI.secMid/3600.
	ii=where((logIHr ge  hrLimL[0]) and (logIHr le hrLimL[1]),cnt)
	if (cnt ne nlogI) then begin
		if cnt eq 0 then begin
			nlogI=0
		    logI=''
			logIHr=0
		endif else begin
			logI=logI[ii]
			logIHr=logIHr[ii]
			nlogI=cnt
		endelse
	endif
	if nlogI eq 0 then begin
		print,"No synLog data found for hours:",hrLimL
	endif
;
; inpput correlator data
;
	gotcordata=0
	if getCorData then begin
 		openr,lun,corFileNm,/get_lun
		istat=corgetm(lun,maxCorRecs,b,/han)
		ncorPwr=n_elements(b)
		if ncorPwr eq 0 then goto,nocordata
		corPwrHr=b.b1.h.std.time/3600.
		ii=where((corPwrHr ge hrLimL[0]) and (corPwrHr le hrLimL[1]),cnt)
		if cnt eq 0 then goto,nocordata
		if cnt ne ncorpwr then begin
			b=b[ii]	
			corPwrHr=corPwrHr[ii]
		endif
		freq=corfrq(b[0].b1.h)
		ii=where((freq ge corFrqRngL[0]) and (freq le corFrqRngL[1]),cnt)
		tp=transpose(total(b.b1.d[ii,*],1)/cnt)
		case corpol  of
			1: corPwr=tp[*,0]/median(tp[*,0])
			2: corPwr=tp[*,1]/median(tp[*,1])
		  else: begin
			   tp[*,0]/=median(tp[*,0])
			   tp[*,1]/=median(tp[*,1])
			   corPwr=total(tp,2)*.5
		  end
		endcase
		free_lun,lun
		gotcordata=1
	endif
nocordata:
	if gotcordata eq 0 then begin
		print,'no cordata found in file, hrlimits:",corfileNm,hrLimL'
		ncorpwr=0l
		corPwr=''
		corpwrHr=''
	endif
	;
	;  now create struct to return data
	;
	txI={ band : band,$ 
		  minTxNum: minTxNum,$
	       n   :ntxPwr,$
		  hrAll:txPwrHrAll,$
	      pwrI:txPwrI,$
		  pwrTot:txPwrTot,$
		  pwrByTx:txPwrByTx }
	logI1={$
			n:nlogI,$
			hr  :logIHr,$
			d   :logI}
	corI={$
	       maxCorRecs:maxCorRecs,$
		   tpFrqRng:corfrqRngL,$
		   tpPol   :corpol,$
			n:ncorPwr,$
			hr:corPwrHr,$
			tp:corPwr}

	hfIDay={yymmdd:yymmdd,$
           hrLim:hrLimL,$
	       txI:txI,$
	      logI:logI,$
		  corI:corI}
	return,1
end
