;+ 
;NAME:
;mascalsclspc - Scale from spectrometer cnts to K using cal spectra
;
;SYNTAX: istat=mascalsclspc(bspc,calI,bK,avgK=avgK)
;ARGS  :
;   bspc[n]:{}: mas struct holding spectra to convert to kelvins.
;   calI{} :    cal info structure returned from mascalonoffspc().
;KEYWORDS:
;              
;RETURNS:
; istat:                : 0 ok, -1 error
; bK[n]:                : spectra returned in degrees K
; avgK[ndump*nrecs,npol]: average spectral values in degK
;                It will use the mask in calI to exclude any bad channels.
;DESCRIPTION:
;   Scale spectra from spectrometer counts to Kelvins. Before calling
;this routine you must call mascalonoffspc() to get the calI structure 
;loaded.
;	There are two sets of cal routines:
;  mascalxxx()  and mascalxxxspc(). 
;	The first set (mascalxx()) computes averages over bandpasses to generate
; the conversion factors to go from spetrometer counts to degK. They are normally
; used when you want to make sure that the calvalue computations do not add to
; the noise of the final spectral results.
;;
;	The second set (mascalxxxspc()) does not average over freq. It retains the
;freq dependence of the cal values. This gives a better idea of tsys freq dependence
; (especially for wider band mesaurements). The drawback is that the cal computation has the 
;potential to increase Tsys (if the cal integration is much shorter than the 
;normal data acquistion). These routines can be used with a hardware winking cal where
;1/2 the time the cal is on, and 1/2 the time the cal is off. They used the cal deflection
;for the band pass correction. If there is any frequency dependence in the signal before
;the cal insertion (eg.. standing waves..) they will not be cancelled.
;
;Examples:
;;0. get the winking cal data
;   istat=masgethwcal(fnmI[ifile],calIhw,bonar,boffar,bavgar=bavgar,brmsar=brmsar,$
;                     /avgrow,/dctinp
;;  get the cal values for calib files
;	istat=mascalvalspc(bavgar[0].h,calI)
;;  compute cal scale factors using avg spectra
;;  exclude 10% edges for sb, 6% for xb
;;  cmp mask using calration and the rms/chan from brmsar
;   cmpmask=2
;   istat=mascalonoffspc(bavgar[0],bavgar[1],calI,edge=edge,cmpmask=cmpmask,brmsar=brmsar)
;;  now scale calOff avg spectra to degK .. (or you could do the 40ms caloff spectra
;   istat=mascalsclspc(bavgar[1],calI,avgK=avgK)
;  
;NOTES:
;
;-
;modhistory
;05jan23 - stole from mascalxxx routines
;
function mascalsclspc,bspc,calI,bK,avgK=avgK
;
;   
	nrecs    =n_elements(bspc)
	nchan    =bspc[0].nchan
	ndumps   =bspc[0].ndump
	nspc     =nrecs*ndumps
	npol     =bspc[0].npol
	exposure=bspc[0].h.exposure
	
;
;   in case cals taken for more/less time than data
;
	scl=calI.cntsToK*calI.exposure/exposure
	if bspc[0].npol eq 4 then begin
		xx=scl[0]
		yy=scl[1]
		xy=sqrt(xx*yy)
		scl=[xx,yy,xy,xy]
	endif
;
; 	have to do the multiply manually..
;
	bk=bspc
	if ndumps eq 1 then begin
		for irec=0,nrecs-1 do begin
			if npol eq 1 then begin
				bk[irec].d*=scl
			endif else begin
				bk[irec].d[*,0]*=scl[*,0]
				bk[irec].d[*,1]*=scl[*,1]
			endelse
		endfor
	endif else begin
		for irec=0,nrecs-1 do begin
			if npol eq 1 then begin
				for idump=0,ndumps-1 do begin
					bk[irec].d[*,idump]*=scl
				endfor
			endif else begin
				for idump=0,ndumps-1 do begin
					bk[irec].d[*,0,idump]*=scl[*,0]
					bk[irec].d[*,1,idump]*=scl[*,1]
				endfor
			endelse
		endfor
	endelse
;	
;   also compute the average Tsys degK if they want
;
	if (arg_present(avgk)) then begin
		iiuse=where(calI.mask ne 0,cnt)
		if (ndumps eq 1) then begin
			avgK=(npol eq 1)?total(bK.d[iiuse])/cnt:total(bK.d[iiuse,*],1)/cnt
			; make nspc,npol
			avgk=reform(avgk)
			if (npol gt 1) and (nrecs gt 1)  then avgK=transpose(avgK)
		endif else begin
			if npol eq 1 then begin
			   avgK=reform(total(bK.d[iiuse,*],1)/cnt,ndumps*nrecs)
		    endif else begin
			   avgK=transpose(reform(total(bK.d[iiuse,*,*],1)/cnt,npol,ndumps*nrecs))
			endelse
		endelse
	endif
	return,0
end
