;+
;NAME:
;mascrossinp - input cross data, compute  totpwr
;SYNTAX: istat=mascrossinp(cFI,cI,baz=baz,bel=bel,brms=brms,edge=edge,$
;                          inpmask=inpmask,$
;                          freqExcl=freqExcl,maskUsed=maskUsed)
;ARGS:
;cFI          : {}    cross file info (see masfindcross)
;KEYWORDS:
;edge[2,nbms] : float fraction of edge to exclude on each side when computing total power.
;                     def .06 (6%)
;inpmask[nchan]:intarr if supplied then use this as the mask (don't bother computing one)
;                      it will still include edge and any freqexcl[] values. Use this if you precompute
;                      a mask using many input scans.(see masmakemask)
;freqExcl[2,m]:float  MHz. if supplied then don't include these freq in total power computation
;                     You can specify up to m freq sections
;                     freqExcl[0,..] is min Freq,freqExcl[1,..] max freq for this section
;                     For each beam, it will exclude all sections within the current band.
;RETURNS:
;istat   :    1  ok
;            -1 error
;cI      : {}    cross info returned here.
;maskUsed: intarr(nchan,nbms) If supplied then return the mask of good channels
;                             we computed. 1-> used, 0-> not used
;                            The same mask is used for polA and polB
;
; -->NOTE:
;
;DESCRIPTION:
;	The routine will input the spectral data files for a cross pattern and then compute
;the total power. It will process a single cross on each call.
;	The user inputs:
; - a cFI (CrossFileInfo structure) for the cross to process (see masfindcross() routine).
; - edge= keyword 
;   -can be used to exclude a fraction of the spectra on each edge of the spectra. 
;   - default value is .06
;   - The fraction to exclude can be 0 to .99 . 
;   - The edge dimension can be:
;     edge=.06 .. exclude 6% of spectra on each band edge for all bands
;     edge=[.1,.06] exclude 10% on the left edge, 6% on right edge for all bands
;     edge=[2,bands] specify the left,right edge for all bands
; - freqExcl= freqExcl[2,m]  freq range sections to exclude in total power computation. Tested
;                            separately for each beam. 
; 
; The routine will then:
; For each bm:
; 1. input the azimuth and el strips
; 2. interpote across the DC freq channel
; 3. using the 1st 1/3 and last 1/3 of the azimuth strip, compute rms/mean by channel
;    - we don't use the el strip since tsys can change vs el.
; 4. compute a mask for freq channels  to use in the total power computation.
;    - intialize the mask using the edge= and freqExcl= keywords.
;    - do a robust linear fit to the rms/mean by channel
;    - exclude any points > 3 sigma from the fit.
;    - use the same mask for polA,B and az,el strips
; 5   if cals are present compute the counts to Kelvins scale factor using the same mask as the
;     strips
; 6. compute the total power for the az, el strips for polA and polB using the above mask.
;    and convert to kelvins if cals are present 
;
; Return the cI structure
;
; If the following keywords are present then return:
;  let n be the number of rows in the fits  file for a strip
; baz=baz .. return the n mas structures holding the individual az strip spectra
; bel=bel .. return the n mas struxtures holding the individual el strip spectra
; brms=brms. return the 1  mas brms structure  computed from the az strip
; maskUsed=  return maskUsed[nchan,nbms] int array. If a channel=1 it was included, 0
;                   means it was not included in the total power measurement.
;EXAMPLE:
;; get the file list
; proj='x101'
; yyyymmdd=20220406l
;; find the files
; ncross=masfindcross(yyyymmdd,proj,cFI,/verb)
;; loop processing the  crosses
;; you could take a subet by source or rcvr , or just do everything
;; eg  ii=where(cFI.srcName='3C144',ncross)
;; you also might want to keep track of the freq channels excluded
; for icross=0,ncross-1 do begin
;    istat=mascrossinp(cFI[icross],cI,maskused=maskUsed) 
;    if icross eq 0 then begin
;       cIAr=replicate(cI,ncross)
;       nbms=cI.nbms
;       nchan=cI.bmI[0].h.chan_in   ; freq chan used
;       maskAr=fltarr(nchan,nbms)
;    endif
;    cIAr[icross]=cI
;    maskAr+=maskUsed
; endfor
; maskAr/=ncross   ; to get a fraction 0..1
;; 
;; plot the the total  power az,el strips for a cross
;;
; plot,cIar[0].bmI[0].tpAz[*,0]               ; polA
; oplot,cIar[0].bmI[0].tpAz[*,1],col=colph[2] ; polB
;
; ----------
; The cI structure:
; the structure shown bel1ow used 7 freq bands (bms) for each cross:
; help,cI,/st
; .. info common to all beams
; RCV        STRING 'xb'            : receiver used xb or sb
; SRCNM      STRING '3C461'         : source name
; AZLEND     DOUBLE 1.2000000       ; azimuth strip length deg
; ELLEND     DOUBLE 1.2000000       : elevation strip length deg
; CROSSCEND  DOUBLE Array[2]        ; az,el center for cross 
; STRIPCEND  DOUBLE Array[2, 2]     ; [i,j]  i= az,el strip center, j=azstrip,elstrip
; AZPOSD     DOUBLE Array[1200, 2]  ; azposition [deg] for each sample az,el strips
; ELPOSD     DOUBLE Array[1200, 2]  ; elposition [deg] for each sample az,el strips
; AZOFFD     DOUBLE Array[1200]     ; az offset azimuth strip .great circle
; ELOFFD     DOUBLE Array[1200]     ; el offset el strip .great circle
; SMPSTRIP   LONG   1200            ; number of samples in each strip
; SMPTM      DOUBLE 0.10000000      ; sample time (secs) 
; MAXTRKERRD DOUBLE Array[2]        ; maxEncoder tracking error [deg] az,el strip 
; NBMS       INT              7     ; number of beams this cross.
; units      int            1,2,3   ; 1 - spectrometer counts, 2=calunits, 3=degK
; .. beam specific info
; BMI        STRUCT  Array[7]       ; beam specific info (see bel1ow)
;
; The beam specific info will depend on whether cals were present
; with no cals:
;help,cI.bmI,/st
; BM     INT     0                  : beam number 0..6
; CFR    DOUBLE  8221.0000          ; rf center frequency MHz
; HDRAZ  STRUCT  -> MASFHDR Array[1]: first header for azimuth strip
; HDREL  STRUCT  -> MASFHDR Array[1]: first header for el strip
; FPNTS  FLOAT   Array[2]           : fraction of freq chan used for tp: polA,B
; FLIST  STRING  Array[2]           : filenames for az,el strips
; TPAZ   DOUBLE  Array[1200, 2]     : computed total power azStrip:polA,B
; TPEL   DOUBLE  Array[1200, 2]     : computed total power elStrip:polA,B
;  with cals present
;help,cI.bmI,/st
; BM     INT     0                  : beam number 0..6
; CFR    DOUBLE  8221.0000          ; rf center frequency MHz
; HDRAZ  STRUCT  -> MASFHDR Array[1]: first header for azimuth strip
; HDREL  STRUCT  -> MASFHDR Array[1]: first header for el strip
; FPNTS  FLOAT   Array[2]           : fraction of freq chan used for tp: polA,B
; FLIST  STRING  Array[2]           : filenames for az,el strips
; FLISTCAL STRING    Array[2]       : filenames for calon,off
; CALI     STRUCT    -> <Anonymous> Array[1] ; cal info
; TPAZ   DOUBLE  Array[1200, 2]     : computed total power azStrip:polA,B
; TPEL   DOUBLE  Array[1200, 2]     : computed total power elStrip:polA,B
;
; and the calI struct will have:
;IDL> help,ci.bmi.cali,/st
;  CALVAL     FLOAT  Array[2]       : cal value in degK (pola,b). if =1 then not yet measured
;  NUMPOL     LONG   2              : number of pols
;  EXPOSURE   FLOAT  0.100000       ; sample time for spectra
;  CNTSTOK    FLOAT  Array[2]       ; convert spectrometer counts to degK
;  NPNTS      LONG   900            ; nfreq points used
;  FLIPPED    BYTE   0              ; was the spectra flipped in freq
;  EDGEFRACT  FLOAT  Array[2]       ; edge fract excluded each side. same for polA,B
;  USEMASK    BYTE   1              ; 0- just used mask, 1 - used mask
;  MASK       BYTE   Array[1024]    ; resultant mask 1=chanused, 0 not used. includes mask and edgefract
;
; Notes on Positions
;  The az,el positions come from the fits header h.azimuth, h.elevatio. These values have been interpolated to crval5 of the hdr.
;These positions do not have the model included (so back computing should give ra,dec without needing the model offsets.
; Finally the reported values have been interpolated to the center of each spectral/totalpower sample.
;-
; mods:
; 220509 - reported azimuth, elevation postions switched from enc_az,el to azimuth,elevatio
;          the new values azimuth,, elevation are interpolated to crval5 and have had the model correction removed.
;          prior to this, using enc_az, enc_el we had to assume the rate was applied at the start of datataking
;         (prior to 6may22 it wasn't) . this way the offsets are correct. 
; 220803  . check that file sizes for cals and sized for strips are the same for all beams. if not skip
function mascrossinp,cFI,cI,baz=baz,bel=bel,brms=brms,edge=edge,$
		  inpmask=inpmask,$
		  freqExcl=freqExcl,maskUsed=maskUsed
;
	forward_function calget1
	if n_elements(cFI) gt 1 then begin
		print,"mascrossinp: cFI should not be an array of  patterns"
		return,-1
	endif
	defEdge=[.06,.06] ; def to 6%
	retMask=(arg_present(maskUsed))
	nchan=cFI.sumicross.h.chan_in
	nbms=cFI.nbms
	useinpmask=n_elements(inpmask) gt 0
	if useinpmask and (n_elements(inpmask) ne nchan) then begin
		print,"--> # of channels in inpmask != to channels in cross"
		return,-1
	endif
;
;   check that all strips, cals have consistant lenghths
;
	ok=1
	fcrAr=file_info(cfi.flistcross[*,0:nbms-1])
	ii=where(fcrAr.size ne fcrAr[0].size,cnt)
	if cnt ne 0 then begin
		print,"-->file size mismatch. src,patid,bms,size0,sizemismath:",cfi.srcnm+ " ",cfi.patid,ii/2,fcrAr[0].size,fcrAr[ii].size
		return,-1
	endif
	npol=2
	edgeL=fltarr(npol,nbms)
	nedge=n_elements(edge)
	iedgAr=lonarr(npol,nbms) ; index first,last chan to use
	defEdge=[.06,.06]
	if nedge eq 1 then defEdge=[edge,edge]
	if nedge eq 2 then defEdge=edge
	iedgAr[0,*]=nchan*defEdge[0]
	iedgAr[1,*]=nchan*(1. - defEdge[1])
;   use same fract start,end
	if nedge eq nbms then begin
		for i=0,nbms-1 do begin
			iedgAr[0,ibm]=nchan*edge[i]
			iedgAr[1,ibm]=nchan*(1. - edge[i])
		endfor
		edgeL[0,*]=edge
		edgeL[1,*]=1. - edge
	endif
;   user specified start,end fract
	if nedge eq nbms*2 then begin
		edgeL=edge
		for ibm=0,nbms-1 do begin
			iedgAr[0,ibm]=nchan*edge[0,ibm]
			iedgAr[1,ibm]=nchan*(1. - edge[1,ibm])
		endfor
	endif
;
; see if they entered and freq ranges to exclue
;
	nExclRange=n_elements(freqExcl)/2
		
;	do we return rms of each az set
	retRms=arg_present(brms)
	retAz=arg_present(baz)
	retEl=arg_present(bel)
;   use rms to exclude bad freq chan.  	exclude middle of strip where source is (1/3)
	nrows=cFI.sumICross.nrows
	nrowsd3=nrows/3			; use 1/3 start,end of az strip
;   get indices in strip to use for rms
	iirms=lindgen(nrows)
	iirms[nrowsd3:nrowsd3*2]=-1
	ii=where(iirms ne -1)
	iirms=iirms[ii]
	xrms=findgen(nchan)/nchan
;
;   loop over the beams
;
	hasCal=cfI.hasCal
	for ibm=0,nbms-1  do begin
		istat=masgetfile(desc,baz1,/double,filename=cfI.flistCross[0,ibm])
		if (istat ne 1) then begin
			print,"mascrossinp:masgetfile returns istat:",istat," for az strip  bm:",ibm 
			return,-1
		endif
		istat=masgetfile(desc,bel1,/double,filename=cfI.flistCross[1,ibm])
		if (istat ne 1) then begin
			print,"masgetfile return istat:",istat," for el strip  bm:",ibm 
			return,-1
		endif
;
;  get the cals
;
		if hascal then begin
			istat=masgetfile(desc,bcalOn,/avg,/double,filename=cfI.flistCal[0,ibm])
			if (istat ne 1) then begin
				print,"mascrossinp:masgetfile returns istat:",istat," for calOn  bm:",ibm 
				return,-1
			endif
			istat=masgetfile(desc,bcalOff,/avg,/double,filename=cfI.flistCal[1,ibm])
			if (istat ne 1) then begin
				print,"mascrossinp:masgetfile returns istat:",istat," for calOff  bm:",ibm 
				return,-1 
			endif
		endif
		if ibm eq 0 then begin
;
;			 need to generate the calInfo struct
;
			if hascal then begin
;			    check if we have cal values for this rcvr
;               kludge for old 12m rcvr
;               for new rcv to alway use rcvnum 20
;
			   if  (strmid(cfi.sumical[0].h.frontend,0,2) eq 'wb') then begin &$
			   		rcvNum =20 &$
			  endif else begin  &$
			    rcvNum=(cfi.cfrAr[0] gt 5000)?11:7 &$
			  endelse
			    caltype=1 ; hcorcal
			    istat=calget1(rcvNum,calType,cfi.cfrar[0],calval,/p12m)
		      
				if istat gt 0  then begin
					units=(calval[0] eq 1.)?2:3  ; cal=1 --> no meausured value unit=cal, 3=K
					istat=mascalonoff(bcalOn,bcalOff,calI,edge=edgeL[*,ibm])
			    endif else begin
					print,"No cal values for rcvr:" + bcalon[0].h.frontend
				 	hascal=0
				endelse
			endif
			if hascal eq 0 then units=1
			nrecs=n_elements(baz1)
			ndump=baz1[0].ndump
			nsmp=nrecs*ndump
			nstrip=2   ; az,el
	        iicen=nrecs/2 ; to get az,el location of center. recorded values at the start
			npol=baz1[0].npol
			nchan=baz1[0].nchan
			if retMask then maskUsed=intarr(nchan,nbms)
			if hascal then begin
			   acal={ calVal   : fltarr(2) ,$;  averaged cal values
           			numPol     : npol,$;
                    exposure  :     0.    ,$; filled in by mascalonoff
                    cntsToK  : fltarr(2),$; filled in by mascalonoff
          			npnts     : 0l,$ ; points used to compute average cal.
                    flipped   : 0b,$ ; 1 if band is flipped
                    edgeFract : fltarr(2),$  ; now same order as spectra
                    usemask   : 0b,$  ; 1--> used freq mask
;                  			                   0--> used edgeFract
                    mask      : bytarr(nchan)}  ; mask. 0 not used,. 1 used

			aa={ bm : 0 ,$; 0..nbms-1 ; .. bmnumber.. 0..6, -1 --> nodata this beam
	              cfr: 0d,$; center freq tp in MHz
                  hdrAz:baz1[0].h,$;
                  hdrEl:bel1[0].h,$;
				  fpnts:fltarr(2),$;  fract of points used for tp accum.polA,polB 
				  flist:strarr(2),$;  filenames for az,el strips
				  flistCal:strarr(2),$; filenames for cal on,off
				  calI    :acal,$   ,; holds cal info
				  tpAz      :dblarr(nsmp,npol),$; total power each spectra
				  tpEl      :dblarr(nsmp,npol)}; total power each spectra
			endif else begin
				aa={ bm : 0 ,$; 0..nbms-1 ; .. bmnumber.. 0..6, -1 --> nodata this beam
	            	  cfr: 0d,$; center freq tp in MHz
                  	 hdrAz:baz1[0].h,$;
                  	hdrEl:bel1[0].h,$;
				  	fpnts:fltarr(2),$;  fract of points used for tp accum.polA,polB 
				  	flist:strarr(2),$;  filenames for az,el strips
				  	tpAz      :dblarr(nsmp,npol),$; total power each spectra
				  	tpEl      :dblarr(nsmp,npol)}; total power each spectra
			endelse
				   
			cI={ rcv: baz1[0].h.frontend,$  ; xb or sb 
				 srcNm: baz1[0].h.object,$;    srcnm
			 	 hasCal:cfi.hascal,$    ; 0-> no cal, 1=cal
;                lengths are great circle
                 azLenD:abs(baz1[0].h.req_OffC1*2.),$; az strip length (great circle) deg
			     elLenD:abs(bel1[0].h.req_OffC2*2.),$; el strip length deg
;                cross center is az,el results of fit
				 crossCenD:dblarr(2),$;el= end of az strip, az=beginng el strip
;                az,el center for each strip
				 stripCenD:dblarr(2,nstrip),$;az,el  center for each strip
;                az,el pos for each sample. interpolated to center of data sample
				 azPosD  : dblarr(nsmp,nstrip),$; azpos for az,el strip
				 elPosD  : dblarr(nsmp,nstrip),$; elpos for az,el strip
				 azOffD  : dblarr(nsmp),$         ; az offset azimuth strip .great circle
				 elOffD  : dblarr(nsmp),$        ; el offset el strip .great circle
				 smpStrip:nsmp,$            ; samples in a strip
				 smpTm   :baz1[0].h.cdelt5,$ ; time between samples .. secs
		         maxTrkErrD: dblarr(2),$    ; max tracking error for az, el strip.
				 nbms:nbms,$
				 units: (hasCal)?3:1,$;            ; 1 spectrometer,2-cal,3-kelvins
				 bmI: replicate(aa,nbms)}   ; info or each beam 
				 cI.maxTrkErrD[0]=max(baz1.h.cur_err);  max tracking error az strips deg.
				 cI.maxTrkErrD[1]=max(bel1.h.cur_err);  max tracking error az strips deg.
;
;	             compute az,el for each smp in strips
;                need to interpolate because we may have many spectra/row header
;
				 smpTm=cI.smpTm
				 tmPos=dblarr(nsmp,2) ; time for each sample
				 tmPos[*,0]=dindgen(nsmp)*smpTm + baz1[0].h.crval5 + smpTm/2. ; center in sample
				 tmPos[*,1]=dindgen(nsmp)*smpTm + bel1[0].h.crval5 + smpTm/2. ; center in sample
				 for istrip=0,1 do begin 
					if istrip eq 0 then begin
			     		haz=baz1.h.azimuth
			     		hel=baz1.h.elevatio
				 		htm=baz1.h.crval5
				    endif else begin
			     		haz=bel1.h.azimuth
			     		hel=bel1.h.elevatio
				 		htm=bel1.h.crval5
					endelse
				 	ii=where(htm lt htm[0],cnt) ;check for midnite crossing
				 	if cnt gt 0 then htm[ii]+=86400d
;   USEMASK   BYTE      1     ; 1 --> used mask,0--> used edgefract
;                now interpolate to center of sample
				 	cI.azPosD[*,istrip]=interpol(haz,htm,tmPos[*,istrip])
				 	cI.elPosD[*,istrip]=interpol(hel,htm,tmPos[*,istrip])
;                   interpolate the az, el offsets.. (greate circle)
					if istrip  eq 0 then begin
						cI.azOffD=interpol(baz1.h.map_offaz,htm,tmPos[*,istrip]) 
					endif else begin
						cI.elOffD=interpol(-bel1.h.map_offza,htm,tmPos[*,istrip])   ; eloff=-zaoff &$
					endelse
				endfor
		
				azCenAz=interpol(baz1.h.azimuth ,baz1.h.map_offaz,0.); 
				elCenAz=interpol(baz1.h.elevatio,baz1.h.map_offaz,0.);
				azCenEl=interpol(bel1.h.azimuth ,bel1.h.map_offza,0.); zaoff=0 --> eloff=0 
				elCenEl=interpol(bel1.h.elevatio,bel1.h.map_offza,0.);
			
		        cI.stripCenD[*,0]=[azCenAz,elCenAz]
		        cI.stripCenD[*,1]=[azCenEl,elCenEl]
		        cI.crossCenD[0]  =cI.azPosD[0,1] ; start of el strip
		        cI.crossCenD[1]  =cI.elPosD[nsmp-1,0];  end of  azstrip
		
		endif
		cI.bmI[ibm].flist=cfI.flistCross[*,ibm]
;		interpolate across dc
		idc=nchan/2l
		if ndump eq 1 then begin
			baz1.d[idc,*,*]=(baz1.d[idc-1,*,*] + baz1.d[idc+1,*,*])*.5
			bel1.d[idc,*,*]=(bel1.d[idc-1,*,*] + bel1.d[idc+1,*,*])*.5
		endif else begin 
			baz1.d[idc,*,*,*]=(baz1.d[idc-1,*,*,*] + baz1.d[idc+1,*,*,*])*.5
			bel1.d[idc,*,*,*]=(bel1.d[idc-1,*,*,*] + bel1.d[idc+1,*,*,*])*.5
		endelse
		cI.bmI[ibm].hdraz=baz1[0].h
		cI.bmI[ibm].hdrel=bel1[0].h
		cI.bmI[ibm].cfr=baz1[0].h.crval1*1d-6 ; cfr Mhz
;
; 		see if any exclude regions are in this band
;
		nExclFnd=0
		if nexclRange gt 0 then begin
			okExcl=intarr(nchan) + 1
			freq=masfreq(baz1[0].h)
			f0=freq[0]
			f1=freq[nchan-1]
	    	if (f0 gt f1) then begin
		  		temp=f0
		   		f0=f1
		  		f1=temp
			endif
			for i=0,nexclRange-1 do begin
				if (f1 lt freqExcl[0,i]) or (f0 gt freqExcl[1,i]) then continue 
				ii=where((freq ge freqExcl[0,i]) and (freq le freqExcl[1,i]),cnt)
				if cnt gt 0 then begin	
					okExcl[ii]=0
				endif
			endfor
			iiexcl=where(okExcl eq 0,nexclFnd)
		endif
		maskInit=(useinpmask)?inpmask:intarr(nchan) + 1
		maskInit[0:iedgAr[0,ibm]]=0
		maskInit[iedgAr[1,ibm]:*]=0
		if (nexclFnd gt 0) then maskInit[iiexcl]=0
		; compute mask from rms
       	ok=intarr(nchan)
		if useinpmask eq 0 then begin
 			iiskip=where(maskInit eq 0)
			brms1=masrms(baz1[iirms])
			deg=1
			for ipol=0,npol-1 do begin
;				find  bad channels using az rms
		   		coef=robfit_poly(xrms,brms1.d[*,ipol],deg,gindx=gindx,ngood=ngood,$
               	 	fpnts=fpnts,mask=maskInit)
        ; make sure edge is still excluded
           		ok[gindx]+=1
			endfor
;       make sure original request still zeroed
        	ok[iiskip]=0
			if nexclFnd gt 0 then ok[iiexcl]=0
		; use inpmask with edge and freqexcl
		endif else begin
			if retRms then brms1=masrms(baz1[iirms])
			ii=where(maskInit ne 0)
			ok[ii]=2
		endelse
       	gindx=where(ok eq 2,ngood)
        fpnts=(ngood*1.)/nchan
		if retMask then maskUsed[gindx,ibm]=1
		scale=[1.,1.]
		if hascal then begin
			cI.bmI[ibm].flistcal=cfi.flistcal[*,ibm]
			istat=mascalonoff(bcalon,bcaloff,calI,mask=ok)
			strtmp=acal
			struct_assign,calI,strtmp
			strtmp.mask[gindx]=1
		    cI.bmI[ibm].calI=strtmp
			scale=calI.cntsToK
		endif
		for ipol=0,1 do begin
			cI.bmI[ibm].fpnts[ipol]=fpnts ;	 fraction of points used for total power
			if ndump eq 1 then begin	
				cI.bmI[ibm].tpaz[*,ipol]=scale[ipo]*total(baz1.d[gindx,ipol],1)/ngood
				cI.bmI[ibm].tpel[*,ipol]=scale[ipol]*total(bel1.d[gindx,ipol],1)/ngood
			endif else begin
				cI.bmI[ibm].tpaz[*,ipol]=scale[ipol]*total(reform(baz1.d[gindx,ipol,*],ngood,nsmp),1)/ngood
				cI.bmI[ibm].tpel[*,ipol]=scale[ipol]*total(reform(bel1.d[gindx,ipol,*],ngood,nsmp),1)/ngood
			endelse
		endfor
;   do we return the rms?
		if retRms then begin
			if ibm eq 0 then begin
			    brms=replicate(brms1,nbms)
			endif 
		    brms[ibm]=brms1
		endif
		if retAz then begin
			if ibm eq 0 then begin
				n=n_elements(baz1)
			    baz=replicate(baz1[0],n,nbms)
			endif
		    baz[*,ibm]=baz1
		endif
		if retEl then begin
			if ibm eq 0 then begin
				n=n_elements(bel1)
			    bel=replicate(bel1[0],n,nbms)
			endif
			bel[*,ibm]=bel1
		endif
	endfor
	return,1
end
