;+
;NAME:
;masfindcross - find cross files 
;SYNTAX: ncross=masfindcross(yyyymmdd,proj,cfI,rcv=rcv,verb=verb,dirI=dirI)
;ARGS:
; yyyymmdd: long   date to search for
; proj    : string project name for files
;KEYWORDS:
; rcv     : string limit to recevier. "sb" or "xb". default: all
; verb    :        if set then print progress as you search
;dirI[2]  : strarr in case files moved .. (see masfilelist )
;RETURNS  :
;ncross   : the number of crosses we found
;cfI[ncross]: {}     array of structs holding Cross File Info.
;
;DESCRIPTION:
;   This routine is written for the 12meter data.
;What is a cross pattern:
;   A cross pattern will consist of:
; - an optional calOn, caloff scan at the beginning.
; - an azimuth scan
; - an elevation scan.
; So there can be 2 or 4 scans in a pattern. 
; 	There can als be multiple frequency bands (bms) in a scan (1 to 7).
; Each scan/bm is stored in separate file so a pattern can have up to 4*7=28 files
;associated with it.
;   All scans in the pattern will have the same pattern_id (equal to the 
;scannumber of the first scan in the pattern). This is used to group scans into
;the pattern.
; 
;	This routine will scan all files for a day and extract the files the 
;cross files belonging to the specified projectd id. It will the group all 
;of the files for a particular pattern into a cfI (crossFileInfo) structure. An
;array of these structures will be returned. This aray can then be passed to
;the cross processing routines.
;	Some tests are done on the scans found for each pattern:
; - it must have an az and el scans.
;   - the number of beams for the az,el scan must be the same.
; - it can optionally have  calon,caloff scans.
;   - the number of beams in the calon,caloff scans must be the same
; - If the scans in a pattern don't meet these requirements then it 
;   is skipped. An example might be the azimuth strip completing, but the
;   source sets before the elevation strip.
;   --> Warning..this only finds files for 1 day. if the pattern straddles
;       midnight then it will be skipped.
;
;   An example of the returned struct array is shown below. These crosses
;were made at xband and had 7 freqbands (bms) for each cross.
;IDL> help,cFI,/st
; PATID      LONG    125300002        .. unique pattern id for pattern
; SRCNM      STRING  '3C461'          .. source name
; NBMS       INT        7             .. number of beams in this pattern
; BMLIST     INT     Array[7]         .. bm numbers 0..6 
; CFRAR      DOUBLE  Array[7]         .. center freq (MHz) each band
; FLISTCROSS STRING  Array[2, 7]      .. filenames: [az,el strips] for the 7 beams
; SUMICROSS  STRUCT  -> Array[1]      .. sumI struct for azstrip 1st bm
; HASCAL     INT        0             .. 1 if pattern has cal..if not ignore below
; FLISTCAL   STRING  Array[2, 7]      .. filenames [calOn,calOff] for the 7 beams
; SUMICAL    STRUCT  ->  Array[1]     .. sumI struct for calOn of 1st beam
;
; The sumI struct hold more detailed info that may be handy. Some of the info
;is duplicated above. See masfilesum for a more complete description.
;
;help,cFI.sumIcross,/st
; OK       INT      1
; FNAME    STRING   '/share/pdata1/pdev/x101.20210910.b0s1g0.0010'...
; FSIZE    LONG64   38856960
; NROWS    LONG     120                 --> number of rows in fits file
; DUMPROW  LONG     10                  --> number of spectra/row
; DATE     LONG     20210910
; BM       INT      0
; H        STRUCT   -> MASFHDR Array[1] --> fitsheader for the file
; HMAIN    STRUCT   -> PDEV_HDRPDEV Array[1]
; HSP1     STRUCT   -> PDEV_HDRSP1 Array[1]
; 
;Example using it:
;  proj='x101'
;  yymmdd=20220406l
;  ncross=masfindcross(yymmdd,proj,cFI)
;; process the crosses
;  mascrossproc(cFI,cI) ;  to be written:)
;-
function masfindcross,yymmdd,proj,cFI,rcv=rcv,verb=verb,dirI=dirI
; 
	useverb=keyword_set(verb)
	if n_elements(rcv) eq 0 then rcv='all'
	yymmddL=yymmdd
	if yymmddL lt 999999l then yymmddL+=20000000l
;
;	obsmodeCr='CROSS'
;	obsmodeCal='CAL'
	maxBms=7
;
	n=masfilelist('',fnmI,proj=proj,yymmdd=yymmddL,/appbm,band=1,grp=0,dirI=dirI)
	if useverb then print,"Done masfilelist. found",n," files."
	if useverb then print,"Starting masfilesum"

	if n eq 0 then goto,nofiles
	nsum=masfilesum('',sumI,fnmI=fnmI)
	if useverb then print,"Done masfilesum"
;   find the obsmode crosses.. this will not include the cals
	if rcv ne 'all' then begin
 		ii=where((sumI.h.obsmode eq 'CROSS') and (sumI.h.frontend eq rcv),cnt)
	endif else begin
		ii=where(sumI.h.obsmode eq 'CROSS',cnt)
	endelse
	if cnt eq  0 then goto,nofiles
;
;   get the uniq patid's for these crosses
;   the cals will also have this patid
;
	patid=sumI[ii].h.pattern_id
	upatid=patid[uniq(patid,sort(patid))]
	ncross=n_elements(upatid)
	ubm=sumI[uniq(sumI[ii].bm,sort(sumI[ii].bm))].bm
	maxbms=n_elements(ubm)
	if useverb then print,"Found ",ncross," potential crosses"
;
;   get the patid's - number of crosses
;
;	struct to return to user
;
	a={ patid: 0l,$
		srcNm: '',$
		nbms  :0,$  ; 1..7 different band
		bmList:intarr(maxBms),$
		cfrAr     :dblarr(maxbms),$ ;  center freq each bm
		flistCross:strarr(2,maxbms),$;  azstrip,el strip
		sumICross: sumI[0],$; summary info azstrip scan (first bm)   
    	hasCal:0,$
		flistCal:strarr(2,maxbms),$; calOn, calOff
	    sumICal  : sumI[0]$; summary scan  info calOn (first bm)
	 }
	cFI=replicate(a,ncross)
	itot=0l
	for icross=0,ncross-1 do begin
		ii=where(sumI.h.pattern_id eq upatid[icross],cnt) 
		sumI1=sumI[ii]
		ii=uniq(sumi1.bm,sort(sumI1.bm))
    	ubm=sumI1[ii].bm
    	cfrAr=sumI1[ii].h.crval1*1d-6
		nbms=n_elements(ubm)
		cFI[itot].patid=upatid[icross]
		cFI[itot].srcNm=sumI1[0].h.object
		cFI[itot].nbms=nbms
		cFI[itot].bmlist[0:nbms-1]=ubm
		cFI[itot].cfrAr[0:nbms-1]=cfrar
		uscan=sumI1[uniq(sumi1.h.scan_id,sort(sumi1.h.scan_id))].h.scan_id
		ii=where(sumI1.h.obsmode eq 'CAL',ncalfiles)
		cFI[itot].hascal=0
;       if cals present, get files and sumI
		if (ncalfiles gt 0) then begin
			if ( (2*nbms) eq ncalfiles) then begin
				cFI[itot].hascal=1
				cFI[itot].sumICal=sumi1[ii[0]] 
			; get the calon,caloff
				for k=0,1 do begin
					step=(k eq 0)?'ON':'OFF' 
					ii=where(sumi1.h.scantype eq step,cnt1)
					flist=sumI1[ii].fname
;					force files to be in beam order
					jj=sort(sumi1[ii].bm)
		    		cFI[itot].flistCal[k,0:nbms-1]=flist[jj]
				endfor
			endif else begin
				lin=string(format=$
		'("-->PatId:",i9," found ",i4," files. expected ",i4," skipping cals")',$	
			           upatid[icross],ncalfiles,2*nbms)
				print,lin				
				continue
			endelse
		endif
;
;
; 		get the az strip,el strip
	   skip=0
       for k=0,1 do begin
           step=(k eq 0)?'AZ':'EL'
           ii=where((sumi1.h.obsmode eq 'CROSS') and (sumI1.h.scantype eq step),cnt)
		   if (cnt ne nbms) then begin
			   lin=string(format=$
		    '("-->skip patId:",i9,1x,a,1x," Found ",i2," files. Expected:",i2)',$
				upatId[icross],step,cnt,nbms)
			    print,lin
				skip=1
			    break
			endif
           flist=sumI1[ii].fname
;                   force files to be in beam order
           jj=sort(sumi1[ii].bm)
           cFI[itot].flistCross[k,0:nbms-1]=flist[jj]
		   if k eq 0 then cFI[itot].sumICross=sumi1[ii[jj[0]]]
       endfor
	   if skip eq 0 then itot++
	endfor
	if itot lt ncross then cFI=cFI[0l:itot-1]
	ncross=itot
	if useverb then print,"Done.. found ",ncross," crosses"
	return,ncross
;
; 		
	
nofiles:	
		lin=string(format=$
			'("-->no crosses found for date:",i8," proj:",a," rcv:",a)',$
			  yymmddL,proj,rcv)
		print,lin
		return,0
end
