;+
;NAME:
;masfindmap - find the files that belong to an radecmap
;SYNTAX: nstrips=masfindmap(proj,yyyymmdd,patternId,mapfI,$
;                noappbm=noappbm,dirI=dirI,bm=bm,band=band,grp=grp)
;ARGS:
; proj: string    project id to search for. files will start with this. eg 'x101'
;yyyymmdd[]:lonarr date(ast) to search for map. It is normally single element.
;                 if your map spanned local midnite then you can enter an array of 2 dates.
;pattern_id: long pattern id for the map.
;KEYWORDS:
; noappbm:        if the data for separate beams has all been moved to a single directory  then this 
;                 keyword should be set. Normally you will ignore it.
; dirI[2] : strarr   if the data has been move to non-standard directories then you can specify the 
;                 location here. see masfilelist for more details.
;bm       : int   limit the returned data to a single bm/mock. numbers are 0 ..6  
;band     : int   limit the returned data to a single mock band. Single pixel observations
;                 only take data in band 1. so setting band=1 will make things run a little faster.
;grp      : int   limit data returned to grp 0 or 1. Multiple groups will be taken when doing 
;                 piggyback observing. Setting grp=0 will cause it to run a little quicker.Currently
;                 the 12meter only uses grp=0.
;RETURNS  : nstrips
;           >0 number of strips in map that were found
;           =0 no data was found
;           -1 error trying to determine the cal,data layout..
;DESCRIPTION:
; 	Search for the files that belong to radecmap taken with the 12meter. The user specifies
;the date yyyymmdd, project name, and pattern id. The routine will search all mock files on disc
;that match these specs. The information is returned in a mapfI structure. The mapfI struct
;can then be passed to masgetmap to load the actual map data into memory. 
;note: this routine returns info on the files,headers that belong to the map.It does not load
;      the data (see masgetmap() for that).
;
;	You can limit the file info that is returned by set the bm= keyword to a single bm.
;mock files look like: x101.20220217.b0s1g0.nnnnn. b0 is the first bm or freq band recorded. bms go
;0 to  6 depending on which mocks you used to record the data.
;
;	The mapfI struct contains:
;IDL> help,mapfi,/st
;** Structure <1c4b658>, 7 tags, length=1099240, data length=1085957, refs=1:
; PATTERN_ID   LONG    204800064   ; pattern id input by caller
; SRC          STRING  '3C461'     ; source name found in the file headers
; NSTRIPS      LONG    25          ; number of strips found in the map
; NBMS         LONG    7           ; number of bms recorded
; CALPERSTRIP  INT     2           ; number of cals per strip.can be 0,1,or 2
; STRIPCODE    INT     6           ; how was the map recorded;
;                                    1 - no cals fired 
;                                    2 - winking cal embeded in the data
;                                    3 - cals only fired at start and end of the map
;                                    4 - cals fired only at start of each strip.
;                                    5 - cals fired only at end of each strip.
;                                    6 - cals fired at start and end of each strip
; STRIPI       STRUCT  -> <Anonymous> Array[25]; info for each strip found
; 
;	The stripI structure contains info for each strip of the map. the entries will depend on 
; the number of cals fired during each strip:
; the example below has cals fired at the start end end of each strip
;IDL> help,mapfI.stripI,/st
;** Structure <2586f38>, 4 tags, length=43968, data length=43437, refs=2:
; STRIPNUM        INT              0          strip number 0 .. nstrips-1
; SUMICAL1        STRUCT    -> <Anonymous> Array[2, 7] cals before the strip 
; SUMIDAT         STRUCT    -> <Anonymous> Array[7]    the files hold the strip data
; SUMICAL2        STRUCT    -> <Anonymous> Array[2, 7] the cals after the strip
;                                               for the cals, the first index is the calOn,caloff
;                                               the 2nd index is the beams
; Note: when the cal is fired, 2 files are generated for each beam. a calON and cal off.
;
;If a different number of cals is used, the the structures will change a bit.
;stripcode=3 (cals at start and end of map only);
;   mapfI has 2 extra fiels containing the file info for the cals at start,end of the map:
;
;  calstart      STRUCT    -> <Anonymous> Array[2, 7]  . sumI for cals before map
;  calEnd        STRUCT    -> <Anonymous> Array[2, 7]  . sumI cals after map
;
;for the stripI struct. 
; - if only one  cal/strip then you have:
; STRIPNUM        INT              0          strip number 0 .. nstrips-1
; SUMICAL         STRUCT    -> <Anonymous> Array[2, 7] cals for stripo
; SUMIDAT         STRUCT    -> <Anonymous> Array[7]    the files hold the strip data
;
; - if no cals are fired then you'd just have:
;  STRIPNUM        INT              0          strip number 0 .. nstrips-1
;  SUMIDAT         STRUCT    -> <Anonymous> Array[7]    the files hold the strip data
; -----------------------------------------------
; One question is where to find the pattern id.
;the pattern id format is: ydddnnnnn.
;                          y is the last digit of the year
;                        ddd is the daynumber in the year
;                      nnnnn is a number that increments by 100 each scan.
;
;looking in the logfile it is the first scan after the line printing the MAPRADEC START:
;
;2022-02-17_14:54:54 MAPRADEC
;2022-02-17_14:54:54 src: ra,dec:232327.90 584842.00 rate:119.67 strip1:1 numStrips:25 code:0x30
;2022-02-17_14:54:57 Mock scan starting filenumber:06100
;2022-02-17_14:55:00 Observation begins Thu Feb 17 14:55:00 2022. scan:204800064
; the pattern_id is 204800064
;
; You could also use the masdmp program for particular day, proj,
;
; masdmp --date=20220217 --proj=x101 --bm=0 --patid
;It will dump all of the scans for the day. setting --bm=0  limits it to just 1 bm 
;(all bms have the same papattern id).  scrolling through the output you'd find your map:
;Proj  Fnum   BSG        SOURCE     PATID     RA       DEC C   nSpc POL Chn    PatNm TopFrq  Bw   RCV
; x101  6000 b0s1g0        3C461 204800061 231557.6  574841 J      3  2 2048 RADECMAP 8219.0 172.0 xb
; x101  6100 b0s1g0        3C461 204800064 231557.3  574842 J      6  2 2048      CAL 8219.0 172.0 xb
; x101  6200 b0s1g0        3C461 204800064 231557.3  574841 J      6  2 2048      CAL 8219.0 172.0 xb
; we started on 204800064 (the previous scans was  testing) 
;
; -----------------------------------------------
;EXAMPLE:
; Suppose you made a map of 3C461 (casA) firing the cals at the start and end of each strip. The
;system had been configured to over 1ghz bandwidth using 7 mocks.
;
; proj='x101'
; yyyymmdd=20220217
; patid=204800064
; nstrips=maspatfind(yyyymmdd,proj,patid,mapfI,band=1,grp=0)
; you could then call masgetmap(mapfI,mapI) to load the actual data (once i write masgetmap :).
;
; The sumI struct hold file info (see masfilesum).
;help,sumI,/st
;   OK      INT    1
;   FNAME   STRING '/share/pdata1/pdev/x108.20080530.b0s0g0.00000.fits'
;   FSIZE   LONG64 84389676   .. bytes in file
;   NROWS   LONG   4          .. rows in fits file
;   DUMPROW LONG   319        .. dumps in first row
;   DATE    LONG   20080530   .. yyyymmdd
;   BM      INT    0          .. 0..6 for alfa
;   BAND    INT    0          .. 0,1 low,hi band (at if)
;   GRP     INT    0          .. grp 0,1
;   NUM     LONG   0          .. file number .nnnnnn.fits
;   H       STRUCT 1st row of fits file (without data,stat)
;   HMAIN   STRUCT pdev main header
;   HSP1    STRUCT pdev sp1 header
;
;-
function masfindmap,proj,yyyymmdd,patid,mapfI,$
	              noappbm=noappbm,dirI=dirI,bm=bmX,band=bandX,grp=grpX 

;  codes used
       nocal=1
	   wcal=2
	   calstartendmap=3
	   calstartStrip=4
	   calEndStrip=5
	   calStartEndStrip=6
;  1  no cal
;  2  winking cal
;  3   cal start,end map
;  4   cal start strip only
;  5   cal end strip only
;  6   cal start, end strips
;
	appbm=~ (keyword_set(noappbm))
	ndates=n_elements(yyyymmdd)
	ntot=0l
	for idate=0,ndates-1 do begin
 	 	n=masfilelist('',fnmI,proj=projId,yymmdd=yyyymmdd[idate],bm=bmX,band=bandX,grp=grpX,$
                appbm=appbm,dirI=dirI)
    	if (n eq 0) then begin
		   print,'No files found for :',yyyymmdd[idate]
		endif else begin
			if ntot eq  0 then begin
				fnmIAll=fnmI
		    endif else begin
				fnmIAll=[fnmIAll,fnmI]
		    endelse
			ntot+=n
		end
	endfor
	if  ntot eq 0 then return,0
;
;	get the file summaries
;
	n=masfilesum(flist,sumI,fnmI=fnmIAll)
;
;   check for our  pattern id
;
	ii=where(sumI.h.pattern_id eq patid,cnt)
	if cnt eq 0 then begin
		print,"No files found with pattern_id:",patid
		return,0
	endif
;
;   find the number of beams
;
	sumI=sumI[ii]
	ubms=sumI[uniq(sumI.bm,sort(sumI.bm))].bm
	nbms=n_elements(ubms)
;
;	work with first bm .. we'll cal it bm0
;
	bm0=ubms[0]
	iibm0=where(sumI.bm eq bm0,nscans)
 	uscans=sumI[iibm0].h.scan_id
;	upat=sumI[iibm0].h.pattern_id
;
; look for when the cals occur
;
;	
	code=nocal
;
	ii=where(sumI[iibm0].h.obsmode eq 'CAL',cnt) 
	stripOrder=''
	case 1 of
		cnt eq 0: begin
		          code=(sumI[0].h.calsrc eq 2)?wcal:nocal
				  striporder=['d']
				  end
		((cnt eq 4) and (ii[0] eq 0) and (ii[1] eq 1) and  $
                       (ii[2] eq (nscans-2)) and (ii[3] eq (nscans-1))):begin
 				  code=calstartendmap
				  striporder=['d']
			    end
		(ii[0] eq 0) and (ii[1] eq 1) and (ii[2] ne 3):begin
				 code=calstartstrip
				  striporder=['c','c','d']
				 end
		(ii[0] eq 1) and (ii[1] eq 2) : begin
				 code=calendstrip
				  striporder=['d','c','c']
				end
		(ii[0] eq 0) and (ii[1] eq 1) and (ii[2] eq 3) : begin
				code=calstartendstrip
				  striporder=['c','c','d','c','c']
			    end
				
		else:  begin
			print,"Could not figure out cal usage.."
			return,-1
		       end	
 	endcase
	scansPerStrip=n_elements(striporder)
	if code eq calstartendmap then begin
		nstrips=nscans - 4 
	endif else begin
		nstrips=nscans/scansPerStrip
	endelse
;   generate strip struct
;   
	case 1 of
	   (code eq nocal) or (code eq wcal) or (code eq calstartendmap): begin
			a={stripNum: 0 ,$;  count from 1
		   	   sumIdat :replicate(sumI[0],nbms)}
	        end
	   (code eq calstartstrip) or (code eq calendstrip): begin
			a={stripNum: 0 ,$;  count from 1
		   	   sumICal :replicate(sumI[0],nbms),$
		   	   sumIdat :replicate(sumI[0],nbms)}
			 end
	   (code eq calstartendstrip): begin
			a={stripNum: 0 ,$;  count from 1
		   	   sumICal1 :replicate(sumI[0],2,nbms),$; calon,caloff
		   	   sumIdat :replicate(sumI[0],nbms),$
		   	   sumICal2 :replicate(sumI[0],2,nbms)}
			 end
	endcase
	calPerStrip=(code le 3)?0:((code eq 4) or (code eq 5))?1:2
	if code eq calstartendmap then begin
		mapfI={ pattern_id:pattern_id,$
		              src:sumI[0].h.object,$
		       nstrips   : nstrips,$
			   nbms      : nbms,$
		      calPerStrip: 0,$
		       stripCode : code , $; 1-no cal,2-winking cal,4-cal before strip,5-cal after strip;
                               ; 6 cal before,after strip
				calStart:replicate(sumI[0],2,nbms),$
				calEnd:replicate(sumI[0],2,nbms),$
		            stripI:replicate(a,nstrips)}
	endif else begin
		mapfI={ pattern_id:patid,$
		              src:sumI[0].h.object,$
		       nstrips   : nstrips,$
			   nbms      : nbms,$
		      calPerStrip: calPerStrip,$
		       stripCode : code , $; 1-no cal,2-winking cal,4-cal before strip,5-cal after strip;
                               ; 6 cal before,after strip
		            stripI:replicate(a,nstrips)}
	endelse
;
; 	now fill in the mapi with the stripI
;
	icur=0
	uscanStrip=uscans  ; uscanStrip will just hold scans for strip,peel off cals,start end map.
	; get start,end map separately
	if code eq calstartendmap then begin
		uscanStrip=uscans[2:nscans-3L] ; exclude cals at start,end
		; do cals st start,end of map
		ii=where(sumI.h.scan_id eq uscans[0],cnt)
		jj=sort(sumI[ii].bm)
		ii=ii[jj]
    	mapfI.calStart[0,*]=sumI[ii]
		ii=where(sumI.h.scan_id eq uscans[1],cnt)
		jj=sort(sumI[ii].bm)
		ii=ii[jj]
    	mapfI.calStart[1,*]=sumI[ii]
		ii=where(sumI.h.scan_id eq uscans[nscans-2],cnt)
		jj=sort(sumI[ii].bm)
		ii=ii[jj]
    	mapfI.calEnd[0,*]=sumI[ii]
		ii=where(sumI.h.scan_id eq uscans[nscans-1],cnt)
		jj=sort(sumI[ii].bm)
		ii=ii[jj]
    	mapfI.calEnd[1,*]=sumI[ii]
	endif 
	uscanStrip=reform(uscanStrip,scansperstrip,nstrips)
	for istrip=0,nstrips-1 do begin
		mapfI.stripI[istrip].stripnum=istrip
		case 1 of 
			; no cal on strip
		 	(code eq nocal) or (code eq wcal) or (code eq calstartendmap): begin
				ii=where(sumI.h.scan_id eq uscanStrip[0,istrip],cnt)
				jj=sort(sumI[ii].bm)
				ii=ii[jj]
		    	mapfI.stripI[istrip].sumIdat=sumI[ii]
				end
			; cal start or end strip
		 	(code eq calStartStrip) or (code eq calendstrip): begin
			    ind=(calStartstrip)?0:3	
				ii=where(sumI.h.scan_id eq uscanStrip[ind,istrip],cnt)
				jj=sort(sumI[ii].bm)
				ii=ii[jj]
		    	mapfI.stripI[istrip].sumIcal[0,*]=sumI[ii]
				ii=where(sumI.h.scan_id eq uscanStrip[ind+1,istrip],cnt)
				jj=sort(sumI[ii].bm)
				ii=ii[jj]
		    	mapfI.stripI[istrip].sumIcal[1,*]=sumI[ii]
			    ind=(calStartstrip)?2:0	
				ii=where(sumI.h.scan_id eq uscanStrip[ind,istrip],cnt)
				jj=sort(sumI[ii].bm)
				ii=ii[jj]
		    	mapfI.stripI[istrip].sumdat=sumI[ii]
				end
		 	(code eq calStartEndstrip): begin
			;   calon
				ii=where(sumI.h.scan_id eq uscanStrip[0,istrip],cnt)
				jj=sort(sumI[ii].bm)
				ii=ii[jj]
		    	mapfI.stripI[istrip].sumIcal1[0,*]=sumI[ii]
				;caloff
				ii=where(sumI.h.scan_id eq uscanStrip[1,istrip],cnt)
				jj=sort(sumI[ii].bm)
				ii=ii[jj]
		    	mapfI.stripI[istrip].sumIcal1[1,*]=sumI[ii]
				; data
				ii=where(sumI.h.scan_id eq uscanStrip[2,istrip],cnt)
				jj=sort(sumI[ii].bm)
				ii=ii[jj]
		    	mapfI.stripI[istrip].sumIdat=sumI[ii]
			;   calon
				ii=where(sumI.h.scan_id eq uscanStrip[3,istrip],cnt)
				jj=sort(sumI[ii].bm)
				ii=ii[jj]
		    	mapfI.stripI[istrip].sumIcal2[0,*]=sumI[ii]
				;caloff
				ii=where(sumI.h.scan_id eq uscanStrip[4,istrip],cnt)
				jj=sort(sumI[ii].bm)
				ii=ii[jj]
		    	mapfI.stripI[istrip].sumIcal2[1,*]=sumI[ii]
				end
		endcase
	endfor
	return,nstrips
end
