;+
;NAME:
;masgetmap - input data from a radecmap
;SYNTAX: nstrips=masgetmap(mapFI,mapDI,edgefract=edgefract,median=median,$
;                calIAr=calIAr,cntsToKar=cntsToKar,cmpmask=cmpmask)
;ARGS:
; mapFI  : {}     structure returned by masfindmap(). contains file info for map
;                 (since we may not have the correct cal values yet.
;KEYWORDS:
;edgefr[2]:fltarr fraction of spectra to ignore on each edge. default:[.06,.06]
;                 if 1 number is supplied then it will be used for both edge.
;median   :       if set then use the median rather than the mean for:
;                 averaging cal records
;                 averaging spectra to get total power.
;remdc    :       if set then interpolate across dc for each spectrag
;noscl    :       if set the don't tpar to kelvins
;RETURNS:
;  istat  : int   num strips. < 0 --> error
;  mapDI  : {}   struct holding the map data that was input
;  calIAr[nsets,nstrips,nbms] {}   return cal info. 
;                if no cals then nothing is returned
;                if cals start/end map then nstrips=1
;                if cals in strip then all 3 dimensions returned.
;cntsToKAr[nstrips,npols,nbms]: fltarr  scale factors used to convert to kelvins.
;                if cal only at start, end map than nstrips=1
;                dividing tpAr by these values will return to spectrometer counts 
;                (if you have questions about the cal scaling)
; cmpmask :      if non zero then compute a mask of good channels using calon/caloff.
;                use this when compute the cal value and the total power.
;DESCRIPTION:
; Input an radecmap taken with the 12meter. The user passes in the mapFI struct
;(returned by masmapFI. The routine inputs the map and returns the info in the
;mapDI structure.
; the mapdI struct contains:
;IDL> help,mapdI,/st
;** Structure <12d5d08>, 3 tags, length=234465008, data length=234289829, refs=1:
; NSTRIPS LONG    25
; BAR     STRUCT  -> <Anonymous> Array[77, 25, 7]; nsmp,nstrips,nbms
; TPAR    FLOAT   Array[77, 25, 2, 7]            ; nsmp,nstrips,npol,nbms
; raPos   dblarr   array[77,25]                   ; raLocation deg
; decPos  float   array[77,25]                   ; decPosition deg
; ------
; Notes:
; terminology
; nsmp - samples per strip
; nsets- number of cals per strip. 1 or 2
; nstrips- number of ra strips in map
; npol   - number of polarizations (usually 2).. we don't add pols 
; nbms   - number of separate freq bands recorded
;
; The ordering of the above arrays is:
; [nsmp or nsets,nstrips,npols,nbms]
; if an array is missing one of these, just drop it from the order
;
; --> need to fix for winking cal and multiple dumps/record
;- 
function masgetmap,mapFI,mapDI,edgeFract=edgeFract,median=median,calIAr=calIAr,$
			 cntsToKAr=cntsToKAr,cmpmask=cmpmask
;
	common colph,decomposedph,colph
	forward_function masgetmapcal
;
; 	the type codes used
;
	  one=1.
	   nocal=1
       wcal=2
       calstartendmap=3
       calstartStrip=4
       calEndStrip=5
       calStartEndStrip=6
;
;   some sanity checks
;
	if (mapFI.nstrips le 0) then begin
		print,"No strips in mapFI structure"
		return,-1
	endif
;   winking cal not yet implemented
	if  (mapFI.stripcode eq wcal) then begin
		print,"Winking cal data not yet implemented.. (bug phil)"
		return,-1
	endif
;   we assume all beam have same nchan,npol
	nchan=mapFI.stripI[0].sumidat[0].h.chan_in
	npol =mapFI.stripI[0].sumidat[0].h.num_pols
	smptm=mapFI.stripI[0].sumidat[0].h.exposure
	; get ndumps from tdim1 of header
	tdim1=mapFI.stripI[0].sumidat[0].h.tdim1
	a=strsplit(tdim1,",)",/extract)
	ndump=long(a[4])
;
   case n_elements(edgefract) of
        1: edgefractL=[edgeFract,edgeFract]*one
        2: edgeFractL=edgeFract *one
       else: edgefractL=[.06,.06]*one
   endcase
;   indices to use in spec
   nbms=mapFI.nbms
;  figure out cal measurements per strip
	calperstrip=mapFI.calperstrip 
; --------------------------------------------------------------------------------------
;  if cal at start/end of map only, compute scale factors here
;
	if (mapFI.stripcode eq calstartendmap) then begin
		istrip=-1
		nsets=2
		istat=masgetmapcal(istrip,nsets,calIar,mapFI.calStart,mapFI.calEnd,$
					median=median,remdc=remdc,edgefract=edgefractl,cmpmask=cmpmask)
		if istat lt 0 then return ,istat
		; average start/end to get 1 scl/pol,bm
		cntsToKAr=transpose(reform(total(calIar[*,*].cntstok,2)/nsets,npol,nbms));[nbms,npol]
	endif
; --------------------------------------------------------------------------------------
;  now loop over the strips of the map
;
	start=1
	nstrips=mapFI.nstrips
	for istrip=0,nstrips-1 do begin
;
;       if strip has cals, input and process them here
;
		case calperstrip of
		 	0: break
			1: begin
				nsets=1
				istat=masgetmapcal(istrip,nsets,calIS,mapFI.stripI[istrip].sumICal,'',$
					median=median,remdc=remdc,edgefract=edgefractl,cmpmask=cmpmask)
				if start then begin
					calIAr=replicate(calIS[0],nstrips,nbms)
				endif
				calIAr[istrip,*]=calIS
			   end
			2:begin
				nsets=2
				istat=masgetmapcal(istrip,nsets,calIS,$
						mapFI.stripI[istrip].sumICal1, mapFI.stripI[istrip].sumICal2,$
					median=median,remdc=remdc,edgefract=edgefractL,cmpmask=cmpmask)
				if start then begin
					calIAr=replicate(calIS[0],nsets,nstrips,nbms)
				endif
				calIAr[*,istrip,*]=calIS
			  end
		endcase
;
;		input the spectra for the strip 
;		
	    fnmAr=mapFI.stripI[istrip].sumidat.fname
		istat=masgetfiles('',bar,fnmAr=fnmAr,remdc=remdc)
		if start then begin
			recStrip=n_elements(bar[*,0])
			smpStrip=recStrip*ndump
			nchan   =bar[0].nchan
			mapDI={nstrips:nstrips,$
					bar    :replicate(bar[0],recStrip,nstrips,nbms),$
					tpAr   :fltarr(smpstrip,nstrips,npol,nbms),$
					raCenD :  0d, $; center of map raDeg should be source loc
					decCenD : 0d,$; center of map decd
				    raPosD :dblarr(smpstrip,nstrips),$
					decPosD:dblarr(smpstrip,nstrips)$
					}
			start=0
			if calPerStrip gt 0 then cntsToKAr=fltarr(nstrips,npol,nbms)
			
		endif
		mapDI.bar[*,istrip,*]=bar
;
;		compute total power, scaling to kelvins. 
;      should be doing a bandpass correction, if not, the freq chan are not weighted 
;      equally.
;
;       average over channels          nchan,npol,nsmp,nbms
;		        smpStrip,nstrips,npol,nbms          npol,nsmp,nbms
;       use same channels as cal used
;		
	 
	    for ibm=0,nbms-1 do begin 
			if calPerStrip lt 2 then begin
				ii=where(caliAr[istrip,ibm].maskused eq 1,nused)
			endif else begin
				ii=where((caliAr[0,istrip,ibm].maskused eq 1) and (calIar[1,istrip,ibm].maskused eq 1),nused)
		    endelse
	       if ndump gt 1 then begin
			   mapDI.tpar[*,istrip,*,ibm]=$
;			                                    npol,ndump,recstrip
			   reform(transpose(total(bar[*,ibm].d[ii,*,*],1)/nused,[1,2,0]),$
			   		             smpstrip,npol)
			endif else begin
				mapDI.tpar[*,istrip,*,ibm]=transpose(total(bar.d[ii,*],1)/nused,[1,0,2])
			endelse
	    endfor
;
		if calPerStrip ge 1 then begin
;      get cntsToKelvin. if 2 cals/strip average values
;      dim [nstrip,npol,nbms)                               npol,nstrip,nbms
			cntsToKAr[istrip,*,*]=(calPerStrip eq 1)?transpose(caliar[istrip,*].cntsToK,[1,0,2]) $
                :total(reform(calIar[*,istrip,*].cntstok,npol,nsets,nbms),2)/nsets
;           now scale
			for ibm=0,nbms-1 do begin
			    for ipol=0,npol-1 do begin
			         mapDI.tpar[*,istrip,ipol,ibm]*=cntsToKAr[istrip,ipol,ibm]
				endfor
			endfor
		endif
	endfor
;    if cal start,end map only, doit here
	if (mapFI.stripcode eq calstartendmap) then begin
		for ibm=0,nbms-1 do begin
			for ipol=0,npol-1 do begin
				mapDI.tpAr[*,*,ipol,ibm]*= cntsToKAr[ipol,ibm]
			endfor
		endfor
	endif
;
;  the header timestamped position  are for the start of each record.
;  interpolate to the center of integration
;  if load the timestamped positions are for the start of the integration , interpolate to the
;  middle
	xh=dindgen(recstrip) ; relative times for position data  in header
	if ndump eq 1 then begin
		xd=dindgen(recstrip) + .5d
	endif else begin
		xd=dindgen(smpstrip)/ndump + (1d/ndump) * .5d
	endelse
	for istrip=0,nstrips-1 do begin &$
		mapDI.raPosD[*,istrip]=interpol(mapDI.bar[*,istrip,0].h.crval2,xh,xd) &$
		mapDI.decPosD[*,istrip]=interpol(mapDI.bar[*,istrip,0].h.crval3,xh,xd) &$
	endfor
;  load the center. should be the request center of map
;
	mapDI.raCenD=mapDI.bar[0].h.req_POSC1
	mapDI.decCenD=mapDI.bar[0].h.req_POSC2
;
	return,nstrips
end
; ----------------------------------------------------------------------------
; masgetmapcal - get cals for map or strip
; compute cals for a set of cals
; istrip:int          strip index 0..nstrips-1. -1 of start/end map cals
; nsets: int number of cal sets to process.. 1 or 2
;        if 2 then it is the first/last of map or first.last of strip
; sumI1[2,nbms]: {}   sumI structs holding info to access cal files
;                     the first dim is the calOn,Off, the 2nd is the beams
; sumI2[2,nbms]: {}   if nsets eq r2 then this is the 2nd set of cal measurements
;KEYWORDS:
; median :            if set then use median rather than mean
; remdc  :            if set then remove dc
; cmpmask:            if set then use calOn/calOff to remove rfi before computing cal
;
; RETURNS
; istat: 
; calIar[nsets,nbms]: {}  struct array holding computed cal info
; 
function masgetmapcal,istrip,nsets,calIar,sumI1,sumI2,median=median ,$
			   remdc=remdc,edgefract=edgefract,cntsToKAr=cntsToKAr,cmpmask=cmpmask
; 
;
	a=size(sumI1)
	nbms=(a[0] eq 1)?1:a[2]
	nchan=sumI1[0].h.chan_in
	fnmAr=sumI1.fname
	istat=masgetfiles('',bcal1,avg=1,median=median,fnmAr=fnmar,remdc=remdc)
	if istat ne 1 then begin
		lab=string(format='(" strip:",i1," returncode:",i2)',$
				istrip,istat)
		print,"Error inputing cal files first set, " + lab
		return,-1
	endif
	bcal1=reform(bcal1,2,nbms)
	if nsets eq 2 then begin
		fnmAr=sumI2.fname
		istat=masgetfiles('',bcal2,avg=1,median=median,fnmAr=fnmar,remdc=remdc)
		if istat ne 1 then begin
			lab=string(format='(" strip:",i1," returncode:",i2)',$
				istrip,istat)
			print,"Error inputing cal files 2nd  set, " + lab
		    return,-1
		endif
		bcal2=reform(bcal2,2,nbms)
	endif
;
	start=1
	for ibm=0,nbms-1 do begin
		for i=0,nsets-1  do begin ; 0 start cal, 1 endcal
           	if i eq 0 then istat=mascalonoff(bcal1[0,ibm],bcal1[1,ibm],calI,edge=edgfract,$
						       cmpmask=cmpmask)
           	if i eq 1 then istat=mascalonoff(bcal2[0,ibm],bcal2[1,ibm],calI,edge=edgfract,$
								cmpmask=cmpmask)
			if istat ne 0 then begin
				lab=string(format= $
				'("set:",i2," bm:",i1," strip:",istrip," -1->start/endmap")',$
							i,ibm,istrip)
				print,"Error processing cal " +  lab
				return,-1
			endif
			if start then begin
			;  calI has an array of indices used that varies with each cal. switch to full mask 
				 	a={calval :fltarr(2),$
 			           numpol    : 0,$
   					   exposure  : 0., $; integration time 
                       cntstok   : fltarr(2),$; convert counts ot kelvin
				      npntsUsed  : 0l       ,$; points in spectra we used
   				      FLIPPED    : 0        ,$; true if spectra was flipped
  				      EDGEFRACT  : fltarr(2),$;
                      maskUsed   : intarr(nchan)}; 0 not used, 1 used
				 	calIAr=replicate(a,nsets,nbms) ; 2 is the start,end cals
				    ; load values that should be constant for all cals
				 	calIAr.numpol=replicate(bcal1[0].npol,nsets,nbms)
				 	calIAr.exposure=replicate(calI.exposure,nsets,nbms); 
				 	if (bcal1[0].accum ne 0.) then calIar.exposure*=abs(bcal1[0].accum)
		 		 	calIAr.edgeFract[0]=replicate(edgefract[0],nsets,nbms)
		 		 	calIAr.edgeFract[1]=replicate(edgefract[1],nsets,nbms)
				start=0
			endif
			caliAr[i,ibm].calval=calI.calval
			caliAr[i,ibm].npntsused=calI.npnts
			caliAr[i,ibm].cntstok=calI.cntstok
			caliAr[i,ibm].flipped=calI.flipped
			ii=calI.indUsed
			calIar[i,ibm].maskUsed[ii]=1
	   endfor
	endfor
	return,1
end
