;+
;NAME:
;masgetmapwc - input data from a winking cal radecmap
;SYNTAX: nstrips=masgetmapwc(mapFI,mapDI,edgefract=edgefract,median=median,$
;                degGainVar=degGainVar,verb=verb)
;ARGS:
; mapFI  : {}     structure returned by masfindmap(). contains file info for map
;KEYWORDS:
;edgefr[2]:fltarr fraction of spectra to ignore on each edge. default:[.06,.06]
;                 if 1 number is supplied then it will be used for both edge.
;median   :       if set then use the median rather than the mean for:
;                 averaging cal records
;                 averaging spectra to get total power.
;remdc    :       if set then interpolate across dc for each spectrag
;noscl    :       if set the don't scale tpar to kelvins
;degGainVar:int   order of polynomial when fitting caldif for gain variation in time.
;                 def:3
;verb     :       if set then output progress of routine.
;RETURNS:
;  istat  : int   num strips. < 0 --> error
;  mapDI  : {}   struct holding the map data that was input
;DESCRIPTION:
; Input an 12meter radecmap taken with the hardware winking cal. The user passes in
; the mapFI struct (returned by masmapFI). The routine inputs the map and returns the 
;info in the mapDI structure.
;
; the mapdI struct contains:
;IDL> help,mapdI,/st
;** Structure <12d5d08>, 3 tags, length=234465008, data length=234289829, refs=1:
; NSTRIPS LONG    25
; BAR     STRUCT  -> <Anonymous> Array[77, 25, 7]; nsmp,nstrips,nbms
; TPAR    FLOAT   Array[77, 25, 2, 7]            ; nsmp,nstrips,npol,nbms
; raPos   dblarr   array[77,25]                   ; raLocation deg
; decPos  float   array[77,25]                   ; decPosition deg
; ------
; Notes
; ------
; 1. Assume there is a gain variation coming after the cal injection that varies much slower than the 
;    cal cycle (40ms).
; 2. assume that Tcal (from the diode) is stable with time (it's temperature  stabilized).
;
; Let
; calOn:  (Tsky + Trcv + Tcal)*Giflo(stable)*Gvar(t)
; calOff: (Tsky + Trcv )*Giflo(stable)*Gvar(t)   (= Tsys)
; calDif=calOn-caloff=Tcal*Giflo(stable)*Gvar(t)
;
; 3.If we divide caloff (tsys) by CalDif at each 40ms cal cycle we can cancel Gvar(t) since we assume it 
;   varies much more slowly than 40ms.
;   - But this will blow up the noise since the noise/mean value for calDif is a factor of 2 to 3
;     bigger than noise on calOff (since the noise is the same and Tcal<Tsys by about a factor of 2 to 3.)
;
; 4.fit a polynomial to  Tpcaldif and then divide polynomial into calOff so we don't affect the noise.
;
;Some steps in the processing:
;
; 1. When computing total power from the spectra, you need to bandpass correct so that all of the 
;    frequency channels are weighted evenly in the average.
;   - we dont' have an  off source to get a bandpass correction. 
;   - calDefl has Tcal*bpIfLo.. so you could use this to remove the if bandpass;
;   --> but you put the ripples of the cal into the off, on banpasses .. but they are much less than 
;       the iflobanpass..maybe later use some edge spectra caloff on the maps.
; 
; 2. using spectra bon, boff.
;    bavgOn=avg(bon)
;    bavgOff=avg(bavgoff)
;    bavgCal=bavgon - bavgoff
;    bavgCalN=normalize bavgCal to unity (since we want spec cnts still)
;  flatten all of the spectra with bavgcalN
;   bonF=bon/bavgcalN
;   boffF=boff/bavgcalN
;
; 3. find channels with rfi;
;    - compute rms/mean by channel for a strip
;    - do a linear fit and exclude channels >3sigma
;      also exclude edgefract channels (4% on each each side)
;
; 4. compute TpOnAr, tpOffAr using good channels from bonF, boffF
;
; 5. compute tpCal(t) =tpONar(t)  - tpOffar(t)
;
; 6. nth order poly fit to tpCal(t)
;
; 7. divide tponar,tpoffar by fit
;
; 8. Get calInK(f) for channels used (avg)
; 9. multiply TponAr,tpOffAr  by calin kel to switch to degK
;
; Notes:
; terminology
; nsmp - samples per strip
; nsets- number of cals per strip. 1 or 2
; nstrips- number of ra strips in map
; npol   - number of polarizations (usually 2).. we don't add pols 
; nbms   - number of separate freq bands recorded
;
; The ordering of the above arrays is:
; [nsmp or nsets,nstrips,npols,nbms]
; if an array is missing one of these, just drop it from the order
;
; --> need to fix for winking cal and multiple dumps/record
;- 
function masgetmapwc,mapFI,mapDI,edgeFract=edgeFract,median=median,$
			 degGainVar=degGainVAr,verb=verb
;
	common colph,decomposedph,colph
	forward_function masgetmapwccal
;
; 	the type codes used
;
   useverb=keyword_set(verb)
   usep12m=masisp12m(mapFI.stripI[0].sumIDat.h)
   wcal=2
   one=1.  ; if double keyword,, makes things double. not yet implemented
   edgeFrDef=.04 ; default exclude 4% each edge
;
;   some sanity checks
;
	if (mapFI.nstrips le 0) then begin
		print,"No strips in mapFI structure"
		return,-1
	endif
;   data must have winking cal.
	if  (mapFI.stripcode ne wcal) then begin
		print,"Routine needs Winking cal data"
		return,-1
	endif
	if n_elements(degGainVar) eq 0 then degGainVar=3
;   we assume all beam have same nchan,npol
	nchan=mapFI.stripI[0].sumidat[0].h.chan_in
	npol =mapFI.stripI[0].sumidat[0].h.num_pols
	smptm=mapFI.stripI[0].sumidat[0].h.exposure
    nbms=mapFI.nbms
;  this is the dumps at 2ms.. we want after  avging over calon, or off
;	tdim1=mapFI.stripI[0].sumidat[0].h.tdim1
;	a=strsplit(tdim1,",)",/extract)
;	ndumpRaw=long(a[4]); before combining cals
;    info needed to get cal values
	rfNum=mapFI.stripI[0].sumidat[0].h.rfNum  ; 11 or 7
	calType=mapFI.stripI[0].sumidat[0].h.calType  ; 'hcorcal'
	yyyymmdd=mapFI.stripI[0].sumidat[0].date
	yr=yyyymmdd/10000l
	dayno=dmtodayno(1,1,1,yymmdd=yyyymmdd)
	; generate freq are for each beam
	freqAr=fltarr(nchan,nbms)
	for ibm=0,nbms-1 do freqAr[*,ibm]=masfreq(mapFI.stripI[0].sumIDat[ibm].h)
;
;    setup the band edge fraction to ignore
   case n_elements(edgefract) of
        1: edgefractL=[edgeFract,edgeFract]*one
        2: edgeFractL=edgeFract *one
       else: edgefractL=[edgeFrDef,edgeFrDef]*one
   endcase
;   indices to use in spec
	iedg0=long(edgeFractL[0]*nchan)
	iedg1=long(nchan -(edgeFractL[1]*nchan) - 1)
;   starting mask for rms fit excludes edge fract to start
	maskSt=intarr(nchan) + 1
	maskSt[0:iedg0]=0
	maskSt[iedg1:*]=0
;  
;   input the cal data (in deg K) (all freq) for this cal
;
	calNum=0
	case (1)  of
		calType eq 'hcorcal':calNum=1
		calType eq 'lcorcal':calNum=0
		else                :begin
			print,"Unknow caltype in header:"+ caltype
			return,-1
		    end
	endcase
	istat=calinpdata(rfNum,calNum,calRcvI,p12m=usep12m,date=[yr,dayno])

	if (istat ne 1 ) then begin
		print,"Error trying to input cal data for rcvr:",rfnum," calTypeNum:",calNum," dayno,yr:",yr,dayno
		return,-1
	endif
; --------------------------------------------------------------------------------------
;  now loop over the strips of the map
;
	start=1
	nstrips=mapFI.nstrips
	deg=1 ; first order fit on rms to remove rfi
	zeroAr=intarr(nchan)
	xchn=findgen(nchan)  ; used for fitting
	bwChnByBm=abs(mapFI.stripI[0].sumIdat.h.cdelt1)*1d-6
;
	for istrip=0,nstrips-1 do begin
;
;		input the spectra for the strip 
;		
	    fnmAr=mapFI.stripI[istrip].sumidat.fname
		if useverb then print,"Start strip:",istrip+1
		istat=masgethwcal('',calI,bonar,boffar,verb=verb,brmsar=brmsar,fnmAr=fnmAr,/dcintp)
;       first time , allocate stucts since we now have the bar structs.
		if start then begin
			nrecs=calI[0].nrows
			ndump=bonar[0].ndump ; after combining cal
			ncalCyclesStrip=nrecs*ndump; masgethwcal 
			tpCalDif=fltarr(ncalCyclesStrip,npol)
;           for interpolating across all calcycles of strip
			xtm   =findgen(ncalCyclesStrip)/ncalCyclesStrip
;           for interpolating position for each cal cycle
;           - nrecs hdrs for strip
;           - there are ndump samples within each rec 
;           - add offset of .5/ndump to put us in the middle of each cal cycle integration
;             
		    xh = findgen(nrecs);  original records
			xhI=findgen(ncalcyclesstrip)/ndump + .5/ndump
;               a=info for 1 beam
		  a={ tpAronK :fltarr(ncalcyclesStrip,nstrips,npol),$
		      tpAroffK:fltarr(ncalcyclesStrip,nstrips,npol),$
			  brmsAr   :replicate(brmsAr[0],2,nstrips),$; calOn, calOff
			  maskAr   :intarr(nchan,nstrips),$ ; 0 chan not used, 1 used
			  edgeFract:edgeFractL,$
			  nchanUsed:lonarr(nstrips),$
			  bwUsed   :fltarr(nstrips),$
			  cntsToK  :fltarr(ncalcyclesStrip,nstrips,npol),$; converting from counts to K
			  calKUsed :fltarr(nstrips,npol) }
		   mapDI={ nstrips:nstrips,$
		   		   nrecsStrip:nrecs,$
				   nbms    : nbms,$
		   		   ncalCyclesStrip:ncalCyclesStrip,$
				   hdrar   : replicate(bonar[0].h,nrecs,nstrips),$; use bm 0 hdr
			       bmI: replicate(a,nbms),$
                   raCenD :  0d, $; center of map raDeg should be source loc
                   decCenD : 0d,$; center of map decd
                   raPosD :dblarr(ncalcyclesStrip,nstrips),$
                   decPosD:dblarr(ncalcyclesStrip,nstrips)$
			}
			start=0
		endif
		mapDI.hdrar[*,istrip]=bonar[*,0].h
;
;       loop over the beams
;
	    for ibm=0,nbms-1 do begin
;
;		 compute avgs, get calspectra, flatten all spectra with this (normalized)
;
			bavgOn=masmath(bonar[*,ibm],/avg)
			bavgOff=masmath(boffar[*,ibm],/avg)
			bcal=masmath(bavgOn,bavgOff,/sub)
			bcalN=masmath(bcal,/norm)
			eps=1e-6
;
;           don't divide by zero on edges. set all of edge  fract to eps
;
			bcalN.d[0:iedg0,*]=eps
			bcalN.d[iedg1:*,*]=eps
;
;			band pass correct all spectra this band , F-> flattened. this is just  1 freq band
;
			bonArF=masmath(bonAr[*,ibm],bcalN,/div)
			boffArF=masmath(boffAr[*,ibm],bcalN,/div)
;
;		 compute mask to exclude rfi . use same mask for polA,B since external rfi will probably
;                                      leak into both.
;        							 linear fit to rms/mean by chan.  throw out 3sigma outliers
			maskCum=zeroAr
			for ipol=0,1 do begin &$
				; find good points to use for tp avg
				a=robfit_poly(xchn,brmsar[ibm,0].d[*,ipol],deg,gindx=gindx,ngood=ngood,fpnts=fpnts,$
                mask=maskSt) &$
        		maskCum[gindx]+=1 &$; good points from cal on
        		a=robfit_poly(xchn,brmsar[ibm,1].d[*,ipol],deg,gindx=gindx,ngood=ngood,fpnts=fpnts,$
                mask=mask) &$
        	    maskCum[gindx]+=1 &$; good points from calon,caloff have 2 
			endfor
       		iigd=where(maskCum eq 4,ngoodchn) 
;           make sure edges are excluded..the fitting starts with excluded, but they can creep back in.
      	    ii=where((iigd gt iedg0) and (iigd lt iedg1),cnt) 
		    if cnt lt ngoodchn then begin 
            	iigd=iigd[ii]
            	ngoodchn=cnt
      		endif

;		 compute tp CalDefl using good channels
;        [ncalcycl,npol]  .. compute cal difference
			tpCalDif=transpose($
					reform((total(bonarF.d[iigd,*,*],1) - total(boffarF.d[iigd,*,*],1))/ngoodchn,$
							 npol,ncalCyclesStrip))

;  			now fit polynomial got tcalDefl to use in correction for tpcal*G(t) . 

			coefAr=fltarr(degGainVar+1,npol)
			tpCalDifFitAr=fltarr(ncalCyclesStrip,npol)
			for ipol=0,npol-1 do begin &$
				coefAr[*,ipol]=robfit_poly(xtm,tpCalDif[*,ipol],degGainVar,yfit=yfit) &$
				tpCalDifFitAr[*,ipol]=yfit &$
			endfor
;
; 		now compute tpCalOnDegK, tpCalOffDegK 
;       use same channels as mask
;           avg over channels, no gain correction or degK yet.
			mapDI.bmI[ibm].tpArOnK[*,istrip,*]=transpose(reform(total(bonarF.d[iigd,*,*,*],1)/ngoodchn,npol,ncalCyclesStrip),[1,0])
			mapDI.bmI[ibm].tpArOffK[*,istrip,*]=transpose(reform(total(boffarF.d[iigd,*,*,*],1)/ngoodchn,npol,ncalCyclesStrip),[1,0])
;
;    compute avg cals for this band using channels for tp avg
;
		 	calKA=total(interpol(calRcvI.calA,calRcvI.freq,freqAr[iigd,ibm]),1)/ngoodchn
			calKB=total(interpol(calRcvI.calB,calRcvI.freq,freqAr[iigd,ibm]),1)/ngoodchn
			sclA=CalKA/tpCalDifFitAr[*,0]
			sclB=CalKB/tpCalDifFitAr[*,1]
			mapDI.bmI[ibm].tpArOnK[*,istrip,0] *= sclA
			mapDI.bmI[ibm].tpArOffK[*,istrip,0]*=sclA
			mapDI.bmI[ibm].tpArOnK[*,istrip,1]*= sclB
			mapDI.bmI[ibm].tpArOffK[*,istrip,1]*= sclB
			mapDI.bmI[ibm].maskAr[iigd,istrip]=1
			mapDI.bmI[ibm].nchanused=ngoodchn
			mapDI.bmI[ibm].bwused=ngoodchn*bwChnByBm[ibm]
			mapDI.bmI[ibm].cntsToK[*,istrip,0]=sclA
			mapDI.bmI[ibm].cntsToK[*,istrip,1]=sclB
			mapDI.bmI[ibm].brmsar[*,istrip]=reform(brmsar[ibm,*])
			mapDI.bmI[ibm].calKUsed[istrip,*]=[calKA,calKB]
    	endfor  ; endfor loop on bms
;
;	 get positions in strip from first bm
;        bon,boff have same locations
;	
	 	mapDI.raposD[*,istrip]=interpol(bonar[*,0].h.crval2,xh,xhi)
		mapDI.decposD[*,istrip]=interpol(bonar[*,0].h.crval3,xh,xhi)
		if (istrip eq 0 ) then begin
			mapDI.raCenD=bonar[0].h.req_POSC1
	       	mapDI.decCenD=bonar[0].h.req_POSC2
		endif
		
	endfor   ; endfor loop on strips
;  
	return,nstrips
end
