;+
;NAME:
;masgetscanhwc - input scan data  using hardware winking cal
;SYNTAX: nbms=masgetscanhwc(fnmI,scanI,edgefract=edgefract,median=median,$
;                degGainVar=degGainVar,verb=verb,row=row,nrows=nrows,filelist=filelist)
;ARGS:
; fnmI[n] : {}    from masfilelist. files in scan (if multiplebeams)
;                 currently assumes 1 file/scan/bm
;KEYWORDS:
;edgefr[2]:fltarr fraction of spectra to ignore on each edge. default:[.06,.06]
;                 if 1 number is supplied then it will be used for both edge.
;median   :       if set then use the median rather than the mean for:
;                 averaging cal records
;                 averaging spectra to get total power.
;remdc    :       if set then interpolate across dc for each spectrag
;noscl    :       if set the don't scale tpar to kelvins
;degGainVar:int   order of polynomial when fitting caldif for gain variation in time.
;                 def:3
; row     :       long . if supplied then limit scan to maxrow recs
; nrows   :       long . if supplied then input nrows rows. def: entire file (starting
;                        at row)
;filelist[n]:    strarr if supplied then ignore fnmI,use filelist
;verb     :       if set then output progress of routine.
;RETURNS:
;  istat  : int   num of beams (freq bands of data returned) < 0 --> error
;  scanI  : {}   struct holding the scan data that was input
;DESCRIPTION:
;	Input an 12meter scan taken with the harware winking cal. The user passes in
;an fnmI[nmbs] array with the file info for the files in theh scan. It currently works
;for multiple bms/scan but only 1 file/bm.
;
;	This routines works similar to masgetmapwc()  except that it inputs a single scan rather than
;and entire map.
;
;
; the scanI struct contains:
;IDL> help,scani,/st
;** Structure <14bbc08>, 7 tags, length=3143128, data length=3140282, refs=1:
; NBMS            LONG     7
; NRECSFILE       LONG     380
; NCALCYCLESFILE  LONG     9500
; HDRAR           STRUCT    -> MASFHDR Array[380]  hdrAr for bm0 of each rec
; XH              FLOAT     Array[380]  ; index 0..380-1
; XHI             FLOAT     Array[9500] ; use to interpolate hdrAr data to calcycle sample times
; BMI             STRUCT    -> <Anonymous> Array[7] ; info from each beam.
;
;IDL> help,scani.bmi,/st
;** Structure <1522548>, 11 tags, length=391256, data length=391230, refs=2:
; TPCALDIF        FLOAT     Array[9500, 2]  totPwr calOn - calOff raw units
; TPCALDIFFIT     FLOAT     Array[9500, 2]  fit to tpcaldif
; TPARONK         FLOAT     Array[9500, 2]  totPwrcalon in deg K
; TPAROFFK        FLOAT     Array[9500, 2]  totPwrcaloff in degk
; BONAVG          STRUCT    -> <Anonymous> Array[1] ; avg calon spc degK (no bpc)
; BOFFAVG         STRUCT    -> <Anonymous> Array[1] ; avf caloff spc degK  (no bpc)
; BRMSAR          STRUCT    -> <Anonymous> Array[2] rms/mean for calon,caloff spectra
; MASKAR          INT       Array[512]       1-> channel used,0 not used in totpwr computation
; EDGEFRACT       FLOAT     Array[2]        0..1. fraction of edge to exclude in totPwr computation
; NCHANUSED       LONG               465    # channels used for totpwr computation
; BWUSED          FLOAT           156.240   bandwidth used MHz.
; CNTSTOK         FLOAT     Array[9500, 2]  convert counts to degK for each calcycle
; CALKUSED        FLOAT     Array[2]        cal values used in degK
;
;Examples:
;* 	Suppose you use the hardware winking cal for an on, off observation
;   You could then call this routine for the Position on scan and the position off scan separately.
;
;
;*  scanI.xh and scanI.xhI  can be used to interpolate header information sampled once a second
;   to the time resolution of the cal cycles
; eg:
; azPosI=interpol(scanI.hdarar.azimuth,scanI.xh,scanI.xhI)
;
; Notes
; ------
; how the gain correction works: see masgetmapwc() description.
;
;Some steps in the processing:
;
; Output spectra
; scanI.bmI[nbms].bonavg .. is the average of each calon cycle spectra after converting to K
;                           with the correction factor for each calcycle.
;                      
; scanI.bmI[nbms].boffavg .. is the average of each caloff cycle spectra after converting to K
;                           with the correction factor for each calcycle
;
;- 
function masgetscanhwc,fnmI,scanI,edgeFract=edgeFract,median=median,$
			 degGainVar=degGainVAr,verb=verb,double=double ,row=row,nrows=nrows,filelist=filelist
;
	common colph,decomposedph,colph
	forward_function masgetmapwccal
;
; 	the type codes used
;
    useverb=keyword_set(verb)
	usefilelist=n_elements(filelist) gt 0
	nfiles=(usefilelist)?n_elements(filelist):n_elements(fnmI)
	if nfiles eq 0 then begin
		print,"-->no files supplied"
	    return,-1
	endif
    if n_elements(degGainVar) eq 0 then degGainVar=3

;
;  get the summary info for the files
;
 	if usefilelist then begin
		nsum=masfilesum(filelist,sumI)
	endif else begin
		nsum=masfilesum('',sumI,fnmI=fnmI)
	endelse

	if ( nsum ne nfiles) then begin
		print,"--> # of summary files does match requested files"
		return,-1
	endif
   usep12m=masisp12m(sumI[0].h)
   wcal=2
   one=(keyword_set(double))?1d:1.  ; if double keyword,, makes things double. not yet implemented
   edgeFrDef=.04 ; default exclude 4% each edge
   if n_elements(degGainVar) eq 0 then degGainVar=3
;
;   some sanity checks nfiles must equal nbms
;   after this use nbms rather than nfiles
;
;	ubms=fnmi[uniq(fnmI.bm,sort(fnmI.bm))].bm
 	ubms=sumI[uniq(sumi.bm,sort(sumI.bm))].bm
    nbms =n_elements(ubms)
	if nbms ne nfiles then begin
		print,"masgetscanhwc: currently only works with 1 file/bm"
		return,-1
	endif
;   data must have winking cal.
	ii=where(sumI.h.calsrc ne  wcal,cnt)
	if  (cnt gt 0) then begin
		print,"Routine needs Winking cal data"
		return,-1
	endif
;   we assume all beam have same nchan,npol
	nchan=sumI[0].h.chan_in
	npol =sumI[0].h.num_pols
	smptm=sumI[0].h.exposure
;  this is the dumps at 2ms.. we want after  avging over calon, or off
;	tdim1=mapFI.stripI[0].sumidat[0].h.tdim1
;	a=strsplit(tdim1,",)",/extract)
;	ndumpRaw=long(a[4]); before combining cals
;    info needed to get cal values
	rfNum  =sumI[0].h.rfNum  ; 11 or 7
	calType=sumI[0].h.calType  ; 'hcorcal'
	yyyymmdd=sumI[0].date
	yr=yyyymmdd/10000l
	dayno=dmtodayno(1,1,1,yymmdd=yyyymmdd)
	; generate freq are for each beam
	freqAr=fltarr(nchan,nbms)
	for ibm=0,nbms-1 do freqAr[*,ibm]=masfreq(sumI[ibm].h)
;
;    setup the band edge fraction to ignore
   case n_elements(edgefract) of
        1: edgefractL=[edgeFract,edgeFract]*one
        2: edgeFractL=edgeFract *one
       else: edgefractL=[edgeFrDef,edgeFrDef]*one
   endcase
;   indices to use in spec
	iedg0=long(edgeFractL[0]*nchan)
	iedg1=long(nchan -(edgeFractL[1]*nchan) - 1)
;   starting mask for rms fit excludes edge fract to start
	maskSt=intarr(nchan) + 1
	maskSt[0:iedg0]=0
	maskSt[iedg1:*]=0
;  
;   input the cal data (in deg K) (all freq) for this cal
;
	calNum=0
	case (1)  of
		calType eq 'hcorcal':calNum=1
		calType eq 'lcorcal':calNum=0
		else                :begin
			print,"Unknow caltype in header:"+ caltype
			return,-1
		    end
	endcase
	istat=calinpdata(rfNum,calNum,calRcvI,p12m=usep12m,date=[yr,dayno])

	if (istat ne 1 ) then begin
		print,"Error trying to input cal data for rcvr:",rfnum," calTypeNum:",calNum," dayno,yr:",yr,dayno
		return,-1
	endif
; --------------------------------------------------------------------------------------
;  now loop over the files
;
	start=1
	deg=1 ; first order fit on rms to remove rfi
	zeroAr=intarr(nchan)
	xchn=findgen(nchan)  ; used for fitting
	bwChnByBm=abs(sumI.h.cdelt1)*1d-6
;
	for ibm=0,nbms-1 do begin
;
;		input the spectra for the file
;		
		if useverb then print,"Start file:",sumI[ibm].fname
		if (usefilelist) then begin 
		    istat=masgethwcal('',calI,bonar,boffar,verb=verb,brmsar=brmsar,/dcintp,$
				row=row,nrows=nrows,fnmar=filelist[ibm])
		endif else begin
			istat=masgethwcal(fnmI[ibm],calI,bonar,boffar,verb=verb,brmsar=brmsar,/dcintp,$
				row=row,nrows=nrows)
		endelse
;       first time , allocate stucts since we now have the bar structs.
		if start then begin
			nrecs=n_elements(bonar[*,0])
			ndump=bonar[0].ndump ; after combining cal
			ncalCyclesFile=nrecs*ndump; masgethwcal 
			tpCalDif=fltarr(ncalCyclesFile,npol)
;           for interpolating across all calcycles of strip
			xtm   =findgen(ncalCyclesFile)/ncalCyclesFile
;           for interpolating position for each cal cycle
;           - nrecs hdrs for strip
;           - there are ndump samples within each rec 
;           - add offset of .5/ndump to put us in the middle of each cal cycle integration
;             
		    xh = findgen(nrecs);  original records
			xsI=findgen(ncalcyclesfile)/ndump + .5/ndump
			bavg=masmath(bonar,/avg)
;               a=info for 1 beam
		  a={ tpCalDif :fltarr(ncalcyclesFile,npol),$ ;raw data 
		      tpCalDifFit:fltarr(ncalcyclesFile,npol),$ ;fit to raw data
              tpAronR :fltarr(ncalcyclesFile,npol),$   ; raw total power in countsconverted to degK
		      tpAroffR:fltarr(ncalcyclesFile,npol),$
              tpAronK :fltarr(ncalcyclesFile,npol),$   ; converted to degK
		      tpAroffK:fltarr(ncalcyclesFile,npol),$
			  bonavg  : bavg,$
			  boffavg  :bavg,$
			  brmsAr   :replicate(brmsAr[0],2),$; calOn, calOff
			  maskAr   :intarr(nchan),$ ; 0 chan not used, 1 used
			  edgeFract:edgeFractL,$
			  nchanUsed:0l,$
			  bwUsed   :0.,$
			  cntsToK  :fltarr(ncalcyclesFile,npol),$; calUsec/difFit .. converting from counts to K
			  calKUsed :fltarr(npol) }
		   scanI={ nbms: nbms,$
                   nrecsfile:nrecs,$
		   		   ncalCyclesFile:ncalCyclesFile,$
				   hdrar   : replicate(bonar[0].h,nrecs),$; headers from bm 0
				   xh      : xh,$    ; index for each header
				   xhI     : xsI,  $ ; index to interpolate samples to header info.eg:spol=interpol(hpos,xh,xhi)   
			       bmI: replicate(a,nbms)}
			start=0
		endif
		if (ibm eq 0) then scanI.hdrar=bonar.h
;
;		 compute avgs, get calspectra, flatten all spectra with this (normalized)
;
		bavgOn=masmath(bonar,/avg)
		bavgOff=masmath(boffar,/avg)
		bcal=masmath(bavgOn,bavgOff,/sub)
		bcalN=masmath(bcal,/norm)
		eps=1e-6
;
;           don't divide by zero on edges. set all of edge  fract to eps
;
		bcalN.d[0:iedg0,*]=eps
		bcalN.d[iedg1:*,*]=eps
;
;			band pass correct all spectra this band , F-> flattened. this is just  1 freq band
;
		bonArF=masmath(bonAr,bcalN,/div)
		boffArF=masmath(boffAr,bcalN,/div)
;
;		 compute mask to exclude rfi . use same mask for polA,B since external rfi will probably
;                                      leak into both.
;        							 linear fit to rms/mean by chan.  throw out 3sigma outliers
		maskCum=zeroAr
		for ipol=0,1 do begin &$
			; find good points to use for tp avg look at cal on and off
			a=robfit_poly(xchn,brmsar[0].d[*,ipol],deg,gindx=gindx,ngood=ngood,fpnts=fpnts,$
                mask=maskSt) &$
       		maskCum[gindx]+=1 &$; good points from cal on
       		a=robfit_poly(xchn,brmsar[1].d[*,ipol],deg,gindx=gindx,ngood=ngood,fpnts=fpnts,$
                mask=mask) &$
        	maskCum[gindx]+=1 &$; good points from calon,caloff have 2 
		endfor
     	iigd=where(maskCum eq 4,ngoodchn) 
;           make sure edges are excluded..the fitting starts with excluded, but they can creep back in.
      	ii=where((iigd gt iedg0) and (iigd lt iedg1),cnt) 
		if cnt lt ngoodchn then begin 
           	iigd=iigd[ii]
           	ngoodchn=cnt
      	endif

;		 compute tp CalDefl using good channels
;        [ncalcycl,npol]  .. compute cal difference
		scanI.bmI[ibm].tpCalDif=transpose($
					reform((total(bonarF.d[iigd,*,*],1) - total(boffarF.d[iigd,*,*],1))/ngoodchn,$
							 npol,ncalCyclesFile))

;  			now fit polynomial got tcalDefl to use in correction for tpcal*G(t) . 

			coefAr=fltarr(degGainVar+1,npol)
			for ipol=0,npol-1 do begin &$
				coefAr[*,ipol]=robfit_poly(xtm,scanI.bmI[ibm].tpCalDif[*,ipol],degGainVar,yfit=yfit) &$
				scanI.bmI[ibm].tpCalDifFit[*,ipol]=yfit &$
			endfor
;
; 		now compute tpCalOnDegK, tpCalOffDegK 
;       use same channels as mask
;           avg over channels, no gain correction or degK yet.
			scanI.bmI[ibm].tpArOnR=transpose(reform( total( bonarF.d[iigd,*,*],1)/ngoodchn,npol,ncalCyclesFile),[1,0])
			scanI.bmI[ibm].tpArOffR=transpose(reform(total(boffarF.d[iigd,*,*],1)/ngoodchn,npol,ncalCyclesFile),[1,0])
;
;    compute avg cals for this band using channels for tp avg
;
		 	calKA=total(interpol(calRcvI.calA,calRcvI.freq,freqAr[iigd,ibm]),1)/ngoodchn
			calKB=total(interpol(calRcvI.calB,calRcvI.freq,freqAr[iigd,ibm]),1)/ngoodchn
			sclA=CalKA/scanI.bmI[ibm].tpCalDifFit[*,0]
			sclB=CalKB/scanI.bmI[ibm].tpCalDifFit[*,1]
			scanI.bmI[ibm].tpArOnK[*,0] = scanI.bmI[ibm].tpArOnR[*,0]*sclA
			scanI.bmI[ibm].tpArOffK[*,0]= scanI.bmI[ibm].tparOffR[*,0]*sclA
			scanI.bmI[ibm].tpArOnK[*,1]= scanI.bmI[ibm].tpArOnR[*,1]*sclB
			scanI.bmI[ibm].tpArOffK[*,1]= scanI.bmI[ibm].tparOffR[*,1]*sclB
			scanI.bmI[ibm].maskAr[iigd]=1
			scanI.bmI[ibm].nchanused=ngoodchn
			scanI.bmI[ibm].bwused=ngoodchn*bwChnByBm[ibm]
			scanI.bmI[ibm].cntsToK[*,0]=sclA
			scanI.bmI[ibm].cntsToK[*,1]=sclB
			scanI.bmI[ibm].brmsar=reform(brmsar)
			scanI.bmI[ibm].calKUsed=[calKA,calKB]
			avgSpcOn=fltarr(nchan,npol)
			avgSpcOff=fltarr(nchan,npol)
			j=0l
			for irec=0,nrecs-1 do begin
				for idump=0,ndump-1 do begin
					avgSpcOn[*,0]+=bonar[irec].d[*,0,idump]*sclA[j]
					avgSpcOn[*,1]+=bonar[irec].d[*,1,idump]*sclB[j]
					avgSpcOff[*,0]+=boffar[irec].d[*,0,idump]*sclA[j]
					avgSpcOff[*,1]+=boffar[irec].d[*,1,idump]*sclB[j]
					j++
				endfor
			endfor 
			scanI.bmI[ibm].bonavg.d=avgspcOn/ncalcyclesFile
			scanI.bmI[ibm].bonavg.h=bonar[0].h
			scanI.bmI[ibm].boffavg.d=avgspcOff/ncalcyclesFile
			scanI.bmI[ibm].boffavg.h=boffar[0].h
   	endfor  ; endfor loop on bms
;  
	return,nbms
end
