;+

;NAME:
;masinterpfreqbands - interpolate multiple freq bands to 1 spectra 
;SYNTAX: masinterpfreqbands,bar,df,dI,f0=f0,f1=f1
;ARGS:
;bar[n]: {}   array of mas structs holding multiple frequencies
;df    : float  delta freq (Mhz) for final interpolated spectrum
;ARGS: 
;f0  : MHz    if supplied then use this as the lowest interpolated chan. Def is
;             the minimum in the initial data.
;f1  : MHz    if supplied then use this as the maximum interpolated chan. Def is
;             the maximumin the initial data.
;RETURNS:
; Di    : {}   struct holding freq and interpolated data (see below for definintion).
;TERMINOLOGY: 
;     bms: when using alfa, the 7 mock boxes were called bms.This terminology
;          has carried over into single pixel observing even though they all point
;          at the same location.
;   bands: With alfa observing, bands refered to the upper and lower 150 Mhz freq bands
;          taken with one mock box. For single pixel observing the lower 150 Mhz band
;          is not used  and the term band is often used in place of bms to refer to 
;           masinterpfreqbands,reform(bar[i,*]),df,dI 
;          different frequency bands.. (sorry for the confusion:)
;DESCRIPTION:
;	Single pixel observing normally has up to 7 different frequency bands (or bms) that
;are typically overlapped. This routine  will interpolate these bands to a single spectra.
;The user supplies the  mas (bar) struct to combine and well as the requested frequency
;spacing for the final spectra. 
;	Most times the bands are overlapped with the overlapping region having the bandpass filter
;falloff. 
;  Prior to 28feb23 the excloverlap keyword specified how much of this overlap region should be
;   excluded from the edge of  each band. this caused problems when the bandpasses were
;   still present. The interpolation could then  jump back and forth between adj values from
;   the different bandpasses.
;  The current version just splits the overlap region in 2, taking the  bottom part from the
;  lower freq spectra, and the upper half from the higher freq spectra.
;
;	The routine will now work with a record with multiple dumps. If you have multiple records to interp,
;you must call this routine multiple times
; If there is no overlap between a set of bands then that portion of the spectrum
;will be a linear interpolation between the endpoints.
;
; The returned structure dI  will have:
;	dI.npnts   ; points in a spectrum
;    dI.npol    ; 2nd dimension of data
;    dI.freq[npnts]: dbl   frequency array
;    dI.d[npnts,npol] ; data
; or dI.d[npnts,npol,ndump] if multiple dumps per record
; -->Note:
;  This routine just  does interpolation between points (idl interpol) it does not do averaging.
;  If you specify a much larger freqspacing than the input spectra then the returned signal will not
;  be averaged over the wider freq step.
;-
;
pro masinterpfreqbands,bar,df,dI,excloverlap=excloverlap,f0=f0,f1=f1
;
	 common colph,decomposedph,colph
	nbms=n_elements(bar)
    nchan=bar[0].nchan
	npol=bar[0].npol
	; ignores stokes for now..
	if npol ge 2 then npol=2
	ndump=bar[0].ndump
	bandflipped=bar.h.cdelt1 lt 0
	if n_elements(exclOverlap)  ne  0  then begin
		print,"Note: masinterpfreqbands now ignores exclOverLap keyword"
	endif
    freqAr=dblarr(nchan,nbms)
    fminmax=dblarr(2,nbms)
	useDouble=(size(bar[0].d[0],/type) eq  5)
	scl=(size(bar[0].d[0],/type) eq  5)?1d:1.
	if ndump eq 1 then begin
		yyA=bar.d[*,0]*scl
		if npol eq 2 then yyB=bar.d[*,1]*scl
	endif else begin
		yyA=reform(bar.d[*,0,*]*scl,nchan,ndump,nbms)
		yya=transpose(yya,[0,2,1])
		if npol eq 2 then begin
			yyB=reform(bar.d[*,1,*]*scl,nchan,ndump,nbms)
		    yyB=transpose(yyB,[0,2,1])
		endif
	endelse
    for i=0,nbms-1 do begin &$
		if bandflipped[i] then begin
        	freqAr[*,i]=reverse(masfreq(bAr[i].h)) &$
			if ndump eq 1 then begin
				yyA[*,i]=reverse(yyA[*,i])
		    	if npol eq 2 then yyB[*,i]=reverse(yyb[*,i])
			endif else begin
				yyA[*,i,*]=reverse(yyA[*,i,*],1)
		    	if npol eq 2 then yyB[*,i,*]=reverse(yyb[*,i,*],1)
			endelse
		endif else begin
        	freqAr[*,i]=masfreq(bAr[i].h) &$
		endelse
        fminmax[0,i]=min(freqAr[*,i]) &$
        fminmax[1,i]=max(freqAr[*,i]) &$
    endfor
;   --> 28feb23.. switch overlap to take left overlap for lower, 
;                upper overlap from right
    bwchan=abs(bar[0].h.cdelt1*1e-6)
    for i=0,nbms-2 do begin
		ovrlap=(fminmax[1,i] -  fminmax[0,i+1])
		fcen=fminmax[1,i] - ovrlap/2.
		ii=where(freqAr[*,i] gt fcen,cnt)
		if (cnt gt 0) then freqAr[ii,i]=-1

		ii=where(freqAr[*,i+1] le fcen,cnt)
		if (cnt gt 0) then freqAr[ii,i+1]=-1
	endfor
;
    freqAr=reform(freqAr,nchan*nbms)
	if ndump eq 1 then begin
    	yyA=reform(yyA,nchan*nbms)
		if npol eq 2 then yyB=reform(yyB,nchan*nbms)
	endif else begin
    	yyA=reform(yyA,nchan*nbms,ndump)
		if npol eq 2 then yyB=reform(yyB,nchan*nbms,ndump)
	endelse
    iis=sort(freqAr)
    freqAr=freqAr[iis]
	if ndump eq 1 then begin
		yyA=yyA[iis]
		if npol eq 2 then yyB=yyB[iis]
	endif else begin
		yyA=yyA[iis,*]
		if npol eq 2 then yyB=yyB[iis,*]
	endelse
    iik=where(freqAr gt 0.,cnt)
    freqAr=freqAr[iik]
	if ndump eq 1 then begin 
		yyA=yyA[iik]
		if npol eq 2 then yyB=yyB[iik]
	endif else begin
		yyA=yyA[iik,*]
		if npol eq 2 then yyB=yyB[iik,*]
	endelse
;   get points we have after interpolation
	if n_elements(f0) eq 0 then f0=min(freqAr)
    if n_elements(f1) eq 0 then f1=max(freqAr)
    nstp=long((f1-f0)/df)
    freqI=f0 + df*dindgen(nstp)
;
	if ndump eq 1 then begin
		if npol eq 2 then begin
			dI={ npnts:nstp,$
			 npol : 2,$
			 freq :freqI,$
			    d :dblarr(nstp,2)$; pola,polB_
		   }
		endif else begin
			dI={ npnts:nstp,$
			 npol : 1,$
			 freq :freqI,$
			    d :dblarr(nstp)$; pola
		   	}
		endelse
		if npol eq 2 then begin
    		dI.d[*,0]=interpol(yyA,freqAr,freqI)
    		dI.d[*,1]=interpol(yyB,freqAr,freqI)
		endif else begin
    		dI.d[*]=interpol(yyA,freqAr,freqI)
		endelse
	endif else begin
		if npol eq 2 then begin
			dI={ npnts:nstp,$
			 npol : 2,$
			 freq :freqI,$
			    d :dblarr(nstp,2,ndump)$; pola,polB_
		   }
		endif else begin
			dI={ npnts:nstp,$
			 npol : 1,$
			 freq :freqI,$
			    d :dblarr(nstp,ndump)$; pola
		   	}
		endelse
		for i=0,ndump-1 do begin
			if npol eq 2 then begin
    			dI.d[*,0,i]=interpol(yyA[*,i],freqAr,freqI)
    			dI.d[*,1,i]=interpol(yyB[*,i],freqAr,freqI)
			endif else begin
    			dI.d[*,i]=interpol(yyA[*,i],freqAr,freqI)
			endelse
		endfor
	endelse
	return
end
