;+
;NAME:
;masmakemask - make a mask for  good channels using a dataset.
;SYNTAX: istat=masmakemask(fnmI,maskI,verb=verb,nsig=nsig,edge=edge,flist=flist)
;ARGS:
; fnmI[n]: {}   array holding files info to process (see masfilelist())
;KEYWORDS:
;  nsig  : float   number of sigmas that separate good from bad channels
; edge[2]: fltarr  fraction of each edge of bandpass to ignore def:[.06,.06]
;  verb  :         if set then plot rms/mean by channel for each file.
;flist[n]: strarr  filenames to use (including path).
;                  if supplied then ignore fnmI[] and use flist
;RETURNS:
;istat  :   int    1 - ok, -1 error
;maskI  : {}       struct holding the mask info (see below).
;
;DESCRIPTION:
;	Given a set of datafiles, this routine will compute a mask of good freq channels to use 
;(when computing total power, or fitting a baseline). The processing steps are:
; (assume there are N files)
; for each file in fnmI[N]
;   - input the file
;   - compute rms/mean by freq channel using the spectra in the file.
;     channels with rfi will have a larger rms/mean .
;   - compute the avg spectra for the file
; After all N files are input:
;   - compute the average rms/mean for the nfiles. call it:rmsAvg
;   - compute the peak value of rms/mean for each file (take the peak from each channel):call it: rmsPk
; Using these 2 averages, find the channels with a large rms/mean:
; -Do a robust linear fit to rmsAvg(nchan)  and rmsPk(nchan).
;   Any fit residuals greater then nsig will be flagged as bad.
;
; maskI returns the following info
;IDL> help,maskI,/st
; NCHAN     LONG     1024   ; number of freq channels
; NFILES    LONG     31     ; number of files used
; NSIGUSED  FLOAT    2.00000 ; value of nsig used for clipping
; MASKAVG   STRUCT -> <Anonymous> Array[1] ;holds maskInfo for the avgerage mask (see below)
; MASKPK    STRUCT -> <Anonymous> Array[1] ;holds maskinfo for the peakHold mask (see below)
; BRMSAVG   STRUCT -> <Anonymous> Array[1] ;average rms/mean spectra (display with masplot,brmsavg
; BRMSPK    STRUCT -> <Anonymous> Array[1] ;peakHold rms/mean spectra 
; BAVG      STRUCT -> <Anonymous> Array[1] ;average spectra
;
; The maskavg, maskpk struct holds:
; help,maski.maskavg,/st
; SIGA      FLOAT  8.52951e-05 ;polA sigma from robust fit to rms/mean avg (or rms/mean peak)
; SIGB      FLOAT  8.04953e-05 ;polB sigma from robust fit to rms/mean avg (or pk)
; NGOODA    LONG   382         ;# of good chan in polA
; NGOODB    LONG   502         ;# of good chan in polB
; NGOOD     LONG   327         ;# of good chan in polA and B
; MASK      INT    Array[1024] ;mask for channels 0=bad, 1=goood, 
;
; The routine makes a mask using the average of the rms/mean by files. 
;It makes a second mask  using the max (in each chan)  of the peakHold rms/mean spectra
;The pkMask will most likely exclude more points than the average mask.
;
; The /verb keyword will plots:
; Each rms/mean will be plotted as it is processed.
; - you should set the ver scale to whatever the limits will be (from the radiometer equation):
;   eg: 172MhzBw/1024chan  rms/mean = .008 so ver,0,.02 will work
; - after plotting all the  files it will plot the average rms/means
; top frame: rms/mean avg
;            - white  + :polA all points 
;            - red    + :polB all points
;            - green  + :polA good freq channels
;            - yellow + :polB good freq channels
;	If you see green or yellow points that stick up above the rms/mean avg then you may
;want to recall the routine with a smaller value for nsig (def=2). But be careful, if you
;make nsig too small, then you may run out of good points.
;                
;
;Some requirements:
; - each file should have the same number of channels.
; - you probably don't want to mix different freq bands.
;
;EXAMPLE:
;	Suppose we took a number of sband crosses on 3C295 on 11nov22
;   - there is a calOn,off at the start of each cross.
;   - use the calOff's to compute the rfi mask.
;  proj='x101'
;  yymmdd=20221111l
;  nfiles=masfilelist('',fnmI,proj=proj,yymmdd=yymmdd,bm=0,band=1,grp=0,/appbm)
;  ;.. get summary info (it includes the headers) to select the cal Offs
;  istat=masfilesum('',sumI,fnmI=fnmI)
;  ii=where((sumI.h.obsmode eq 'CAL') and (sumI.h.scantype eq 'OFF'),nfiles)
;  ; make a subset of only the cal offs (note sband only has 1 mock band) 
;  fnmICaloff=fnmI[ii]
;  ; make the mask, setup hor ,ver for /verb plotting.
;  ; exclude 8%. 10% from the 2 edges (bw of rf < mock bw)
;  hor
;  ver,0,.02
;  edge=[.08,.1]
;  istat=masmakemask(fnmICalOff,maskI,/verb,edge=edge0i
;  ; use the peak hold spectra
;  mask=maskI.maskavg.mask
;  ; call stripchart routine to monitor with mask
;  masmonstripch,mask=mask,proj=x101
;  
;-
function masmakemask,fnmI,maskI,verb=verb,nsig=nsig,edge=edge,flist=flist
;
    common colph,decomposedph,colph

	if n_elements(nsig) eq 0 then nsig=2
	if n_elements(edge) eq 0 then edge=[.06,.06]
	edgel=edge
	if n_elements(edgel) eq 1 then edgeL=[edge[0],edge[0]]
	useflist=n_elements(flist) gt 0
	nfiles=(useflist)?n_elements(flist):n_elements(fnmI)
	for ifile=0,nfiles-1 do begin
		if useflist then begin
			istat=masgetfile('',b,filename=flist[ifile])
		endif else begin
			istat=masgetfile('',b,fnmI=fnmI[ifile])
		endelse
		brms=masrms(b)
		if ifile eq 0 then begin
		   brmsAr=replicate(brms,nfiles)
		   bavgAr=brmsAr ; want floats
		endif
		brmsAr[ifile]=brms
		bavgAr[ifile]=masmath(b,/avg)
		fname=(useflist)?basename(flist[ifile]):fnmI[ifile].fname	
		if keyword_set(verb) then masplot,brms,tit=fname
	endfor
;
; 	now compute avg and max rms
;
	brmsAvg=masmath(brmsAr,/avg)
	brmsPk=brmsAvg
	brmsPk.d[*,0]=max(brmsAr.d[*,0],dim=2)
	brmsPk.d[*,1]=max(brmsAr.d[*,1],dim=2)
	nchan=b[0].nchan
	maskSt=intarr(nchan)
	i0=(long(edgeL[0]*nchan + .5) -1) > 0
	i1=(nchan - (long(edgeL[1]*nchan + .5) -1))  < (nchan-1)
	maskSt=intarr(nchan)+1
	maskSt[0:i0]=0
	maskSt[i1:*]=0
	x=findgen(nchan)
    deg=1
	maxiter=40
	coefA=robfit_poly(x,brmsavg.d[*,0],deg,sig=sigA,gindx=gindxA,nsig=nsig,yfit=yfitA,$
			iter=iter,maxiter=maxiter,mask=maskst)
	coefB=robfit_poly(x,brmsavg.d[*,1],deg,sig=sigB,gindx=gindxB,nsig=nsig,yfit=yfitB,$
			iter=iter,maxiter=maxiter,mask=maskst)
	coefAPk=robfit_poly(x,brmsPk.d[*,0],deg,sig=sigAPk,gindx=gindxAPk,nsig=nsig,yfit=yfitApk,$
			iter=iter,maxiter=maxiter,mask=maskst)
	coefBPk=robfit_poly(x,brmsPk.d[*,1],deg,sig=sigBPk,gindx=gindxBPk,nsig=nsig,yfit=yfitBpk,$
			iter=iter,maxiter=maxiter,mask=maskst)
;   avg
	ok=intarr(nchan)
	ok[gindxA]=1
	ok[gindxB]+=1
;   force edge
	ok[0:i0]=0;
	ok[i1:*]=0;
	iiavg=where(ok eq 2,cnt)
;   pk
	ok=intarr(nchan)
	ok[gindxAPk]=1
	ok[gindxBPk]+=1
;   force edge
	ok[0:i0]=0;
	ok[i1:*]=0;
	iiavg=where(ok eq 2,cnt)
	iiPk=where(ok eq 2,cntPk)
	if keyword_set(verb) then begin
		!p.multi=[0,1,2]
		freq=masfreq(b[0].h)
		symA=1
		symB=2
		ver,0,(coefA[0]+coefB[0]) 
		plot,freq,brmsavg.d[*,0],psym=symA
		oplot,freq,brmsavg.d[*,1],psym=symB,col=colph[2]
		oplot,freq,yfita
		oplot,freq,yfitB,col=colph[2]
		oplot,freq[gindxA],brmsavg.d[gindxA,0],psym=symA,col=colph[3]
		oplot,freq[gindxB],brmsavg.d[gindxB,1],psym=symB,col=colph[5]
;
		ver,0,(coefAPk[0]+coefBPk[0]) 
        plot,freq,brmsPk.d[*,0],psym=symA
        oplot,freq,brmsPk.d[*,1],psym=symB,col=colph[2]
        oplot,freq,yfitaPk
        oplot,freq,yfitBPk,col=colph[2]
        oplot,freq[gindxAPk],brmspk.d[gindxAPk,0],psym=symA,col=colph[3]
        oplot,freq[gindxBPk],brmsPk.d[gindxBPk,1],psym=symB,col=colph[5]
	endif

	mask1={siga:0.,$
		   sigb:0.,$
		   ngoodA:0l,$
		   ngoodB:0l,$
		   ngood:0l,$
		   mask :intarr(nchan)}
	maskavg=mask1
	maskavg.siga=siga
	maskavg.sigB=sigb
	maskavg.ngood=cnt
	maskavg.ngoodA=n_elements(gindxa)
	maskavg.ngoodB=n_elements(gindxB)
	maskavg.mask[iiavg]=1
	maskPk=mask1
	maskPk.siga=sigaPk
	maskPk.sigB=sigbPk
	maskPk.ngood=cntPk
	maskPk.ngoodA=n_elements(gindxaPk)
	maskPk.ngoodB=n_elements(gindxBPk)
	maskPk.mask[iiPk]=1
	maskI={$
			nchan:nchan,$
			nfiles:nfiles,$
	        nsigUsed:nsig,$
			maskAvg:maskavg,$
			maskPk:maskPk,$
            brmsavg:brmsAvg,$
            brmsPk :brmsPk,$
			bavg   :masmath(bavgar,/avg)}
	return,1
end
