;+
;NAME:
;masmon2dfits - plot 2d gauss fits from mascrossfit2d
;SYNTAX: masmon2dfits,fitiAr,delay=delaysecs,freqtouse=freqtouse,pwr=pwr,$
;                     remtsys=remtsys
;ARGS:
;fitiar[n] :  {} array of fit structs from mascrossfit2() 
;freqToUse: Mhz  freq to use. def=all bands
;pwr      :      if set then data in in power units. def: degK 
;                (used for yaxis labels)
;remTsys  :      if set then remove the constant Tsys term from the data
;                before plotting.
;DESCRIPTION:
;	The mascrossfit2d() routine returns an array of structures with the fitting
;info. This routine will plot the input data and resulting fit. Each plot will 
;have the data from 1 freq band and fit. There will be 3 frames on a plot:
; - upper  frame: az strip data vs offset with the 2d fit for az overplotted.
; - middle frame: el strip data vs offset with the 2d fit for el overplotted.
; - bottom frame: the az and el data and fits are plotted vs the el offset from
;                 the center of the cross. The fitting routine includes a
;                 linear term in elevation when fitting.
; There are a number of parameters that will control what and how things
;are plotted.  After entering the routine, hitting any key will bring up a
;menu that lets you change these parameters. You can then enter a command 
;and value.
;  
;CMD      ARG    Description
;i      index    move to index (0 to nfits-1= 209)
;src      all    srcToPlot,all or                  
;                3C274 3C286 3C348 3C405 CasA 
;f        all    Freq to plot or  all. frqList:
;                8223 8363 8505 8647 8789 8931 9073 
;delay   secs    delay between plots. def:.1 secs
;r        0,1    remove tsys:0,1
;s        0,1    single step (0,1)
;v               min,max .. set vertical scale, blank is autoscale
;d               debug. stop in routine
;q               quit monfits
;any otherChar continue plotting
;  currently stepping:       0
;
;
;i    : lets you skip around in the fitiar[] array
;src  : you can limit the plots to a single source. Entering "all" will
; i     plot all sources
;f    : limit plots to this freq. Entering "all" will return to all frequencies.
;delay: Put a delay of delay secs between each plot (fractional secs ok).
;r    : 1-> remove tsys from data and fits before plotting. 0.. don't remove
;       Since it uses the fit value,if the fit failed, the plot may not appear.
;s    : 1-> single step the plots. It will redisplay the menu after each plot and wait for
;       you to enter a key.
;       0-> go back to continuous mode.
;v    :  vmin,vmax set the vertical scale for the plots. Entering a blank
;        will return to autoscaling.
;d    :  go to debug mode. will stop within the routine.
;        enter .continue to continue on.
;q    :  quit this routine.
;
;	The routine uses masplot2dfit() to plot a single fit.
;
;Example:
;; if you've saved the fits somewhere
; restore,'fitiar.sav' ,/verb
;% RESTORE: Portable (XDR) SAVE/RESTORE file.
;% RESTORE: Save file written by phil@megs3, Fri Oct  7 08:55:20 2022.
;% RESTORE: IDL version 7.1 (linux, x86_64).
;% RESTORE: Restored variable: FITIAR.
;
; masmon2dfits,fitiar 
; .. plottings starts
; .. you can resize the window with the cursor to get a better view
; .. hit space bar to bring up the menu:
;
;CMD      ARG    Description
;i      index    move to index (0 to nfits-1= 209)
;src      all    srcToPlot,all or
;                3C274 3C286 3C348 3C405 CasA 
;f        all    Freq to plot or  all. frqList:
;                8223 8363 8505 8647 8789 8931 9073 
;delay   secs    (0-> no delay)
;r        0,1    remove tsys:0,1
;s        0,1    single step (0,1)
;v               min,max .. set vertical scale, blank is autoscale
;d               debug. stop in routine
;q               quit monfits
;any otherChar continue plotting
;  currently stepping:       0
;v -.2 2     .. change vertical scale -.2 to 2 K
;-
pro masmon2dfits,fitIAr ,delay=delaysecs,freqtouse=freqtouse,pwr=pwr,$
			remTsys=remTsys


	curSrc='all'
	remTsysL=(keyword_set(remTsys))?1:0
	singleStep=0
	ldelay=(n_elements(delaysecs) gt 0)?delaysecs:.1
	nfits=n_elements(fitiar)
	nbms=n_elements(fitiar[0].fitibm)
	npnts=fitiar[0].pntstrip
	icur=0
	var=0
	usrc=fitiar[uniq(fitiar.srcnm,sort(fitiar.srcnm))].srcnm
	nsrc=n_elements(usrc)
	usrcL=strlowcase(usrc)
	labSrc=''
	for i=0,nsrc-1 do begin
		labSrc=labsrc + strtrim(usrc[i],2) + " "
	endfor
;   freq
	curFrq='all'
	ufrq=long(fitiar[0].fitIbm.cfrmhz + .5)
	nfrq=n_elements(ufrq) 
	if n_elements(freqtouse) gt 0 then freql=long(freqtouse + .5)
	labFrq=''
	for i=0,nfrq-1 do labFrq=labFrq + string(format='(i4)',ufrq[i]) + ' '
		

	while (icur lt nfits) do begin &$
		for ibm=0,nbms-1 do begin &$
			lab=string(format='(" index:",i3,"/",i3)',icur,nfits)
;           do we skip src 
			src=strlowcase(fitiar[icur].srcnm)
			skipSrc=(src ne curSrc) and (curSrc ne 'all')
;           do we skip freq
			if (not skipSrc) and (curFrq ne 'all') then begin
				lfrq=string(format='(i4)',long(fitiar[icur].fitibm[ibm].cfrMhz +.5))
				skipSrc=lfrq ne curFrq
			endif
	    	if (not skipSrc) then	$
			    masmon2dplot1,fitiar[icur],ibm,lab=lab,pwr=pwr,var=var,remTsys=remTsysL &$
			key=''
			brkbm=0
			newcur=0
			if singleStep and (not skipSrc) then begin
				key=' '
			endif else begin
			    key=checkkey() 
			endelse
			if key ne '' then begin
				gotit=0
			    while not gotit do begin
				  print," "
				  print,'CMD      ARG    Description'
			      print,'i      index    move to index (0 to nfits-1='+string(format='(i4)',nfits-1)+')'
				  lab=string(format='(a8)',cursrc)
				  print,'src ' + lab + '    srcToPlot,all or'
				  print,'                ' + labSrc
				  lab=string(format='(a4)',curfrq)
				  print,'f       ' + lab + "    Freq to plot or  all. frqList:"
				  print,'	        ' + labfrq
			      print,'delay   secs    (0-> no delay)'
				  print,'r        0,1    remove tsys:0,1'
				  print,'s        0,1    single step (0,1)'
				  print,'v               min,max .. set vertical scale, blank is autoscale'
				  print,'d               debug. stop in routine'
			      print,'q               quit monfits'
				  print,'any otherChar continue plotting'
				  print,'  currently stepping:',singlestep 

				  inpstr=''
    			  read,inpstr
    			  toks=strsplit(inpstr,' ,',/extract)
    			  cmd=toks[0]
				  ntoks=n_elements(toks)
			   	  case cmd of 
				   'delay': begin
						if ntoks ne 2 then begin
							print,"-->delay  secs"
						    break
						endif
						ldelay=float(toks[1])
				        gotit=1
						end
				   'f': begin
						if ntoks ne 2 then begin
							print,"-->f freq or all"
						    break
						endif
						val=strlowcase(toks[1])
						if val eq 'all' then begin
							curFrq='all'
					    endif else begin
							ival=long(val)
						    ii=where(ival eq ufrq,cnt)
		 					if cnt eq 0 then begin
								print,"-->illegal freq"
				            	print,"--> legal values: all or "+labfrq
						    	break
							endif
							curFrq=string(format='(i4)',long(ufrq[ii] + .5))
						endelse
				       	gotit=1
						end
				   'r': begin
						if ntoks ne 2 then begin
							print,"-->step   0 or 1"
						    break
						endif
						remTsysL= long(toks[1]) ne 0
				        gotit=1
						end
				   'src': begin
						if ntoks ne 2 then begin
							print,"-->src srcNm or all"
						    break
						endif
						val=strlowcase(toks[1])
						aa=[usrcl,'all']
						im=where(val eq aa,cnt)
					    if cnt eq 0 then begin
							print,"-->illegal source name (or all)"
						    break
						endif
						curSrc=aa[im]
				        gotit=1
						end
				   's': begin
						if ntoks ne 2 then begin
							print,"-->step   0 or 1"
						    break
						endif
						singlestep= long(toks[1]) ne 0
				        gotit=1
						end
				   'i': begin
						if ntoks ne 2 then begin
							print,"-->i IndexPosition"
						    break
						endif
					    index=long(toks[1])
					    if (index lt 0 ) or (index ge nfits) then begin
						   print,"--> index is 0 to nfits-1:",nfits-1
					       break
					    endif
						icur=index
						brkbm=1
						gotit=1
					    newcur=1
					  end
				   'q': return
				   'd': begin
						  stop
					      gotit=1
						end
				   'v': begin
						  case ntoks of
						  1:begin
							 var=0
						     gotit=1
				 		     break 
						    end
					      3: begin
							 var=float(toks[1:2])
						     gotit=1
				 		     break 
						     end
					     else: print,"--> bad values for v keyword ( 0 or 2 ok)"
						endcase
					    end
			      else: begin
						gotit=1
					  end
				  endcase
				endwhile
			endif 
			if brkbm ne 0 then break
			if (ldelay gt 0.) and (singleStep eq 0) and (not skipSrc)  then wait,ldelay &$
		endfor ; beam loop &$
 		if newcur eq 0 then icur++
	endwhile
	return
end
