;+
;NAME:
;maspatidday - get the pattern id's for a day and obsmode
;SYNTAX: npatid=maspatidday(yymmdd,proj,obsmode,patIday,src=src,bmtotest=bmtotest,$
;                           patid=patid)
;
;ARGS:
;  yymmdd   long    date to test
;  proj     string  project id to test
;  obsmode  string  the pattern must include this obsmode type:see list below
;                   (it can contain additional obsmodes eg ONOFF  with CAL
;                   The exception is if patid= keyword is used.
;KEYWORDS: 
;     src:  string  if supplied then the pattern must also be on this source.
;                   def: any source
;bmToTest:  int     which bm (mock box to check) 0..6
;patid   : long     if present then ignore obsmode.
;                   return info for patid.. If src= is supplied then this patid
;                   must have src as its object..
;RETURNS:
;  npatid   int     number of unique patterns we found
; patIdDay: {}      struct holding the info
;
; Search for all patterns for date yymmdd and project=proj.
;To be included the pattern must include the obsmode=obsmode (see below)
;If the keyword src is included then the pattern must also have src for the
;source observered.
;	The routine only scans a single mock bm (by default bm0). the keyword bmtotest=
;will use that beam instead (0..6).
;
;	If the patid=  keyword is present, then the routine ignores obsmode
;and searches for the single patid=patid (in bmToTest). If src= is supplie then
;this pattern id must have src as its object.
;
;	The info is returned in the structure patIday
;
;  help,patIday,/st
;  NPAT   LONG       2          number of patterns found
;  PROJ   STRING    'sun'       project id used
;  SRC    STRING    'sun'       source requested
;  PATI   STRUCT     Array[2]   holds the info on each pattern
;the patI struct holds:
; help,patIday.patI,/st 
; PATID   LONG      211700002   patid for this pattern
; FNUM0   long      0           first filenumber for pattern
; FNUM1   long      2800        last  filenumber for pattern
; HDR0    STRUCT MASFHDR Array[1] fits header first scan, bm=bmToTest
; HDR1    STRUCT MASFHDR Array[1] fits header last  scan  
;
;  obsmode values are:
; ONOFF    - on off position switching
; RADECMAP - map driving in ra, stepping in dec
; ON       - just tracking a source
; CAL      - calon,off will probably be part of another pattern. eg, onoff	 
; CROSS    - az,el cross on a source
;
;Note:
; this routine  will not work with pulsar observations since the psrfits files
; have a different header
;-
function maspatidday,yymmdd,proj,obsmode,patIday,src=src,bmtotest=bmtotest,patid=patid  
;
;	
	useSrc=n_elements(src) gt 0
	usePatId=n_elements(patid) gt 0
	retFnumAr=arg_present(fnumAr)
	if n_elements(bmToTest) eq 0 then bmToTest=0
	yymmddL=yymmdd
	if yymmddL lt 999999 then yymmddL+=20000000l
    n=masfilelist('',fnmI0,proj=proj,yymmdd=yymmddL,bm=bmToTest,band=1,grp=0,/appbm)
	if n eq 0 then return,0
;
;   get the summary info
;
    nsum=masfilesum('',sumI0,fnmI=fnmI0)
;
	patIdAr=sumI0[uniq(sumi0.h.pattern_id,sort(sumi0.h.pattern_id))].h.pattern_id
	if usepatid then begin
		ii=where(patIdAr eq patid,cnt)
		if cnt eq 0 then return,0
		patIdAr=patId
;       need proj,obsmode
		ii=where(sumI0.h.pattern_id eq patId)
		proj=fnmI0[ii[0]].proj
		obsmode=sumI0[ii[0]].h.obsmode
	    src=    sumI0[ii[0]].h.object
	endif
    npat=n_elements(patIdAr)
;
;   make sure pattern has obs mode and optionally src
;
    ok=intarr(npat)
    for i=0,npat-1 do begin
        ii =where(sumI0.h.pattern_id eq patIdAr[i],cnt)
		if (useSrc) then begin
        	jj =where((sumI0[ii].h.obsmode eq obsmode) and (sumI0[ii].h.object eq src),cnt)
		endif else begin
        	jj =where((sumI0[ii].h.obsmode eq obsmode),cnt)
		endelse
        if cnt gt 0 then ok[i]=1
    endfor
    ii=where(ok eq 1,npat)
	if npat eq 0 then return,0
	patIdAr=patIdAr[ii]
;  
;  define the return struct
;
	a={ patId: 0l,$
		fnum0: 0L,$;  first filenumber
		fnum1: 0L,$;  last filenumber
		hdr0: sumI0[0].h,$ ; first fits header
		hdr1: sumI0[0].h $ ; last fits header
	  }
	
    patIday={npat : npat,$; we found
		     proj : proj,$; we used
		     src  : '',$;   we used, '' if any source
			 patI :replicate(a,npat)$
			}
   	patIday.src=(usesrc)?src:''
	for i=0,npat-1 do begin
        ii =where(sumI0.h.pattern_id eq patIdAr[i],nfiles) 
		a=min(sumI0[ii].num,ind0)
		a=max(sumI0[ii].num,ind1)
   	    patIday.patI[i].patid=patIdAr[i]
   	    patIday.patI[i].fnum0= sumI0[ii[ind0]].num
   	    patIday.patI[i].fnum1= sumI0[ii[ind1]].num
   	    patIday.patI[i].hdr0 = sumI0[ii[ind0]].h
   	    patIday.patI[i].hdr1 = sumI0[ii[ind1]].h
	endfor
	return,npat
end
