;+
;NAME:
;mastsyshwcal - compute tsys from files with hardware winking cal 
;SYNTAX: n=mastsyshwcal(fnmAr,tsysI,fnmIAr=fnmIAr,verb=verb,$
;                       edgefract=edgefract,rmsdigar=rmsdigar)
;ARGS:
; fnmAr[n]: strarr  list of files to process
;KEYWORDS:
; fnmIAr[n] :{}       files to input. returned from masfilelist. If
;                     present, ignore fnmAr[]
; edgefract[2]:fltarr fraction of bandpass on each edge to ignore when computing
;                     tsys. default [.06,.06]. if single value entered then use
;                     same fraction on each edge.                     
; verb      : int     if set then output file info as they are processed
;RETURNS:
; n       : int > 0 number of entries in tsysI[n]. it will skip any files that do not
;                   include the winking cal.
;             < 0 error
; tsysI[n]:  struct arr  holding tsys info for each file
; rmsdigar[2,2,n]: fltarr   return dig rms [i/q,POlA/B,nfiles]
;DESCRIPTION:
;	Compute tsys from files with hardware winking cal data. Each file
;in fnmAr[n] (or fnmIAr[]) is input using masgethwcal(). The
;cal value is input and then tsys is computed using the cal off specdtra. Tsys info
;is returned in the tsysI[] structure. It contains:
;   YYMMDD    LONG      20220711   ; ast date start of file
;   SECMID    LONG         84093   ; ast secmid start of file
;   SEC1970   DOUBLE   1.6575961e+09 ; secs 1970 start of file
;   H         STRUCT -> MASFHDR Array[1]; mas header
;   SCAN      LONG     219200151        ; scan number
;   BM        LONG             0        ; bm/band 0..6
;   CFR       FLOAT       8219.00       ; center freq of band in MHz
;   AZ        FLOAT       50.7062       ; az start of file
;   EL        FLOAT       53.9142       ; el start of file
;   TSYSA     FLOAT       103.902       ; tsys polA deg K avg over band
;   TSYSB     FLOAT       97.4181       ; tsys polB deg K (if 2 pol)
;   CALI      STRUCT-> <Anonymous> Array[1] ; cal info, includes how much of band was used
;                                             to compute tsys (edge, rfi ignored).
;EXAMPLE:
;	1. process tsys for a given day for proj sscontin
; proj='sscontin'
; yymmdd=20220711
; nfiles=masfilelist('',fnmIAr,proj=proj,yymmdd=yymmdd,band=1,grp=0,/appbm)
; nTsys=mastsyshwcal('',tsysI,verb=verb,fnmIar=fnmIAr)
; ver, 80,125
; plot,tsysI.el,tsysI.tsysA
; oplot,tsysI.el,tsys.tsysB,col=colph[2]
;-
function  mastsyshwcal,fnmAr,tsysI,fnmIar=fnmIar,verb=verb,edgefract=edge,rmsdigar=rmsdigar
;
	edgel=[.06,.06]
	hwcalport=2
	retRmsDig=arg_present(rmsdigar)

	if n_elements(edge) eq 2 then edgeL=edge
	if n_elements(edge) eq 1 then edgeL=[edge,edge]
	
	fnmArL=fnmAr
	if (n_elements(fnmIAr) gt 0) then fnmArL=fnmIar.dir + fnmIAr.fname
	nfiles=n_elements(fnmArL)
	if nfiles eq 0 then return,0
;
;   if lots of files,, first check which ones have hw wcal
;   in case they are passing in all files from a  day.
;
	if nfiles gt 14 then begin
       nfiles=masfilesum(fnmArL,sumI)
	   ii=where(sumI.h.calsrc eq hwcalport,cnt)
	   if cnt eq 0 then return,0
	   if cnt ne nfiles then begin
		   fnmArL=fnmArL[ii]
		   nfiles=cnt
	   endif
	endif
;
	cmpmask=1
	icur=0
	for ifile=0,nfiles-1 do begin
		   istat=masgethwcal( "",calIhw,bonar,boffar,bavgar=bavgar,/avgrow,$
				fnmAr=fnmArL[ifile],sumI=sumI,/dcintp,verb=verb,hdrAr=hdr)
		   if istat lt 0 then begin
				if keyword_set(verb) then print," ..masgethwcal err, skip:",fnmArL[ifile]
			    continue
		   endif
		   if istat le 0 then continue
		   bavgar=reform(bavgar)
		   istat=mascalonoff(bavgAr[0],bavgAr[1],calI,edge=edgeL,cmpmask=cmpmask)
		   if istat lt 0 then begin
				if keyword_set(verb) then print," ..mascalonoff err, skip:",fnmArL[ifile]
			    continue
		   endif
		   if icur eq 0 then begin 
;          info to return to caller
;                 info about cal
			   a1={$
               calVal: fltarr(2),$; averaged cal values,pola,b
               numPol   : 0l,$;
               exposure  : 0.    ,$; filled in by mascalonoff
               cntsToK  : fltarr(2),$; filled in by mascalonoff
               npnts     : 0l,$ ; points used to compute average cal.
			   fractUsed : 0.         ,$ ; npntUsed/nchan
               flipped   : 0l,$ ; 1 if band is flipped
               edgeFract : edgeL,    $  ; now same order as spectra
               usemask   : 0 }   ; 1--> used freq mask
;                                     0--> used edgeFract
			   a={ yymmdd: 0l,$ ; ast date
				   secMid: 0l,$ ; sec midnite ast
			      sec1970: 0d,$ ; sec from 1970
			         h   : bavgar[1].h,$ ; hdr for cal off
				  scan   : 0l,$ ; scan number
				     bm  : 0l,$ ; which freq band
		       cfr       : 0.,$; cfr 
		       az        : 0.,$; azimuth
		       el        : 0.,$; elevation 
			   tsysA     : 0.,$; degK
		       tsysB     : 0.,$
		       calI      : a1}  ; info on how cal computed
			   tsysI=replicate(a,nfiles)
			   if retRmsDig then rmsDigAr=fltarr(2,2,nfiles)
		   endif
;
		   code= 3  ; no band pass correction , just return total power tsys
		   istat=mascalscl(bavgAr[1],calI,code,bK)
		   if (istat lt  0) then begin
				if verb then print,".. skipped mascalscl returns err"
				continue
		   endif
		   npol=calI.numpol
		   tsysI[icur].sec1970=tosecs1970(0,0,mjd=bavgar[1].h.mjdxxobs)
		   a=fromsecs1970(tsysI[icur].sec1970)
		   tsysI[icur].secMid=hms1_secmid(a[0])
		   tsysI[icur].yymmdd=a[1]
		   tsysI[icur].h     =bavgar[1].h
		   tsysI[icur].scan=bavgar[1].h.scan_id
		   tsysI[icur].bm  =bavgar[1].h.beam
		   tsysI[icur].cfr =bavgar[1].h.crval1*1e-6
		   tsysI[icur].az  =bavgar[1].h.azimuth
		   tsysI[icur].el  =bavgar[1].h.elevatio
		   tsysI[icur].TsysA=bk[0]
		   if npol gt 1 then tsysI[icur].TsysB=bk[1]
			
		   TsysI[icur].calI.calval[0]=calI.calval[0]
		   if npol eq 2 then TsysI[icur].calI.calval[1]=calI.calval[1]
		   TsysI[icur].calI.numpol=calI.numpol
		   TsysI[icur].calI.exposure=calI.exposure
		   TsysI[icur].calI.cntsToK =calI.cntsToK
		   TsysI[icur].calI.npnts   =calI.npnts
		   TsysI[icur].calI.fractUsed=(calI.npnts*1.)/bavgar[0].h.chan_in
		   TsysI[icur].calI.flipped =calI.flipped
		   TsysI[icur].calI.edgeFract=calI.edgeFract
		   TsysI[icur].calI.useMask  =calI.useMask
;
		   if retRmsDig then begin
				a=strpos(hdr,"ADRMS_")
			    ii=where(a ne -1,cnt)
			    if cnt ne 4 then begin
					  print,"mastsyshwcal:no ADRMS_ in header. file:",$
                        fnmArL[ifile]
                	return, -1
		        endif
				a=stregex(hdr[ii],"ADRMS_[AB][IQ]= *([0-9.]*)",/extract,/sub)
                rmsDigAr[*,*,ifile]=float(reform(a[1,*]))
            endif
		   icur++
	endfor
	if icur eq 0 then return,0
	if icur ne nfiles then begin
		TsysI=tsysI[0:icur-1]
	    if retRmsDig then begin
			rmsDigAr=rmsDigAr[*,*,0:icur-1]
			if icur eq 1 then rmsDigAr=reform(rmsDigAr,2,2)
		endif
	endif
	return,icur
end
