;+
;NAME:
;p12mfitmod - fit model to az,el positions and pointing errors
;SYNTAX: p12mfitmod,az,el,azErr,elErr,modFitI,resAz,resEl
;ARGS:
;       azReq[npts]: flt/dbl  requested azimuth positions degrees

;       elReq[npts]: flt/dbl  requested el  angle positions degrees
;    azErr[npts] : flt/dbl    azimuth errors (great circle) deg.(azPk - azUsed)gc
;    elErr[npts] : flt/dbl el      angle errors deg.            (elPk - elUsed)
;                              the refraction correction has not been removed
;                           
;RETURNS:
;      modFitI : {modelFitInfo} return model fit results here
;                               The model coef will output Asec values
;   resAz[npts]: hold the azimuth residuals from the fits (asecs). (GC) 
;                           azErr - modelCoraz
;   resEl[npts]: hold the el residuals from the fits (asecs).  
;                computed as: (elErr - Refraction) - modelCorel
;
;Notes: - Lc=Little circle
; 		- Gc= great circle
;       - When making a new model, we normally fit to to raw azerr,elerr..
;         so the old model and the refraction correcdtion has been added to the 2dfit errors.
;       
;DESCRIPTION:
;   fit the 12meter model to the offsets measured by the 2d cross fits on sources.
;Input: azReq,elReq            :requested position encoder values for 2d fits
;       azerr                  :measured az error (great circle),
;       elerr                  :measured el error. from 
;
;the 2d cross fits compute the error (gc) as:
;       FitPosPk= azReq + azErrGc .. where error is gc
;  so you add the error to the requestedposition to get the correct (peak) position
;Output:
;       model,
;       model residuals:  great circle
; -----------------------------------------------------------
;   What is being measured and what is being computed.
; -----------------------------------------------------------
;The input values from the 2d fits:
; let AzPk, elPk be the az,el that will cause you to point at the source
;
; azPk = azReq+azErrGc .. where azErr is great circle(div by cos(el)
; elPk = elReq +elErr 
; -----------------------------------------------------------
; what the model computes
;  to point at the source you need to move to azPk,elPk
; Let elRefr be the elevation after the refraction correction has been made:
;  elRefr = elReq + dElRefr 
; 
; azPk= azReq   + azMod/cos(elPr)   .. to go great cird to little circ
; elPk= elPr              + elMod  .. since model fits el after refraction correction.
;     = (ElReq + dElRefr) + elMod
;     so add mod az, mod el + refrCor to azreq,elReq
; -----------------------------------------------------------
; mapping azErr, elErr to model values to fit
;         model                      2d fits
; azPk=(azReq + azMod/cos(elPr)  = (azReq + azErrGc)
; -->            azmod           =  azErrGc
; elPk=(elReq + dElRefr) + elMod = (elReq + elErr)
; --            dElRefr  + elMod =  elErr
; -->           elMod            =  elErr - dElRefr .. so remove refraction correction.
; -----------------------------------------------------------
; model: (marks params P1..P9, indices I0..I8,  P6/I5 not used
;The model is:
;azM= P1        + P2*cos(elP) + P3*sin(elP) + P4*sin(elP)*cos(az) + P5*sin(elP)*sin(az)
;elM=-P4*sin(az)+ P5*cos(az)  + P7          + P8*cos(elP)         + P9*cot(elP)
;
;ind pNum  azCoef    
;0   p1    offset  
;1   p2    cosEl     
;2   p3    sinEl
;3   p4    sinEl*cosAz
;4   p5    sinEl*sinA   
;
;ind pNum  elCoef    
;0   p4    sinAz 
;1   p5    cosAz     
;2   p7    offset
;3   p8    cosEl
;4   p9    cotEl
; ------------------------------------------------
;The residuals are defined as:
;  res=  errStart - errFit 
;    so a positive residual --> the fits is too low.
;    If you try to compute the correct position from the residual, You need to add
;    the el residual to the elcorrected for refraction.
; ------------------------------------------------
;We try fitting everything at once rather than azerr,elerr separately
; This ends up being a non linear fit. what we do
; 1. do linear fit to azerrs
; 2. to linear fit to el errs
; 3. avge p4,p5 (the common params)
; 4. use the above params as the starting value vor curvefit
;
;   Model params are 0..8
;     0..4 az  , 3,4,6,7,8 el
; to fit:
; 0  = (azErr - modfit)  + (elErr 0 modfit)
;    modFitI.
;NOTES:
;  25feb22 - switched to be p12mfitmod. now the version to use
;            older version that fit az,el separately in p12mfitmod.pro.old
;-
; ----------------------------------------
; fit just az params
;
function fitmodAz_func,x,m
	common fitmodcom,azRd,elRefrRd,azerrAsec,elerrAsec

;
;   compute the refracted el.
;   that is what the model fits
;
    i=long(x+.5d)
    az=azRd[i]
    el=elRefrRd[i]
    sinEl=sin(el)
    cosEl=cos(el)
    cosAz=cos(az)
    sinAz=sin(az)
    return,[[1.D]       , $
            [cosEl]     , $
            [sinEl]     ,$
            [sinEl*cosAz],$
            [sinEl*sinAz]]
end
; ----------------------------------------
; fit just el params
; 
function fitmodEl_func,x,m
	common fitmodcom,azRd,elRefrRd,azerrAsec,elerrAsec

    i=long(x+.5d)
    az=azRd[i]
    el=elRefrRd[i]
    cosEl=cos(el)
    cosAz=cos(az)
    sinAz=sin(az)
    cotEl=1d/tan(el)
    return,[    [-sinAz] , $
                [cosAz], $
                [1.D]   ,$
                [cosEl] ,$
                [cotEl]]
end
; ----------------------------------------
; fit az,el together
;
pro fitmodazel_func,x,A,f,pder
	common fitmodcom,azRd,elRefrRd,azerrAsec,elerrAsec
;
;fazM= P1        + P2*cos(elP) + P3*sin(elP) + P4*sin(elP)*cos(az) + P5*sin(elP)*sin(az)
;felM=-P4*sin(az)+ P5*cos(az)  + P7          + P8*cos(elP)         + P9*cot(elP)
;     0  1  2  3  4  5 6  7
; a=[p1,p2,p3,p4,p5,p7,p8,p9]
;-->Note that we are fitting to Asecs
;  
	ii=long(x+.5d)
	az=azRd[ii]
	el=elRefrRd[ii]
	azErr=azErrAsec[ii]
	elErr=elErrAsec[ii]
	sinEl=sin(el)
	cosEl=cos(el)
	cosAz=cos(az)
	sinAz=sin(az)
	cotEl=1d/tan(el)
	maz= (a[0] + a[1]*cosEl + a[2]*sinEl + a[3]*sinEl*cosAz + a[4]*sinEl*sinAz)
	mel= (-a[3]*sinaz + a[4]*cosaz + a[5]  + a[6]*cosEl + a[7]*cotel)
	f  = (azErrAsec - maz)^2 + (elErrAsec - mel)^2
	return
end
;
;
function p12mfitmod,az,el,azErrInpAsec,elErrInpAsec,modFitI,resAzAsec,resElAsec,$
		yfit=yfit
;  az,refracted el in radians
	common fitmodcom,azRd,elRefrRd,azErrAsec,elErrAsec
;
;fazM= P1        + P2*cos(elP) + P3*sin(elP) + P4*sin(elP)*cos(az) + P5*sin(elP)*sin(az)
;felM=-P4*sin(az)+ P5*cos(az)  + P7          + P8*cos(elP)         + P9*cot(elP)
;     0  1  2  3  4  5 6  7
; a=[p1,p2,p3,p4,p5,p7,p8,p9]
;
;order
;linear fit az for az vales
;linear fit el for el values
; average common params
; use az,el params for starting values for full fit
;
	ddtor=!dpi/180d
	dToAsec=3600d
	npnts=n_elements(az)
	numCoef=8
;--> be carefull moving these coef to the model struct 
;    We have 8 coef (p6 is not used and exclueded here
;    in the model eval p6 is included (but ignored)
	modFitI={$ 
	      coefAzEl:dblarr(5,2),$; p1,p2,p3,p4,p5
;                                 p4,p5,p7,p8,p9
	      coefAll:dblarr(8),$; p1,p2,p3,p4,p5,p7,p8,p9
	       npnts : 0L     ,$; number az,el points for fit
	       chisqAzEl:dblarr(2),$; for az,el fit
	       coefSigazel:dblarr(5,2),$; az,el ..Asecs
		   rmsModazel: dblarr(2),$; asecs az,el
		   meanModazel: dblarr(2),$;az,el should be 0 asecs
		   chisq  : 0d,$; all at once
		   coefSig:dblarr(8),$; asecs
		   rmsModAll:dblarr(2),    $;az,el asecs
		   meanModAll:dblArr(2)}    ;az,el asecs

	modFitI.npnts=npnts
	modFitI.coefAzEl*=0d
	modFitI.coefAll*=0d
;       do az strip first
 	azrd=az*ddtor
	elRefr= p12mrefract(el)
	elRefrRd= elRefr*ddtor
	elErrNoRefrAsec= elErrInpAsec - (elRefr - el)*DtoAsec
	x=dindgen(npnts)
	azErrAsec=azErrinpAsec
	elErrAsec=elErrNoRefrAsec
	cosEl=cos(elRefrRd)
	sinEl=sin(elRefrRd)
;
;	linear azimuth errors fit
;
	numcoef=5
	coefAz=svdfit(x,azErrAsec,numCoef,chisq=chisq,sigma=sigma, singular=singular,$
			yfit=yfitaz,/double,function_name='fitmodAz_func')
	if singular ne 0 then begin
		print,'Warning..azfit had', singular,' singular coefficeints'
	endif
	modFitI.coefAzEl[*,0]=coefAz
	modFitI.chisqAzEl[0]=chisq
    modFitI.coefSigazel[*,0]= sigma
;	resAzAsec=(azErrAsec - yfitAz)
;	a=moment(resAzAsec,sdev=stdev)
;	modFitI.rmsmodazel[0]= stdev
;
;	el angle error fit
	coefEl=svdfit(x,elErrAsec,numCoef,chisq=chisq,sigma=sigma,singular=singular,$
				   yfit=yfitEl,/double,function_name='fitmodEl_func')
	if singular ne 0 then begin
		print,'Warning..azfit had', singular,' singular coefficeints'
	endif
	modFitI.coefAzEl[*,1]=coefEl
	modFitI.chisqAzEl[1]=chisq
    modFitI.coefSigAzEl[*,1]= sigma
;	resElAsec=ElErrAsec - yfitEl
;	a=moment(resElAsec,sdev=stdev)
;	modFitI.rmsmodazel[1]= stdev
;
;  now fit all params together using starting values of linear fits
	w=dblarr(npnts) + 1d
	y=dblarr(npnts) ; function should eval to 0.
	params=dblarr(8)
	params[0]=modFitI.coefAzEl[0,0]     ;P1  az constant
	params[1]=modFitI.coefAzEl[1,0]     ;P2*cosE
	params[2]=modFitI.coefAzEl[2,0]     ;P3*sinE
	params[3]=(modFitI.coefAzEl[3,0] + modFitI.coefAzEl[0,1])*.5 ;P4*sineEl*cosAz,el:-sinaz
	params[4]=(modFitI.coefAzEl[4,0] + modFitI.coefAzEl[1,1])*.5 ;P5
	params[5]= modFitI.coefAzEl[2,1]                          ;P7
	params[6]= modFitI.coefAzEl[3,1]                          ;P8
	params[7]= modFitI.coefAzEl[4,1]                          ;P9
	paramsStart=params										  ; save for later
;
; now do non linear fit
;
	yfit=curvefitpp(x,y,w,params,sigma,function_name='fitmodazel_func',/noderivative,$
	           chisq=chisq,trouble=trouble)
	if trouble ne 0 then begin
		print,"--> curvefitpp for az,el together returned trouble:",trouble
		return,-1
	endif
	modFitI.coefAll=params
	modFitI.chisq=chisq
	modFitI.coefSig=sigma  ; garbage si
;
;  compute rms for both types of fits
;  for az,el separate we need to recompute using the average for p4,p5
;  Using the updated params evaluated the model and take difference from
;  the input errors
;  -->fitall
	modI={p12mmodeldata}
	modI.type=1
	modI.nargs=9
	modI.coefAsec[0:4]=params[0:4]
	modI.coefAsec[5]=0.
	modI.coefAsec[6:8]=params[5:*]
	p12mmodeval,az,el,modI,azCorMAsec,elCorMAsec
	resAzAsec =azErrInpAsec - azCorMAsec
	resElAsec =elErrInpAsec - elCorMAsec
	a=rms(resAzAsec,/quiet)
	modFitI.meanModAll[0]=a[0];  ; azres imean value
	modFitI.rmsModAll[0]=a[1] ;  ; rms
	a=rms(resElAsec,/quiet)
	modFitI.meanModAll[1]=a[0];  ; azres imean value
	modFitI.rmsModAll[1]=a[1] ;  ; rms
;
;   fitaz,el
;
	modI.type=1
    modI.nargs=9
	modI.coefAsec[0:4]=paramsStart[0:4]
	modI.coefAsec[5]=0.
	modI.coefAsec[6:8]=paramsStart[5:*]
    p12mmodeval,az,el,modI,azCorMAsec,elCorMAsec
    resAzAsecAzel =azErrInpAsec - azCorMAsec
    resElAsecAzEl =elErrInpAsec - elCorMAsec
    a=rms(resAzAsecAzEl,/quiet)
    modFitI.meanModazel[0]=a[0];  ; azres imean value
    modFitI.rmsModazel[0]=a[1] ;  ; rms
    a=rms(resElAsecAzEl,/quiet)
    modFitI.meanModazel[1]=a[0];  ; azres imean value
    modFitI.rmsModazel[1]=a[1] ;  ; rms

	return,0
end
