;+
;NAME:
;p12mfittsys - fit to tsys vs elevation data
;SYNTAX: istat=p12mfittsys(tsys,el,fitI,weights=weights,yfit=yfit,$
;                          tol=tol,itmax=itmax,Tatm=Tatm
;ARGS:
; tsys[n]: dbl/fltarr   tsys data to fit
; el[n]  : dbl/fltarr   elevation values for each tsys value
;KEYWORDS:
;weights[n]:  dblarr weights for each Tsys value passed to curvefit() routine.
;                     def:1. 
;tol       :  double  tolerance for fit convertion. def: .001
;itmax     : integer  maximum interation value for fit. def:40
;Tatm      : double   fixed Tatm value to use in fit. Def:270 K
;RETURNS;
;istat     : int      1 -ok
;                    -1  fit error.
;fitI      : {}      structure holding fitinfo .. see below
;yfit[n]   : fltarr  fit evaluated at input points.
;DESCRIPTION:
;	Fit tsys data to elevation. The input Tsys should aleady be in deg K.
;The routine fits:
; Tsys=A[0] + Tatm*(1. - exp(-A[1]*secza)) + Tcmb*exp(-A[1]*secza)
; coef to fit:
; A[0]:     Tsys constant value
; A[1]:     opacity
;
;Constants used by the fit:
;  Tatm: atm temp to use. default is 270K
;  Tcmb: 2.725   cmb temp
; 
; secza=1/cos(za) = 1/sin(el)
; 
; the returned fitI struct contains:
; help,fiti,/st
;** Structure <190aed8>, 8 tags, length=72, data length=66, refs=1:
; NPNTS     LONG   1375        ; number of points used to fit
; TCMB      FLOAT  2.72500     ; cmb temp used:
; TATM      DOUBLE 270.00000a  ; tatm value used
; COEF      DOUBLE Array[2]    ; coef from fit [Tconsta,tau] 
; SIGCOEF   DOUBLE Array[2]    ; sigmas for coef
; SIGFIT    DOUBLE 0.13066463  ; sigma for fit
; CHISQ     DOUBLE 0.017085681 ; chisq (not meaningful unless you supply valid weights
; ITER      INT    4           ; number of times the curvefit() routine interated 
;-
pro p12mfittsysfunc,el,a,f,pder
	common p12mfittsys ,tatmCom
;
	Tcmb=2.725d
	Tatm=tatmCom
	ddtor=!dpi/180d
	secza=1d/sin(el*ddtor)
	vexp=exp(-a[1]*secza)
    f=a[0] + Tatm*(1d - vexp) + Tcmb*vexp
    if n_params() ge 4 then begin
        pder=fltarr(n_elements(el),2)
        pder[*,0]=1.                ;  d/dconstant
        pder[*,1]=(Tatm -Tcmb)*vexp*secza; d/dtau
    endif
    return
end
function p12mfittsys,tsys,el,fitI,weights=weights,yfit=yfit,$
			tol=tol,itmax=itmax ,est=est,Tatm=Tatm

	common p12mfittsys ,tatmCom

	one=1d
	n=n_elements(tsys)
	weightsl=dblarr(n) + one
	if n_elements(weights) eq n then weightsL=weights
    if n_elements(tol)   eq 0 then tol=(1e-3)*one
    if n_elements(itmax) eq 0 then itmax=40
    if n_elements(sigma) eq 0 then sigma=0.
    if n_elements(tatm) eq 0 then tatm=270d
  	a=dblarr(2)
	a[0]=median(tsys)
	a[1]=.02d
    tatmCom=Tatm
	if n_elements(est) eq 2 then a=est
	fitNm='p12mfittsysfunc'
	yfit=curvefit(el,tsys,weightsl,a,sigma,chisq=chisq,$
              /noderiv,$
; 			   covar=covar,$
               function_name=fitNm,iter=iter,tol=tol,itmax=itmax)
	 aa=rms(tsys-yfit,/quiet)
	 fitI={ npnts: n_elements(el),$
		    Tcmb:2.725 ,$
		    Tatm:tatmCom,$
           coef : a,$
           sigCoef:sigma,$
           sigFit:aa[1],$
           chisq:chisq,$
           iter :iter} ; interations
	return,1
end
