;+
;NAME:
;p12minpday - read p12m log data for the requested day
;SYNTAX: nrecs=p12minpday(yymmdd,d,dcd=dcd)
;ARGS: 
; yymmdd : long  data to input yymmdd or yyyymmdd
;KEYWORDS:
;   dcd   :       if set then return structs with the device, programs words
;                 decoded (takes more time). if not supplied then the status words come
;                 back as unsigned int32. 
;RETURNS:
;nrecs: long	> 0 number of records returned
;                 0 : hit eof
;              <  0 : error
;                 -1  filename does not exist
;                 -2  file exists but no data
;d[nrecs]:{p12mLogI}   array of structs holding log info for each second of day
;
;Terminology
; p12m  : patriot 12meter antenna system
;
;DESCRIPTION:
;	The p12mProg runs on a linux computer (currently galfas2). It has 3 components:
; - talk to the outside world taking requests and returning queries.
; - loop driving the p12m telescope. On each 1 second tick it will read the p12m state
; - log info to a disc file and broadcast on the net. This is mainly the 1 second tick data read 
;   from the p12m as well as some p12mProg state info
;	The log data is written in binary with filenames of /share/phildat/logdata_yyyymmdd.dat where the yyyymmdd
;	date changes each day.
;	This routine will read in the logfile for a particular data.
;The data is returned to the caller as an array of structures. The structure contains data convering:
;  - the program p12MProg that is talking to the 12meter amplifiers, status, what is is trying to do
;  - data from the 12meter amplifier controllers. 
;There are timestamps, locations, and a number of 32bit status words. By default the status words
;are returned as 32Bit integers (which means you need to decode them yourself:). If you include the
;/dcd keyword then each status word will be decoded into a structure holding name,values for the various bits
;(this takes a little longer. If you only want the locations, times then you can skip it).
;The routione p12mdcdstat() will decode the status words if you input the binary format.
;
;	The returned structure (with the /dcd set) is:
;Notes:
;  tm : these are normally returned as unix seconds since 1970. They are normally doubles with 
;       sub second resolution.
;     : if you want to convert the secs from 1970 to human readable:
;       - print,systime(0,d[10230].tickTmisec) will give (only 1 element at a time)
;            Wed Jan 13 02:50:30 2021 
;       - convert to hhmmss,yymmdd
;         [hhmmss,yyyymmdd]=fromsecs1970(secs1970,utc=utc,daynoyr=daynoyr,str=str)
;       - to convert an entire day to secMidnite:
;          secMid=d.tickmtmisec - d[0].ticktmisec)
;          then add on secmidnite for 1st entry
;         [hhmmss0,yyyymmdd0]=fromsecs1970(secs1970,utc=utc,daynoyr=daynoyr,str=str)
;          secMid0=hms1_secmid(hhmmss0)
; 	       secMid+=secmid0	
;
;** Structure P12MLOGI, 17 tags, length=400, data length=392:
;   CPUTMATWAITTICK DOUBLE  1.6105104e+09  :When p12mProg start to wait for the tick before read:
;                                           Normally 100ms before tick
;   CPUTMATTICK     DOUBLE  1.6105104e+09  :Time when program started read (after tick)
;   DURRDDEV        DOUBLE  0.021378517    : how long it took to read the device 
;   DURWRLAST       DOUBLE  6.6757202e-06  ; how long the last write to device took
;   ST              STRUCT  -> P12MSTBLK Array[1]: struct holding status read from p12m
;                                            look here for the telescope position, stat,etc
;   TICKTMISEC      LONG64  1610510400     : integer seconds from 1970 for this tick
;   PRSTATE         STRUCT  -> P12MPRSTWD Array[1]; program status for p12mProg
;   NUMIOTHRDS      LONG    1              : how many threads are talking to outside world (5 max)
;   FRLISTFRBUFS    LONG    1024           : free list # of free buffers for p12mprog to use
;   NDEVCONNECTOK   LONG    4              : # of times p12mProg has connected to p12m
;                                            It only tries a reconnect when the i/o fails..typically
;                                            someone turns off power when working on telescope.
;   NDEVCONNECTFAIL LONG    1              : # of times p12mProg failed on a connection attempt
;   TRARFREEPNTS    LONG    0              : # of free points in the track array
;   PL              STRUCT  -> P12MPLDAT Array[1]: what p12mProg has requested p12m to do
;   AZERRD          FLOAT   0.00000        : pointing error (radians) in azimuth (if tracking)(little circle?)
;   ELERRD          FLOAT   0.00000        : pointing error (radians) in elevation ( if tracking)
;   GCERRD          FLOAT   0.00000        : great circle pointing error (if tracking).
;   FILL            FLOAT   0.00000        : filler
;
; The p12mStblk is :
;DL> help,d.st,/st
;** Structure P12MSTBLK, 11 tags, length=184, data length=180:
;   MJD             DOUBLE 59227.167        :modified julian date for tick 
;   STWDS           STRUCT -> P12MDEVSTWDS Array[1] :structs hold status words
;   AZPOS_D         DOUBLE -179.99740       : az position degrees
;   AZERR_D         DOUBLE -0.0025000000    : az error degrees
;   AZFDBACKVEL_DS  DOUBLE 0.0000000        : az fdback velocit deg/sec
;   AZMOTCUR_A      DOUBLE 0.0000000        : az master motor current (amps)
;   AZSLMOTCUR_A    DOUBLE 0.0000000        : az slave motor current (amps)
;   ELPOS_D         DOUBLE 20.002200        : elevation encoder degrees
;   ELERR_D         DOUBLE -0.0023000000    : elevation error 
;   ELFDBACKVEL_DS  DOUBLE  0.00030000000   : elevatin fdbackvel deg/sec
;   ELMOTCUR_A      DOUBLE  0.0000000       : elevatio motor current
; -----------------------
;There are 4 status words from the p12m: az master,az slave, elevation ,and central
; DL> help,d.st.stwds,/st
;** Structure P12MDEVSTWDS, 4 tags, length=100, data length=100:
;   AZM             STRUCT    -> P12MSTAZM Array[1]
;  AZSL            STRUCT    -> P12MSTAZSL Array[1]
;  EL              STRUCT    -> P12MSTEL Array[1]
;  CEN             STRUCT    -> P12MSTCEN Array[1]
; -----------------------
; az master Status word:
;IDL> help,d.st.stwds.azm,/st
;** Structure P12MSTAZM, 25 tags, length=28, data length=28:
;   STATUS32        ULONG    4194315 ; before decoding
;   HIGHHARDLIM     BYTE     0
;   LOWHARDLIM      BYTE     0
;   HIGHSOFTLIM     BYTE     0
;   LOWSOFTLIM      BYTE     0
;   MOTSHAFTBRAKEON BYTE     1
;   LOCAL           BYTE     0
;   DRIVENOTENERGIZED BYTE   1
;   DRIVENOTHEALTHY BYTE     1
;   TURNCOUNTSERR   BYTE     0
;   POSDEMANDLIMHIGH BYTE    0
;   POSDEMANDLIMLOW BYTE     0
;   NODIGLOCKVIRTAXIS BYTE   0
;   OFFSETPROFNOTATSPEED BYTE 0
;   OFFSETPROFNOTATPOS BYTE  0
;   MAINPROFNOTATSPEED BYTE  0
;   MAINPROFNOTATPOS BYTE    0
;   FREE2_7         BYTE     0
;   MOTBRAKEON      BYTE     1
;   MOTBRAKEALARM   BYTE     0
;   FREE2_4         BYTE     0
;   VIRTAXISSPEEDDEMANDLIM BYTE 0
;   FREE_2_2        BYTE     0
;   RUNNOTPERMITTED BYTE     0
;   DRIVENOTENABLED BYTE     0
; -----------------------
; az slave Status word:
;IDL> help,d.st.stwds.azsl,/st
;** Structure P12MSTAZSL, 9 tags, length=12, data length=12:
;   STATUS32        ULONG 19 ; before decoding
;   FREE0_7         BYTE   0
;   FREE0_6         BYTE   0
;   FREE0_5         BYTE   0
;   MOTBRAKEON      BYTE   1
;   MOTBRAKEALARM   BYTE   0
;   DRIVENOTENABLED BYTE   0
;   DRIVENOTENERGIZED BYTE 1
;   DRIVENOTHEALTHY BYTE   1
; -----------------------
; elevation  Status word:
;IDL> help,d.st.stwds.el,/st
;** Structure P12MSTEL, 25 tags, length=28, data length=28:
;   STATUS32        ULONG    4194315
;   HIGHHARDLIM     BYTE     0
;   LOWHARDLIM      BYTE     0
;   HIGHSOFTLIM     BYTE     0
;   LOWSOFTLIM      BYTE     0
;   MOTSHAFTBRAKEON BYTE     1
;   LOCAL           BYTE     0
;   DRIVENOTENERGIZED BYTE   1
;   DRIVENOTHEALTHY BYTE     1
;   FREE1_7         BYTE     0
;   POSDEMANDLIMHIGH BYTE    0
;   POSDEMANDLIMLOW BYTE     0
;   NODIGLOCKVIRTAXIS BYTE   0
;   OFFSETPROFNOTATSPEED BYTE 0
;   OFFSETPROFNOTATPOS BYTE  0
;   MAINPROFNOTATSPEED BYTE  0
;   MAINPROFNOTATPOS BYTE    0
;   FREE2_7         BYTE     0
;   MOTBRAKEON      BYTE     1
;   MOTBRAKEALARM   BYTE     0
;   MAINBRAKEALARM  BYTE     0
;   VIRTAXISSPEEDDEMANDLIM BYTE 0
;   MAINBRAKEON     BYTE     0
;   RUNNOTPERMITTED BYTE     0
;   DRIVENOTENABLED BYTE     0
; -----------------------
; central  Status word:
;IDL> help,d.st.stwds.cen,/st
;** Structure P12MSTCEN, 29 tags, length=32, data length=32:
;   STATUS32        ULONG        808742917
;   AUTOSTOWINPACTIVE BYTE    0
;   TRKARRAYLESS10PNTS BYTE   0
;   TMOUT30SEC      BYTE      0
;   SNTPRESPONSESLOW BYTE     0
;   SYSCLKNOTINITIALIZED BYTE 0
;   SUMOPERATENOTREADY BYTE   1
;   SUMNOTREMOTE    BYTE      0
;   PWR3PHASEOFF    BYTE      1
;   AZTRKSTARTOUTOFRANGE BYTE 0
;   AZTRKSTARTTURN  BYTE      3
;   TMOUT30SECDISABLED BYTE   1
;   RADECTRKINGDATA BYTE      0
;   TRKCOORDEQUAT   BYTE      0
;   STOWINPROGRESS  BYTE      0
;   CURRUNMODE      BYTE      0
;   AZSLVONLINE     BYTE      1
;   AZMASTERONLINE  BYTE      1
;   FREE2_3         BYTE      0
;   ELONLINE        BYTE      1
;   FREE2_1         BYTE      0
;   TRKARRREINIT    BYTE      0
;   CONNECTFILTERON BYTE      0
;   FREE3_6         BYTE      0
;   CORRECTIONSDISA BYTE      3
;   CURAZELOFFSETMODE BYTE    0
;   CURRADECOFFSETMODE BYTE   0
;   FREE3_1         BYTE      0
;   CUROFFSETMODE   BYTE      0
;-
function  p12minpday,yymmdd,logI,dcd=dcd
;
;   make the filename
;
	retStat=0l
	idir='/share/phildat/p12m/'
	yymmddl=yymmdd
	if yymmddl lt 999999 then yymmddl+= 20000000l
	fname=idir + string(format='("logdata_",i08)',yymmddl) + ".dat" 
	lun=-1
	iostat=0
	openr,lun,fname,/get_lun,err=iostat
	if iostat ne 0 then begin
		print,!ERROR_STATE.MSG 
		retstat=-1
		goto,done
	endif
	fs=fstat(lun)
	recsize=n_tags({p12mLogIU},/len)
	nrecs=fs.size/recsize
	if nrecs eq 0 then begin
		retstat=-2
		goto,done
	endif
	on_ioerror,ioErr
	dodcd=keyword_set(dcd)
	logIU=replicate({p12mLogIU},nrecs)
	logIU.cpuTmAtWaitTick=-1.
	readu,lun,logIU
ioErr:
	; use 0 with -1 match. last rec still had 0.
	ii=where(logiU.cpuTmAtWaitTick gt 0.,nread)
	if nread ne nrecs then begin
		if nread gt 0 then begin
			logIU=logIU[ii]
		endif else begin
			logIU=''
		endelse
		nrecs=nread
	endif
	if not dodcd then begin
		logI=logIU
		return,nrecs
	endif
	logI=replicate({p12mLogI},nrecs)
	;// copy and decode
	struct_assign,logIU,logI
	;// now need to decode
	logI.st.stwds=p12mdcdstat(logIU.st.stwds)
	;// decode prState
	logI.prState=p12mdcdprstwd(logIU.prState)
	retstat=nrecs
done:
	if lun ne -1 then free_lun,lun
	return,retstat
end
