;+ 
;NAME:
;psrfhwcal - input and process hardware winking cal file.
;
;SYNTAX: stat=psrfhwcal(fname,calI,fnmI=fnmI,blankcor=blankcor,edge=edge,$
;                     hdr=hdr,tpar=tpar)
;
;ARGS:
;    fname: string	name of file to use (unless fnmI supplied
;KEYWORDS:
;	fnmI:  {}       structure from masfilelist for file to use
;					In this case ignore fname
;blankcor:          If set the correct for adc blanking
;edge[2] :float     fraction of spectra on each edge to ignore when computing
;                   tp. if only 1 dimension. then apply this fracct on both edges.
;                   def:0.
;RETURNS:
;  calI       : holds scale factor   calDefl/masCnts .. multiply by this to tget
;                   power in cal units. then mult by calInKelvins to get kelvins
;                    used in total power sum).
; hdr         : struct primary and extension headers
; tpar[n,npol]; float  .. the raw total power samples
;
; stat:   =1 ok
;      < 0  error
;
;DESCRIPTION:
;   Read in a hardware winking cal file and compute the scale factors. If the edge= keyword is present
;then exclude this fraction of freq channels from each edge of the spectra when computing the total
;power.
;	Return the scale factors in calI (see below). If hdr= is present then also return the file headers.
;If tpAr= is present then return the raw total power sampled points where n is samples in the file.
;
;  The calI struct contains:
;   calI.NPOL   LONG           2      ; number of pols 
;   calI.CALSCL FLOAT     Array[2]    ; 1./(calOn-caloff) ..
;   calI.EDGE   FLOAT     Array[2]    ; fraction of bp on each side not used when compute tp.
; Multiplying calI.calscl  will convert from spectrometer counts to calunits.
; If you then multiply by Tcal you'll get kelvins.
; 
;	We cheat a bit when looking for the calon and  cal off samples.
;1. compute the total power for all samples
;2. compute the averge value
;3. let calH=the median of all points above the avg
;4.     calL=the median of all points beloe the avg.
; this is to get around sampled points during the transition.
; instead we could have folded the data into bins that span the 40ms calon,off.
;
; Notes:
;  -you should be consistant in the use of the edge= keyword. If you use it for the cal,
;   then you should also use it when processing pulsar data total power.
;  - This will probably not work for stokes data.. 
;-
;
function psrfhwcal,fname,calI,fnmI=fnmI,hdr=hdr,blankcor=blankcor,$
				 edge=edge,tpar=tpar
;
;;    on_error,2
	useEdge=n_elements(edge) ge 1
	n=psrftpfile(fname,tpAr,fnmI=fnmI,hdr=hdr,blankcor=blankcor,edge=edge)
	if n lt 0 then begin
		print,"Err:psrftpfile returned stat:",n
		return,-1
	endif
;
	edgeL=[0.,0.]
	if useEdge then begin
		edgeL=[edge[0],edge[0]]
		if n_elements(edge) gt 1 then edgeL[1]=edge[1]
	endif
	npol=hdr.hpri.nrcvr
	if npol eq 1 then begin
		meanVal=mean(tpar)
	    calI={ npol:npol,$
               calScl:0.,$
               edge  :edgeL}
	endif else begin
		meanVal=fltarr(2)
		meanVal[0]=mean(tpar[*,0])
		meanVal[1]=mean(tpar[*,1])
	    calI={ npol:npol,$
               calScl:fltarr(2),$
               edge  :edgeL }
	endelse
	for ipol=0,npol-1 do begin
		iiH=where(tpAr[*,ipol] gt meanVal[ipol],cntH)
		iiL=where(tpAr[*,ipol] lt meanVal[ipol],cntl)
		cali.calScl[ipol]=1./(median(tpAr[iiH,ipol]) - median(tpAr[iiL,ipol]))
	endfor
	return,1
end
