;+ 
;NAME:
;psrftpfile - compute total power for file
;
;SYNTAX: n=psrftpfile(fname,tp,fnmI=fnmI,hdr=hdr,maxrows=maxrows,blankcor=blankcor,
;                      edge=edge)
;
;ARGS:
;    fname: string	name of file
;KEYWORDS:
;maxrows: long		max number of rows to input
;	fnmI:  {}       structure from masfilelist for file to use
;					In this case ignore fname
;blankcor:          If set the correct for adc blanking
;edge[2] :float     fraction of spectra on each edge to ignore when computing
;                   tp. if only 1 dimension. then apply this fracct on both edges.
;                   def:0.
;RETURNS:
;  tp[n,npols]: float total power. returned as mean value of spectra (we dividedby number of chan
;                    used in total power sum).
; hdr         : struct primary and selected extension headers
;
; n:   >=0 number of sample points returned
;      < 0  error
;
;DESCRIPTION:
;   Read in a file and compute the total power. If edge=[] is suppied then 
;exclude this fraction of freq chan on each side of the bandpass.
;eg . egde=[.06,.08]. If 256 channels then 15,20 channels will be excluded from the left,right
;of the spectra when computing tp.
;
;Mods:
; 28dec21: added edge=keyword to exlude channels on edge when computing total power.
;          - also switched tp to return the mean values of the spectra. Needed
;            when edge= added.
;-
;
function psrftpfile,fname,tp,hdr=hdr,fnmI=fnmI,verb=verb,maxrows=maxrows,blankcor=blankcor,$
				 edge=edge
;
;;    on_error,2
	useEdge=n_elements(edge) ge 1
	openOk=0
	if (psrfopen(fname,desc,fnmI=fnmI) ne 0) then begin
			print,"Error opening file"
			return,-1
	endif
	blankcorL=keyword_set(blankcor)?1:0
	openOk=1
;
	numRows=keyword_set(maxRows)?maxRows<desc.totRows:desc.totRows
	if numRows lt 1 then begin
		print,"No rows in file"
		goto,errout
	endif
;
;   see if we position before start
;
	nsblk=desc.hsubint.nsblk
	npol=desc.hsubint.npol
	nchan=desc.hsubint.nchan
	if useEdge then begin
		edgeL=(n_elements(edge) eq 1)?[edge,edge]:edge
        ie0=long(edgeL[0]*nchan)
	    ie1=nchan - long(edgeL[1]*nchan) - 1
        nedge=ie1-ie0 + 1
	endif 
	tp=fltarr(nsblk*numrows,npol)
	icur=0L
	for irow=0,numrows-1 do begin
		istat=psrfget(desc,b,blankcor=blankcor)
	 	if keyword_set(verb) then print,irow 
		if istat ne 1 then break
		if (icur eq 0 ) and (arg_present(hdr)) then begin
			hdr={hpri: desc.hpri,$
			     hsubint: desc.hsubint,$
			     haogen: desc.haogen,$
				 azSt: b.tel_az,$
				 zaSt: b.tel_zen}
		endif
		if npol eq 1 then begin
			if useEdge then begin
				tp[icur:icur+nsblk-1]=total(b.data[ie0:ie1],1)
			endif else begin
				tp[icur:icur+nsblk-1]=total(b.data,1)
	        endelse
		endif else begin
			for ipol=0,npol-1 do begin  $
				if useEdge then begin
					tp[icur:icur+nsblk-1,ipol]=reform(total(b.data[ie0:ie1,ipol,*],1),nsblk)
				endif else begin
					tp[icur:icur+nsblk-1,ipol]=reform(total(b.data[*,ipol,*],1),nsblk)
	        	endelse
			endfor
		endelse
		icur+=nsblk
	endfor
	if icur ne (numrows*nsblk) then begin
		if icur eq 0 then begin
			print,"no data found"
			goto,errout
		endif
		tp=(npol eq 1)?reform(tp[0l:icur-1]):reform(tp[0L:icur-1],icur,npol)
	endif
;  scale to averge value in spectra. added when edge keyword added
	scl=(useEdge)?1./nedge:1./nchan
	tp*=scl
	if openOk then psrfclose,desc
	return,icur
errout: 
	if openOk then psrfclose,desc
	return,-1
end
