;+
;NAME:
;rm12mon - monitor the 12m receiver temperatures.
;SYNTAX: rm12mon,yymmdd,temp=temp,volt=volt,cur=cur,pressure=pressure,hepressure=hepressure,$
;               disp=disp,win=win,tlab=tlab,ver=ver,$
;               min=min,delay=delay,hshift=hshift,smo=smo,debug=debug,log=log
;ARGS:
;   yymmdd: long    date to display. -1 --> current day
; 
;KEYWORDS:
;     temp :    int  temp to monitor 1:16K,2:70K,3=casetemp
;     volt :    int  voltage to monitor: 1-gate,2-drain,3-lna drain
;  pressure:    int  if set then monitor vacuum pressure
;  hepressure:    int  if set then monitor helium  pressure (hi,lo)
;  cur     :    int  if set then monitor drain current
;    ver[2]:    float vertical scale to use (min,max). If 1 number then
;                     go 0 to ver.
;     disp :    int  1 ->upper magnified display only
;                    2 ->lower daily display only
;                    default both
;       win:    long window number to display in. This allows the user to
;                    setup the window dimensions and position before
;                    calling this routine. The default is 0. Window 4
;                    is used for the pixwin, so do not use it as the 
;                    display window number.
;  tlab[4]:   float  specify temp label text position
;                    [0] - start line 0..31   def 3
;                    [1] - line step          def 1
;                    [2] - xpos left (0..1)   def  .02
;                    [4] - charsize           def 1.3
;   log   :          if set the make vertical display logarithmic.
; -->keywords for HI RESOLUTION upper display
;      min :    long number of minutes for upper hi res display. default:
;                    60.
;     delay:    long The seconds to wait between each display of the magnified
;                    display. The default is 20 seconds.
;    hshift:    long The number of minutes to leave blank on the right 
;                    side of the magnified display. The default is 1 minute
; -->keywords for DAILY lower display
;       smo:    long the number of points to smooth,decimate the daily display.
;                    The default is 5 or 15  seconds (with 3 sec updates). This is also the update
;                    rate for the daily display.
;     debug:    If set then output some debugging info about the the number
;               of points we read at each i/o.
;DESCRIPTION:
;   rm12mon will monitor the 12m receiver data. The monitor data is 
;being written to disc once every 3 seconds for each receiver. The user can
;select temperature, voltage, current, or pressure.
;
;   rm12mon can be used to display an entire days worth of data by setting
;the parameter yymmdd to the data of interest. If yymmdd is set to -1 then
;the monitoring will display the current day and then continue updating 
;every DELAY seconds. The default delay is about 15  seconds. 
;
;   There are two displays each showing data versus hour of
;day: a high resolution plot and a plot of the entire day. The DISP keyword
;can be used to limit the output to just one of these plots (the default
;is both plots are output). By default the 16K temperautre stage of the amplifiers are
;displayed. 
;
;   The top plot is a high resolution display showing the last MIN minutes
;of data. The dislay will be updated every DELAY seconds. The *'s at the 
;right of the plot are the last data points. 
;   
;   The lower plot is the data versus hour of day for the entire day.
;The data has been smoothed and decimated by SMO seconds. This plot will 
;be updated every SMO sample points.
;
;EXAMPLES:
;   - Plot the temp  data for 02jan23
;   rm12mon,230102
;   - monitor the current day.
;   rm12mon,-1
;   - monitor the current day, smoothing the magnified plot to 9 secs (3 samples) 
;     and displaying 3 hours of data. 
;   rm12mon,-1,smo=3,min=180
;
;NOTE:
;   If you call rm12mon,-1... you will need to ctrl-c to get out of it.
;After this you should do a retall to get back to the main idl procedure.
;-
pro rm12mon,yymmdd,temp=temp,volt=volt,cur=cur,pressure=pressure,min=hrmin,$
		  hepressure=hepressure,$
		  smo=smo,delay=delay,ver=ver,$
          hshift=hshift,debug=debug,axis=axis,win=win,disp=disp,tlab=tlab,$
		  rcvmonstr=rcvmonstr,log=log
;   
;    on_error,1
	common colph,decomposedph,colph
    if not keyword_set(debug) then debug=0
    xdimdef=640
    ydimdef=512
    !x.style=1
    !y.style=1
    wpixwintouse=4
    maxrecs=86400/3 + 100       ; 1 rec every 3 secs
    cs=1.4
    labtemp=['16K','70K','case']
	labVolt=['gate','drain','lna']
	labCur=['mAmps']
    hfract=.17                          ; 10 % used for labels,temp
;
;   label positions
;
    if n_elements(tlab) eq 4 then begin
        tlabln   = tlab[0]
        tlabscl  = tlab[1]
        tlabxp   = tlab[2]
        tlabcs   = tlab[3]
    endif else begin
        tlabln = 3
        tlabscl=1 
        tlabxp =.02
        tlabcs=1.3
    endelse
;
;   default vertical scaling for different temps
;
	ylog=(keyword_set(log))?1:0
;   check if they specified a ver scale
	gotver=1
    case n_elements(ver) of
        1: begin
            if string(ver) eq 'auto' then begin
                ver
            endif else begin
                ver,0,ver
            endelse
           end
        2: begin 
			ver,ver[0],ver[1]
		   end
     else: begin
;       we'll use a default depending on the type
		gotver=0
	     end
     endcase
;

    if n_elements(smo) eq 0 then   smo=5
	dec=smo
    if not keyword_set(hrmin) then hrmin=60.
    if not keyword_set(delay) then delay=20
    if not keyword_set(hshift) then hshift=2
    if not keyword_set(disp) then disp=0
	dotemp=0
	docur=0
	dovolt=0
	dovac=0 
	dohepres=0 
	if ((keyword_set(temp) + keyword_set(volt) + keyword_set(cur) + keyword_set(pressure) + keyword_set(hepressure)) eq 0) then begin
			temp=1
	endif
;   temp
	if (keyword_set(temp)) then begin
		if  (temp lt 1 ) or (temp gt 3) then begin
			print,'temp= keywords values are 1 (16k) or 2 (70k) or 3(case)'
			return
		endif
        verdef   =[[0,30],[0,100],[15,50]] 
        verLogdef=[[5,300],[5,300],[5,50]]
		if (gotver eq 0 ) then begin
			if (ylog eq 0) then begin
				ver,verdef[0,temp-1],verdef[1,temp-1]
			endif else begin
				ver,verLogdef[0,temp-1],verLogdef[1,temp-1]
			endelse
		endif
		dotemp=1
	endif
;   voltage
	if (keyword_set(volt)) then begin
		if ((volt lt 1)	 or (volt gt 3)) then begin
			print,'volt= keywords values are 1 (gate), 2 (drain), or 3(lnadrain)'
			return
		endif
		if keyword_set(log) then begin
			print,'log= keyword not allowed for voltages, ignoring'
			ylog=0
		endif
      	verdef =[[-.2,-.1],[1.5,2.0],[.8,1.3]] 
		if (gotver eq 0 ) then begin
			ver,verdef[0,volt-1],verdef[1,volt-1]
		endif
		dovolt=1
	 endif
;   current
	if (keyword_set(cur)) then begin
		if keyword_set(log) then begin
			print,'log= keyword not allowed for current, ignoring'
			ylog=0
		endif
      	verdef =[20,40.]
		if (gotver eq 0 ) then begin
			ver,verdef[0],verdef[1]
		endif
		docur=1
	 endif
;   vac pressure
	if (keyword_set(pressure)) then begin
      	verdef =[0,.03]
      	verLogdef =[.001,1000]
		if (gotver eq 0) then begin
			if (ylog eq 0) then begin
				ver,verdef[0],verdef[1]
			endif else begin
				ver,verLogdef[0],verLogdef[1]
			endelse
		endif
		dovac=1
	 endif
;   helium pressure
    if (keyword_set(hepressure)) then begin
        verdef =[0,500]
        verLogdef =[.1,1000]
        if (gotver eq 0) then begin
            if (ylog eq 0) then begin
                ver,verdef[0],verdef[1]
            endif else begin
                ver,verLogdef[0],verLogdef[1]
            endelse
        endif
        dohepres=1
     endif

;
;    they specified a window to use??
;
    if n_elements(win) eq 0 then  begin
        win=0
        window,0,xsize=xdimdef,ysize=ydimdef
    endif else begin
        if win eq 4 then begin
            message,$
        'window 4 is use for the internal pixwin,pick another window...'
        endif
        wset,win
    endelse
    xdim=!d.x_size
    ydim=!d.y_size
    wpixwin=-1
    ilast=-1L
    done=0
    totpnts=0
    botcnt=0
;   number of delays before max bot cnt
;
    if delay gt 0 then begin
    	maxbotcnt= long((smo*20)/delay  + .5)     ; how often to update lower plot
    endif else begin
    	maxbotcnt=1
    endelse
    t4=0
    t3=0
;
;   loop here on the data
;
    append=0
    repeat begin
        t1=systime(1)
		if debug then print,"Here 2, t1:",t1
        newpnts=rm12moninp(yymmdd,b,curpos=curpos,inprecsize=inprecsize,$
                         append=append,rcvmonstr=rcvmonstr)
		if debug then print,"Here 3, newpnts:",newpnts
        append=1L
        if yymmdd eq -1 then begin
            a=bin_date()
            yymmddl=string(format='(i2.2,i2.2,i2.2)',a[0] mod 100,a[1],a[2])
        endif else begin
            yymmddl=string(yymmdd)
        endelse
        totpnts=n_elements(b)
        i2=totpnts-1
        if (newpnts eq 0) or (totpnts lt (2)) then begin
            goto,nexttm
        endif
        t2=systime(1)
;
;   setup the pixwin
;
        if (!d.x_size ne xdim) or (!d.y_size ne ydim) or (wpixwin lt 0) $
            then begin
           xdim=!d.x_size
           ydim=!d.y_size
           window,wpixwintouse,/pixmap,xsize=xdim,ysize=ydim
           wpixwin=!d.window
           botcnt=0
        endif
        wset,wpixwin
;     get the data
		if (dotemp) then begin
        	case temp of
           		1: yy1=b.temp2k
            	2: yy1=b.temp1k
            	3: yy1=b.tempcase
        	endcase
			ytit=(temp eq 3)?"DegC":"DegK"
        	lab0=string(format='(a," ",a," dewar temps ")',$
            yymmddl,labtemp[temp-1])
		endif
		if (dovolt) then begin
        	case volt of
           		1: begin
					yy1=b.vgate[0]
					yy2=b.vgate[1]
				   end
            	2: begin
					yy1=b.vdrain[0]
					yy2=b.vdrain[1]
			       end
            	3: begin
					yy1=b.vdrainlna[0]
					yy2=b.vdrainlna[1]
				   end
        	endcase
			ytit='volts'
        	lab0=string(format='(a," ",a," voltage ")',$
            yymmddl,labvolt[volt-1])
		endif
		if (docur) then begin
			yy1=b.idrain[0]
			yy2=b.idrain[1]
        	lab0=string(format='(a," Drain cur ")',$
            yymmddl)
			ytit='milliAmps'
		endif
		if (dovac) then begin
			yy1=b.ptorr
        	lab0=string(format='(a," Vaccum pressure ")',$
            yymmddl)
			ytit='torr'
		endif
		if (dohepres) then begin
			yy1=b.hepress[0]
			yy2=b.hepress[1]
        	lab0=string(format='(a," Helium pressure ")',$
            yymmddl)
			ytit='psi'
		endif
		useyy2=(dovac + dotemp) eq 0 
;
        if (disp eq  1) or (disp eq 2) then begin
            !p.multi=0
        endif else begin
            !p.multi=[0,1,2]
        endelse
        if disp eq 2 then goto,pltlow
;
;       high res points to display
;
        ind=where( b.day ge ( b[i2].day - hrmin/(60.*24)),count)
        if count lt 2 then goto,nexttm
        i1=ind[0]
        tm1=(b[i1:i2].day - fix(b[i1].day)) *24.
        if tm1[0] gt tm1[1] then tm1[0]=tm1[1]

        hmax=max(tm1,min=hmin)
        hmax1=hmax+hshift/60.
        hor,hmin-(hmax1-hmin)*hfract,hmax1
		lab=lab0 +  string(format='(f5.1," minutes")',hrmin)
		ii0=n_elements(tm1)-1
		ii1=n_elements(yy1)-1

        plot,tm1,yy1[i1:i2],charsize=cs,ylog=ylog,$
            xtitle='hour',ytitle=ytit,title=lab
        plots,tm1[ii0],yy1[ii1],psym=2
		if (useyy2) then begin
            oplot,tm1,yy2[i1:i2],col=colph[2]
            plots,tm1[ii0],yy2[ii1],psym=2,col=colph[2]
		endif

pltlow: if debug then print,'botcnt: ',botcnt
        hor
        if (totpnts gt (smo*10) and ((botcnt mod maxbotcnt) eq 0) and $
            (disp ne 1)) then begin
            tm= (b[0:i2].day - fix(b[0].day))*24
            if tm[0] gt tm[1] then tm[0]=tm[1]
            lab1=lab0 + string(format='(" (smoothed to",i4," secs)")',$
                smo*20)

            lab2=lab0 + "(no smoothing)"
            newplot=1
			count=n_elements(b)
            if count gt smo then begin
               if smo gt 1 then begin
                   x=select(smooth(tm,smo,/edge_truncate),dec/2,dec)
                   yy1s=select(smooth(yy1,smo,/edge_truncate),dec/2,dec)
                   if useyy2 then yy2s=select(smooth(yy2,smo,/edge_truncate),dec/2,dec)
                   lab=lab1
               endif else begin
                    x=tm
                    yy1s=yy1
				    if (useyy2) then yy2s=yy2
                    lab=lab2
               endelse
               plot,x,yy1s,charsize=cs,ylog=ylog,$
                   xtitle='hour',ytitle=ytit,title=lab
               if (useyy2) then oplot,x,yy2s,col=colph[2]
            endif
        endif
        wset,win
        if (botcnt  eq 0) or (disp eq 1) or (disp eq 2)  then begin 
            device,copy=[0,0,xdim,ydim,0,0,wpixwin]
        endif else begin
            device,copy=[0,ydim/2-1,xdim,ydim/2,0,ydim/2-1,$
                wpixwin]
        endelse
        t3old=t3
        t3=systime(1)
nexttm:
        botcnt=(botcnt+1) mod maxbotcnt
        if yymmdd ne -1 then goto,done
        if debug then begin
lab=string(format=$
'("new:",i5," tot:",i5," tm1:",f5.3," tm2:",f5.3," tm3:",f5.3," tmt:",f5.3)',$
            newpnts,totpnts,t2-t1,t3-t2,t4-t3old,t3-t1)
            print,lab
        endif
        wait,delay
        t4=systime(1)
    endrep until done
done:
    return
end
