;+
;NAME:
;rm12plot - plot 12mdewar temperatures.
;SYNTAX: rm12plot,d,temp=temp,cur=cur,volt=volt,pressure=pressure,over=over,$
;               sym=sym,autoscale=autoscale,color=col,ln=ln,adate=adate,$
;               drange=drange,mframe=mframe,cs=cs,log=log,labadate=labadate,$
;               nolab=nolab,xtitle=xtitle,title=title,pol=pol,abpolcol=abpolcol,$
;				font=font
;ARGS:   
;   d[]:{rcvmon} array of data input via rminpday.
;KEYWORDS:
;   temp    : int    temperature to plot:
;                    1 16  deg stage (default) (degK) 
;                    2 70  deg stage (degK) 
;                    3  case temperature (degC)
;   cur     : int    if set then plot the drain currents in milliAmps
;   volt    : int    amplifier bias voltage to plot 1-vgate,2=vdrain,3=vdrainlna
;  pressure : int   if set display vaccum pressure (in microTorr).
;   over    : int   if set then overplot rather than start a new plot
;   sym     : int   1,2,3 .. plot a symbol at each measured position.
;                   1 +, 2 *, 3 . , Negative number -1,-2,-3 will plot 
;                   the symbol and the connecting lines.
; autoscale : int   if set the autoscale the y axis to min,max + .03*(max-min)
;  color    : int   if provided then plot the current data set in this color.
;                   colors are: (1-white, 2-red, 3-green,4-blue,5-yellow...
;  ln       : int   line number for rcvr name labels. values 3..33
;  adate    : int   if set then plot using alphanumeric dates: ddMMMyy
;  labadate : string if supplied, then this is the output format
;                   for label_date call:
;                   %M-monName,%N-monNun,%D-dayNo,%Z-yy,%H-hr,%I-min
;  drange[3]: float Date range to plot. Use with /adate to limit the
;                   dates plotted.  format:[year,daynum,Numdays]
;                   keep numdays about year,daynum.
;                   (numdays can be positive or negative)
;                   note that numdays=0 or -1, or 1 keeps just the current day
;  mframe   : int   if set then plot multiple frames on a page with (not yet implemented)
;  cs       : float character size scaling if single plot. default=1
;  log      :       if true then use log for vertical scale.. 
;                   (warning.. don't set lower vertical scale to 0..)
;  nolab    :       if set then don't print receiver names at the 
;                   bottom.. handy if you supply title=
; xtitle    : string Use this for the xtitle. Handy if you want to specify
;                    the dd:hh for labadate
; title     : string Use this for the reciever name
;	 				(but not oplot)
; pol       :  1,2   if present then just plot pola(1) or polB 2
; abpolcol  :        if set  and single frame, then plot always plot polb in red.
;	 				 normally used for 1 rcv/page pola=black,polb=red
; font      : int   1-truetype, 0 regular
;
;DESCRIPTION:
;   Plot the 12m dewar info data versus day of year. The data 
;must be first input with rm12inpday. By default it will plot the 16 degree 
;temperature stage. You can use the keywords to select the following data to
;plot:
; temp: 1,2,3  : T1,T2, case  temperatures
; cur : 
; volt: 1,2,3  : vgate,vdrain,vdrainlna bias voltages
;
;The data can be plotted versus daynumber of the year (default) or
;versus an ascii date ddMonyy (using /adate). 
;
;   The axis can be scaled using hor,min,max and ver,min,max (these are two
;idl functions). Entering hor or ver without any arguements will autoscale to
;the min,max.
;
;   The adate keyword plots the x axis in a more human readable form
;ddMonyy. To plot a subrange of the data with this format, hor,min,max
;will not work (since the x data gets converted to julian date). Use the
;drange[day1st,daylast] keyword to limit the day range in this case.
;
;The /mframe keyword will plot all info on one page (temps,voltages, currents 
;(not yet implemented)
;
;   When using the adate keyword, the tickmark end up being spaced by
;some reasonable number of days (5,10,20..etc).
;
;   When plotting the voltage or currents, both polarizations will
;be plotted (black polA, red polB).
;
;   Making hardcopy requires that you set the destination of the output to
;a postscript file before you call the routine.
;   pscol,'filename.ps'
;   rmplot,d
;   hardcopy
;   x
;The data will then be in the postscript file filename.ps.
;
;   If you used the /mframe keyword, then you don't need color postscript,
;but you will probably want to use the entire page for plotting:
;   ps,'filename.ps',/full
;   rmplot,d,/mframe
;   hardcopy
;   x
;
;EXAMPLES:
;   To start idl:
;   idl
;   @phil
;   @rcvmon12init
;   To exit idl:
;   exit
;   ... 
;nrecs=rm12inpday,230101,d,ndays=6         .. input jan 1 to 6
;hor
;ver,0,50                          .. vertical scale 0 to 50 K
;rm12plot,d                        .. plot 16 deg stage
;rm12plot,d,/adate .. plot  use, ascii dates
;rm12plot,d,temp=2                   .. plot 70 k temps
;
;; plot the gate voltage 
;ver,0,5
;rm12plot,d,volt=1
;
;; overplot 16,70 k stages
;  ver,0,120
;  rm12plot,d,temp=1
;  rmp12lot,d,temp=2,/over
;
;NOTE:
;    The datasets contain 70 Mbytes per month. You should run this on 
;a computer with lots of memory;
;-
;history:
; --> 9de22 need to check whether we need label date offset > 30 days
;    does the tick mark appear to 00: or 12:  
;
pro rm12plot,d,temp=temp,cur=cur,volt=volt,pressure=pressure,over=over,sym=sym,$
	    autoscale=autoscale,$
        color=col, ln=ln,adate=adate,drange=drange,mframe=mframe,cs=cs,$
		log=log,labadate=labadate,xtitle=xtitle,title=title,nolab=nolab,$
		pol=pol,abpolcol=abpolcol,font=font
;
	common colph,decomposedph,colph

	mframe=0 ; not yetimplemented
	useAutoscale=keyword_set(autoscale)
	usextit=keyword_set(xtitle)
;   for titles
	labTemp=['16K Temp','70K Temp','Case TempC']
    labVolt=['Gate Voltage','Drain Voltage','Lna Voltage']
    labCur=['Drain current']
    labVac=['Vacuum pressure']

	csL=1.
	npol=2
	ipol=[0,1]
	if keyword_set(cs) then csL=cs
	if n_elements(pol) eq 1 then begin
		if (pol ne 1) and (pol ne 2) then begin
			print,"pol must be 1 or 2" 
			return;
		endif
		ipol[0]=(pol eq 1)?0:1
		npol=1
	endif
    docur=0
	dovolt=0
    dotemp=0
	dovac=0
    !x.style=1
    !y.style=1
    if n_elements(ln) eq 0 then ln=26
    if n_elements(mframe) eq 0 then mframe=0
    if n_elements(temp) eq 0 then temp=0
	ylog=(keyword_set(log))?1:0
; make sure they only specify one things to plot
;
	val=(keyword_set(temp) + keyword_set(volt) + keyword_set(cur) + keyword_set(pressure))
	if val gt 1 then begin
		print,"--> you can only select one thing to plot: temp,volt,cur, or pressure)"
	    return
	endif
	if val eq 0 then temp=1  ; default 16k temp
    case 1 of
       keyword_set(cur) gt 0: begin
			 if keyword_set(log) then begin
                  print,'log= keyword not allowed for current, ignoring'
                  ylog=0
             endif
            docur=1
            ltit=labCur
            end
       keyword_set(pressure) gt 0: begin
            dovac=1
            ltit=labVac
            end
       keyword_set(volt) gt 0: begin
            indv=volt-1
			if (volt lt 1) or (volt gt 3) then begin
				print,'bad keyword val for volt= ..vals=1,2,3 (vgate,vdrain,vdrainlna)'
				return;
			endif
            ltit=labVolt[indv]
			if keyword_set(log) then begin
            	print,'log= keyword not allowed for voltages, ignoring'
            	ylog=0
			endif
			dovolt=1
            end
        else:begin
			 dotemp=1
		     indT=temp-1
			 if (temp lt 1) or (temp gt 3) then begin
				print,'bad keyword val for temp= ..vals=1,2,3 (16k,70k,caseC)'
				return;
		 	 endif
			 ltit=labTemp[indT]
			end
    endcase
	if usextit then xtitleL=xtitle

    if n_elements(drange) eq 3 then begin
;	   Note: we're treating drange, and the d.day,d.year as
;            gmt times below.It's ok since it's all relative.
;
       x =daynotojul(d.day,d.year)  ; data dayno
       x1=daynotojul(drange[1],drange[0]) ; start of yymmdd1 
       x1=x1[0]
       delta=drange[2]
       if delta lt 0. then begin 
;                                             keep current day
        ind=where((x ge (x1+(delta+1d))) and (x lt (x1 + 1d)),count)
       endif else begin
        ind=where((x ge x1) and (x lt (x1+ (delta+1d))),count)
       endelse
       if count le 0 then begin
            printf,-2,"no data between daynumbers:",drange
            return 
        endif
        dd=d[ind]
    endif else begin
        dd=d
    endelse

    if keyword_set(adate) then begin
		if (n_elements(labadate) gt 0) then begin
			aa=labadate
			if usextit eq 0 then xtiteL="Date"
		endif else begin
			; 
			iday=long(dd.day)
			ndays=n_elements(iday[uniq(iday,sort(iday))])
			if ndays lt 30 then begin
				nn=n_elements(dd)
			    a1=daynotodm(long(dd[0].day),dd[0].year)
				mon1=monname(a1[1]) 
			    a2=daynotodm(long(dd[nn-1].day),dd[nn-1].year)
				mon2=monname(a2[1])
				aa='%D:%H'
				if usextit eq 0 then xtitleL=mon1 + "  " + "DD:HH" + "   " +  mon2
			endif else begin
				aa='%D%M'
				if usextit eq 0 then xtitleL="Date (ddMon)"
			endelse
		endelse
;		labDateOff=(strpos(aa,"H") ne -1)?0.:.5
		labDateOff=(strpos(aa,"H") ne -1)?0.:.0
       	a=label_date(date_format=aa)
        xtformat='label_date'
    endif else begin
        xtformat=''
        if usextit eq 0 then xtitleL='daynumber of year'
    endelse

	if n_elements(title) gt 0 then titleL=title

    xp=.02
    xpinc=.1
    scl=.8
    n=n_elements(d)
    ind=lindgen(n)
;
    if n_elements(sym)  eq 0 then sym=0
    lcol=-1
    if n_elements(col)  ne 0 then lcol=col
;
    if keyword_set(adate) then begin
        x=daynotojul(dd.day,dd.year)  
;       print,'min x:',min(x)
;       print,'max x:',max(x)
;
;      idl plotting with just mon:day puts the major ticks at
;      12 noon (start of julday). We put a .5 offset so the 
;      plot starts at midnite..
;      If our adate has Hours in it, then we don't need this..
        x=x+ labDateOff      
		print,"labDateOff:",labDateOff
     endif else begin
        x=dd.day
    endelse

    if mframe then begin
; not yet implemented
    endif else begin
; ----------------------------------------------------------------
; single frame color
;
        case 1 of
            dotemp: begin
                case temp of
                    1: y=dd.temp2k
                    2: y=dd.temp1k
                    3: y=dd.tempcase
                endcase
                nplot=1
                ytitle='deg K'
                end
            docur:  begin
                 y =dd.idrain[ipol[0]]
                 yb=dd.idrain[ipol[1]]
                nplot=npol
                ytitle='milliAmps'
                end
            dovolt:  begin
				case volt of
				 1:begin
					 y =dd.vgate[ipol[0]]
					 yb=dd.vgate[ipol[1]]
				   end
				 2:begin
					 y =dd.vdrain[ipol[0]]
					 yb=dd.vdrain[ipol[1]]
				   end
				 3:begin
					 y =dd.vdrainlna[ipol[0]]
					 yb=dd.vdrainlna[ipol[1]]
				   end
			     endcase
                 ytitle='Volts'
                 nplot=npol
                end
            dovac:  begin
                 y =dd.ptorr*1e6
                nplot=1
                ytitle='microTorr'
                end
        endcase
;
; if ver, (!y.range=[0,0] then use min,max
;
		if useautoscale then begin
			ymin=min(y,max=ymax)
			if nplot eq 2 then begin
				ymin2=min(yb,max=ymax2)
				ymin=ymin < ymin2
				ymax=ymax > ymax2
			endif
			eps=.03*(ymax - ymin)
			ver,ymin-eps,ymax +eps
		endif
		year=dd[0].year
		tit='12m ' + ltit + string(format='(" (year ",i4,")")',year)
        if not keyword_set(over) then begin
         plot,x,y,/nodata,xtickformat=xtformat,ylog=ylog,$
            xtitle=xtitleL,ytitle=ytitle,title=tit,charsize=csL,font=font
        endif
		icol=(lcol lt 0)?ipol[0]+1:lcol
        oplot,x,y,color=colph[icol],psym=sym
        if nplot eq 2 then begin
		   oplot,x,yb,color=colph[2],psym=sym
		endif
		if useautoscale then ver
    endelse
    return
end
