;+
;NAME:
;rfirdrinfo - return info on requested radars
;SYNTAX:rfirdrinfo,rdrI,faa=faa,pb=pb,aero=aero
;ARGS:
;KEYWORDS:
;faa:   if true return faa info (def).
;pb :   if ture return punta borinquen nifo
;aero:  if true return aerostat info
;
;
;RETURNS:
; rdrI: struct  holding info for this  radar
;
;DESCRIPTION:
;   Return info for the requested radar. If no keyword is supplied then faa is returned.
;
;The rdrI structure holds:
;IDL> help,rdrI,/st
;** Structure <1ebda58>, 11 tags, length=116, data length=114, refs=1:
;   NAME            STRING         "faa"  ;  name of radar
;   NFREQ           INT              4     ;  number of frequencies. they may not all be in 1 ipp
;   FREQAR          FLOAT     Array[4]     ; Mhz. frequencies used
;   PULSEDUR        INT       Array[4]     ; Usecs. how long each freqPulse is.
;   PULSEDURTOT     FLOAT           279.000; total pulse length in ipp. may contain blanking between freqs.
;   NFREQINPULSE    INT              4     ;  how many freq are in an ipp
;   FREQSEQ         INT       Array[4]     ;  order of freq in ipp . eg [0,2,1,3].. 
;                                             indices (count from 0) into freqAr
;                                             if multi dimensional, then 1 ipp has the 1st dimension
;   NIPPS           INT             15     ; number of ipps  before repeating.
;   IPPAR           FLOAT     Array[15]    ; ipps in usecs.
;   ROTPER          FLOAT           12.0000; rotation period in secs (approx).
;   BMWIDDEG        FLOAT           1.40000; beam width in degrees (approx)
;   BANDWIDTH       FLOAT           1.00000; bandwidth used at each freq.
;   notes           string     "           ; some notes on the radar
; }
;
;	Some radars output all freq each ipp (faa,pb). For faa the values are:

;
; For the punta salinas radar use:e
;rfipuntasalinasfreq,rinfo   for freq of the punta salinas 
; rdrI.freqar  : 1257.59      1252.41      1349.59      1344.41
; rdrI.pulseDur:  117           19          117           19
; rdrI.freqSeq:   0       2       1       3                    indices info freqAr
; For the aerostat, there are 4 freq, but only two are output on an ipp 
; the next ipp will output the same  2 freq (dual freq mode) or the 2nd set of 2 freq (quad freq mode)
; rdrI.freqSeq:    0       1     1 ipp
;                  2       3     next ipp if 4 freq
;
;-
pro rfirdrinfo,rdrI,faa=faa,pb=pb,aero=aero

	toUse='faa'
	if keyword_set(pb) then   touse='pb'
	if keyword_set(aero) then touse='aero'
	if keyword_set(faa) then touse='faa'
	if toUse eq 'faa' then begin
	    name='faa'
        nfreq=4
        freqAr =[1257.59,1252.41,1349.59,1344.41]
        pulseDur=[117     ,19    ,117     ,19]
        pulseDurTot=279.    ; include 2usec delay between f1,f1n
        nfreqInPulse=4
        freqSeq=[1,3,2,4]-1 ; check
        nipps=15
        ippAr=fltarr(5,3)
        ippAr[*,0]=2746.9166 * fltarr(5)
        ippAr[*,1]=3154.7360 * fltarr(5)
        ippAr[*,2]=3473.4905 * fltarr(5)
        ippAr=reform(ippar,nipps)
        rotPer=12. ; secs
        bmWidDeg=1.4  ;
        bandWidth= 1. ; mhz (each freq)
    endif
	if toUse eq 'pb' then begin
	    name='puntaBorinquen'
		nfreq=4
		freqAr =[1274.58,1269.41,1332.59,1327.41]
		pulseDur=[117     ,19    ,117     ,19]
		pulseDurTot=279.	; include 2usec delay between f1,f1n
		nfreqInPulse=4
		freqSeq=[1,3,2,4]-1 ; check
	    nipps=15
		ippAr=fltarr(5,3)
		ippAr[*,0]=2746.911 * fltarr(5)
		ippAr[*,1]=3154.7289 * fltarr(5)
		ippAr[*,2]=3473.4831 * fltarr(5)
		ippAr=reform(ippar,nipps)
		rotPer=12. ; secs
		bmWidDeg=1.4  ;
		bandWidth= 1. ; mhz (each freq)
	endif
	if toUse eq 'aero' then begin
	    name='aerostat'
		nfreq=4
		freqAr =[1261.25,1246.2,1256.5,1241.74]
		pulseDur=[160    , 160  ,160   ,160]
		pulseDurTot=320
		nfreqInPulse=2
		freqSeq =[[1,2],[3,4]]-1  ; one or the other set in 
	    nipps=7
		ippAr=[3771.,3504.,3076,2809.,2903.5,3289.,3676.] 
		rotPer=12.
		bandwidth=1.667  ; per freq
		rotPer=12. ; secs
		bmWidDeg=0  ; not measured
	endif
	rdrI={$
			name       :name ,$
			nfreq      :nfreq,$
			freqAr     :freqAr,$
			pulseDur   :pulseDur,$        ; entire pulse of ipp
			pulseDurTot:pulseDurTot,$   ; may include some blank periods between pulses
			nfreqInPulse:nfreqInPulse,$ ;
			freqSeq     :freqSeq,$      ; count from 1. order of freq in pulse
			nipps       :nipps,$        ; before repeat
			ippAr       :ippAr,$        ; before repeat
		    rotPer      :rotPer,$       ; secs 
		    bmWidDeg    :bmWidDeg,$     ; beam width of beam (deg)
		    bandwidth   :bandwidth $    ; Mhz each freq
		}
	return
end
