;+
;NAME:
;ff_2port_inp - input field fox 2port network analyzer file
;SYNTAX: n=ff_2port_inp(fname,naI,date)
;ARGS:
;fname: string file to input
;RETURNS:
; n: int   > 0 entries in file found
;               0 - file does not exist
;             -1 - file exists, but size is too small
;
; nai[n] :   struct holding the s parameters
;date    :   string ascii date when data recorded.
;DESCPRIPTION:
;	Input a field fox 2port network analyzer filel
;The program will check to format of the data in the file using line 6:
;# Hz S  dB   R 50
;
;  The returned structure will contain:
; help,nai,/st
;** Structure <d062a8>, 5 tags, length=72, data length=72, refs=1:
;   FREQ            DOUBLE           20000000.    in hz.
;   S11             DCOMPLEX  (      0.15347931,    -0.043506600)
;   S21             DCOMPLEX  (      -9.8280163,      -1.5181556)
;   S12             DCOMPLEX  (     0.058216736,     0.010209074)
;   S22             DCOMPLEX  (    -0.011068294,    0.0095901256)
;
; the S parameters will be returned as complex voltages.
;
; examples start of file:
;Agilent Technologies,N5230A,US43500289,A.05.05
;!Agilent N5230A: A.05.05
;!Date: Monday, May 03, 2021 13:08:59
;!Correction: S11(Full 2 Port(1,2)) S21(Full 2 Port(1,2)) S12(Full 2 Port(1,2)) S22(Full 2 Port(1,2))
;!S2P File: Measurements: S11, S21, S12, S22:
;# Hz S  dB   R 50
;20000000 -1.594334e+001 -1.582638e+001 1.995173e+001 -1.712188e+002 -2.456750e+001 9.946437e+000 -3.668618e+001 1.390927e+002
;-
function ff_2port_inp,fname,naI,date,finp=finp
;
;    my codes for format type: db,angle mag,angle, real,img
	format_db=1
	format_ma=2
	format_ri=3
	n=readasciifile(fname,finp)
	if n le 0 then begin
		print,"no data input. istat=:",n
	    return,n
	endif
;
;   search for !s2p line
;	ltype='!S2P File: Measurements: S11, S21, S12, S22:'
;    remove spaces
	ltype='!S2PFile:Measurements:S11,S21,S12,S22:'
	irecMax=20
	irec=0

	for i=0,irecMax-1   do begin
		irec=i
		line=strcompress(finp[i],/remove_all)
		if strcmp(line,ltype,strlen(ltype)) eq 1 then begin
			gotit =1
			break
		endif
	endfor
	if gotit eq 0 then begin
		print,"wrong file format:"
		print,"Expected:",ltype
		print,"Did not find line in first 20 records"
		return,-1
	endif
;
;  get the format 
;  dB (db,phase) 
;  Ma (magnintued,angle)
;  ri (real,img)
;
	irec++
	lin=strlowcase(finp[irec])
	a=strsplit(lin,/extract)
	case strmid(a[1],0,1) of 
       'h':freqscl=1d 
       'k':freqscl=1d3
       'm':freqscl=1d6
       'g':freqscl=1d9
	  else:begin
			print,"Illegal freq type in line6:" + line
			print,"Legal values  col2:h,k,m,g" + line
			return,-1
		   end
	endcase
;
; 	get the format
;
	case strmid(a[3],0,1) of 
       'd':form=FORMAT_DB
       'm':form=FORMAT_MA
       'r':form=FORMAT_RI
	  else: begin
			print,"Illegal format type in line 6:" + line
			print,"Legal values  col3:d,m,r" + line
			return,-1
		   end
	 endcase
;
	irec=2
	a=stregex(finp[irec],"!Date: (.*)",/sub,/extr)
	date=a[1]
;
	astr={ freq:0D,$
		   s11 :dcomplex(0.,0.),$
		   s21 :dcomplex(0.,0.),$
		   s12 :dcomplex(0.,0.),$
		   s22 :dcomplex(0.,0.)$
		 }
;
	icur=0
;          0       1        2        3        4         5        6        7      8  +1 for a[]
;    FREQ    S11re    S11im    S21re    S21im     S12re    S12im    S22re   S22im'
;'^([0-9]+) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*)'
	regex=$
'^([0-9]+) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*) +([^ ]*)'
;   switch tabs to space
	a=stregex(strcompress(finp),regex,/sub,/ext)
	ii=where(a[0,*] ne '',cnt)
	naI=replicate(astr,cnt)
	i=1
	nai.freq=reform(double(a[1,ii]))*freqscl 
	; move to double array then interpret format
	tmpar= reform(double(a[2:9,ii]),2,4,cnt)
	if form ne format_ri then  begin
		; first number db, 2nd number phase in degrees
		mag=(form eq format_db)?sqrt(10d^(tmpar[0,*,*]*.1d)):tmpar[0,*,*]  
		ph=tmpar[1,*,*]*!dtor
		tmpAr[0,*,*]=mag*cos(ph)
		tmpAr[1,*,*]=mag*sin(ph)
	endif	
	nai.s11 =complex(reform(tmpar[0,0,*]),reform(tmpar[1,0,*]))
	nai.s21 =complex(reform(tmpar[0,1,*]),reform(tmpar[1,1,*]))
	nai.s12 =complex(reform(tmpar[0,2,*]),reform(tmpar[1,2,*]))
	nai.s22 =complex(reform(tmpar[0,3,*]),reform(tmpar[1,3,*]))
	return,cnt
end 
