;+
;NAME:
;ff_inputspc_sta - input a fieldfox spectra from .sta zip file
;SYNTAX: istat=ff_inputspc_sta(zipfile,zipOdir,saI)
;ARGS:
;zipfile: string name of zip file input from field fox. normally ends in .sta
;zipOdir: string name of directory to unzip the files. It will not write over
;                any directories that currently exist.        
;RETURNS:
; istat: int   1 - input ok
;              <0  erorr. a message will be printed to stdout.
; saI  : {}    structure  holding the input info (see below)
;DESCPRIPTION:
;   Unzip a field fox zipped .sta (state) file and read in the spectral info.
;You can retrieve the .sta using ftp to the fieldfox.  This routine will
; 1. unzip the .sta file to the specified directory
;    - it will error out if the directory already exists.
; 2. search the subdir  340cdb5e-84ed-410f-9e89-d0f76a1ab16d/ for any .xml files
; 3. input and parse the SaState.xml file
;    -not all info in the file is parsed
; 4. input and parse all SaTraceN.xml  N=1,2,3,4  trace files
; 5 return the data in saI struct.
; 
; Frequencies have been converted to MHz.
; An example saI structure with 1 trace is:
;IDL> help,sai,/st
;** Structure <220cc98>, 19 tags, length=16168, data length=16155, refs=2:
;   STATEFILE       STRING    '12M23_3.sta'  ; name of zip file used
;   STARTFREQ       DOUBLE    2000.0000
;   STOPFREQ        DOUBLE    11000.000
;   CENFREQ         DOUBLE    6500.0000
;   SPAN            DOUBLE    9000.0000
;   RBW             DOUBLE    5.0000000
;   VIDEOBW         DOUBLE    5.0000000
;   RFATTN          LONG      0
;   PREAMPON        BYTE      1
;   NUMPOINTS       LONG      1001
;   DETMODE         STRING    'Average'   ; detector mode used
;   REFLEV          DOUBLE    -20.000000
;   AMPSCALE        STRING    'Log'
;   AVGNUM          LONG      20          ; number averages for any average traces
;   NTRACE          LONG      1           ; # traces returnedh
;   TRNUM           INT       Array[1]    ; the trace # (1-4) for the n traces returned
;                                         ; if traces 2,3,4 returned,then these will be in
;                                         ; trnum array 
;   TRACETYPE       STRING    Array[1]    ; type for each trace returned: Average...
;   FREQ            DOUBLE    Array[1001] ; freq inMhz
;   SPCAR           DOUBLE    Array[1001] ; spectra (normally in dbm)
;-
; 
;  Function used to help parse a line
;
; parse field fox state file
; icur will autoincrement on return to next line
;
; keyistat=ffparsesta(zipfile,zipdir,sa3)
;words:
; novalue: if set don't search for value
; inpline: if present then return the line we processed
; optkey : if present then search for optkey, return in optval
; optval : return opt value here. remove double quotes
;
function ffparsematch,inp,icur,maxline,key,value,novalue=novalue,curline=curline,optkey=optkey,optval=optval
;
	if icur ge maxline then begin
		print,"Hit end of file looking for:",key
		return,-1
	endif
;
    while (strpos(inp[icur],key) eq -1 ) do begin
		 icur++
         if icur eq maxline then begin
			print,"Hit end of file searching for key:",key
			return,-1
		 endif
	endwhile
	if (arg_present(curline)) then curline=inp[icur]
	if (n_elements(optkey) eq 1) then begin
		regex=".*"+ optkey + '="([^"]*)"'
		a=stregex(inp[icur],regex,/sub,/extr)
		if (strlen(a[1]) eq -1) then begin
			lcurline=string(format='"line ",i4,":")',icur)
			print,lcurline + "Did not find optvalue key,opt:",key + ","+optkey 
		    return,-1
		endif
		optval=a[1]
	endif
	if keyword_set(novalue) then return,1
;
;  look for value
;
    a=stregex(inp[icur],"^[^>]*>([^<]*)",/sub,/extr)
    if (strlen(a[1]) eq 0) then begin
		lcurline=string(foGrmat='"line ",i4,":")',icur)
		print,lcurline+ "Missing value when parsing:",key
		return,-1
	endif
	value=a[1]
	icur++
	return,1
end
;
function ff_inputspc_sta,zipfile,zipDir,saState
;
	if file_test(zipdir) eq 1 then begin
		print,"output zipDir already exits"
		return,-1
	endif
	saDir='340cdb5e-84ed-410f-9e89-d0f76a1ab16d/' ; sub dir after unzip
	cmd="unzip " + zipfile + " -d " + zipDir
	spawn,cmd,result,errs
	odir=zipDir + '/' + saDir
	flist=file_search(odir,'*.xml')
	a=stregex(flist,".*/(.*).xml",/sub,/extr)
	fileAr=reform(a[1,*])
	ii=where(fileAr eq 'SaState',cnt)
	if cnt eq 0 then begin
		print,"Did not file SaState.xml in subdir:",subdir
		return,-1
	endif
	if cnt gt 1 then begin
		print,"Found more than one SaState.xml files in subdir:",flist[ii]
		return,-1
	endif

	nlines=readasciifile(flist[ii[0]],inp)
	if nlines le 0 then begin
		print,"Error reading ",fname
		return, -1
	endif
;   skip binary magic numbers at start
	if  (strmid(inp[0],3,5) ne "<?xml") then begin
		print,"File doesnot start with <?xml"
		return, -2
	endif
	maxline=n_elements(inp)
	icur=0
;
;   now scan the file for the keywords we want
;
	key='<StartFrequency>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	startFreq=double(value)
;
	key='<StopFrequency>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	stopFreq=double(value)
;
	key='<CenterFrequency>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	cenFreq=double(value)
;
	key='<SpanFrequency>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	spanFreq=double(value)
;
	key='<ResolutionBandwidth>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	rbw=double(value)
;
	key='<VideoBandwidth>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	videobw=double(value)
;
	key='<RfAttenuation>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	rfattn=long(value)
;
	key='<PreampOnState>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	preampOn=strcmp(value,"tr",2) eq 1
;
	key='<NumberOfPoints>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	npnts=long(value)
;
	key='<DetectorMode '
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	detMode=value
;
	key='<AvgOrHoldNumber>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	avgNum=long(value)
;
	key='<NumActiveTraces>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	ntrace=long(value)
;    
	key='<TheTraceStates>'
    if (ffparsematch(inp,icur,maxline,key,value,/novalue) lt  0)  then goto,errout
	n=long(value)
;  0->not used 1,2,3,4
	traceNum=intarr(n)
	traceType=strArr(n)
	for i=0,n-1 do begin ; search for traces that are not blank
		key='<Item '
    	if (ffparsematch(inp,icur,maxline,key,value,optKey="default",optval=optval) lt  0)  then goto,errout
		if (optval ne 'Blank') then begin
			traceNum[i]=i+1
			traceType[i]=value
		endif 
	endfor
	ii=where(tracenum ne 0,ntrace)
	if ntrace eq 0 then begin
		print,"No traces in use (all blank)"
		return,-1
	endif
	traceNum=traceNum[ii]
	traceType=traceType[ii]

	
;    
	key='<ReferenceLevel_dBm>'
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
	refLev=double(value)
;    
	key='<OutputAmplitudeScale '
    if (ffparsematch(inp,icur,maxline,key,value) lt  0)  then goto,errout
;	Log
	ampScale=value
;
;  read the trace files
;
	spcAr=dblarr(npnts,ntrace)
	for itr=0,ntrace-1 do begin
		traceNm=string(format='("SaTrace",i1)',traceNum[itr])
		ii=where(fileAr eq traceNm,cnt)
		if cnt eq 0 then begin
			print,"Did not find trace file:",traceNm
			return,-1
		endif
		fname=flist[ii[0]]
		nlines=readasciifile(fname,inptr)
		if nlines le 0 then begin
			print,"Error reading tracefile:",FNAME
			return,-1
		endif
;
;       process first line separately
		icur=0l
		for i=0,nlines-1 do begin
			a=strsplit(inptr[i],",",/extr,count=nitems)
			if nitems eq 0 then begin
				print,format='("debug: trace",i1," line:",i4," no items. line:",a)',$
						itr,i,inptr[i] 
			    continue
			endif
			; first line may have some xml
			if i eq 0 then begin
				ipos=strpos(a[0],">",/reverse_search)
				if ipos ne -1 then a[0]=strmid(a[0],ipos+1)
			endif
			spcAr[icur:icur+nitems-1,itr]=double(a)
		    icur+=nitems
		endfor
	endfor
;
	scl=1d-6  ;convert  hz to mhz
	df=spanFreq/(npnts-1d)
	freqAr=scl*(dindgen(npnts)*df + startFreq)
	saState={$
		stateFile:zipfile,$
		startFreq:scl*startFreq,$
		stopFreq:scl*stopFreq,$
	    cenFreq:scl*cenFreq,$
		span: scl*spanFreq,$	
		rbw : scl*rbw,$
	 	videobw:scl*videobw,$
		rfAttn: rfAttn,$
		preampon:preampOn,$
		numpoints:npnts,$
		detMode  :detMode,$
		refLev   :refLev,$
		ampScale: ampScale,$
		avgNum   :avgNum,$
		ntrace   :ntrace,$
		trNum  :traceNum,$
	    traceType:traceType,$
		freq     :freqAr,$
		spcAr    :spcar}
	return,1
errout:return,-1
end
