;+
;NAME:
;aosunexamples - Using the idl sun mapping routines 
;SYNTAX: none
;ARGS: none
;DESCRIPTION:
;   The sun is mapped daily at xband using the 12meter telescope and the mock
;spectrometer. 
; (<A HREF=http://www.naic.edu/~phil/hardware/12meter/datataking/howtoexample.html#mapsun> mapping description.</A>
; These maps are automatically processed by a cron job (phil on megs3) at 19:15 each evening with
;the save files written to /share/phildat/sunmaps.
;
;	The idl sun routines will:
; Input the map spectra
; scale to kelvins using the cals
; compute total power at each point
; compute the ra,dec offset position (from the center of the sun)
; for each point. The info is returned in a mapI structure.
;
; idl .sav files are generated for each map taken and stored in
;/share/phildat/sunmaps/ 
; The .sav file naming convention is:
; yymmdd_nn_sun.sav  where yymmdd is the date, and nn is  1..nmaps for the day
; eg 
;220420_01_sun.sav  1st map
;220420_02_sun.sav  2nd map
; ...
;
;You can recall a map for from a .sav file using:
;IDL> restore,'/share/phildat/sunmaps/220420_01_sun.sav',/verb
;% RESTORE: Portable (XDR) SAVE/RESTORE file.
;% RESTORE: Save file written by phil@gpuserv0, Thu Apr 21 17:28:28 2022.
;% RESTORE: IDL version 7.1 (linux, x86_64).
;% RESTORE: Restored variable: MAPI.
;
; The structure contains:
;IDL> help,mapI
;** Structure <1055a88>, 16 tags, length=1542944, data length=1532385, refs=1:
;   NSTRIPS    LONG                25    number of strips in map
;   NPOL       LONG                 2    number of pols
;   SMPSTRIP   LONG                60    number of samples in ra strip 
;   NBMS       LONG     Array[2]         min,max filenumber used.includes cals and map
;   FILENUMS   LONG                 7    number of freq bandsrr0r
;   YYYYMMDD   LONG          20211224    date for start of map
;   MJDSTART   DOUBLE           59572.611 mjd start time of 1st strip of map
;   SECSTART   DOUBLE       1.6403568e+09 secsonds 1970 for start of 1st strip of map
;   HARBM0     STRUCT  MASFHDR Array[60, 25] fits headers bm 0, 60smp/strip,25 strips
sungetmap.pro;   HARBANDS   STRUCT  MASFHDR Array[7]  fits header first sample,first row, all 7 freq bands.
;   RAOFFD     FLOAT     Array[60, 25]     measured RA offset in deg for each sample
;   DECOFFD    FLOAT     Array[60, 25]     measured Dec offset in deg for each sample
;   CFRARBM    DOUBLE    Array[7]          center freq (MHz) each band
;   SCLFACTOR  FLOAT     Array[2, 7]       scale factor cntsToKelvin if units=3
;                                          cntsToCalUnits if  units =2
;   UNITS      LONG                        mockunits      if  units =1
;   TPAR       FLOAT     Array[60, 25, 2, 7] total power data (smpStrip,Nstrips,Npol,Nbms)
;
;The maps conver +/-1 deg in ra (great circle) and +/- 1 deg in dec.
;The ra strips are spaced by 5 Amin in declination (fwhm of xband beam is 10Amin)
;So there are 25 ra strips in total. The 12th strip (counting from 0) goes through the
;center of the sun.
;  The total power at  each point is returen in mapI.tpar[60,25,2,7]
; 60= 60 samples across the strip.
; 25= 25 strips .. index (12 ..cnt from 0) goes through the center of the sun,
; 2 =polA, polB
; 7 =7 frequency bands.
;
;	When taking the data, the telescope drives from -RaOffset to +RaOffset and then reverses
;direction and drives fro +Raoffset to -RaOffset. That means that  the 60 points in adjacent
;strips start at opposite ra locations. 
;	The RaOffD[60,25] array has the actual raOffsets (in deg) for each totalPwr point. 
;
;Examples using the routines:
; to enter idl:
; idl
; @phil
; @suninit
;
;Example 1:	process a days worth of maps and return then in mapIAr
; IN idl..
; yymmdd=220428L
; nmaps=sunmapsday(yymmdd,mapIar)
; -->Found:  2 sun maps for date:20220428
; -->220428 map  0 fnums:    0 2800 Nnums:     29
; -->220428 map  1 fnums: 2900 5700 Nnums:     29
; -->maps done: 2  no save files generated
;
;Example 2: process a single map using the pattern id:
; Look in the logfile in the proj directory:/share/obs4/usr/p12m/sun/ (or whatever project you used)
; Look at the logfile;
; more sun.p12mlog_20220428.log
; line:
; 2022-04-28_09:16:36 mapradec: start. pattern_id:211800018
; This is the pattern id for the first map.
; In idl:
; yymmdd=220428L
; patId=211800018L
;  nmaps=sunmapsday(yymmdd,mapIar,patid=patId)
; -->Found patId:211800018 sun map for date:20220428
; -->220428 map  0 fnums:    0 2800 Nnums:     29
; -->maps done: 1  no save files generated
;
;Example 3: process a day and save it
; in idl
;  nmaps=sunmapsday(yymmdd,mapIar,/dosav)
; --> Found:  2 sun maps for date:20220428
; --> 220428 map  0 fnums:    0 2800 Nnums:     29
; --> 220428 map  1 fnums: 2900 5700 Nnums:     29
; --> maps done:  2  save files generated
;
;	Normal user should not try to generate save files in the
; default location (you probably don't have write  access).
;If you've done your own maps, then you can save them to one of your
;directories using the savDir=  keyword.
;
; More info (from within idl):
; explain,sundoc   ..to get a listing of the routines
; ----- Documentation for /pkg/rsi/local/libao/phil/doc/sundoc.pro -----
;NAME 
;sundoc - routine list (single line)
;
; aosunexamples    - Using the idl sun mapping routines
; sungetmap        - input mock scans of map, compute totalPwr
; sunmapsday       - process sunmaps for a given day
;
; explain,sunmapsday
; ----- Documentation for ./sunmapsday.pro -----
;NAME 
;sunmapsday - process sunmaps for a given day
;SYNTAX: nmaps=sunmapsday(yymmdd,mapIar,proj=proj,dosav=dosav,savDir=savDir,patId=patId
;ARGS:
; ...
; ...
;-
; --> if ;NAME: multiple times in file, get make doc error. just remove :
