;+
;NAME:
;sungetmap - input mock scans of map, compute totalPwr
;SYNTAX: nstrips=sungetmap(yymmdd,proj,fnum0,fnum1,mapI,tcal=tcal,minstrips=minstrips)
;ARGS:
;  yymmdd: long   date for data   yyyymmdd  (ast)
;   proj : string   project name for files
;   fnum0: int    first filenum of day (include any cal on,offs
;   fnum1: int    last filenum of day
;KEYWORDS:
;  verb  : int    if set then plot tp as we go
;  tcal  :        if set then scale mapI.tpar to units of the cal rather than degK
;                 (since we may not have the correct cal values yet.
;minstrips: int   minimum number of radecstrips map needs default:13
;RETURNS:
;  nstrips: int   num strips. < 0 --> error
;  mapI   : {}  struct holding the map info (including tpar)
;DESCRIPTION:
; Input a map of the sun takne with the 12 meter. This assumes you've used the
;tcl script mapsun.proc to input the data.
; The datatakingi does:
; 1. a cal on, off at  the beginnning (2 files)
; 2. a map of nstrips covering an area, driving in ra, stepping in dec.
; 3. another calon ,off at the end of the map
;
; The routine will:
; 1. input the cals, compute the scale factor from mock units to cal units
;    exclude 6% of the spectra on each edge.
;    it averages the scale factor computed from the 2 cal measurements.
; 2. input the map 
; 3. compute the total power for each sample (excluding 6% of the spectra on each edge).
; 4. scale the total power to Kelvins or calunits depending on tcal=
; 5. compute the raoff, dec off for each sample (this are currently little circle).
; 6. return the info in mapI.
; The mapI struct contains:
;IDL> help,mapI
;** Structure <632c008>, 15 tags, length=1536168, data length=1525660, refs=1:
;   NSTRIPS    LONG                25    number of strips in map
;   NPOL       LONG                 2    number of pols
;   SMPSTRIP   LONG                60    number of samples in ra strip 
;   NBMS       LONG                 7    number of freq bandsrr0r
;   FILENUMS   LONG     Array[2]         min,max filenumber used.includes cals and map
;   YYYYMMDD   LONG          20211224    date for start of map
;   MJDSTART   DOUBLE           59572.611 mjd start time of 1st strip of map
;   SECSTART   DOUBLE       1.6403568e+09 secsonds 1970 for start of 1st strip of map
;   HARBM0     STRUCT  MASFHDR Array[60, 25] fits headers bm 0, 60smp/strip,25 strips
;   HARBANDS   STRUCT  MASFHDR Array[7]  fits header first sample,first row, all 7 freq bands.
;   RAOFFD     FLOAT     Array[60, 25]     measured RA offset in deg for each sample
;   DECOFFD    FLOAT     Array[60, 25]     measured Dec offset in deg for each sample
;   CFRARBM    DOUBLE    Array[7]          center freq (MHz) each band
;   SCLFACTOR  FLOAT     Array[2, 7]       scale factor cntsToKelvin if units=3
;                                          cntsToCalUnits if  units =2
;   UNITS      LONG                        mockunits      if  units =1
;   TPAR       FLOAT     Array[60, 25, 2, 7] total power data (smpStrip,Nstrips,Npol,Nbms)
;
; When making the map:
; - drives from negative ra offset to positive The ra,dec offsets are the measuredPos - SunCenterPos
; So a negative ra is at a lower ra than the center of the sun.
; The sun center is interpolated from the jpl ephemeris used to track the sun.
; The strips
;- 
;
; History
;  27dec21 - added raOffd,decOffD  curPos - EphemCenter  for every sample
;  03jan22 - changed ra,offset to be the center of the ra samples (it had been the beginning)
;            with 1 second sampling this is now at the measured positions .5,1.5,2.5,3.5....
;  14apr22 - return data in kelvins, scltocal -> sclfactor, added hdr each band
;
function sungetmap,yymmdd,proj,fnum0,fnum1,mapI,verb=verb,tcal=tcal,minstrips=minstrips
;
	common colph,decomposedph,colph

	if n_elements(minstrips) eq 0 then minstrips=13
; 
;  tpUnits 
;  1 - mock
;  2 - cal
;  3 - kelvins
;  
	tpUnits=3  ; default Kelvins
	yymmddL=yymmdd
	if yymmddL lt 999999l then yymmddL+=20000000l
	if keyword_set(tcal) then tpUnits=2
	if n_elements(verb) eq 0 then verb=0
	edgfr=.06 ; ignore this much of  bandpass on each edge.
;
;	scan the files for this date
;		`
	n=masfilelist('',fnmI,proj=proj,yymmdd=yymmddL,band=1,grp=0,/appbm)
	ii=where((fnmI.num ge fnum0) and (fnmI.num le fnum1),nfiles)
	if nfiles eq 0 then begin
		print,"No file for ",yymmddL," " + proj," between fnums:",fnum0,fnum1
		return,0
	endif
	fnmI=fnmI[ii]
	fmin=min(fnmI.num,max=fmax)

; get the summary files

	n=masfilesum('',sumI,fnmI=fnmI)
	ubms=sumI[uniq(sumI.bm,sort(sumI.bm))].bm
	nbms=n_elements(ubms)

;   check the number of strips in map. will be nstrips*nbms files
;
	iimap=where((sumI.h.obsmode eq 'RADECMAP'),nmap)
	if (nmap lt 25*nbms) then begin
			if (nmap lt minstrips*nbms) then begin
			lab=string(format='("Found only ",i3," strips in map. Min required:",i3)',$
					nmap/nbms,minstrips)
			print,lab
			return,-1
		endif
		lin=string(format='("Warning:",i08," fnum0:",i6," found ",i3," strips, expected 25")',$
				yymmddL,fnum0,nmap/nbms)
		print,lin
		nstrips=nmap/nbms
 	endif
	sumImap=sumI[iimap]

;  see if we have any cals

	iical=where((sumI.h.obsmode eq 'CAL'),ncal)
	useCal=(ncal/nbms ge 2)
	if usecal then  begin
		sumIcal=sumI[iical]
	endif else begin
		tpUnits=1 	; if no cal mock units.
	endelse
;                                  1       2         3       4      
	uscanmapid=sumImap[uniq(sumImap.h.scan_id,sort(sumImap.h.scan_id))].h.scan_id
	nstrips=n_elements(uscanmapid)
;
	a=stregex(sumI[0].h.tdim1,"\(([0-9]+),([0-9]+),([0-9]+),([0-9]+),([0-9]+)",/sub,/ext)
	nchan=long(a[1])
	npol=long(a[4])
	ndump=long(a[5])
;
	ie0=long(edgfr*nchan)  ; to ignore on edges
	ie1=nchan - ie0 - 1
	nchuse=ie1 -ie0 + 1
;
;  do the cals first
;
	sclFactAr=dblarr(npol,nbms) + 1d
	if useCal then begin
		iion=where(sumIcal.h.scantype eq 'ON',cnton)
		iioff=where(sumIcal.h.scantype eq 'OFF',cntoff)
		istat=masgetfiles('',bon,/avg,fnmAr=sumIcal[iion].fname,/remdc)
		istat=masgetfiles('',boff,/avg,fnmAr=sumIcal[iioff].fname,/remdc)
		; if more than 1 cal, average them
		if ncal gt (2*nbms) then begin
			bon1=replicate(bon[0],nbms)
			boff1=replicate(boff[0],nbms)
			for ibm=0,nbms-1 do begin &$
				ii=where(bon.h.beam eq ibm,cnt) &$
				bon1[ibm]=masmath(bon[ii],/avg) &$
				ii=where(boff.h.beam eq ibm,cnt) &$
				boff1[ibm]=masmath(boff[ii],/avg) &$
			endfor
			bon=bon1
			boff=boff1
			bon1=''
			boff1=''
		endif
		; compute scale factors for each bm
		for ibm=0,nbms-1 do begin &$
			iion=where(bon.h.beam eq ibm,cnt) &$
			iioff=where(boff.h.beam eq ibm,cnt) &$
			istat=mascalonoff(bon[iion],boff[iioff],calI,edge=edgfr) &$
;           if they want cal units
	        sclFactAr[*,ibm]=calI.cntsToK &$
			if tpUnits eq 2  then sclFactAr[*,ibm]/=calI.calval&$  ; convert to cal units
			if ibm eq 0 then calIar=replicate(calI,nbms) &$
			calIar[ibm]=calI &$
		endfor
	endif
	; now input the map
	cfrar=dblarr(nbms)
	start=1
	for istrip=0,nstrips-1 do begin
	;   get this strips all beams
		ii=where(sumImap.h.scan_id eq uscanmapid[istrip],cnt)
		sumI1=sumIMap[ii]
        istat=masgetfiles('',b,fnmAr=sumI1.fname,/remdc)
		for ibm=0,nbms-1 do begin
		  ii=where(sumI1.bm eq ibm)
		  if start then begin
			smpStrip=n_elements(b[*,0])*ndump
			mapI={ nstrips:nstrips,$
				   npol   :npol,$
				   smpStrip: smpStrip,$
			       nbms   : nbms,$
				   fileNums:[fmin,fmax],$;  min,max filenumber for cals and maps
                   yyyymmdd: yymmddL,$; ast date for map 
                   mjdStart: b[0].h.MJDXXOBS, $; start of first strip
                   secStart: tosecs1970(0,0,mjd=b[0].h.mjdxxobs),$; secs 1970 start of first strip
		           harbm0 :replicate(b[0].h,smpstrip,nstrips),$; only keep hdrs for first bm (for pos)
		           harBands:reform(b[0,*].h),$ ; keep first header of each freq band for freq info
			       raOffD :fltarr(smpstrip,nstrips),$;  ra offset (curPos - center) in deg
			       decOffD :fltarr(smpstrip,nstrips),$;  dec offset (curPos - center) in deg
		           cfrArBm:dblarr(nbms),$
				   sclFactor: sclFactAr,$; values used to scale  tpar to cal or degK 
                   units   : tpUnits,$;  units 1- mockCnts,  2- cal deflection,3: kelvins
			       tpar   :fltarr(smpstrip,nstrips,npol,nbms)} 
			start=0
		  endif
		  nsmp=n_elements(b[*,0])
		  tp=transpose(reform(total(b[0:nsmp-1,ibm].d[ie0:ie1,*],1)/nchuse,npol,nsmp))
		  if useCal then begin
		  	tp[*,0]*=mapI.sclFactor[0,ibm]
		  	tp[*,1]*=mapI.sclFactor[1,ibm]
		  endif
		  mapI.tpar[0:nsmp-1,istrip,*,ibm]=tp 
		  if (ibm eq 0) then mapI.harBm0[0:nsmp-1,istrip]=b[*,ibm].h
		  if istrip eq 0 then mapI.cfrArBm[ibm]=b[0,ibm].h.crval1*1d-6
		  if (verb) then begin
			tit=string(format='("istrip:",i,"ibm:",i)',istrip,ibm)
			plot,mapI.tpar[*,istrip,0,ibm],tit=tit
			oplot,mapI.tpar[*,istrip,1,ibm],col=colph[2]
	    	empty
		  endif
		endfor
	endfor
;
; compute the ra,dec offsets req_posc1/2  has the ephemeris center of the sun for each sec
; 
;	interpolate to ra offset to center of each sample
    raoff=mapI.harBm0.crval2 - mapI.harBm0.req_posc1
    raoffI=raOff
	x=findgen(smpstrip)
	xI=x + .5
	for i=0,nstrips-1 do begin &$
	  raOffI[*,i]=interpol(raoff[*,i],x,xI) &$
	endfor
	mapI.raOffD= raOffI
	mapI.decOffD= mapI.harBm0.crval3 - mapI.harBm0.req_posc2
	return,nstrips
end
