;+1.6511536e+09
;NAME:
;sunmapsday - process sunmaps for a given day
;SYNTAX: nmaps=sunmapsday(yymmdd,mapIar,proj=proj,dosav=dosav,savDir=savDir,patId=patId
;ARGS:
;  yymmdd: long   date for data   yyyymmdd  (ast)
;KEYWORDS:
;   proj : string project name for files to search
;   dosav: int    if set then generate idl .sav files
;                 If no savdir= is specified then it will default to /share/phildat/sunmaps/
;                 You need to have permissions to write into the savdir.
;                 If patId= is used then a single map is done, and no save files are allowed.
; savDir : string if dosav= is set, then write the save files to this directory.
;                 If not supplied, the default is /share/phildat/sunmaps. 
;                 You must have write permissions to this directory
; patId  : long   if supplied then just return 1 map
;                 - that has patid=patid, (see logfiles)
;                 - it must also be in proj=proj, and have object='sun'
;minstrips: int   minimum number of radecstrips map needs default:13
;RETURNS:
;  nmaps: int   < 0 --> error
;               >=0 number of maps processed
;
;  mapIAr[nmaps]: {}  array of structs holding info from each map (see below)
;DESCRIPTION:
; 	This routine will input and compute totalpower for all sun maps taken on the specified day.
;The results are returned in the mapIAr array of structures. You can optionally have each map written
;to an idl .sav  file. 
; 	If the patId= keyword is supplied then only one map will be returned (that has the patId specified)
;and no sav files will be written (see reasons below).
;
;	For a map to be included in the data set (assuming patid= is not used) the following must occur:
;  -The map was taken on yymmdd
;  -the project matches the requested project (def='sun')
;  -The source for the map is 'sun'
;  -The map was taken with the mapradec tcl routine.
;
;	Each structure in mapIar will contain:
;IDL> help,mapIAr,/st
;Structure <214e168>, 16 tags, length=1542944, data length=1532385, refs=1:
;   NSTRIPS    LONG                25    number of strips in map
;   NPOL       LONG                 2    number of pols
;   SMPSTRIP   LONG                60    number of samples in ra strip
;   NBMS       LONG     Array[2]         min,max filenumber used.includes cals and map
;   FILENUMS   LONG                 7    number of freq bandsrr0r
;   YYYYMMDD   LONG          20211224    date for start of map
;   MJDSTART   DOUBLE           59572.611 mjd start time of 1st strip of map
;   SECSTART   DOUBLE       1.6403568e+09 secsonds 1970 for start of 1st strip of map
;   HARBM0     STRUCT  MASFHDR Array[60, 25] fits headers bm 0, 60smp/strip,25 strips
;   HARBANDS   STRUCT  MASFHDR Array[7]  fits header first sample,first row, all 7 freq bands.
;   RAOFFD     FLOAT     Array[60, 25]     measured RA offset in deg for each sample
;   DECOFFD    FLOAT     Array[60, 25]     measured Dec offset in deg for each sample
;   CFRARBM    DOUBLE    Array[7]          center freq (MHz) each band
;   SCLFACTOR  FLOAT     Array[2, 7]       scale factor cntsToKelvin if units=3
;                                          cntsToCalUnits if  units =2
;   UNITS      LONG                        mockunits      if  units =1
;   TPAR       FLOAT     Array[60, 25, 2, 7] total power data (smpStrip,Nstrips,Npol,Nbms)
;  
; See sumgetmap() for more info on how each map is taken.
;
;Notes on the .sav files:
;  If there are N maps made in a given day, then the .sav file names will be
; savDir/yymmdd_nn_sun.sav   where yymmdd is the date and nn is 1 thru Nmaps
; eg: on 220420 11maps were made. the names are:
;220420_01_sun.sav
;220420_02_sun.sav
; ...
;220420_11_sun.sav
;
;	You are not allowed to generate a save file if you ask for a single map with patid=  keyword
;This could disrupt the file naming of the .sav files for a day
; eg:
; suppose you used the patid= for the 4th map of 220420.
;If you were allowed to generate a save file with the specified format, it would end up
;being call 220420_01.sav .. This would overwrite the true 1st map. For this reason
;save files can only be written when the entire day is processed.
;	You can always call the routine with patid=   and then do your own save using whatever name you choose,
;- 
function sunmapsday,yymmdd,mapIar,proj=proj,dosav=dosav,savDir=savDir,patid=patid
;
	savDef='/share/phildat/sunmaps/'
	projDef='sun'
	minstrips=13
	usePatId=n_elements(patId) gt 0
	dosav=keyword_set(dosav)
	if (usePatid and dosav) then begin
		print,"save files not written when patid= keyword used"
		print,"data will be returned mapIAr"
		dosav=0
	endif
	if n_elements(proj) eq 0 then proj=projDef
	tcal=3  ; kelvins
	ibmToPlot=3
	src='sun'
	obsmode='RADECMAP'
	if n_elements(savDir) eq 0 then savDir=savDef
	if (dosav) then begin
;       need to protect savDef from other projects, and single patid writes
		if ((savDir eq savDef)) then begin
 		   if (proj  ne  projDef)  then begin
				print,"Only proj='sun' can write  to " + projDef
				return,-1
			endif
		endif
		if ( file_test(savDir,/dir,/write) eq 0) then begin
			print,"You don't have write permissions to save file dir:",savDir
			return,-1
		endif
	endif
	yymmdd=yymmdd mod 1000000l
	yymmddL=yymmdd
	if yymmddL lt 999999 then yymmddL+=20000000l
	lyymmdd=string(format='(i06)',yymmdd)
		
;
	nmaps=maspatidday(yymmdd,proj,obsmode,patIday,src=src,bmToTest=bmToTest,patId=patId)
;
	yymmdd=yymmdd mod 1000000l
	if (usePatId eq 0) then begin
		lin=string(format='("Found: ",i2," sun maps for date:",i8)',nmaps,yymmddL)
	endif else begin
		lin=string(format='("Found patId:",i9," sun map for date:",i8)',patId,yymmddL)
	endelse
	print,lin
;
;	loop doing one map at a time
;
	mapsdone=0
	mapsStored=0
	for imap=0,nmaps-1 do begin
;
;       first,last file numbers
; 
		fnum0=patiDay.patI[imap].fnum0
		fnum1=patiDay.patI[imap].fnum1
; 
;      input map, compute total power
;
		nn=(fnum1-fnum0)/100 +1
	    print,lyymmdd + string(format='(" map ",i2," fnums:",i5,i5," Nnums:",i)',$
				imap,fnum0,fnum1,nn)
		nstrips=sungetmap(yymmddl,proj,fnum0,fnum1,mapI,minstrips=minstrips)
		if nstrips le 0 then begin
			print,"Skipping patternid",patIDay.patI[imap].patid
		    continue
		endif
		tr=findgen(60)
		if dosav then begin
			fsav=savDir + lyymmdd + string(format='("_",i02,"_sun.sav")',imap+1)
			save,mapI,file=fsav
		endif
	    mval=max(mapI.tpar[*,*,*,ibmToPlot])
		ver,0,mval*1.05	
		ytit=(mapI.units eq 3)?"Kelvins":(mapI.units eq 2)?"CalUnits":"spectrometer counts"
		xtit='Ra offset from center[deg]'
		y=mapI.tpar[*,*,*,ibmToPlot]
; 	    flip to have +ra, -ra
	    if mapI.raoffd[0,0] lt 0 then begin &$ ; first strips neg to pos
			ii=findgen(nstrips/2l)*2  &$ ; start flipping r0,2,4 if first 
			x=mapI.raoffd[*,1] &$
		endif else begin &$
			ii=findgen(nstrips/2l)*2 + 1 &$  ; start flipping 1,3,5
			x=mapI.raoffd[*,0] &$
		endelse
		y[*,ii,*]=reverse(y[*,ii,*],1)
		hor,max(x),min(x)
		fname=(dosav eq 0)?"nosavFile":basename(fsav)
		tit=string(format='(a," (AST)  file:",a)',systime(0,mapI.secstart),fname)
		inc=0.
		step=0
		stripsxy,x,y[*,*,0],0,inc,step=step,$
				xtit=xtit,ytit=ytit,tit=tit
		stripsxy,x,y[*,*,1],0,inc,/over,/step,colar=intarr(100)+2 
			empty
		if mapsDone eq 0 then begin
			mapIAr=replicate(mapI,nmaps)
			mapsize=n_tags(mapI,/length)
		endif
		curSize=n_tags(mapI,/length)
		if curSize ne mapSize then begin
			lin=string(format='("1stMapSize:",i," map#:",i," has size:",i)',$
						mapsize,imap+1,cursize)	 
			print,lin
			print,"map will not be stored in mapIAr for return" 
		endif else begin
			mapIAr[imap]=mapI
	        mapsStored++
		endelse
	    mapsdone++
	endfor
	lab=" save files generated"
	if (dosav) eq 0 then lab=" no" + lab
	lin=string(format='("maps done:",i2,1x,a)',mapsdone,lab)
	print,lin 
	if mapsStored lt mapsDone then begin
		mapIar=mapIar[0:mapsStored-1]
	endif

	return,mapsdone
end
