;+
;NAME:
;turplotsum - plot a summary of the tiedown data.
;SYNTAX: turplotsum,d,title=title,tmrange=tmrange,$
;                  vpos=vpos,vvel=vvel,vtq=vtq,tqinc=tqinc,tqsmo=tqsmo,$
;                  win=win,wait=wait,page=page,fl=fl,zerooff=zerooff
;ARGS:
;    d[] :{turlog} data to display. read via turinpday()
;
;KEYWORDS:
;   title: string title for each plot (eg date..)'
;tmrange[2]:long  if provided, then limit the plots to
;                 the tmrange[0],tmrange[1] time range. the units are hours.
;   vpos[2]:float min max for pos vs time
;   vvel[2]:float min max for commanded and feedback velocity..-2048,2048
;    vtq[2]:float min max for torque plots
;    tqinc :float display offset between each motor tq plot. def=0
;   tqsmo  :int   number of seconds to smooth motor tqs. def=1 (no smoothing)
;     win[]:int   window numbers to plot each page in..
;     wait :      if set then wait for user to hit a key between each plot
;      fl[]:float list of hours to flag in each page
;     page :int   plot just one page of the info. default is all
;zerooff   :float zero offset for vel cmd. default=50
;fittime   :      if set then do linear fit for time. Useful
;                 if there are time dropouts
;
;DESCRIPTION:
;    Plot the turret data and status info. The data comes from:
;n=turinpday(yymmdd,tur) .. 
;
;    You can plot a subset of the data in tur by using the tmr=[hr1,hr2] keyword
;to limit the plots to between hr1 and hr2 hours of the day.
;
;    the keyword page=n will output a single page (1..7)
;    By default pages are output one after another with stopping. This is
;normally used when plotting to a file. For inspection from a terminal,
;use /wait so the program pauses after each page waiting for keyboard input.
;
;    The output consists of analog data as well as status bits. 
;
;    The analog data is output for both motors.The outputs are:
;        torques: black=master, red=slave
;		
;    The status plots:
;   -  Each bit is plotted as a line vs time
;   -  The left side shows the bit label as well as a colored *.
;   -  The * is the value=0 position the bit, value=1 moves the line 
;      farther up.
;   -  Labels that start with - (eg -CRmEmgS) negate the label.
;       eg. -CRmEmgSt 1 value means control room emergency stop is off.
;
;The pages are:
;   
;page 1; position, commanded and feedback velocity
;page 2: the tiedown device status vs time
;page 3: the little star digital inputs 
;page 4: digital uio 1 and 2 inputs
;page 5: little star digital output
;page 6: drive and fault status
;page 7: motor torques (for all 3 tiedowns)
;
;To use these routines:
;idl71
;@phil
;@turinit
;yymmdd=150913
;n=turinpday(yymmdd,tur)
;turplotsum,td,/wait
;
;to send output to a ps file:
;
;pscol,'junk.ps',/full
;turplotsum,d
;hardcopy
;x
;-
pro  turplotsum,tur,title=title,xplab=xplab,loff=loff,$
				 vpos=vpos,vvel=vvel,vtq=vtq,tqsmo=tqsmo,tqinc=tqinc,$
                 win=win,wait=wait,page=page,sym=sym,fl=fl,tmrange=tmrange,$
				 zerooff=zerooff,fittime=fittime,cs=cs,ncs=ncs,font=font
;
	forward_function turlab
    common colph,decomposedph,colph

	usePs=!d.flags and 1
	if n_elements(cs) eq 0 then cs=(usePs)?1.4:1.4
	if n_elements(ncs) eq 0 then ncs=(usePs)?1.1:1.5
	hor
	ver
	font=1
	!p.multi=0
;	tdparms,inchPerEncCnt=inchPerEncCnt
;   fix ??
    cntToCurI=.01788                ; comes from forcing foldback=10amps
                                    ; see tieProg.h bottom.
	n=n_elements(tur)
    if n lt 2 then begin
        print,'not enough data to plot..'
        return
    endif
	if n_elements(zero_off) eq 0 then zero_off=50.
;;;/*************************************************************************/
;;LOCAL   TTT_LABELS   turFltStat [16]={
;;                    {"SafetyO ","        ",0},
;;                    {"Mot12SpG","        ",0},
;;                    {"CmdSpdEr","        ",0},
;;                    {"OvrSpd  ","        ",0},
;;                    {"Drv1Fail","        ",0},
;;                    {"Drv2Fail","        ",0},
;;                    {"PwrSFail","        ",0},
;;                    {"IndFail ","        ",0},
;;                    {"BrkFail ","        ",0},
;;                    {"        ","        ",0},
;;                    {"        ","        ",0},
;;                    {"        ","        ",0},
;;                    {"        ","        ",0},
;;                    {"        ","        ",0},
;;                    {"        ","        ",0},
;;                    {"        ","        ",0}};

	tm=tur.secm/3600.
	if (keyword_set(fittime)) then begin
		coef=poly_fit(findgen(n),tm,1,yfit=tm)
	endif
;   in case last sec of prev day
    if (tm[0] gt tm[1]) then tm[0]=tm[1]-1./3600.
;
   if n_elements(tmrange) eq 2 then begin
        ind=where((tm ge tmrange[0]) and (tm le tmrange[1]),n)
        if n eq 0 then return
        i1=ind[0]
        i2=ind[n-1]
    endif else begin
        i1=0L
        i2=n-1
    endelse

    if n_elements(page) eq 0 then page=0
    if not keyword_set(loff) then loff=.06
	nwin=n_elements(win)
	if nwin gt 0 then begin
		for i=0,nwin-1 do window,win[i],xsize=640,ysize=800
	endif
	tm=tm[i1:i2]
;
;
;the program will then plot pages containing:
; page 1: tur position
;         tur velfeedback and command
;
	wini=0
	useflag=n_elements(fl) gt 0
	if n_elements(tqinc) eq 0 then tqinc=0
    fls=2

    if not keyword_set(title) then title=''
    if not keyword_set(xplab) then xplab=-.2
;--------------------------------------------------------
; 
; page 1   tm vs pos, 
;          tm vs velCmd  and velFback
;

	if nwin gt 0 then wset,win[wini]
	wini=(wini+1 ) < (nwin-1)
	if (page eq 0) or (page eq 1) then begin
      !p.multi=[0,1,2]
	  pos=transpose(tur[i1:i2].pos)
      if (n_elements(vpos) eq 2) then  begin
		ver,vpos[0],vpos[1] 
	  endif else begin 
		ver,min(pos)*.9,max(pos)*1.1
	  endelse 
	  plot,tm,pos,$
		chars=cs,font=font,$
        xtitle='hour',ytitle='deg',psym=sym,$
        title=title + ' Tur position vs hour (page 1)'
		if useflag then flag,fl,linestyle=fls
;
;     page 1 velfeedback, velCmd - 2048
;
      if (n_elements(vvel) eq 2) then begin
		 ver,vvel[0],vvel[1] 
	endif else begin
		 ver,min(tur[i1:i2].dat.ai_velfb)*.9,max(tur[i1:i2].dat.ai_velfb)*1.1
	endelse
	zero_off=50.
	plot,tm,tur[i1:i2].dat.ai_velfb,$
		chars=cs,font=font,$
        xtitle='hour',ytitle='counts',psym=sym,$
        title=title+ ' velFeedback and velCmd Sent to dac (-2048+zerooff) (page 1)'
	oplot,tm,tur[i1:i2].dat.ao_velcmd -2048 + zero_off,psym=sym,col=colph[2]

	note,17      ,"vel feedback",chars=ncs,font=font,xp=.04
	note,17+1*.8,"velCmd -2048",chars=ncs,font=font,xp=.04,col=colph[2]

	if useflag then flag,fl,linestyle=fls
	if keyword_set(wait) then begin
		print,'hit return to continue'
		wait,1
		key=checkkey(/wait)
	endif
    
;;    hor
    ver
	if nwin gt 0 then wset,win[wini]
	wini=(wini+1 ) < (nwin-1)
	endif
    !p.multi=0

;--------------------------------------------------------
; page 2 device stat
;   decode mode
;   ignore encoder feedback
;   move around reboot, trackdata
;
	if (page eq 0 ) or (page eq 2) then begin
    lab=turlab('devstat')
	mask='ff7f'X
;
;   get mode from .status, turn it into an index
;   we can use in devstat lablels. I put the
;   modes at top of devstat labels
;
	ivalM=tur[i1:i2].statWd 
	ivalD=tur[i1:i2].devStat
	modeBits=ishft(ivalM and '1c'XL,-2); 1-stop,2-pt,3-slew,4-autoscan
	rebtr=ishft(ivalD and '4000'X,-8)
	ivalD= ((ivalD and '0f3f'X) or rebtr) or (ishft(2^modeBits,11))
	
    pltbits,tm,ivalD,mask,maxbits=16,lab=lab,psym=sym,$
		chars=cs,ncs=ncs,font=font,$
    xtitle='hour',title=title + ' device status (page 2)'
	if useflag then flag,fl,linestyle=fls
	 if keyword_set(wait) then begin
		print,'hit return to continue'
		wait,1
		key=checkkey(/wait)
    endif
    endif
;
;--------------------------------------------------------
; page 3 digital input little star
;
;
	if (page eq 0 ) or (page eq 3) then begin
	if nwin gt 0 then wset,win[wini]
	wini=(wini+1 ) < (nwin-1)
    lab=turlab('dils')
	mask='fcff'X
    pltbits,tm,tur[i1:i2].dat.di_ls,mask,maxbits=16,lab=lab,psym=sym,$ 
		chars=cs,ncs=ncs,font=font,$
    xtitle='hour',title=title + ' Little Star digital input (page 3)'
	if useflag then flag,fl,linestyle=fls
	 if keyword_set(wait)  then begin
		print,'hit return to continue'
		wait,1
		key=checkkey(/wait)
    endif
  endif
; 
;--------------------------------------------------------
; page 4 uio digital  input 1 
;
;
    if (page eq 0 ) or (page eq 4) then begin
    if nwin gt 0 then wset,win[wini]
    wini=(wini+1 ) < (nwin-1)
    lab=turlab('di1')
    mask='ffff'XL 
    pltbits,tm,tur[i1:i2].dat.di_uio1,mask,maxbits=16,lab=lab,$
		chars=cs,ncs=ncs,font=font,$
	psym=sym,$
    xtitle='hour',title=title +  ' UIO digital 1 (page 4)'
    if useflag then flag,fl,linestyle=fls
     if keyword_set(wait)  then begin
        print,'hit return to continue'
        wait,1
        key=checkkey(/wait)
    endif
  endif
;--------------------------------------------------------
; page 5 uio digital  input 2 
;
;
    if (page eq 0 ) or (page eq 5) then begin
    if nwin gt 0 then wset,win[wini]
    wini=(wini+1 ) < (nwin-1)
    lab=turlab('di2')
    mask='ffc3'XL 
    pltbits,tm,tur[i1:i2].dat.di_uio2,mask,maxbits=16,lab=lab,$
		chars=cs,ncs=ncs,font=font,$
	psym=sym,$
    xtitle='hour',title=title +  ' UIO digital 2 (page 5)'
    if useflag then flag,fl,linestyle=fls
     if keyword_set(wait)  then begin
        print,'hit return to continue'
        wait,1
        key=checkkey(/wait)
    endif
  endif
;--------------------------------------------------------
; page 6 uio digital  input 3 
;
;
    if (page eq 0 ) or (page eq 6) then begin
    if nwin gt 0 then wset,win[wini]
    wini=(wini+1 ) < (nwin-1)
    lab=turlab('di3')
    mask='1fcf'XL 
    pltbits,tm,tur[i1:i2].dat.di_uio3,mask,maxbits=16,lab=lab,$
		chars=cs,ncs=ncs,font=font,$
	psym=sym,$
    etitle='hour',title=title +  ' UIO digital 3 (page 6)'
    if useflag then flag,fl,linestyle=fls
     if keyword_set(wait)  then begin
        print,'hit return to continue'
        wait,1
        key=checkkey(/wait)
    endif
  endif

;
;--------------------------------------------------------
; page 7 digital out LS and uo_2_3
;
;
    if (page eq 0 ) or (page eq 7) then begin
    if nwin gt 0 then wset,win[wini]
    wini=(wini+1 ) < (nwin-1)
    lab=turlab('dols_2')
    mask='1ffc0'XL  
	ival=(tur[i1:i2].dat.do_lsuio1*1L and 'fff'xl) or $
           ishft((tur[i1:i2].dat.do_uio2_3*1L) and '7c'xl,10)
    pltbits,tm,ival,mask,maxbits=17,lab=lab,$
		chars=cs,ncs=ncs,font=font,$
    psym=sym,$
    xtitle='hour',title=title +  ' digOut:littleStar, uio2_3 (page 7)'
    if useflag then flag,fl,linestyle=fls
     if keyword_set(wait)  then begin
        print,'hit return to continue' 
        wait,1 
        key=checkkey(/wait)
    endif
  endif
;--------------------------------------------------------
; page 8  driveStat master,slave cram togethter
;  - each drive 9 bits ..dcd bits 0,1
;      decode enable,disable,pwr,flt
;      shift up to make room for decoded value
;  - fault stat put at top
;    push pwrSfail,BrkFail together.
;
;
    if (page eq 0 ) or (page eq 8) then begin
    if nwin gt 0 then wset,win[wini]
    wini=(wini+1 ) < (nwin-1)
    lab=turlab('drv_stat')
;
;   get master, slave stats
;
	ivalM=tur[i1:i2].dat.st_mdrv
	ivalS=tur[i1:i2].dat.st_sldrv
	stBitsM=ivalM and '3'X
	stBitsS=ivalS and '3'X
	ivalM= ishft(ivalM and '7C'X,2) or 2^stBitsM
	ivalS= ishft(ivalS and '7C'X,2) or 2^stBitsS
	ival= ishft(ivalS,9) or ivalM 
    mask='3ffff'X  
    pltbits,tm,ival,mask,maxbits=18,lab=lab,psym=sym,$
		chars=cs,ncs=ncs,font=font,$
    xtitle='hour',title=title + ' master, slave Drive  Status (page 8)'
    if useflag then flag,fl,linestyle=fls
    if keyword_set(wait)  then begin
        print,'hit return to continue' 
        wait,1
        key=checkkey(/wait)
    endif
    !p.multi=0
  endif
;
;--------------------------------------------------------
; page 9  faults
;
;
    if (page eq 0 ) or (page eq 9) then begin
    if nwin gt 0 then wset,win[wini]
    wini=(wini+1 ) < (nwin-1)
    lab=turlab('faults')
;
;   get master, slave stats
;
	ival=tur[i1:i2].dat.st_fault
    mask='1ff'X  
    pltbits,tm,ival,mask,maxbits=9,lab=lab,psym=sym,$
		chars=cs,ncs=ncs,font=font,$
    xtitle='hour',title=title + ' faults (page 9)'
    if useflag then flag,fl,linestyle=fls
    if keyword_set(wait)  then begin
        print,'hit return to continue' 
        wait,1
        key=checkkey(/wait)
    endif
    !p.multi=0
  endif
;

;--------------------------------------------------------
;   page 10 motor torques
    if ((page eq 0 ) or (page eq 10)) then begin
    if nwin gt 0 then wset,win[wini]
    wini=(wini+1 ) < (nwin-1)
    mlabln = 2.5
    mlabscl=.7
    mlabxp1=.02
    mlabxp2=mlabxp1 + .1
    mlabxpb=mlabxp1 + .2
    mlabcs=1.3
	tqM=tur[i1:i2].dat.AI_mampcurmon * cntToCurI
	nn=n_elements(tqM)
	tqS=tur[i1:i2].dat.AI_sampcurmon * cntToCurI
	tq=reform([tqM,tqS],nn,2)
!p.multi=0
	if n_elements(vtq) eq 2 then begin
			ver,vtq[0],vtq[1]
	endif else begin
		vmin=min(tq)
		vmin=(vmin lt 0)?1.1*vmin:.9*vmin
		ver,vmin,max(tq)*1.01 
	endelse
	if n_elements(tqsmo) eq 0 then tqsmo=0
	stripsxy,tm,tq,0,tqinc,/step,xtitle='hour',ytitle='Torque Amps',$
		chars=cs,csn=ncs,font=font,$
			title=title + ' Motor torques page 10',smo=tqsmo
	if useflag then flag,fl,linestyle=fls
;
; 		label motors
;
		nmot=n_elements(mot)
	   lab=['Master',"Slave"]
	   for i=0,1 do begin
         note,mlabln+i*mlabscl,lab[i],xp=mlabxp1,color=colph[i +1],$
                charsize=mlabcs&$
       endfor
	endif
;
    !p.multi=0
    return
end
