;+
;NAME:
;usrp_iobuf_get - read raw data from usrp dataset
;SYNTAX: istat=usrp_iobuf_get(fI,nsmp,buf,pI)
;ARGS
; fI:  {}   File info from  usrp_iobuf_init()
;nsmp: long number of samples to return in buf
;buf[nsmp]:complexarr  return data here. User should preallocate
;RETURNS:
; istat: int    
; pI : {}   position info for this databuf
; buf: complex buffer they want
;
; istat: 1  nsmp samples  data returned in buf
;        2  pI.smpRet < nsmp returned
;           (pI.smpRet is always the actual number returned)
;        0  hit end of dataset, no data left
;       -1  error returned from usrp_iobuf_inpfile()
;buf[] :complexarr   return data. The number of samples =pI.smpRet
;                 the user should preallocate this array.
;pI    : {}  structure holding info on the returned buffer
;DESCRIPTION:
;	Return the next data from the usrp dataset.
;The data is returned in the complex arr buf[]. The user requests
;nsmp samples. Reads can span files and directories. 
;The actual number of samples  returned will be recorded in pI.smpRet.
;The routine can return fewer than the requested samples if the data would end up 
;being non contiquous in time.
;	The user must call usrp_iobuf_init() once to initialize fI, before
;calling this routine.
;   Position info on the returned buf is set in pI:
;IDL> help,pI,/st
;   IDIR            LONG        0   .. index into fI.dI[] for directory for this buf. 0 based
;   IFILE           LONG        0   .. index for file in directory. 0 based
;   ISMP            LONG        0   .. sample within file. 0 based
;   SMPRET          LONG  1048576   .. number of samples returned in buf[]
;   MFILE           INT         0   .. multiple files. set to 1 if buf crossed file boundary.
;   TMJUMP          INT         0   .. 1 if this buffer is discontinuous in time from the
;                                      previous buffer.
;See usrp_iobuf_init() for a description of fI.
;-
function usrp_iobuf_get,fI,nsmp,buf,pI
;
;  return info for start of buffer.
;
	nsmpI=long(nsmp + .5) ; in case float
	tmJump=0 
	pI={idir : 0l,$; directory index start of this buf
	    ifile: 0l,$; file index start of this buf
	    ismp : 0l,$; smp index start of this buf
		smpRet:0l,$; may be less if we hit a time jump
		mfile:0,  $ ;  1 if this buf spans multiple files
		tmJump:0}   ; true if this buf is discontinuous in time from
;                     previous
	start=1

	smpNeeded=nsmpI
	ib0=0l			; index into return buf
	while (smpNeeded gt 0L) do begin
		smpLeftInBuf=fI.smpInBuf - fI.ismp
		if smpLeftInBuf le 0 then begin
			istat=usrp_iobuf_inpfile(fI)
			if istat lt 0 then return, -1
			fi.tmJump=(fI.tmJump or tmJump)
			if not start then pI.mfile=1
			if istat eq 0 then begin
				return,(pI.smpRet eq 0)?0:(pI.smpRet eq nsmpI)?1:2
			endif
			smpLeftInBuf=fI.smpInBuf
		endif
		if (start) then begin
			pI.idir=fI.idir
    		pI.ifile=fI.ifile
    		pI.ismp =fI.ismp
			start=0
		endif
	
		i0=fI.ismp		; first smp
		smpUse=(smpNeeded <  (fI.smpInBuf-fI.ismp))
		if (smpUse eq smpNeeded) then begin
			i1=i0+smpUse-1l
			buf[ib0:ib0+smpUse-1l]=fI.buf[i0:i1]; this empties fi.buf
			smpNeeded=0l
		endif else begin
			i1=fI.smpInBuf -1
			smpUse=i1 - i0 + 1l
			buf[ib0:ib0+smpUse-1l]=fI.buf[i0:i1]
			smpNeeded-=smpUse
		endelse
		ib0+=smpUse
		fi.ismp+=smpUse
		pI.smpRet+=smpUse
	endwhile
	return,(fI.smpInBuf eq fI.maxSmpFile)?1:2
end
