;+
;NAME:
;usrp_iobuf_init - initialize for usrp_iobuf i/o
;SYNTAX: istat=usrp_iobuf_init(dirl,fI)
;ARGS:
;dirL[]: strarr array of directories holding the data
;               they should be in time order.
;RETURNS:
;fI    : {}     structure holding the file info for the
;               iobuf routines
;DESCRIPTION:
;	The usrp_iobuf_xxxx() routines will read raw data from 
;the usrp data files. The routines will read across files and directories.
;To use the routines:
;	- file_search('/media/USRPyymmdd/gr/*,/test_dir)
;     get a list of directories to read from
;   - istat=usrp_iobuf_init(dirl,fI) 
;     call this routine to initialize things
;   - loop calling istat=usrp_iobuf_get(fI,buflen,buf,pI)
;	The status info for each buf comes back in pI structure.
;
;	The routines read and buffer a 1 second file at a time.
;The user then requests data. It will be returned from the buffer, and 
;another file will be read if needed to complete the sample request.
;
;	If a buffer request would create a gap in time (needs a new file to 
;complete the buf, and the file is not the next second),then the
;routines return the samples left before the gap. The next call
;will set a flag saying that this new buffer is a time jump from
;the previous buffer.
;
;	The fI struct contains:
;** Structure <21c1708>, 12 tags, length=200000320, data length=200000314, refs=1:
;   NDIR         LONG       11   .. number of directories in dirl=[]
;   MAXFILESDIR  LONG       3600 .. each dir is max of 3600 1 sec files
;   MAXSMPFILE   LONG       25000000 .. 1 sec of data at 25 Mhz
;   DI           STRUCT -> <Anonymous> Array[11]. info on each directory
;   IDIR         LONG          0     index into fi.dI[idir] current dir. 0based.
;   IFILE        LONG          0     index into file in fi.dI[idir] directory. 0based   
;   FNAME        STRING    '/media/USRPJUHA/eregion/gr/2018-05-04T14-35-46/rf@15'... cur file
;   TMSEC        LONG      1525444546  file time start of buf
;   ISMP         LONG      1048576     index for next sample in buf
;   SMPINBUF     LONG      25000000    samples in buf
;   TMJUMP       INT       0           if 1 this buf has time jump from previous.
;   BUF          COMPLEX   Array[25000000]
;
; The struct array fI.dI[] contains:
;** Structure <19ca4f8>, 3 tags, length=24, data length=24, refs=2:
;  DIRN            STRING    '/media/USRPJUHA/eregion/gr/2018-05-04T14-35-46'.. dir name
;  SEC1            LONG        1525444546                    .. first sec in dir
;  NFILES          LONG              3600                    .. number of files in dir
;-
function usrp_iobuf_init,dirl,fI
;
	smpInUsec= 25l
	maxSmpFile=smpInUsec*1000000l
	maxFilesD=3600l  ;   1 sec files
    ndir=n_elements(dirL)
    dI1={ $
          dirN:'' ,$  ; name of directory   
          sec1: 0L,$  ; secs midnite first file
          nfiles:0l$ ; number of files in directory
    }
;
; file info
; 	
	fI={ndir : ndir,$;  number of directories 
		maxFilesDir:3600l,$ ; 1 hour /dir
		maxSmpFile:maxSmpFile,$ ; 25000000
        dI   :replicate(dI1,ndir),$ ; directory info
        idir : 0l,$; index of dir for current buf
        ifile: 0l,$; index of file for current buf
		fname: '',$; filename where buf data came from
		tmSec:0l,$;  sec 1970 this file. used to test for jumps
		ismp : 0l,$; index for next sample in buf to return
		smpInBuf:0l,$; maybe less than max if last file of set
		tmJump:0 ,$; if true then this buf was a time jump from previous
		buf  :complexarr(maxSmpFile)};
;
;  initialize the directory/file info
;
	for idir=0,ndir-1 do begin
        fI.dI[idir].dirN=dirL[idir]
;       get the files in dir
        flist=file_search(dirL[idir]+'/rf@*')
        fI.dI[idir].nfiles=n_elements(flist)
;        get the start time first file.. used to generate filenames
        fbase=basename(flist[0])
        pat='rf@([0-9]+)'
        a=stregex(fbase,pat,/extr,/sub)
        fI.dI[idir].sec1=long(a[1])
	endfor
;
; 	initialize the buffer to the first file
;
	istat=usrp_iobuf_inpfile(fI,/firsttime)
	if istat ne 1 then begin
		print,"usrp_iobuf_init,usrp_iobuf_inpfile returned:",istat
		return,-1
	endif
	return,1
end
