;+
;NAME:
;usrpclpavgspc - average clp spectra files
;SYNTAX: nout=usrpclpavgspc(inpdir,outdir,yyyymmddst,hhmmddst,toavg,toskip)
;ARGS:
;inpdir: string   directory to read spectra
;outdir: string   directory to write spectra
;yyyymmddst:long  year,month,date to start
;hhmmssst  :long  hour,min,sec to start
;toAvgInCycle :long  number of files in cycle the we can average
;toSkipInCycle:long    number of files inc cycle after the avg to skip 
;toavg     :long   files to avg
;RETURNS:
;nout      : >= 0  number of files output
;          -1 error
;DESCRIPTION:
; 	Average usrp spectra file together.
;The user specifies the input directory and the output directory for the data.
;They must both exist before calling this routine.
;The start date,time to begin averaging is also input.
;	This is mainly used to average heater on or heater off spectra.
;heating typically uses an on off cycle:
; eg:  1 3secOn, 1 3secOff  ( 2 files in cycle)
;      1 1 secOn, 3 1 secOffs (4 files in cycle)
; You specify the cycle length vai toavg and toskip number of files
;The routine will avg toavg files and then output the averaged spectra. The averaging 
;can span multiple cylces.
;	Note that the start time is the first file to avg. If you have on,offs and you want to avg
;the offs, you need to specify the start time of the first off (not the first on).
; 	Once it starts averaging, it will go to end of directory... maybe want to add
;a stop time parameter?
;-
function usrpclpavgspc,inpdir,outdir,yyyymmddSt,hhmmssSt,toAvgInCycle,toSkipInCycle,toAvg,$
			verb=verb
;
	if n_elements(verb) eq 0 then verb=0
	sufHdr=".1.hdr"
	sufDcd=".1.dcd"
	inpDirL=inpDir
	outDirL=outDir
;   make sure end in /
	if (strmid(inpDirL,0,1,/reverse) ne '/') then inpDirL+='/'
	if (strmid(outDirL,0,1,/reverse) ne '/') then outDirL+='/'
	if file_test(inpDirL) ne 1 then begin
		print,"Inpdir:",inpDirL," does not exist"
		return,-1
	endif
	if file_test(outDirL) ne 1 then begin
		print,"outDir:",outDirL," does not exist"
		return,-1
	endif
;
;   get list of files in inpdir	
;
	flist=file_search(inpdirL + "*.hdr")
;
	secSt=long(tosecs1970(yyyymmddSt,hhmmssSt) +.5)  + 4*3600l
	nfileInCycle=toAvgInCycle + toSkipInCycle
	; get the file times
	a=stregex(flist,".*usrp/([0-9]*)",/sub,/extr)
; 
	fnum=reform(long(a[1,*]))
	; get how long files are.. assume first two are contiguous
	tmFileSec=fnum[1]-fnum[0] ; assume contiguous
; 	skip files before start time.
	ii =where(fnum ge secSt,nfiles)
	fnum=fnum[ii]
	okAvg=intarr(nfiles)
	; use the file times to verify the start of each cycle. 
	cycleTm=tmFileSec*(toAvgInCycle + toSkipInCycle)
	tmpTm=fnum - fnum[0]
; 	get index for start of each cycle
;
	iicycle=where((tmpTm mod cycleTm) eq 0,ncycle)
;   we want to select only the files that we average, we throw out the others
;   use the time stamps to try and catch any time jumps
	for i=0,ncycle-1 do begin &$
		i0=iicycle[i] &$
		t0=tmpTm[i0] &$
		; mark avg in cycle to include
		for j=0,toAvgInCycle-1 do begin &$
			if tmpTm[i0 + j] lt (t0 + toAvgInCycle*tmFileSec) then okAvg[i0+j]=1 &$
		endfor &$
	endfor
; now just grab the files to avg
	ii=where(okAvg eq 1,filesToUse)
	fnum=fnum[ii]
	tmpTm=tmpTm[ii]
;
; get a header file.. to get the size of the arrays
; if gaps, just integrate across them..
;
	fhdr=inpdirL + string(format='(i010)',fnum[0]) + sufHdr
	istat=usrpclprdhdr(fhdr,hdr)
	if istat ne 0 then begin
		print,"Error reading 1st hdrfile:",fhdr
		return,-1
	endif
	blklen=hdr.fftlen * hdr.nhghts
	inpDat=fltarr(blklen)
	avgDat=fltarr(blklen)
	nout=0l
 	for i=0,filesToUse-1,toAvg do begin
		gotFirstHdr=0 
		nadd=0l
		for j=0,toAvg-1 do begin
			if (j+i) ge filesToUse then goto,done
			if verb ne 0 then print,"inp:",i,j,fnum[i+j]
	 	    fbaseInp=inpdirL + string(format='(i010)',fnum[i+j])
			fhdrInp=fbaseInp + sufhdr
			fdcdInp=fbaseInp + sufdcd
			openr,lun,fdcdInp,/get_lun
			readu,lun,inpDat
			free_lun,lun
			istat=usrpclprdhdr(fhdrInp,hdr)
		    if gotFirstHdr eq 0 then begin
			   hdrOut=hdr
			   avgDat=inpDat
			   gotFirstHdr=1
			   fnumOut=fnum[i+j]
			endif else begin
			  hdrOut.nippsaccum+=hdr.nippsaccum
			  avgDat+=inpDat
			endelse
			nadd++
	   endfor
	   if nadd gt 0 then begin
		 fbaseOut=outDirL + string(format='(i010)',fnumOut)
		 fdcdOut=fbaseOut + sufDcd
		 fhdrOut=fbaseOut + sufHdr
		 if verb ne 0 then print,"  out:",fnumOut
		 openw,lunout,fdcdOut,/get_lun
		 if nadd gt 1 then avgDat/=nadd
		 writeu,lunout,avgDat 
		 free_lun,lunout
		 istat=usrpclpwrhdr(fhdrout,hdrout)
		 nout++
    	endif
	endfor
done:
return,nout
end
