;+
;NAME:
;usrpclpimg- make image of decoded clp file
;SYNTAX:istat=usrpclpimg,file,img=img,hdr=hdr,$
;			bpcind=bpcind,medlen=medlen,cfr=cfr,useimg=useimg,
;			nsig=nsig,zx=zx,zy=zy,title=title,nhghts=nhghts,$
;           wintouse=wintouse,freqToUse=freqToUse,rangeToUse=rangeToUse,fast=fast,combine=combine,$
;           maxFreq=maxfreq,freqsmo=freqsmo,tmI=tmI,freqAr=freqAr
;ARGS:
;file: string	    filename holding decoded info (xxx.dcd)
;
;KEYWORDS:
;bpcInd[2]:long     first last height indices to use for bpc
;                   default last 100 hghts
;medlen:   long     length median filter each spc. default=71 channels
;cfr : float        cfr in Mhz for band. 
;useimg:            if set then user is passing in img. useit
;nsig[2]: float		for scaling the image. default=[-nsig[0],nsig[1]]*sig
;                   def. 6 sigma.         
;zx  : int   	    scaling for x axis (neg is smaller)
;zy  : int   	    scaling for y axis (neg is smaller)
;nhghts:int         limit to this number of heights (def all)
;title: string      title
;wintouse: int      window number to use for display. Default=1
;freqToUse[2] :float freq range to use (in Mhz). Currently combine does not work with this
;rangeToUse[2]:float ranges to use (in km). overrides nghts
;                    [0.,x] will start with first range
;                    [x,999] will end with last range
;fast         :int   if supplied then compute median bandpass to subtract
;                    every fast rows
;combine      :int   if set then combine up and downshifted spectra
;                    note: if you set /useimg, you still need to set
;                    this to reflect how the image was made (so we get the
;                    freq correct).
;maxFreq      :float 0..12.5 Mhz .. max freq offset to display
;                    default is 12.5 Mhz, <= 0 --> use default
;freqsmo      :int   number of freq chan to smooth,decimate before output.
;                    the medlen length above will still be relative to the original 
;                    number of chan.
;                    zx should be relative to the smoothed/decimated value
;
;RETURNS:
;
;istat:  0 ok, -1 error
;img[*,*]:  float selected channel image. this is the image from the file
;                 before max freq selection or smoothing/decimation
;hdr: {}    struct from .hdr file
;tmI:{}     struct holding tming info.. read, median filter
;freqAr[]: float   freq of each channel in img
;
;DESCRIPTION:
;	Make a dynamic spectra image of decoded usrp file (.dcd).
;Take info from the .hdr file (it should be in the same directory
;as the .dcd file).
;-
;
function usrpclpimg,dcdfile,img=img,hdr=hdr,$
            cfr=cfr,useimg=useimg,$
			bpcind=bpcind,medLen=medlen,$
			nsig=nsig,zx=zx,zy=zy,title=title,border=border,$
			nhghts=nhghts,wintouse=wintouse,freqToUse=freqToUse,rangeToUse=rangeToUse,$
			fast=fast,combine=combine,maxFreq=maxFreq,freqsmo=freqsmo,tmI=tmI,freqAr=freqAr
;
	tmStart=systime(/sec)
	if n_elements(fast) eq 0 then fast=1
	usecToKm=.15
	double=1	; use double for meanrob in imgdisp
	if n_elements(wintouse) eq 0 then wintouse=1
	csn=1.5
	if n_elements(nsig) eq 0 then nsig=6
	if n_elements(title) eq 0 then title='' 
	if n_elements(cfr) eq 0 then cfr=430.
	if n_elements(border) eq 0 then border=75
	if n_elements(medlen) eq 0 then medlen=71L
	if n_elements(useimg) eq 0 then useimg=0
	if n_elements(combine) eq 0 then combine=0
	useSmo=(n_elements(freqsmo) gt 0)
	if useSmo then useSmo=(freqSmo[0] gt 1)
    basenm=basename(dcdfile)
	len=strlen(dcdfile)
	hdrfile=strmid(dcdfile,0,len-3) + "hdr"
	if (usrpclprdhdr(hdrfile,hdr) ne 0) then return,-1
	fftlen=hdr.fftlen
	if (useSmo) then begin
		if (fftlen mod long(freqSmo)) ne 0 then begin
		  print,"Err: freqsmo length must divide into fftlen:",fftlen
		  return,-1
		endif
	endif
	nhghtsTot=hdr.nhghts
	bw=1./(hdr.smpTmUsec)
;   get the initial freq array of the data.
	cenChan=fftlen/2l - 1 ;	 since we flipped the original data
	chanWd=bw/fftlen
	fftlen0=fftlen
	freqAr=(findgen(fftlen) - cenChan)*chanWd  + cfr
;   if combine, we take dc bw/2.. be careful with center channel.
	if (combine) then begin
		fftlen0=fftlen/2l
		freqAr=freqAr[cenChan:cenChan + fftlen0-1]
	endif
	if n_elements(nhghts) eq 0 then nhghts=nhghtsTot
	if nhghts le 0 then nhghts=nhghtsTot
	hghtsToUseL=long([1,nhghts] + .5)
	ih0=hghtsToUseL[0]-1
    ih1=hghtsToUseL[1]-1
;	they specified the freq range to use
	if0=0
	if1=fftlen0-1
	selectFreq=n_elements(freqToUse) eq 2
	if selectFreq then begin
		if (useSmo) then begin
		   print,"Error: smoothing currently not supported when selecting freq range"
		   return,-1
		endif
		if (combine) then begin
			ii=where(freqToUse  lt cfr,cnt)
			if cnt gt 0 then begin
				print,"Error:When using combine bands, the freqToUse must be >= ",cfr
				return,-1
			endif
		endif 
		ii=where((freqAr ge freqToUse[0]) and (freqAr le freqToUse[1]),cnt)	
		if cnt le 0 then begin
				print,"Error:freqToUse outside allowable band:",min(freqAr),max(freqAr)
				return,-1
		endif
		if0=ii[0]
		if1=ii[cnt-1]
	endif
;
	if n_elements(rangeToUse) gt 0 then begin
		rangeTot=([0L,nhghts-1]*hdr.hghtresusec  + hdr.hghtdelayusec)$
				 *usecToKm
		rangeToUseL=rangeTot
		if n_elements(rangeToUse) eq 1 then $
			rangeToUseL=[rangeToUse,rangeTot[1]]
		if n_elements(rangeToUse) eq 2 then rangeToUseL=rangeToUse
;       compute indices relative to first height
		aa=(rangeToUseL/usecToKm - hdr.hghtdelayusec)/$
				hdr.hghtresusec
		ih0=long((aa[0] > 0L) + .5)
		ih1=long((aa[1] > 0L) + .5) < (nhghtsTot-1)
		hghtsToUseL=[ih0,ih1]+1L
	endif
		
	nhghts=hghtsToUseL[1]-hghtsToUseL[0] + 1L
	if (nhghts lt 0) or (nhghts gt hdr.nhghts) or $
		(hghtsToUseL[0] lt 1) or (hghtsToUseL[1] gt hdr.nhghts)$
		then begin
		if n_elements(rangeToUse) gt 0 then begin
			print,"rangeToUse illegal. Valid vales are:",rangeTot
		endif else begin
			print,"illegal number heights requested. maxval:",$
		        hdr.nhghts
	    endelse
		return,-1
	endif
	tmRd=0.
	if (not useimg) then begin
;
;	bpc should always come from end of array? 
;
		if n_elements(bpcind)  ne 2 then begin
			; be careful with signal pulse typically 175-185 hghts
			if hdr.nhghts gt 1000 then begin
				bpcInd=[-100,-1] + hdr.nhghts
			endif else begin
				; normally 100./3960 = .025

				aa=long(hdr.nhghts * .025) > 10
				bpcInd=[-aa,-1] + hdr.nhghts
		    endelse
		endif

		err=0
		openr,lun,dcdfile,error=err,/get_lun
		if err ne 0 then begin	
			printf,-2,!ERROR_STATE.MSG
			return,-1
		endif
;
;       assume spectra always flipped... 
;        1    8191                 8192
;       |dc|reverseNegFreq..8191|revPosFreq|
;       revPosFreq|dc|revNegFreq|    shiftr by 8192
;       then reverse
;       negfreq|dc|posfreq
		img=fltarr(fftlen,nhghtsTot)
		tm1=systime(/sec)
		readu,lun,img
		tmRd=systime(/sec)- tm1
		free_lun,lun
		tm1=systime(/sec)
		if not combine then begin
			img=shift(img,fftlen/2L,0)
		    img=reverse(img,1)
		endif else begin
			i1=fftlen/2L -1 ; last neg poitn
 			i2=i1+2         ; first pnt 2nd half...
		; img has dc,reverseNegFreq
		;    reverse pos freq upper halfsneg pnt at 8192 is extra 
			img[i2:*,*]= reverse(img[i2:*,*],1)
		endelse

		bpc=total(img[*,bpcind[0]:bpcind[1]],2)/(bpcind[1]-bpcind[0] +1.)
		bpc=reform(bpc,fftlen)

;
;	now extract the heights that actually want to display
;
		ih0=hghtsToUseL[0]-1
		ih1=hghtsToUseL[1]-1
		if nhghts ne nhghtsTot then begin
			img=img[*,ih0:ih1]
		endif
	    bpcScl=1./bpc[*]
 		; combine upshifted, downshifted
	    if (combine) then begin
			img1=fltarr(fftlen0,nhghts)
			for ih=0,nhghts-1 do begin
				y=img[*,ih]*bpcScl			; band pass correction
				if medlen gt 1 then $ y-=median(y,medlen)         ; remove baseline
                ; add 2 hakves
				img1[*,ih]    = y[0:i1] 
				img1[1:i1,ih]+= y[i2:*]
				img1[0,ih]*=2.             ; dc scale by 2
			endfor
			img=img1
			img1=''
		endif else begin
			if (fast gt 1 )then begin
				midval=fast/2
			    if (medlen gt 1) then begin
					medbp= median(img[*,midval],medlen)
			    endif else begin
					medbp= img[*,midval]
				endelse
				for ih=0,nhghts-1 do begin
					img[*,ih]*=bpcScl
					if (ih mod fast) eq midval then begin
				 		print,midval
						medbp=(medlen gt 1)? median(img[*,ih],medlen):img[*,ih]
					endif
					img[*,ih]-= medbp
				endfor
			endif else begin
				; if selectFreq, try to speed things up.. don't median filter the entiree array..
				if selectFreq then begin
				   ; limit to medlen/2 beyond freq they wanted..
				   ij0=(if0 - medlen/2) > 0l
				   ij1=(if1 + medlen/2) < (fftlen0-1)
				   for ih=0,nhghts-1 do begin
					   img[ij0:ij1,ih]*=bpcScl[ij0:ij1]
				       if medlen gt 1 then $
					  		img[ij0:ij1,ih]-= median(img[ij0:ij1,ih],medlen)
				    endfor
				endif else begin
				for ih=0,nhghts-1 do begin
					img[*,ih]*=bpcScl
					if medlen gt 1 then $
					 img[*,ih]-= median(img[*,ih],medlen)
				endfor
				endelse
			endelse	
		endelse
	endif
	if medlen lt 1 then img-=1.
	yr=([ih0,ih1]*hdr.hghtresusec  + hdr.hghtdelayusec)*.15
	if (useSmo) then begin
		imgsmo=reform(total(reform(img,freqsmo,fftlen0/freqsmo,nhghts),1)/freqsmo)
		fftlen0=fftlen0/freqSmo
	endif
	useMaxFreq=(n_elements(maxFreq) gt 0) and (not selectFreq)
	if (useMaxFreq) then useMaxFreq=(maxFreq*2 ge bw)?0:1
	if (useMaxFreq) then useMaxFreq=(maxFreq le 0)?0:1
	if (not useMaxFreq) then begin		
		if (selectFreq) then begin
			i0=if0
		    i1=if1
		endif else begin
			i0=0
			i1=fftlen0-1
		endelse
		xr=[freqAr[i0],freqAr[i1]]	
	endif else begin
		bwScl=MaxFreq/(bw/2.)
		if (combine) then begin
			i0=0
			i1=long(fftlen0 *bwScl + .5) - 1l
;			xr=[0,1.]*bw*bwScl + cfr
			xr=[freqAr[i0],freqAr[i1]]
		endif else begin
			ii=where(abs(freqAr - cfr) le maxFreq,cnt)
			i0=ii[0]
			i1=ii[cnt-1]
			xr=[freqAr[i0],freqAr[i1]]
		endelse
	endelse
	tm2=systime(/sec)
	tmProc=tm2 - tm1
	tit0=" file:"+basenm;
	if (useSmo) then begin
		imgdisp,imgsmo[i0:i1,*],nsigclip=nsig,zx=zx,zy=zy,xr=xr,yr=yr,win=wintouse,$ &$
			border=border,xstyle=9,xtitle='Freq Mhz',ytitle='Range Km',$
			double=double
	endif else begin
		imgdisp,img[i0:i1,*],nsigclip=nsig,zx=zx,zy=zy,xr=xr,yr=yr,win=wintouse,$ &$
			border=border,xstyle=9,xtitle='Freq Mhz',ytitle='Range Km',$
			double=double
	endelse
	note,1.5,title + tit0,chars=csn
	tm1=systime(/sec)
	tmDisp=tm1 - tm2
	tmTot=tm1 - tmStart
	tmI={ rd:tmRd, proc:tmProc,disp:tmDisp,tot:tmTot}
	return,0
end
