;+ 
;NAME:
;usrpclpimg_hc- make hardcopy image of decoded usrp clp file
;SYNTAX:n=usrpclpimg_hc,inpDir,outDir,startTm=startTm,endTm=endTm,cfr=cfr,
;                  bpcind=bpcind,medlen=medlen,combine=combine,$
;                  maxFreq=maxFreq,freqsmo=freqsmo,$
;                 rangeToUse=rangeToUse,freqToUse=freqToUse,$
;                 nsig=nsig,zx=zx,zy=zy,title=title,nhghts=nhghts,$
;                 logtmfile=logtmfile,$
;				   noimg=noimg,nsig=nsig,ftype=ftype,useHdrArout=usehdrArout,hdrArOut=hdrArOut
;ARGS:
;inpDir: string     input directory of reduced files to process
;outDir: string     where to write the images
;
;KEYWORDS:
;startTm  :long     use files >= this time. default is first file
;endTm    :long     last time to include . default all
;cfr      :float    cfr for band. default=430
;bpcInd[2]:long     first last height indices to use for bpc
;                   default last 100 hghts
;medlen:   long     length median filter each spc. default=71 channels
;combine:  int      if set then combine upshifted, downshifted spectra
;nsig: float        for scaling the image. default=6 sigma
;zx  : int          scaling for x axis (neg is smaller)
;zy  : int          scaling for y axis (neg is smaller)
;title: string      title
;nhghts:int         limit to this number of heights. (<=0 --> all)
;maxFreq:float      0..12.5 Mhz .. max freq offset to use for image
;                   default is 12.5 Mhz
;rangeToUse[2]: float range in km to use for the image. overrides nhghts
;freqToUsep2]: float  freq range to use(MHz at 430) to use. Overrides
;                   maxfreq. If provided, smoothing not supported.
;freqsmo: int       number of freq channels to smo/decimate. zx should be
;                   relative to this final number of channels
;                   eg. maxFreq/25. * fftlen/freqsmo
;noimg: int         if set then don't make images, just return
;                   fiarout.. files we'll use 
;nsig[2]: float     clip to -nsig[0],nsig[1] sigmas
;ftype  : string    file type: gif,png,jpg. default=jpg
;useHdrArOut:int    if set then use hdrArOut as files to process.
;                   user passes this in
;verb   :  int       if set then  print number of files found to process
;logtmfile: string   file name to log times. '' --> no logging
;                   
;RETURNS:
;
;n :    >=0         number of images we made
;          -1       error
;hdrArOut[n]: {}    array of .hdr from each ,dcd file, (unless useHdrArOut 
;                   is set. in this case just pass back what is passed in.
;tmI:{}             times for i/o and bandpass/medianviltering..
;
;DESCRIPTION:
;   Make a dynamic spectra image of decoded usrp files (.dcd) and then write
;the graphics file to disc. Do this for all of the files in the inpdir subject
;to:
; - start at >= startTm
; - end   at <= endTm
;
;-
function usrpclpimg_hc,inpDirU,outDir,startTm=startTm,endTm=endTm,cfr=cfr,$
		 bpcind=bpcind,medlen=medlen,combine=combine,$
         nsig=nsig,zx=zx,zy=zy,title=title,nhghts=nhghts,hdrArOut=hdrArOut,$
		 maxFreq=maxFreq,freqsmo=freqsmo,$
		 noimg=noimg,useHdrArout=usehdrArout,ftype=ftype,verb=verb,$	
         logtmfile=logtmfile,$
		rangeToUse=rangeToUse,freqToUse=freqToUse,tmI=tmI
;
  forward_function usrpclpmonimgscan

	luntm=-1
	if n_elements(ftype) eq 0 then ftype='jpg'
	verbL=keyword_set(verb)?verb:0l
	ofileSuf=''
	useGif=0
	usePng=0
	useJpg=0
	if (ftype eq 'gif') then begin
		useGif=1
		ofileSuf='.gif'
	endif 
	if (ftype eq 'png') then begin
		usePng=1
		ofileSuf='.png'
	endif
	if (ftype eq 'jpg') then begin
		useJpg=1
		ofileSuf='.jpg'
	endif
	if ofileSuf eq '' then begin
		print,"Unknown output file type. allowable values: gif png jpg"
		if luntm gt 0 then free_lun,luntm
		return,-1
	endif
    if n_elements(nhghts) eq 0 then nhghts=-1
    if n_elements(startTm) eq 0 then startTm=0L 
    if n_elements(endTm) eq 0 then   endTm=2000000000L;
	startTm=(startTm > 0)
	if n_elements(zx) eq 0 then zx=-12
	if n_elements(zy) eq 0 then zy=-5
	if n_elements(logtmfile) eq 0 then logtmfile=''
	logTm=logtmfile ne ''
    winsToUse=1                     ; 1 or 2 simultaneous windows
	inpDir=inpDirU
	if strmid(inpDir,0,1,/reverse_offset) ne '/' then inpDir+='/'
	if not keyword_set(useHdrArout) then begin
    	nfiles=usrpclpimgmonscan(inpDir,hdrArOut,firsttime=1)
		if nfiles le 0 then begin
			if nfiles eq 0 then print,"no files found for",inpdir
			if luntm gt 0 then free_lun,luntm
			return,nfiles
		endif	
;
;   now find set of files they want to process
;
;	>= fnumstart <= fnumend
;
		ii=where((hdrArOut.filenum ge startTm) and $
		         (hdrArOut.filenum le endTm) ,nfiles)
		if nfiles eq 0 then begin
			print,"no files between fileNums::",startTm,endTm
			if luntm gt 0 then free_lun,luntm
			return,0
		endif	
		hdrArOut=hdrArOut[ii]
	endif
	if keyword_set(verbL) then print,"Found ",nfiles," .hdr files to process"
	if keyword_set(noimg) then begin
		if luntm gt 0 then free_lun,luntm
		return,0
	endif
;
;	now start making the images
;
	if logtm then openw,luntm,logtmfile,/append,/get_lun
		
	ldir=outdir
	if strmid(ldir,0,1,/reverse_offset) ne '/' then ldir+='/' &$
	nfiles=n_elements(hdrArOut)
	for ifile=0,nfiles-1 do begin
		starttm=systime(/sec)
		fnameB=string(format='(i10,".",a)',hdrArOut[ifile].filenum,hdrArOut[ifile].lblkinfile)
	    bw=1./hdrArOut[ifile].smpTmUsec
		ltm=fisecmidhms3(hdrArOut[ifile].secMid,hr,min,sec) &$
		lpos=string(format=$
			'(" pos(az,zagr,zach):(",f5.1,",",f4.1,",",f4.1,")")',$
			(hdrArOut[ifile].az mod 360.),$
			 hdrArOut[ifile].zaGreg,$
			 hdrArOut[ifile].zaCh)

        title=string(format='("date:",i08," tm:",a," naccum:",i04)',hdrArOut[ifile].date,ltm,hdrArOut[ifile].nippsaccum) $
							 + lpos
	    istat=usrpclpimg(inpDir+fnameB+".dcd",cfr=cfr,$
       			    zx=zx,zy=zy,title=title,nhghts=nhghts,wintouse=win,nsig=nsig,$
					bpcind=bpcind,freqsmo=freqsmo,$
					freqToUse=freqToUse,rangeToUse=rangeToUse,$
 					combine=combine,maxFreq=maxFreq,medlen=medlen,tmI=tmI)
		if (istat lt 0 ) then begin
			print,"usrpclpimg returned error processing:",inpDir+fnameB
			if luntm gt 0 then free_lun,luntm
			return,-1
		endif
		empty
		if logtmfile ne '' then begin
			lin=string(format='(d14.3," tot:",f7.2, " read:",f7.2," proc:",f6.2," disp:",f6.2)',$
				starttm,tmI.tot,tmI.rd,tmI.proc,tmI.disp)
			printf,luntm,lin
			flush,luntm
		endif
	    ofileName=fnameB + ofileSuf
	    if useGif then write_gif,ldir + ofileName,tvrd() 
		if usePng then write_png,ldir + ofileName,tvrd() 
		if useJpg then write_jpeg,ldir + ofileName,tvrd() 
		if verbL ge 2 then begin
			line=string(format='("tot:",f6.2, " read:",f6.2," proc:",f6.2," disp:",f6.2)',$
				tmI.tot,tmI.rd,tmI.proc,tmI.disp)
			print,line
			flush,-1
		endif
	endfor
	if luntm gt 0 then free_lun,luntm
;
	return,nfiles
end
