;+
;NAME:
;usrpcountpeaks - count peaks in ipps.
;SYNTAX  nipps=usrpcountpeaks(dir,subdir,si,indFile=indFile,nfiles=nfiles,$
;		minTxPwr=minTxpwr,ippusec=ippusec,rfusec=rfusec,$
;       kmAvg=kmAvg,limPk=limPk,kmPk=kmPk,verb=verb,var=var,har=har)
;ARGS:
; dir  : ''  base dir for usrp data
;subDir: ' ' subdir to process
;KEYWORDS:
;indFile: int file index in subdir to start on (count from 0)
;             def=0
;nfiles : int number of files (seconds) to process.  def=3600
;             or we hit end of directory
;minTxPwr: float  to define tx pulse. def=.8e6
;ippusec:  float  ipp in usec. def=10000.
;rfUsec :  float  rf in usec . def=440.
;kmAvg[2]: float  if supplied then avg all samples between these
;                 2 ranges. It will be stored for each ipp,.
;                 def= no averging done.
;limPk   : float  defines a peak. This is a power level.
;                 def=[1.5e6]
;kmPk[2] : float  search for peaks between these two ranges.
;                 def=[360,1200.]  km. lets you exclude fregion, cals.
;verb    : int    if > 0 the do plots every this many ipps.
;                 def: no plots.. A warning message will be output
;                 if verb  >0 and  < 100. it will slow down processing.
;har[2]  : float  horizontal scale if plottting. def:[0,10000.] usecs
;var[2]  : float  vertical scale if plotting. def:[0,3e6]
;RETURNS:
;nipps   : long   -1  error initializing dataset
;                 number of ipps returned in pkI. 
;si      : {}     si struct opened by usrpinitset. It is left
;                 pointed after the last read. (probably end of set).
;pkI[nipps]:{}    struct holding info for each ipp read.
;
;DESCRIPTION:
;	This routine will read data in a subdir, counting the peaks between
;the specified km range. It will also optionally average pwr over a hght
;range (giving a single number). This info will then be stored fore each
;ipp.
; the program flow is:
; - initialize a dataset (usrpinitset)
;     if si was previously used and has an lun open, it will be closed.
; - call usrpsync() to synchronize to the first rf pulse found
;   minTxPwr is used for this.
; - loop reading ipps until nfiles have been processed or we hit the end of
;   the subdir.
; For each ipp:
; - compute the power
;   search for levels above the limPk. save this count
; - optionally compute average over the specified height range.
;
;	This will continue until nfiles have been processed or we hit the 
;end of the  subdir.
;
; pkI[nipps] is returned with each entry containg:
;IDL>  help,pkI,/st
;** Structure <22cf808>, 5 tags, length=20, data length=20, refs=2:
;   FIND       LONG        0     ; file index in dir. count from 0
;   TIME       ULONG 1487867130  ; secs 1970. filename=rf@time.000.h5 
;   kmAvg      FLOAT    33587.7  ; power average over km range
;   PKCNT      LONG          0   ; samples above pkLim
;   BYTEPOS    ULONG    973196   ; in file start of ipp.
;
;the index of pkI[] is the continuous ipp number.
;You can use the bytepos to position into the file and then read
;the entire ipp. The bytepos includes the header offset.
;EXAMPLE:
; run on cpu (gpuserv0) that has access to the usrp raw data.
; @usrpinit
; dir='/media/USRP170223/ch/'
; subDir='2017-02-23T16-25-30' 
; indFile=0 ; def
; kmPk= [400,1000.]
; kmAvg=[120,300]
; verb=1000   ; print every 10 secs of data
;
; nipps=usrpcountpeaks(dir,subdir,si,indFile=indFile,nfiles=nfiles,$
;		minTxPwr=minTxpwr,ippusec=ippusec,rfusec=rfusec,$
;       kmAvg=kmAvg,limPk=limPk,kmPk=kmPk,verb=verb,var=var,har=har)
;- 
function usrpcountpeaks,dir,subdir,si,indFile=indFile,nfiles=nfiles,$
		minTxPwr=minTxpwr,ippusec=ippusec,rfusec=rfusec,kmAvg=kmAvg,$
		kmPk=kmPk,limPk=limPk,verb=verb,var=var,har=har
		

	expI={expI, rfUsec: 440D,$
               ippUsec: 10000D,$
	          minTxPwr:.8d6}

	if n_elements(mintxpwr) eq 1 then expI.minTxPwr=minTxPwr
	if n_elements(ippusec)  eq 1 then expI.ippusec =ippusec
	if n_elements(rfUsec)  eq 1  then expI.rfUsec  =rfUsec
	if n_elements(indFile) eq 0 then  indFile=0L
	if n_elements(nfiles) eq 0 then nfiles=3600l
	verbL=0
	if n_elements(verb) ne 0 then verbL=verb
	if (verbL lt 100) and (verbL gt 0) then begin
		print,"Warning: verb=",verbL," will plot every verb ipps"
		print,"This will slow things down.."
	endif
	doplot=verbL gt 0
;   km to avg to return on pulse value
	useKmAvg= n_elements(kmAvg) eq 2

	a={ find: 0l, $; file index into subdir
    	time: 0ul, $; name of file
	  tppuls:0.,$
	   pkCnt:0l ,$ ; peaks above threshold
     bytePos:0ul};start of ipp
;
;   free previous lun if open
;
	if (size(si,/type) eq 8) then begin
		if si.cfi.lun gt 0 then free_lun,si.cfi.lun
		si.cfi.lun = -1
	endif
	istat=usrpinitset(dir,sI,expI=expI,subDirSt=subDir,indFile=indFile)
	if istat ne 0 then begin
		print,"error initializing set:",istat
		return,-1
	endif
;	sync to rf pulse
	nsmp=usrpsynctorf(si)
	smpIpp=sI.smpIpp
	maxIpp=long(1d/(expI.ippusec*1d-6)*3600L)
	pkI=replicate(a,maxIpp)
	if useKmAvg then begin
		indAvg=(kmAvg/.15)*25
		nAvg=(indAvg[1]-indAvg[0]) + 1l
	endif
;
;	find where to search for peaks
;
	if n_elements(kmPk) eq 0 then kmPk=[360.,1200.]
	usecPk=kmPk/.15 
	if usecPk[1] gt expI.ippusec then usecPk[1]=expI.ippUsec - 1.
	indPk=long(usecPk*25.)
;
	if doPlot then begin
		x=dindgen(smpipp)/25.
		varL=(n_elements(var) eq 2)	?var:[0,3e6]
		harL=(n_elements(har) eq 2)	?har:[0,10000.]
		hor,harl[0],harl[1]
		ver,varl[0],varl[1]
	endif
	iCur=0L
	while (icur lt maxIpp) do begin
		pkI[icur].bytePos=si.cfi.curPos
		pkI[icur].time=si.cfi.time
		pkI[icur].find=si.cfi.find
		nsmp=usrpgetdata(si,smpIpp,d)
		if nsmp ne smpIpp then break
		tp=abs(d)^2
		ii=where(tp[indPk[0]:indpk[1]] gt limPk,cnt)
		pkI[icur].pkCnt=cnt
		if (useKmAvg) then begin
			pkI[icur].kmAvg=total(tp[indAvg[0]:indAvg[1]])/navg
		endif
	    
		if (doplot) and (icur mod verbL) eq 0 then begin
			tit=string(format=$
			'("ippCur:",i7," find:",i4," pkCnt:",i6," pulsPwr:",f9.1)',$          
			icur,pkI[icur].find,pkI[icur].pkCnt,pkI[icur].tpPuls)
			plot,x,tp,tit=tit,xtit='usecs',ytit='pwr'
		endif
		icur++
	endwhile
	if (icur lt maxipp) then pkI=pkI[0L:icur-1]
	return,icur
end
