;+
;NAME:
;windplotstorm - plot a days worth of wind data.
;SYNTAX: windplotstorm,d,rain=rain,maxvel=maxvel,day=day
;ARGS:
; d[n]:{}	wind data input by windinpraw()
;KEYWORDS:   
;  rain  : int  if true then include rainfall
; maxvel : float  max wind velocity to use for vertical scale
; day    :      if set , then just plot day, don't bother with timestamp
;RETURNS:
;		plots the wind data.
;DESCRIPTION:
;	Plot the wind data for a day. The top plot is the wind data versus hour 
;of data (ast). The data has bin binned to 1 minute steps. The white (black)
;plot is the average over each minute. The red plot is the peak hold over
;each minute.
;	The bottom plot is the median wind velocity for each minute versus 
;ast hour of day. The direction is where the wind is coming from.
;
;EXAMPLE:

;
; yymmdd=070412
; n=windinpraw(yymmdd,d)
; windplotday,d
;
;-
pro wstplotstorm,d,maxvel=maxvel,rain=rain,day=day

    common colph,decomposedph,colph

	cs=1.8
	userain=keyword_set(rain)
	if (userain) then cs=2.0
	csn=1.5
	font=1
;    maxaccel  =30.              ; max accel vel/sec^2 before we ignore data
    maxaccel  =300.              ; max accel vel/sec^2 before we ignore data
	vel=d.wiavg3sec.spd
    dir=d.wiavg3sec.dir
	velG10=d.wigust10min.spd
	accel=(vel - shift(vel,1))
    accel[0]=accel[1]
	ii=where( abs(accel) gt maxaccel,cnt)
	if cnt gt 0 then begin
 	 	d[ii].windspd=0
 	 	d[ii].windiradj=0
	endif
	vel=d.wiavg3sec.spd
    dir=d.wiavg3sec.dir
	velG10=d.wigust10min.spd
;
	
; figure out the date use middle of data string
;  
n=n_elements(d)
; jd0 is now ast date of center of date range
jd0=d[n/2].jd - 4./24D 
caldat,jd0,mon,day,year,hour0,min0,sec0
ldate=string(format='(i02,a,i02)',day,monname(mon),year mod 100L)
;
; convert to hours of jd0 day this jd to minutes
;
hr=((d.jd - 4D/24D) - jd0)* 24D + hour0 + (min0/60. + sec0/3600.)
;
velPk=velG10
dirMd=d.winddiradj
ii=where(velPk gt 0.,cnt)
;
colPk=2
vmax=(n_elements(maxvel) gt 0)?maxvel: (long(max(velpk)/5) + 1 )*5
ver,0,vmax
;
; plot velocity 
;
!p.multi=[0,1,3]
if userain then !p.multi=[0,1,4]
iimax=max(ii)
maxhr=hr[iimax]
ihr=long(maxhr)
imin=long((maxhr-ihr)*60)
lab=(keyword_set(day))?"":string(format='(" @:",i02,":",i02)',ihr,imin)
;
plot,hr[ii],vel[ii],$
	chars=cs,font=font,$
	xtitle='hour of day [AST]',ytitle='velocity [mph]',$
	title='Wind velocity for ' + ldate + lab
oplot,hr[ii],velG10[ii],col=colph[2]
xp=.02
ln=1.5
scl=.8
xpinc=.5
note,ln,'3 sec average',chars=csn,font=font,xp=xp
note,ln,'max wind gust(10min span)',chars=csn,font=font,col=colph[2],xp=xp+xpinc
n=n_elements(ii)
;  be careful with starting enw day.. rain gets reset to 0. just take max
lrain=string(format='("TotalRain:",f5.2," inches")',max(d[ii].rainToday))
note,ln+1*scl,lrain,chars=csn,font=font,col=colph[4],xp=xp
;
; pressure
;
ver
plot,hr[ii],d[ii].barpresadj,$
	chars=cs,font=font,$
	xtitle='hour of day [AST]',ytitle='Adjust pressure [inches]',$
	title='Barometric pressure (adjusted)'
;
; grab the dir data
;
dd=dirMd[ii]
; get rid of jumps of 360

ver,0,360
sym=3
tosmo=1
plot,hr[ii],smooth(dirMd[ii],tosmo),psym=sym,$
	chars=cs,font=font,$
	xtitle='hour of day [AST]',ytitle='Wind from direction [deg]',$
	title='Wind direction for ' + ldate
oplot,[0,24],[90,90],col=colph[2]
oplot,[0,24],[180,180],col=colph[3]
oplot,[0,24],[270,270],col=colph[4]
x=!x.range[0] + 1
y=90
yeps=5
csn=1.5
xyouts,x,y+yeps,'From the East',col=colph[2],charsize=csn,font=font
y=180
xyouts,x,y+yeps,'From the South',col=colph[3],charsize=csn,font=font
y=270
xyouts,x,y+yeps,'From the West',col=colph[4],charsize=csn,font=font
;
if (userain) then begin
rmax=fix(max(d[ii].raintoday) + 1.)
ver,0,rmax
plot,hr[ii],d[ii].raintoday,$
	chars=cs,font=font,$
	xtitle='hour of day [AST]',ytitle='Cumulative inches of rain',$
	title='cumulative rain for ' + ldate
endif
return
end
